define(['baja!','baja!electronicSignature:SecureFacet',
        'jquery',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/webEditors/rc/fe/baja/FacetsEditor'], function (
         baja,
         types,
         $,
         BaseEditor,
         FacetsEditor) {

  'use strict';


////////////////////////////////////////////////////////////////
// FacetsEditor definition
////////////////////////////////////////////////////////////////

  /**
   * A field editor for working with `Facets` values.
   *
   * A `FacetsEditor` supports the following `bajaux` `Properties`:
   *
   * - `allowAdd`: set to `false` to prevent new facets from being added.
   * - `allowChangeType`: set to `false` to prevent facet types from being
   *   changed.
   * - `allowDelete`: set to `false` to prevent facets from being removed.
   * - `allowRename`: set to `false` to prevent facet names from being changed.
   * - `showHeader`: set to `false` to prevent the header from being shown.
   * - `uxFieldEditors`: set this to a `baja.Facets` instance where the keys
   *   correspond to the keys in the edited `Facets`. The values should be
   *   type specs or RequireJS module IDs. This allows you to specify exactly
   *   what kind of editor to use to edit facets.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
   * @alias module:nmodule/webEditors/rc/fe/baja/FacetsEditor
   */
  var ESignFacetsEditor = function ESignFacetsEditor(params) {
    BaseEditor.call(this, $.extend({
      keyName: 'ESignFacetsEditor'
    }, params));
  };
  ESignFacetsEditor.prototype = Object.create(FacetsEditor.prototype);
  ESignFacetsEditor.prototype.constructor = ESignFacetsEditor;

  /**
   * Creates a table to hold the array of facet values.
   *
   * @param {jQuery} dom
   */
  ESignFacetsEditor.prototype.doInitialize = function (dom, params) {
    return FacetsEditor.prototype.doInitialize.call(this,dom);
   };


  /**
   * Loads the `Facets` into the editor, creating a table row for each key
   * in the `Facets`. Each row has field editors for key, type, and value.
   *
   * @param {baja.Facets} value
   * @returns {Promise} promise to be resolved when all the field editors
   * have finished loading into the DOM
   */

  ESignFacetsEditor.prototype.doLoad = function (value,params) {
    return FacetsEditor.prototype.doLoad.call(this,value);
  };

  return ESignFacetsEditor;
});
