/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * PointListEditor manages functionality to manage secure point list.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/PointListEditor
 */
define(['baja!', 'baja!electronicSignature:SecureString',
    'lex!baja',
    'jquery',
    'Promise',
    'dialogs',
    'bajaux/Widget',
    'bajaux/mixin/subscriberMixIn',
    'hbs!nmodule/electronicSignature/rc/fe/template/PointListTemplate',
    'nmodule/electronicSignature/rc/fe/baja/loader',
    'nmodule/webEditors/rc/fe/baja/BaseEditor',
    'nmodule/electronicSignature/rc/wb/mgr/editor/CustomPointSelectorEditor',
    'nmodule/electronicSignature/rc/wb/mgr/commands/ZoneEditCommand',
    'nmodule/electronicSignature/rc/wb/mgr/commands/ZoneAddCommand',
    'nmodule/webEditors/rc/fe/feDialogs',
    'lex!electronicSignature',
    'css!nmodule/electronicSignature/rc/css/widgets'], function (baja,
                                                             types,
                                                             lexs,
                                                             $,
                                                             Promise,
                                                             dialogs,
                                                             Widget,
                                                             subscriberMixIn,
                                                             template,
                                                             bootstrap,
                                                             BaseEditor,
                                                             CustomPointSelectorEditor,
                                                             ZoneEditCommand,
                                                                 ZoneAddCommand,
                                                             feDialogs,
                                                             lex)
{

    'use strict';

    var pointEditorLex = lex[0];

    /**
     * A field editor for working with secured point list.
     *
     * @private
     * @class
     * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
     * @alias module:nmodule/electronicSignature/rc/fe/baja/PointListEditor
     */
    var PointListEditor = function PointListEditor(params)
    {
        BaseEditor.call(this, $.extend({
            keyName: 'PointListEditor'
        }, params));
        subscriberMixIn(this);
    };

    PointListEditor.prototype = Object.create(BaseEditor.prototype);
    PointListEditor.prototype.constructor = PointListEditor;

    PointListEditor.prototype.doInitialize = function (dom)
    {
        var that = this;
        var jsonObject={choosePoints:pointEditorLex.get('choosePoints')};
        dom.html(template({jsonObject:jsonObject}));
        dom.find("#pointChooserButton").click(function ()
        {
            var zone = that.getComplex();

            if(zone === null || zone === undefined)
            {
                if(ZoneEditCommand.zoneEditor!==null){
                    zone = ZoneEditCommand.zoneEditor.value().getComponentSource().getContainer()[0];
                }
                else if(ZoneAddCommand.zoneEditor!==null){
                    zone = ZoneAddCommand.zoneEditor.value().getComponentSource().getContainer();
                }
            }
            if(!(zone === null || zone === undefined))
            {
                // License check.
                // If eSignature feature is missing or expired then show error and return.
                loadComponentSlots(zone).then(function(zone)
                {
                    loadComponentSlots(zone.getParent()).then(function(customer)
                    {
                        loadComponentSlots(customer.getParent()).then(function(eSignatureComponent)
                        {
                            if(!eSignatureComponent.getIsLicensed())
                            {
                                dialogs.showOk("eSignature license is missing or expired.");
                                return;
                            }
                            return savePoints.call(that);
                            /*if(eSignatureComponent.getUsedPointCount() >= eSignatureComponent.getMaxPointCount())
                            {
                                dialogs.showOk("Maximum allowed points are already secured.\nPlease purchase additional eSignaure point count or remove security from existing points.")
                                  .ok(function()
                                  {
                                      console.log("Calling Save Point.");
                                      return savePoints.call(that);
                                  });
                            }
                            else
                            {
                                return savePoints.call(that);
                            }*/
                        });
                    });
                });
            }
            else
            {
                console.log("Zone is null");
            }
        });
    };

    PointListEditor.prototype.doLoad = function (pointList, params)
    {
        var dom = this.jq();

        dom.find("#pointListText").val(pointList);
        if (this.getComplex() === null)
        {
            dom.find("#pointChooserButton").attr("disabled", true);
            dom.find("#pointChooserButton").css('cursor', 'not-allowed');
        }
        if(ZoneAddCommand.zoneEditor!==null){
            var isZoneMounted = ZoneAddCommand.zoneEditor.value().getComponentSource().getContainer().isMounted();
            if(!isZoneMounted){
                dom.find("#pointChooserButton").attr("disabled", true);
                dom.find("#pointChooserButton").css('cursor', 'not-allowed');
            }
        }
        else if(ZoneEditCommand.zoneEditor!==null){
            var isZoneMounted = ZoneEditCommand.zoneEditor.value().getComponentSource().getContainer()[0].isMounted();
            if(!isZoneMounted){
                dom.find("#pointChooserButton").attr("disabled", true);
                dom.find("#pointChooserButton").css('cursor', 'not-allowed');
            }
            else{
                dom.find("#pointChooserButton").attr("disabled", false);
                dom.find("#pointChooserButton").css('cursor', 'pointer');
            }
        }
        return Promise.resolve(pointList);
    };

    PointListEditor.prototype.doRead = function ()
    {
        var dom = this.jq();
        return dom.find("#pointListText").val();
    };

    PointListEditor.prototype.doSave = function (pointList, params)
    {
        return pointList;
    };
    PointListEditor.prototype.doDistroy = function (enabled) {
        ZoneEditCommand.zoneEditor = null;
        ZoneAddCommand.zoneEditor = null;
    };

    function savePointsToStation(zone, allComps)
    {

        return zone.invoke({
            slot: "addSecuredPoint",
            value: baja.$("baja:String").make(JSON.stringify(allComps))
        });
    }

    function fillList(pointArray, reasons, destinationArray)
    {
        var obj = null, i = 0, j = 0;
        for (i = 0; i < pointArray.length; i++)
        {
            obj = {};
            obj.ord = pointArray[i];
            for (j = 0; j < reasons.length; j++)
            {
                if (pointArray[i] === reasons[j].pointSlotPath)
                {
                    obj.reasonHandles = reasons[j].reasonHandles;
                }
            }
            destinationArray.push(obj);
            console.log("points to under process :: " + obj.ord, " Handle :" + obj.reasonHandles); // jshint ignore:line
        }
    }

    function savePoints()
    {
        var that = this;
        var val = that.jq().find("#pointListText").val();
        var wrapper = {};
        wrapper.listOfPoints = val;
        wrapper.zone = this.getComplex();
        if(wrapper.zone === null || wrapper.zone === undefined)
        {
            wrapper.zone = ZoneEditCommand.zoneEditor.value().getComponentSource().getContainer()[0];
        }
        return feDialogs.showFor({
            value: wrapper,
            type: CustomPointSelectorEditor
        }).then(function (listOfPrimarySecondaryToAdd)
        {
            if (listOfPrimarySecondaryToAdd !== null)
            {
                var allComps = {};
                var reasons = listOfPrimarySecondaryToAdd.reasonPointMapHash;
                var primCompToAdd = [], secCompToAdd = [], primCompToRem = [], secCompToRem = [],editedReasonPointMap=[];

                fillList(listOfPrimarySecondaryToAdd.priAdd, reasons, primCompToAdd);
                fillList(listOfPrimarySecondaryToAdd.secAdd, reasons, secCompToAdd);
                fillList(listOfPrimarySecondaryToAdd.priRem, reasons, primCompToRem);
                fillList(listOfPrimarySecondaryToAdd.secRem, reasons, secCompToRem);
               // Array.prototype.push.apply(primCompToRem, secCompToRem);
               //Add the required history details from pointHistoryDetails to secCompToRem
                addHistoryDetailsToSecCompToRem(listOfPrimarySecondaryToAdd.pointHistoryDetails,secCompToRem);
                allComps.primCompToAdd = primCompToAdd;
                allComps.secCompToAdd = secCompToAdd;
                allComps.secCompToRem = secCompToRem;
                allComps.primCompToRem = primCompToRem;
                allComps.compWithEditedReasons = listOfPrimarySecondaryToAdd.editedReasonPointMap;
                allComps.loggedInUserName=baja.getUserName();
                return savePointsToStation(wrapper.zone, allComps).then(function (val)
                {
                    wrapper.zone.setPointList(listOfPrimarySecondaryToAdd.pointList).then(function ()
                    {
                        console.log("After setting point list"); // jshint ignore:line
                        var element = ZoneEditCommand.zoneEditor.$jq.parent().parent().parent().parent();
                        if (element !== null)
                        {
                            element.remove();
                        }
                    });
                });
            }
            else
            {

            }
        });
    }

    function loadComponentSlots(comp){
        return new Promise(function(fullfil,reject){
            comp.lease().then(function(){
                comp.loadSlots({
                    ok:function(){
                        fullfil(comp);
                    },
                    fail:function(){
                        reject(comp);
                    }
                });
            });
        });
    }

    //Add the required history details from pointHistoryDetails to secCompToRem
  function addHistoryDetailsToSecCompToRem(pointHistoryDetails,secCompToRem)
  {
    var obj={};
    for(var i=0;i<pointHistoryDetails.length;i++)
    {
      for(var j=0;j<secCompToRem.length;j++)
      {
        if(pointHistoryDetails[i].pointSlotPath === secCompToRem[j].ord)
        {
          secCompToRem[j].primComments=pointHistoryDetails[i].primComments;
          secCompToRem[j].secComments=pointHistoryDetails[i].secComments;
          secCompToRem[j].managerName=pointHistoryDetails[i].managerName;
        }
      }
    }

  }

  return PointListEditor;

});