/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * SecuredBooleanEditor manages functionality like primary, secondary authentication ,Change new value and maintain the secure point history.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredBooleanEditor
 */
define(['baja!','baja!electronicSignature:SecureBoolean',
        'lex!baja',
        'jquery',
        'Promise',
        'dialogs',
        'lex!electronicSignature',
        'bajaux/Widget',
        'underscore',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredBooleanTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         electronicSignatureLex,
         Widget,
         _,
         subscriberMixIn,
         template,
         loader,
          BaseEditor,
         SecureParentEditor) {
  'use strict';
    var lex = electronicSignatureLex[0];
    var SecuredBooleanEditor = function (params) {
      BaseEditor.call(this, $.extend({
        keyName: 'SecuredBooleanEditor'
      }, params));
      subscriberMixIn(this);
    };

    SecuredBooleanEditor.componentFE = null;

    SecuredBooleanEditor.prototype = Object.create(SecureParentEditor.prototype);

    SecuredBooleanEditor.prototype.constructor = SecuredBooleanEditor;

    SecuredBooleanEditor.prototype.doInitialize = function (dom) {
    var jsonObject={pointName:lex.get('fe.pointName.label'),
    newValue:lex.get('fe.newValue.label'),
    reason:lex.get('fe.reason.label'),
    otherReason:lex.get('fe.otherReason.label'),
    authLevel1:lex.get('fe.authorization.level1.label'),
    authLevel2:lex.get('fe.authorization.level2.label'),
    userName:lex.get('fe.userName.label'),
    password:lex.get('fe.password.label'),
    comment:lex.get('fe.comment.label'),
    sign:lex.get('fe.sign.button'),
    sign2:lex.get('fe.sign2.button'),
    remote:lex.get('fe.remote.button'),
    chooseUser:lex.get('remote.chooseUser.label'),
    local:lex.get('remote.local.button'),
    sendRequest:lex.get('remote.sendRequest.button'),
    acknowledgement:lex.get('eSignature.ack.label'),
    label1:lex.get('eSignature.ack.label1'),
    label2:lex.get('eSignature.ack.label2'),
    label3:lex.get('eSignature.ack.label3'),
    label4:lex.get('eSignature.ack.label4'),
    label5:lex.get('eSignature.ack.label5'),
    label6:lex.get('eSignature.ack.label6'),
    label7:lex.get('eSignature.ack.label7')

    };
    dom.html(template({jsonObject:jsonObject}));
      this.doInitializeContent(dom);
    };

    SecuredBooleanEditor.prototype.doLoad = function(secureBoolean,params){
      var widget = this, dom = widget.jq();
      SecuredBooleanEditor.componentFE = secureBoolean;
      var result = this.doLoadValue(dom,secureBoolean);
      if(result === "NOT_CONNECTED")
        return Promise.resolve("");
      var trueText =  secureBoolean.getSecureSlotFacets().$map.get("trueText");
      var falseText =  secureBoolean.getSecureSlotFacets().$map.get("falseText");
      if(trueText === null || trueText === undefined)
      {
        trueText = "true";
      }
      if(falseText === null || falseText === undefined)
      {
        falseText = "false";
      }

      dom.find("#newValue").html('<option value="true">'+trueText+'</option><option value="false">'+falseText+'</option>');
      dom.find("#newValue").find('option[value="'+secureBoolean.getNewValue()+'"]').attr("selected",true);

      return this.validateSecondaryUsers(secureBoolean);
    };


    SecuredBooleanEditor.prototype.doIsValueValid = function(dom){
        var invalidList=[];

        var newValueField = dom.find('#newValue').val();
        if(newValueField==="")
        {
          var invalidListObj = {};
          invalidListObj.id="newValue";
          invalidListObj.errorMsg=lex.get('editor.pointValue.blank');
          invalidList.push(invalidListObj);
        }
        return invalidList;
    };

    SecuredBooleanEditor.prototype.doRead = function(){
      var that = this;
      var dom = this.jq();
      var secureBoolean = that.value();
      secureBoolean.setNewValue(dom.find("#newValue").val());
      return secureBoolean;
    };

    SecuredBooleanEditor.prototype.doSave = function (secureBoolean,params) {
      var dom = this.jq();
      secureBoolean = this.doSaveValue(secureBoolean,dom);
      return secureBoolean;
    };

   /* function getPointNewValue(dom) {
      return dom.find(".newValue");
    }*/

    return SecuredBooleanEditor;
  });