/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * SecuredBooleanOverrideEditor manages functionality like primary, secondary authentication ,Change new value and maintain the secure point history.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredBooleanOverrideEditor
 */
define(['baja!','baja!electronicSignature:SecureBooleanOverride,control:Override',
        'lex!baja',
        'jquery',
        'Promise',
        'dialogs',
        'lex!electronicSignature',
        'bajaux/Widget',
        'underscore',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredBooleanOverrideTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'bajaux/events',
        "nmodule/webEditors/rc/fe/fe",
        'nmodule/webEditors/rc/fe/baja/OverrideEditor',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         electronicSignatureLex,
         Widget,
         _,
         subscriberMixIn,
         template,
         loader,
         BaseEditor,
         SecuredParentEditor,
         events,
         fe,
         OverrideEditor) {
  'use strict';
    var lex = electronicSignatureLex[0];
    var SecuredBooleanOverrideEditor = function (params)
    {
      BaseEditor.call(this, $.extend({
        keyName: 'SecuredBooleanOverrideEditor'
      }, params));
      subscriberMixIn(this);
    };

    SecuredBooleanOverrideEditor.componentFE = null;

    SecuredBooleanOverrideEditor.prototype = Object.create(SecuredParentEditor.prototype);

    SecuredBooleanOverrideEditor.prototype.constructor = SecuredBooleanOverrideEditor;

    SecuredBooleanOverrideEditor.prototype.doInitialize = function (dom)
    {
        var jsonObject={pointName:lex.get('fe.pointName.label'),
            newValue:lex.get('fe.newValue.label'),
            reason:lex.get('fe.reason.label'),
            otherReason:lex.get('fe.otherReason.label'),
            authLevel1:lex.get('fe.authorization.level1.label'),
            authLevel2:lex.get('fe.authorization.level2.label'),
            userName:lex.get('fe.userName.label'),
            password:lex.get('fe.password.label'),
            comment:lex.get('fe.comment.label'),
            sign:lex.get('fe.sign.button'),
            sign2:lex.get('fe.sign2.button'),
            remote:lex.get('fe.remote.button'),
            chooseUser:lex.get('remote.chooseUser.label'),
            local:lex.get('remote.local.button'),
            sendRequest:lex.get('remote.sendRequest.button'),
            acknowledgement:lex.get('eSignature.ack.label'),
            label1:lex.get('eSignature.ack.label1'),
            label2:lex.get('eSignature.ack.label2'),
            label3:lex.get('eSignature.ack.label3'),
            label4:lex.get('eSignature.ack.label4'),
            label5:lex.get('eSignature.ack.label5'),
            label6:lex.get('eSignature.ack.label6'),
            label7:lex.get('eSignature.ack.label7')

        };
        dom.html(template({jsonObject:jsonObject}));
      this.doInitializeContent(dom);
    };

    SecuredBooleanOverrideEditor.prototype.loadOverrideFE = function(dom, override)
    {
      var overrideDom = $("#overrideValue",dom);
      return fe.buildFor({
        dom: overrideDom,
        value: override,
        type: OverrideEditor,
        formFactor: "mini"
      }).then(function(overrideEditor){
        if($(overrideDom).find(".col-display").length>0)
        {
          $(overrideDom).find(".col-display").hide();
          $(overrideDom).find(".key-value").hide();
          $('.easNewValue', dom).find("#nvLabel").html(lex.get("duration"));
        }
        overrideEditor.jq().on(events.MODIFY_EVENT, function(){
            return overrideEditor.read().then(function(override){
                window.setTimeout(function(){ // jshint ignore:line
                 $('.js-dialog-button-ok').attr('disabled', true);
              },100);
            });
         });
        return overrideEditor;
      });
    };

    SecuredBooleanOverrideEditor.prototype.doLoad = function(securedBooleanOverride,params)
    {
      var widget = this, dom = widget.jq();
      SecuredBooleanOverrideEditor.componentFE = securedBooleanOverride;
      var result = this.doLoadValue(dom,securedBooleanOverride);
        if(result === "NOT_CONNECTED")
            return Promise.resolve("");

      var promise = widget.loadOverrideFE(dom,securedBooleanOverride.getOverrideParent());
      var overrideEditor;
      return Promise.resolve(promise).then(function(fe){
        overrideEditor = fe;
        return overrideEditor.setReadonly(false);
      }).then(function(){
        overrideEditor.$builder.getEditorFor('duration').setModified(true);
        overrideEditor.setModified(true);
        return widget.validateSecondaryUsers(securedBooleanOverride);
      });
    };


    SecuredBooleanOverrideEditor.prototype.doIsValueValid = function(dom)
    {
      var overrideWidget = $("#overrideValue", dom).data('widget');
      return overrideWidget.save().then(function(){
        return overrideWidget.doRead().then(function(newValueField){
          try
          {
            baja.$("baja:RelTime").make(overrideWidget.$builder.$dataSource.$map.get("duration").$val.$ms);
          }
          catch(err)
          {
            // No logging as overrideWidget.$builder.$dataSource.$map becomes null when duration is permanent.
          }
          return null;
        });
      });
    };

    SecuredBooleanOverrideEditor.prototype.doRead = function()
    {
      var that = this;
      var dom = this.jq();

      var FE = $("#overrideValue",dom).data('widget');
      FE.setModified(true);
      FE.$builder.getEditorFor('duration').setModified(true);

      return FE.read().then(function(overrideValue){

        var securedBooleanOverride = that.value();

        if(!_.isEmpty(overrideValue.$map))
        {
          var newValueField = baja.$("baja:RelTime").make(overrideValue.$map.duration.$ms);
          var overrideVal = baja.$("control:Override",{ duration : newValueField });
          return securedBooleanOverride.setOverrideParent(overrideVal).then(function()
          {
            return securedBooleanOverride;
          });
        }
        return securedBooleanOverride;
      });
    };

    SecuredBooleanOverrideEditor.prototype.doSave = function (securedBooleanOverride,params)
    {
      var dom = this.jq();
      securedBooleanOverride = this.doSaveValue(securedBooleanOverride,dom);
      return securedBooleanOverride;
    };

    return SecuredBooleanOverrideEditor;
  });