function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/*global niagara*/

/**
 * NOTE: This context menu appears on the secured electronic signature components.
 * So it does not have Cut, Copy and Duplicate commands.
 * Plus, it has custom commands to authenticate before the rename, set display name and delete.
 * API Status: **Private**
 * @module nmodule/webEditors/rc/wb/menu/componentMenuAgent
 */
define(['baja!', 'lex!', 'log!nmodule.webEditors.rc.wb.menu.componentMenuAgent', 'bajaux/commands/Command', 'bajaux/commands/CommandGroup', 'Promise', 'underscore', 'nmodule/js/rc/asyncUtils/asyncUtils', 'nmodule/webEditors/rc/wb/commands/transfer/TransferCommand', 'nmodule/webEditors/rc/wb/menu/menuUtils', 'nmodule/webEditors/rc/wb/mixin/mixinUtils'], function (baja, lex, log, Command, CommandGroup, Promise, _, asyncUtils, TransferCommand, menuUtils, mixinUtils) {
  'use strict';

  var doRequire = asyncUtils.doRequire,
      ifAllowedByProfile = menuUtils.ifAllowedByProfile,
      makeActionsGroup = menuUtils.makeActionsGroup,
      makeNewGroup = menuUtils.makeNewGroup,
      makeViewsGroup = menuUtils.makeViewsGroup,
      hasMixin = mixinUtils.hasMixin,
      NEW_COMMAND_GROUP_DISPLAY_NAME = menuUtils.NEW_COMMAND_GROUP_DISPLAY_NAME,
      EDIT_TAGS_COMMAND_DISPLAY_NAME = menuUtils.EDIT_TAGS_COMMAND_DISPLAY_NAME,
      COMPOSITE_COMMAND_DISPLAY_NAME = menuUtils.COMPOSITE_COMMAND_DISPLAY_NAME,
      SHOW_ON_WIRESHEET_COMMAND_DISPLAY_NAME = menuUtils.SHOW_ON_WIRESHEET_COMMAND_DISPLAY_NAME,
      wiresheetViewCommandName = menuUtils.WIRESHEET_COMMAND_DISPLAY_NAME,
      resolveBestViewId = menuUtils.resolveBestViewId;
  var logSevere = log.severe.bind(log);

  function isCommand(cmd, displayNameFormat) {
    return cmd.getDisplayNameFormat() === displayNameFormat;
  }

  function isParented(comp) {
    return !!comp.getPropertyInParent();
  }
  /**
   * Check to see if the component is frozen (either unparented, or a frozen
   * slot).
   *
   * @inner
   * @param {baja.Complex} comp
   * @returns {Boolean}
   */


  function isFrozen(comp) {
    var prop = comp.getPropertyInParent();
    return !prop || prop.isFrozen();
  }

  function isESignature(comp) {
      return (comp.getType().$typeSpec.equals("electronicSignature:SecuredDashboardConfiguration"));
  }

  function isReadonly(comp) {
    var space = comp.getComponentSpace();
    return !comp.getPermissions().hasAdminWrite() || space && space.isSpaceReadonly();
  }

  /**
   * Asynchronously require the appropriate command modules and instantiate
   * the `CommandGroup`. Assign the appropriate enabled/disabled status for
   * the different commands depending on the write/invoke permissions on this
   * component.
   *
   * @inner
   * @param {*} owner
   * @param {baja.Component} comp
   * @returns {Promise}
   */


  function makeGroup(owner, comp) {
    return doRequire(['nmodule/webEditors/rc/transform/ExportCommand',
    'nmodule/webEditors/rc/wb/commands/AddSlotCommand',
    'nmodule/electronicSignature/rc/fe/baja/SecuredRenameCommand',
    'nmodule/webEditors/rc/wb/commands/LinkMarkCommand',
    'nmodule/webEditors/rc/wb/commands/LinkFromCommand',
    'nmodule/webEditors/rc/wb/commands/LinkToCommand',
    'nmodule/webEditors/rc/wb/commands/RelationMarkCommand',
    'nmodule/webEditors/rc/wb/commands/RelateFromCommand',
    'nmodule/webEditors/rc/wb/commands/RelateToCommand',
    'nmodule/webEditors/rc/wb/commands/ReorderSlotsCommand',
    'nmodule/electronicSignature/rc/fe/baja/SecuredSetDisplayNameCmd',
    'nmodule/webEditors/rc/wb/commands/CompositeEditorCommand',
    'nmodule/webEditors/rc/wb/menu/Separator',
    'nmodule/electronicSignature/rc/fe/baja/SecuredDeleteCommand',
    'lex!electronicSignature']).then(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 15),
          ExportCommand = _ref2[0],
          AddSlotCommand = _ref2[1],
          SecureRenameCommand = _ref2[2],
          LinkMarkCommand = _ref2[3],
          LinkFromCommand = _ref2[4],
          LinkToCommand = _ref2[5],
          RelationMarkCommand = _ref2[6],
          RelateFromCommand = _ref2[7],
          RelateToCommand = _ref2[8],
          ReorderSlotsCommand = _ref2[9],
          SecuredSetDisplayNameCmd = _ref2[10],
          CompositeEditorCommand = _ref2[11],
          Separator = _ref2[12],
          SecuredDeleteCommand = _ref2[13],
          electronicSignatureLex = _ref2[14];

      var group = new CommandGroup(),
          newGroup,
          deleteCmd,
          duplicateCmd,
          renameCmd,
          setDisplayNameCmd,
          reorderCmd,
          compositeCmd,
          exportCmd,
          cutCmd,
          copyCmd,
          pasteCmd,
          pasteSpecialCmd,
          eSignLex = electronicSignatureLex[0];

      var ESignature;
      if (!isReadonly(comp)) {
        var frozen = isFrozen(comp),
            ESignature = isESignature(comp),
            parented = isParented(comp),
            hasAdminWrite = comp.getPermissions().hasAdminWrite(),
            hasAdminWriteOnParent = parented && comp.getParent().getPermissions().hasAdminWrite();
        newGroup = makeNewGroup(comp, new AddSlotCommand(comp, {
          undoable: true
        }));
        setDisplayNameCmd = new SecuredSetDisplayNameCmd(comp);
        reorderCmd = new ReorderSlotsCommand(comp);
        compositeCmd = new CompositeEditorCommand(comp);
        deleteCmd = new SecuredDeleteCommand(comp);
        exportCmd = new ExportCommand(comp);
        renameCmd = new SecureRenameCommand(comp);

        setDisplayNameCmd.setEnabled(hasAdminWrite && parented && !ESignature);
        reorderCmd.setEnabled(hasAdminWrite);
        compositeCmd.setEnabled(hasAdminWrite && parented);
        exportCmd.setEnabled(hasAdminWrite);
        deleteCmd.setEnabled(hasAdminWrite && !frozen && !ESignature);
        renameCmd.setEnabled(hasAdminWrite && !frozen && !ESignature);

        if (hasMixin(owner, 'transfer')) {
          var subject = [comp];

          if (owner.isTransferOpSupported('paste', subject)) {
            pasteCmd = TransferCommand.make(owner, 'paste', {
              subject: subject,
              module: 'webEditors',
              lex: 'commands.paste'
            });
          }

          if (owner.isTransferOpSupported('pasteSpecial', subject)) {
            pasteSpecialCmd = TransferCommand.make(owner, 'pasteSpecial', {
              subject: subject,
              module: 'webEditors',
              lex: 'commands.pasteSpecial'
            });
          }
        } else {
          renameCmd = new SecureRenameCommand(comp);
          renameCmd.setEnabled(hasAdminWrite && !frozen && !ESignature);
        }
      }

      var makeEditTagsCommmand = _.once(function (component, enabled) {
        // Edit Tags should not be available for VirtualComponents
        if (!componentMenuAgent.$supportsEditTagsCommand(component) || isReadonly(component)) {
          return Promise.resolve(null);
        }

        return baja.$requireFromJsBuild('nmodule/tagdictionary/rc/commands/TagManagerCommand', 'tagdictionary:TagDictionaryJsBuild').then(function (TagManagerCommand) {
          return TagManagerCommand.make(component, enabled);
        })["catch"](function (ignore) {
          return null;
        });
      });

      function makeRelationCommands(comp) {
        if (isReadonly(comp)) {
          return [];
        }

        return [new Separator(), RelationMarkCommand.make(comp), RelateFromCommand.make(comp), RelateToCommand.make(comp)];
      }

      function makeLinkCommands(comp) {
        if (isReadonly(comp)) {
          return [];
        }

        return [new Separator(), LinkMarkCommand.make(comp), LinkFromCommand.make(comp), LinkToCommand.make(comp)];
      }

      var preloadWiresheetDisplayName = _.once(function () {
        return lex.module('wiresheet').then(function (wiresheetLex) {
          wiresheetViewCommandName = wiresheetLex.get('WebWiresheet.displayName');
        });
      });

      return Promise.all([ifAllowedByProfile('viewSelection', function () {
        return preloadWiresheetDisplayName().then(function () {
          return makeViewsGroup(comp.getNavOrd());
        });
      }), ifAllowedByProfile('viewSelection', function () {
        return Promise.all([componentMenuAgent.$isWebWiresheetPresent(), preloadWiresheetDisplayName()]).then(function (_ref3) {
          var _ref4 = _slicedToArray(_ref3, 1),
              wiresheetPresent = _ref4[0];

          return wiresheetPresent && makeShowOnWiresheetCommand(comp);
        });
      }), makeActionsGroup(comp), new Separator(), newGroup, new Separator(), makeEditTagsCommmand(comp, hasAdminWrite), new Separator(),  pasteCmd, pasteSpecialCmd, deleteCmd, new Separator()].concat(makeLinkCommands(comp)).concat(makeRelationCommands(comp)).concat([new Separator(), renameCmd, setDisplayNameCmd, reorderCmd, compositeCmd, new Separator(), exportCmd])).then(function (allKids) {
        group.add.apply(group, _.compact(allKids));
        return group;
      });
    });
  }
  /**
   *
   * @param {baja.Component} comp
   * @returns {module:bajaux/commands/Command}
   */


  function makeShowOnWiresheetCommand(comp) {
    if (!canShowOnWiresheet(comp)) {
      return;
    }

    var parentComp = comp.getParent();
    return new Command({
      module: 'webEditors',
      lex: 'commands.showOnWiresheet',
      func: function func() {
        var viewParams = {
          'tHandle': comp.getHandle()
        };
        return resolveBestViewId(parentComp, 'wiresheet:WebWiresheet').then(function (viewId) {
          var viewQuery = new baja.ViewQuery({
            id: viewId,
            params: viewParams
          }),
              ord = baja.Ord.make({
            base: parentComp.getNavOrd(),
            child: viewQuery
          });
          return niagara.env.hyperlink(ord);
        })["catch"](logSevere);
      }
    });
  }

  function canShowOnWiresheet(comp) {
    var parentComp = comp.getParent();
    return !isWb() && !baja.hasType(comp, 'baja:Vector') && parentComp && parentComp.getPermissions().hasAdminWrite();
  }

  function isWb() {
    return typeof niagara !== 'undefined' && niagara.env && niagara.env.type === 'wb';
  }
  /**
   * Create a command group that can be used to construct a context menu. Note
   * that the component, if mounted, will be leased at this time so any
   * dynamic actions can be included in the menu.
   *
   * The value may also be a `NavContainer`, e.g., a nav node corresponding to
   * a component inside a .bog file inside the station's file space, that
   * BajaScript has not fully resolved yet. In this case, the `NavContainer`'s
   * nav ORD will be resolved and the menu will be generated from the result.
   *
   * @alias module:nmodule/webEditors/rc/wb/menu/componentMenuAgent
   * @param {*} owner if the owner has the `TransferSupport` mixin, then
   * transfer commands will be included
   * @param {baja.Component|baja.NavContainer} comp
   * @returns {Promise} promise to be resolved with a `CommandGroup`
   * for the given component
   * @see module:nmodule/webEditors/rc/wb/mixin/TransferSupport
   */


  var componentMenuAgent = function componentMenuAgent(owner, comp) {
    if (comp instanceof baja.NavContainer) {
      return comp.getNavOrd().get({
        lease: true
      }).then(function (comp) {
        return makeGroup(owner, comp);
      });
    }

    return Promise.resolve(comp.isMounted() && comp.lease()).then(function () {
      return makeGroup(owner, comp);
    });
  };
  /**
   * @private
   * @param {baja.Component} comp
   */


  componentMenuAgent.$supportsEditTagsCommand = function (comp) {
    return !baja.hasType(comp, 'baja:VirtualComponent') && !baja.hasType(comp, 'baja:VirtualGateway');
  };
  /**
   * @private
   * @returns {Promise<boolean>}
   */


  componentMenuAgent.$isWebWiresheetPresent = _.once(function () {
    return baja.importTypes(['wiresheet:WebWiresheet']).then(function () {
      return true;
    })["catch"](function () {
      return false;
    });
  });
  /**
   * @param {module:bajaux/commands/Command} cmd
   */

  componentMenuAgent.IS_NEW_COMMAND = function (cmd) {
    return isCommand(cmd, NEW_COMMAND_GROUP_DISPLAY_NAME);
  };
  /**
   * @param {module:bajaux/commands/Command} cmd
   */


  componentMenuAgent.IS_EDIT_TAGS_COMMAND = function (cmd) {
    return isCommand(cmd, EDIT_TAGS_COMMAND_DISPLAY_NAME);
  };
  /**
   * @param {module:bajaux/commands/Command} cmd
   */


  componentMenuAgent.IS_COMPOSITE_COMMAND = function (cmd) {
    return isCommand(cmd, COMPOSITE_COMMAND_DISPLAY_NAME);
  };
  /**
   * @param {module:bajaux/commands/Command} cmd
   */


  componentMenuAgent.IS_WIRESHEET_VIEW_COMMAND = function (cmd) {
    return isCommand(cmd, wiresheetViewCommandName);
  };
  /**
   * @param {module:bajaux/commands/Command} cmd
   */


  componentMenuAgent.IS_SHOW_ON_WIRESHEET_COMMAND = function (cmd) {
    return isCommand(cmd, SHOW_ON_WIRESHEET_COMMAND_DISPLAY_NAME);
  };

  return componentMenuAgent;
});
