/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

define(['baja!',
        'baja!electronicSignature:SecuredNumericWritable,electronicSignature:SecuredBooleanWritable,electronicSignature:SecuredEnumWritable,electronicSignature:SecuredStringWritable,electronicSignature:SecuredNumericPoint,electronicSignature:SecuredBooleanPoint,electronicSignature:SecuredEnumPoint,electronicSignature:SecuredStringPoint',
        'lex!webEditors',
        'jquery',
        'dialogs',
        'lex!electronicSignature',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/wb/commands/ComponentEditorCommand',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/electronicSignature/rc/fe/baja/Util',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         dialogs,
         electronicSignatureLex,
         Promise,
         _,
         ComponentEditorCommand,
         template,
         loader,
         Util) {

  'use strict';
  var webEditorsLex = lexs[0];
    var lex = electronicSignatureLex[0];
 /* var webEditorsLex = lexs[0],
  _SecondaryAuthenticatorRoles="",
  isValidUserSecondary=false;*/

////////////////////////////////////////////////////////////////
// Support functions
////////////////////////////////////////////////////////////////

  /**
   * Remove all the given slots from the component, batching into one network
   * call.
   *
   * @inner
   * @param {baja.Component} comp
   * @param {Array.<baja.Slot|String>} slots
   * @returns {Promise}
   */
  function removeAll(comp, slots) {
    var batch = new baja.comm.Batch(),
        removes = _.map(slots, function (slot) {
          return comp.remove({ slot: slot, batch: batch });
        });

    batch.commit();

    return Promise.all(removes);
  }

  //TODO: optional confirmation dialog
  //TODO: found bug: add slot, rename, and delete all without refreshing multisheet. wrong slot goes away

  /**
   * A command for deleting slots from an editor's `Component` value.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/wb/commands/ComponentEditorCommand
   * @alias module:nmodule/webEditors/rc/wb/commands/DeleteCommand
   * @param {module:bajaux/Widget} ed the `Widget` to bind this `Command` to
   * @param {baja.Component} the component from which to delete slots
   * @throws {Error} if no `Widget` provided
   */
  var SecuredDeleteCommand = function SecuredDeleteCommand(component) {
    ComponentEditorCommand.call(this, {
      module: 'webEditors',
      lex: 'commands.delete'
    }, component);
  };
  SecuredDeleteCommand.prototype = Object.create(ComponentEditorCommand.prototype);
  SecuredDeleteCommand.prototype.constructor = SecuredDeleteCommand;
  SecuredDeleteCommand.isAuthenticatedUser = false;

  /**
   * Make sure we have admin write permissions and we're not deleting a frozen
   * slot.
   *
   * @param {baja.Component} comp
   * @param {baja.Slot} slot
   * @returns {Boolean} true if I can delete this slot
   */
  SecuredDeleteCommand.prototype.canPerformCommand = function (comp, slot) {
      return comp.getPermissions().hasAdminWrite() && !slot.isFrozen();
  };

  /**
   * Deletes the selected slots from the editor's loaded Component.
   *
   * @param {baja.Component} comp
   * @param {Array.<baja.Slot>} slots the slots to delete
   * @returns {Promise}
   */
  SecuredDeleteCommand.prototype.performCommand = function (comp, slots, params) {
    var slot=slots[0];
    if (!slot) {
      return Promise.reject(lex.get('delete.slot.required'));
    }
    if(slot.getType().is(baja.$("electronicSignature:SecuredNumericWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredNumericPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringPoint").getType())){
     slots[0].$val.getActionParameterDefault(slots[0].$val.getSlot("call")).then(function(actionPoint){
         var primarySignLabel = lex.get('fe.sign1.button');
         if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
         {
             primarySignLabel = lex.get('fe.sign.button');
         }
         var jsonObject={
             authLevel1:lex.get('fe.authorization.level1.label'),
             authLevel2:lex.get('fe.authorization.level2.label'),
             userName:lex.get('fe.userName.label'),
             password:lex.get('fe.password.label'),
             comment:lex.get('fe.comment.label'),
             sign:primarySignLabel,
             sign2:lex.get('fe.sign2.button'),
             acknowledgement:lex.get('eSignature.ack.label'),
             label1:lex.get('eSignature.ack.label1'),
             label2:lex.get('eSignature.ack.label2'),
             label3:lex.get('eSignature.ack.label3'),
             label4:lex.get('eSignature.ack.label4'),
             label5:lex.get('eSignature.ack.label5'),
             label6:lex.get('eSignature.ack.label6'),
             label7:lex.get('eSignature.ack.label7')
         };
          dialogs.showOkCancel({title:lex.get('authenticate.user.title'),content:template({jsonObject:jsonObject})}).ok(function(val){
             return removeAll(comp, slots).then(function(){
                var dashboardHandle=actionPoint.getSecureSlotFacets().$map.$map.dashboardHandle;
                getDashBoardComponent(dashboardHandle).then(function(dashBoardComponent){
                var primaryComments=val.jq().find("#primaryComment").val();
                actionPoint.setActionName("delete");
                if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                {
                 actionPoint.setPrimaryComments(primaryComments);
                }else
                {
                  var secondaryComments=val.jq().find("#secondaryComments").val();
                  var managerName= val.jq().find("#secondaryUsername").val();
                  actionPoint.setPrimaryComments(primaryComments);
                  actionPoint.setSecondaryComments(secondaryComments);
                  actionPoint.setManager(managerName);

                }
                 //actionPoint.setNewValue(newName);
                 var comp= baja.$("baja:Component", {
                    actionPoint: actionPoint,
                 });
                dashBoardComponent.invoke({slot:"updateHistory", value: comp});

                });

                });
          });
          Util.loadInitialEventsAndData(actionPoint,"delete");
     }).catch(function (err) {
        return dialogs.showOk({
           title: lex.get("error.title"),
           content:lex.get('editor.primary.user.validation')
        });
     });
    }
    else{
      return dialogs.showOkCancel({
        title: webEditorsLex.get('commands.delete.confirm.title'),
        content: webEditorsLex.get('commands.delete.confirm.content')
      })
        .ok(function () {
            let componentShouldDelete = isZoneContainsPointList(slots);
            if(!componentShouldDelete){
                return dialogs.showOk({
                    title: lex.get("error.title"),
                    content:lex.get('Text.error.addZone.zone.delete')
                });
            }
            else{
                return removeAll(comp, slots);
            }
        })
        .promise();
    }
  };

  function isZoneContainsPointList(Slots) {
      let componentShouldDelete = true;
      _.forEach(Slots,function (slot) {
          let value = slot.$val;
          if(value.getType().is("electronicSignature:Zone")){
              if(value.getPointList()!==null && value.getPointList()!==""){
                  componentShouldDelete = false;
              }
          }
      });
      return componentShouldDelete;
  }

  function getDashBoardComponent(handle) {
    var ord="station:|h:"+handle;
    return baja.Ord.make(ord).get({lease: true});
  }

  return SecuredDeleteCommand;
});