/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Balaji Gite
 */

/**
 * SecuredBooleanOverrideEditor manages functionality like primary, secondary
 * authentication ,Change new value and maintain the secure point history.
 * 
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredBooleanOverrideEditor
 */

define(
		[
				'baja!',
				'baja!electronicSignature:SecureEnumOverride,baja:EnumRange,baja:DynamicEnum,baja:StatusEnum,control:Override,control:EnumOverride',
				'lex!baja',
				'jquery',
				'Promise',
				'dialogs',
        'lex!electronicSignature',
				'bajaux/Widget',
				'underscore',
				'bajaux/mixin/subscriberMixIn',
				'hbs!nmodule/electronicSignature/rc/fe/template/SecuredEnumOverrideTemplate',
                'nmodule/electronicSignature/rc/fe/baja/loader',
				'nmodule/webEditors/rc/fe/baja/BaseEditor',
				'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
				'bajaux/events', 'nmodule/webEditors/rc/fe/fe',
				'nmodule/webEditors/rc/fe/baja/OverrideEditor',
				'css!nmodule/electronicSignature/rc/css/widgets'],
		function(baja, types, lexs, $, Promise, dialogs, electronicSignatureLex,Widget, _,
				subscriberMixIn, template, loader, BaseEditor, SecuredParentEditor,
				events, fe, OverrideEditor) {
			'use strict';

  var lex = electronicSignatureLex[0];
  var controlChoiceVal=null;
  var valuePresent = false;
  var overrideFE = null;
  //var encString = null;

  /**
   * A field editor for working with enum overrides.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
   * @alias module:nmodule/electronicSignature/rc/fe/baja/SecuredEnumOverrideEditor
   */
  var SecuredEnumOverrideEditor = function SecuredEnumOverrideEditor(params) {
    BaseEditor.call(this, $.extend({
      keyName : 'SecuredEnumOverrideEditor'
    }, params));
    subscriberMixIn(this);
  };

  SecuredEnumOverrideEditor.isAuthenticatedUser = false;
  SecuredEnumOverrideEditor.componentFE = null;
  SecuredEnumOverrideEditor.prototype = Object.create(SecuredParentEditor.prototype);
  SecuredEnumOverrideEditor.prototype.constructor = SecuredEnumOverrideEditor;

  SecuredEnumOverrideEditor.prototype.doInitialize = function(dom)
  {
      var jsonObject={pointName:lex.get('fe.pointName.label'),
          overrideValue:lex.get('editor.override.val'),
          overrideDuration:lex.get('editor.override.duration'),
          reason:lex.get('fe.reason.label'),
          otherReason:lex.get('fe.otherReason.label'),
          authLevel1:lex.get('fe.authorization.level1.label'),
          authLevel2:lex.get('fe.authorization.level2.label'),
          userName:lex.get('fe.userName.label'),
          password:lex.get('fe.password.label'),
          comment:lex.get('fe.comment.label'),
          sign:lex.get('fe.sign.button'),
          sign2:lex.get('fe.sign2.button'),
          remote:lex.get('fe.remote.button'),
          chooseUser:lex.get('remote.chooseUser.label'),
          local:lex.get('remote.local.button'),
          sendRequest:lex.get('remote.sendRequest.button'),
          acknowledgement:lex.get('eSignature.ack.label'),
          label1:lex.get('eSignature.ack.label1'),
          label2:lex.get('eSignature.ack.label2'),
          label3:lex.get('eSignature.ack.label3'),
          label4:lex.get('eSignature.ack.label4'),
          label5:lex.get('eSignature.ack.label5'),
          label6:lex.get('eSignature.ack.label6'),
          label7:lex.get('eSignature.ack.label7')

      };
      dom.html(template({jsonObject:jsonObject}));
      this.doInitializeContent(dom);
  };

  SecuredEnumOverrideEditor.prototype.loadOverrideFE = function(dom, override)
  {
    var overrideDom = $("#overrideValue",dom);
    return fe.buildFor({
      dom: overrideDom,
      value: override,
      type: OverrideEditor,
      formFactor: "mini"
    }).then(function(overrideEditor){
      overrideFE = overrideEditor;
      if($(overrideDom).find(".col-display").length>0)
      {
        $(overrideDom).find(".col-display").hide();
        $(overrideDom).find(".key-value").hide();
        $('.easNewValue', dom).find("#nvLabel").html(lex.get("duration"));
      }
      overrideEditor.jq().on(events.MODIFY_EVENT, function(){
          return overrideEditor.read().then(function(override){
            setTimeout(function(){ // jshint ignore:line
               $('.js-dialog-button-ok').attr('disabled', true);
            },100);
          });
       });
      return overrideEditor;
    });
  };

  SecuredEnumOverrideEditor.prototype.doLoad = function(secureEnumOverride, params)
  {

    var widget = this, dom = widget.jq();
    SecuredEnumOverrideEditor.componentFE = secureEnumOverride;
    controlChoiceVal=secureEnumOverride.getValue();

    var result = this.doLoadValue(dom,secureEnumOverride);
      if(result === "NOT_CONNECTED")
          return Promise.resolve("");

    if(controlChoiceVal!== null)
    {
      valuePresent = controlChoiceVal.getRange().getOrdinals().length>0;
      var enumOrdinals = controlChoiceVal.getRange().getOrdinals();
      var ordinal = controlChoiceVal.getOrdinal();
      var tempHtml ="";
      _.each(enumOrdinals,function(value,key){
        if(value === ordinal)
          {
          tempHtml+="<option ordinal="+value+" value="+value+" selected>"+controlChoiceVal.getRange().getTag(value)+"</option>";
          }
        else
          {
          tempHtml+="<option ordinal="+value+" value="+value+">"+controlChoiceVal.getRange().getTag(value)+"</option>";
          }

        dom.find("#txtNewValue").html(tempHtml);
      });
      if(valuePresent === false)
      {
        dom.find("#txtNewValue").html("<option ordinal="+ordinal+" value="+ordinal+" selected>"+ordinal+"</option>");
      }
      dom.find('#txtNewValue').editableSelect({ filter: false });
    }

    var promise = widget.loadOverrideFE(dom,secureEnumOverride.getOverride());
    var overrideEditor;
    return Promise.resolve(promise).then(function(fe){
      overrideEditor = fe;
      return overrideEditor.setReadonly(false);
    }).then(function(){
      overrideEditor.$builder.getEditorFor('duration').setModified(true);
      overrideEditor.setModified(true);
      return widget.validateSecondaryUsers(secureEnumOverride);
    });
  };
			
  SecuredEnumOverrideEditor.prototype.doIsValueValid = function(dom)
  {
     var invalidList=[],
     invalidListObj = {},
     ordinal=saveToOrdinal(dom);

     if(ordinal === "InvalidValue")
     {
       invalidListObj = {};
       invalidListObj.id="ddlEnumNewValue";
       invalidListObj.errorMsg=lex.get('enum.editor.value.validation');
       invalidList.push(invalidListObj);
     }
     else
     {
      dom.find('#txtNewValue').removeClass("errorMsg");
     }

      var overrideWidget = $("#overrideValue", dom).data('widget');
      var promise = new Promise(function(resolve, reject){
        overrideWidget.save().then(function(){
          overrideWidget.doRead().then(function(newValueField){
            try
            {
              baja.$("baja:RelTime").make(overrideWidget.$builder.$dataSource.$map.get("duration").$val.$ms);
            }
            catch(err)
            {
              // No logging as overrideWidget.$builder.$dataSource.$map becomes null when duration is permanent.
            }
            resolve(invalidList);
          });
        },function(err){
           if(err)
           {
               invalidListObj = {};
               invalidListObj.id="staticParent";
               invalidListObj.errorMsg=err;
               invalidList.push(invalidListObj);
           }
           else
           {
             dom.find('#staticParent').removeClass("errorMsg");
           }
           resolve(invalidList);
        });
      });

     return promise;
  };


  function getEnum(dom)
  {

    var ordinal,dynamicEnum;
    if(valuePresent===true)
    {
      ordinal=saveToOrdinal(dom);
      dynamicEnum= baja.$("baja:DynamicEnum").make({ordinal:parseInt(ordinal), range:SecuredEnumOverrideEditor.componentFE.getValue().getRange()});
    }
    else
    {
      ordinal=dom.find("input#txtNewValue").val();
      dynamicEnum = baja.$("baja:DynamicEnum").make(parseInt(ordinal));
    }
    return dynamicEnum;
  }

  SecuredEnumOverrideEditor.prototype.doRead = function() {

    var that = this;
    var dom = this.jq();

    var FE = $("#overrideValue",dom).data('widget');
    FE.setModified(true);
    FE.$builder.getEditorFor('duration').setModified(true);

    return FE.read().then(function(overrideValue){

      var secureEnumOverride = that.value();
      secureEnumOverride.setValue(getEnum(dom));
      if(!_.isEmpty(overrideValue.$map))
      {
        var duration = baja.$("baja:RelTime").make(overrideValue.$map.duration.$ms);
        var overrideVal = baja.$("control:EnumOverride",{ duration : duration, value: getEnum(dom) });
        return secureEnumOverride.setOverride(overrideVal).then(function()
        {
          return secureEnumOverride;
        });
      }
      else
      {
        return secureEnumOverride.getOverride().setValue(getEnum(dom)).then(function(){
          return secureEnumOverride;
        });
      }
    });
  };

  SecuredEnumOverrideEditor.prototype.doSave = function(secureEnumOverride, params)
  {
    var dom = this.jq();
    secureEnumOverride = this.doSaveValue(secureEnumOverride,dom);
    return secureEnumOverride;
  };

  function saveToOrdinal(dom)
  {
    var tag = dom.find("input#txtNewValue").val();
    if(valuePresent === true)
    {
        var enumOrdinals = controlChoiceVal.getRange().getOrdinals();
        var index = _.find(enumOrdinals,function(value){
            return tag.equals(controlChoiceVal.getRange().getTag(value, null));
        });
        if(!(index === undefined || index === null))
            return index;

        return "InvalidValue";
    }

    //Validation for hex numbers and normal numbers.
     var pattHex= /^(0[x])?[a-fA-F0-9]+$/;
     var resHex = pattHex.test(tag);
     var patt = /^[-+]?[0-9]+$/;
     var res = patt.test(tag);
    if(res === true || resHex === true)
    {
      if(isNaN(parseInt(tag)) === false)
      {
      if(parseInt(tag)>2147483647 || parseInt(tag)<(-2147483648))
      {
       return "InvalidValue";
      }
      return tag;
      }
    }
    return "InvalidValue";
  }
			
  SecuredEnumOverrideEditor.prototype.doReadonly = function(readonly) {
    //this.$getTextElement().prop('readonly', readonly);
    this.jq().find("#txtNewValue").prop("disabled", readonly);
    overrideFE.setReadonly(readonly);
  };

  return SecuredEnumOverrideEditor;
});
