define(['baja!','baja!electronicSignature:SecureFacet',
        'lex!baja',
        'bajaux/Properties',
        'bajaux/Widget',
        'jquery',
        'bajaux/events',
        'dialogs',
        'lex!electronicSignature',
        'Promise',
        'underscore',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        "nmodule/webEditors/rc/fe/fe",
        'nmodule/webEditors/rc/fe/baja/FacetsEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'hbs!nmodule/electronicSignature/rc/fe/template/FacetEditorTemplate',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         Properties,
         Widget,
         $,
         events,
         dialogs,
         electronicSignatureLex,
         Promise,
         _,
         loader,
         BaseEditor,
         fe,
         FacetsEditor,
         SecureParentEditor,
         //ESignFacetsEditor,
         template) {
         'use strict';
  var MODIFY_EVENT = events.MODIFY_EVENT;
  var lex = electronicSignatureLex[0];

////////////////////////////////////////////////////////////////
// FacetsEditor definition
////////////////////////////////////////////////////////////////

  /**
   * A field editor for working with `Facets` values.
   *
   * A `FacetsEditor` supports the following `bajaux` `Properties`:
   *
   * - `allowAdd`: set to `false` to prevent new facets from being added.
   * - `allowChangeType`: set to `false` to prevent facet types from being
   *   changed.
   * - `allowDelete`: set to `false` to prevent facets from being removed.
   * - `allowRename`: set to `false` to prevent facet names from being changed.
   * - `showHeader`: set to `false` to prevent the header from being shown.
   * - `uxFieldEditors`: set this to a `baja.Facets` instance where the keys
   *   correspond to the keys in the edited `Facets`. The values should be
   *   type specs or RequireJS module IDs. This alZlows you to specify exactly
   *   what kind of editor to use to edit facets.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
   * @alias module:nmodule/webEditors/rc/fe/baja/FacetsEditor
   */
  var SecuredFacetsEditor = function SecuredFacetsEditor(params) {
    BaseEditor.call(this, $.extend({
      keyName: 'SecuredFacetsEditor'
    }, params));
  };

  SecuredFacetsEditor.prototype = Object.create(SecureParentEditor.prototype);
  SecuredFacetsEditor.prototype.constructor = SecuredFacetsEditor;
  SecuredFacetsEditor.componentFE = null;

  /**
   * Creates a table to hold the array of facet values.
   *
   * @param {jQuery} dom
   */
  SecuredFacetsEditor.prototype.doInitialize = function (dom, params) {

    console.log("Do Initialize"); // jshint ignore:line
    var that = this;
    var jsonObject={pointName:lex.get('fe.pointName.label'),
    newValue:lex.get('fe.newValue.label'),
    reason:lex.get('fe.reason.label'),
    otherReason:lex.get('fe.otherReason.label'),
    authLevel1:lex.get('fe.authorization.level1.label'),
    authLevel2:lex.get('fe.authorization.level2.label'),
    userName:lex.get('fe.userName.label'),
    password:lex.get('fe.password.label'),
    comment:lex.get('fe.comment.label'),
    sign:lex.get('fe.sign.button'),
    sign2:lex.get('fe.sign2.button'),
    remote:lex.get('fe.remote.button'),
    chooseUser:lex.get('remote.chooseUser.label'),
    local:lex.get('remote.local.button'),
    sendRequest:lex.get('remote.sendRequest.button'),
    acknowledgement:lex.get('eSignature.ack.label'),
    label1:lex.get('eSignature.ack.label1'),
    label2:lex.get('eSignature.ack.label2'),
    label3:lex.get('eSignature.ack.label3'),
    label4:lex.get('eSignature.ack.label4'),
    label5:lex.get('eSignature.ack.label5'),
    label6:lex.get('eSignature.ack.label6'),
    label7:lex.get('eSignature.ack.label7')
      };
    dom.html(template({jsonObject:jsonObject}));
    this.doInitializeContent(dom);
    var keys = ['one', 'two'],
        values = [1, 2],
        facets = baja.Facets.make(keys, values),
        facetsDom = $('#facetEditor', dom);
    return fe.buildFor({
        dom: facetsDom,
        value: facets,
        type: FacetsEditor,
        properties: { allowAdd: true, allowChangeType : true, allowDelete: true, allowRename: true },
        formFactor: "compact"
      }).then(function(ed){
          that.$facetsFe = ed;
          dom.on(MODIFY_EVENT, '.editor', function (e) {
            that.setModified(false);
            return false;
          });
      });
  };


/**
   * Loads the `Facets` into the editor, creating a table row for each key
   * in the `Facets`. Each row has field editors for key, type, and value.
   *
   * @param {baja.Facets} value
   * @returns {Promise} promise to be resolved when all the field editors
   * have finished loading into the DOM
   */

  SecuredFacetsEditor.prototype.doLoad = function (facetValue,params) {
    var widget = this,
      dom = widget.jq();

    SecuredFacetsEditor.componentFE = facetValue;

    var result = this.doLoadValue(dom,facetValue);
      if(result === "NOT_CONNECTED")
          return Promise.resolve("");

    return this.$facetsFe.load(facetValue.getOldFacets()).then(function(){
      return widget.validateSecondaryUsers(facetValue).then(function(){
          setTimeout(function () {
              $('.js-dialog-button-ok').attr('disabled', true);
          },100);
          return Promise.resolve("");
      });
    });
  };

/**
   * Reads key and value editors from each row and assembles them into a
   * `Facets` instance.
   *
   * @returns {Promise} promise to be resolved with a `baja.Facets`
   * instance read from key and value editors, or rejected in the case of
   * duplicate keys or invalid values
   */

  SecuredFacetsEditor.prototype.doRead = function () {
    var that = this;
    var dom = that.jq();
    return Promise.resolve(this.$facetsFe.isInitialized() && this.$facetsFe.read()).then(function(facets){
        return that.value().setNewFacets(facets);
    }).then(function(){
        return that.value().setPrimaryUser(dom.find("#username").val());
    }).then(function(){
        return that.value().setIsAuthenticated(that.isValidUser());
    }).then(function(){
        if(that.value().getReasonForChange()!==null){
          return that.value().setReasonForChange(getReasonValue(dom));
        }
        return Promise.resolve("");
    }).then(function(){
        if(that.value().getSecureSlotFacets().$map.$map.isSecuredSecondary)
        {
          var manager  =  dom.find("#secondaryUsername").val();

          return that.value().setSecondaryUser(manager).then(
              function(){
                  return that.value().setPrimaryPassword(btoa(dom.find('#password').val()));
              }
          ).then(function(){
              return that.value().setSecondaryPassword(btoa(dom.find('#secondaryPassword').val()));
          });
        }
        else
        {
            return that.value().setPrimaryPassword(btoa(dom.find('#password').val()));
        }
    }).then(function(){
        that.value().set({slot:"primaryComments",value :dom.find("#primaryComment").val()});
    }).then(function(){
        if(that.value().getSecureSlotFacets().$map.$map.isSecuredSecondary)
        {
            that.value().set({slot: "secondaryComments", value: dom.find("#secondaryComments").val()});
        }
        return Promise.resolve("");
    }).then(function(){
        return that.value();
    });
  };

  SecuredFacetsEditor.prototype.doIsValueValid = function(dom){
      var invalidList=[],
      invalidListObj = {};
      return Promise.resolve(this.$facetsFe.validate()).then(function(facets){
      },function(error){
         invalidListObj = {};
         invalidListObj.id="staticParent";
         invalidListObj.errorMsg=error.message;
         invalidList.push(invalidListObj);
         return invalidList;
      });
  };

  SecuredFacetsEditor.prototype.doSave = function (secureFacets,param) {
      var that = this;
      return this.$facetsFe.doRead().then(function (facets) {
          return that.$facetsFe.doSave(facets)}).then(function () {
          return secureFacets});
  };

  function getReasonValue(dom){
    if(dom.find("#idReason").val()==="Other"){
      return dom.find("#idTextarea").val();
    }
    else{
      return dom.find("#idReason").val();
    }
  }

  return SecuredFacetsEditor;
});
