/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * API Status: **Private**
 * @module nmodule/webEditors/rc/wb/menu/navMenuAgent
 */
define(['baja!',
    'bajaux/commands/CommandGroup',
    'underscore',
    'nmodule/electronicSignature/rc/fe/baja/Util',
    'nmodule/webEditors/rc/transform/ExportCommand',
    'nmodule/webEditors/rc/wb/menu/menuUtils',
    'nmodule/webEditors/rc/wb/menu/Separator'],
    function (baja, CommandGroup, _, Util, ExportCommand, menuUtils, Separator) {
    'use strict';

    var makeViewsGroup = menuUtils.makeViewsGroup;
    /**
     * @alias module:nmodule/webEditors/rc/wb/menu/navMenuAgent
     * @param {module:bajaux/Widget} widget
     * @param {baja.Component} comp
     * @returns {Promise.<module:bajaux/commands/CommandGroup>}
     */

    var SecuredHistoryMenuAgent = function SecuredHistoryMenuAgent(widget, comp) {
        return menuUtils.ifAllowedByProfile('viewSelection', function () {
            return makeViewsGroup(comp.getNavOrd());
        }).then(function (viewsGroup) {
            console.log("Context Menu: Secured History Nav Menu Agent");
            console.log(viewsGroup);

            return Util.getESignatureComponent().then((eSignComponent)=>{

                let filtered = viewsGroup.$kids;
                if(comp.getType().is("history:HistorySpace") ||
                    comp.getType().is("history:HistoryFolder"))
                {
                    filtered = viewsGroup.$kids.filter(function(value, index, arr){
                        return value.getDisplayNameFormat().toLowerCase().indexOf("maintenance")==-1;
                    });
                    viewsGroup.$kids = filtered;
                    var commands,
                        exportCommand = new ExportCommand(comp);
                    commands = viewsGroup ? [viewsGroup, new Separator(), exportCommand] : [exportCommand];
                    return new CommandGroup({
                        commands: commands
                    });
                }
                else //if (comp.getType().is("history:IHistory"))
                {
                    return getHistoryNames().then(function(histories){
                        console.log("Secured histories:", histories);
                        if(comp.getNavName() == histories.alarmHistory ||
                            comp.getNavName() == histories.pointHistory)
                        {
                            filtered = viewsGroup.$kids.filter(function(value, index, arr){
                                return value.getDisplayNameFormat().toLowerCase().indexOf("history editor")==-1;
                            });
                        }

                        viewsGroup.$kids = filtered;
                        var commands,
                            exportCommand = new ExportCommand(comp);
                        commands = viewsGroup ? [viewsGroup, new Separator(), exportCommand] : [exportCommand];
                        return new CommandGroup({
                            commands: commands
                        });
                    });

                }
            }).catch((err)=>{
                var commands,
                    exportCommand = new ExportCommand(comp);
                commands = viewsGroup ? [viewsGroup, new Separator(), exportCommand] : [exportCommand];
                return new CommandGroup({
                    commands: commands
                });
            });
        });
    };

    function getHistoryNames()
    {
        return Util.getESignatureComponent().then(function(eSign){
            return eSign.loadSlots().then(function(){
                let histories = {pointHistory: eSign.getSecuredHistoryName(),
                    alarmHistory: eSign.getSecuredAlarmHistoryName()
                };
                return histories;
            });
        });
    }

    function isHistoryNode(comp) {
        if(comp.getType().$typeSpec.equals("history:HistorySpace") ||
            comp.getType().$typeSpec.equals("history:IHistory") ||
            comp.getType().$typeSpec.equals("history:HistoryFolder")){
            return true;
        }
        return false;
    }

    return SecuredHistoryMenuAgent;
});
