/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

/**
 * SecuredNumericEditor manages functionality like primary, secondary authentication ,Change new value and maintain the secure point history.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredNumericEditor
 */
define(['baja!','baja!electronicSignature:SecureDouble',
        'lex!baja',
        'jquery',
        'Promise',
        'dialogs',
        'bajaux/Widget',
        'underscore',
        'lex!electronicSignature',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredNumericTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'nmodule/webEditors/rc/fe/baja/util/numberUtils',
        'nmodule/webEditors/rc/fe/baja/util/FloatingPointHelper',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         Widget,
         _,
         electronicSignatureLex,
         subscriberMixIn,
         template,
         loader,
         BaseEditor,
         SecureParentEditor,
         numberUtils,
         FloatingPointHelper) {
   'use strict';

     var convertUnitTo = numberUtils.convertUnitTo,
             convertUnitFrom = numberUtils.convertUnitFrom,
             getDisplayUnits = numberUtils.getDisplayUnits,
             DEFAULT_PROPERTIES = {
                 forceSign: false,
                 precision: 2,
                 showSeparators: false
             },
             lex = electronicSignatureLex[0],
             min, max;
    var SecuredNumericEditor = function SecuredNumericEditor(params) {
      BaseEditor.call(this, $.extend({
        keyName: 'SecuredNumericEditor'
      }, params));
      subscriberMixIn(this);
    };

    SecuredNumericEditor.componentFE = null;
    SecuredNumericEditor.prototype = Object.create(SecureParentEditor.prototype);
    SecuredNumericEditor.prototype.constructor = SecuredNumericEditor;

    SecuredNumericEditor.prototype.doInitialize = function (dom) {
        var jsonObject={pointName:lex.get('fe.pointName.label'),
            newValue:lex.get('fe.newValue.label'),
            reason:lex.get('fe.reason.label'),
            otherReason:lex.get('fe.otherReason.label'),
            authLevel1:lex.get('fe.authorization.level1.label'),
            authLevel2:lex.get('fe.authorization.level2.label'),
            userName:lex.get('fe.userName.label'),
            password:lex.get('fe.password.label'),
            comment:lex.get('fe.comment.label'),
            sign:lex.get('fe.sign.button'),
            sign2:lex.get('fe.sign2.button'),
            remote:lex.get('fe.remote.button'),
            chooseUser:lex.get('remote.chooseUser.label'),
            local:lex.get('remote.local.button'),
            sendRequest:lex.get('remote.sendRequest.button'),
            acknowledgement:lex.get('eSignature.ack.label'),
            label1:lex.get('eSignature.ack.label1'),
            label2:lex.get('eSignature.ack.label2'),
            label3:lex.get('eSignature.ack.label3'),
            label4:lex.get('eSignature.ack.label4'),
            label5:lex.get('eSignature.ack.label5'),
            label6:lex.get('eSignature.ack.label6'),
            label7:lex.get('eSignature.ack.label7')

        };
        dom.html(template({jsonObject:jsonObject}));
      this.doInitializeContent(dom);
    };

    SecuredNumericEditor.prototype.$getUnits = function () {
      return this.properties().getValue('units');
    };

    SecuredNumericEditor.prototype.$getHelper = function () {
      var that = this,
          helper = that.$helper;

      if (!helper) {
        var value = that.value(),
            ctor = (value !== null) && value.constructor;

          helper = new FloatingPointHelper(baja.Double,
              $.extend({}, DEFAULT_PROPERTIES, that.properties().toValueMap()));

        that.$helper = helper;
      }

      return helper;
    };

    SecuredNumericEditor.prototype.$getUnitConversion = function () {
      return this.properties().getValue('unitConversion');
    };

    SecuredNumericEditor.prototype.doLoad = function(secureDouble,params){
      var widget = this,
          dom = widget.jq();
          SecuredNumericEditor.componentFE = secureDouble;
      var result = this.doLoadValue(dom,secureDouble);
        if(result === "NOT_CONNECTED")
            return Promise.resolve("");
      min =  secureDouble.getSecureSlotFacets().$map.get("min").$val;
      max =  secureDouble.getSecureSlotFacets().$map.get("max").$val;

      if(min === undefined){
          min =  secureDouble.getSecureSlotFacets().$map.get("min");
          max =  secureDouble.getSecureSlotFacets().$map.get("max");
      }

      if(min === null || !$.isNumeric(min))
      {
        min = null;
      }
      else
      {
        if(!_.isNumber(min))
          min = widget.$getHelper().parse(min);
      }

      if(max === null || !$.isNumeric(max))
      {
        max = null;
      }
      else
      {
        if(!_.isNumber(max))
          max = widget.$getHelper().parse(max);
      }

      widget.$getHelper().format(parseFloat(secureDouble.getNewValue())).then(function(value){
        return convertUnitTo(value, widget.$getUnits(), widget.$getUnitConversion())
          .then(function (converted) {
              return widget.$getHelper().format(value.make(converted.valueOf()));
          }).then(function(formattedValue){
             dom.find("#txtNewValue").val(formattedValue);
              return widget.validateSecondaryUsers(secureDouble);
          });
      });
    };

    SecuredNumericEditor.prototype.isWithinRange = function()
    {
        var that = this,
            helper = that.$getHelper(),
            result = helper.parse(this.jq().find('#txtNewValue').val());

      if(isNaN(result))
      {
        return false;
      }
      if(min !== null && min !== undefined && result < min)
      {
        return false;
      }
      return !(max !== null && max !== undefined && result > max);
    };


    SecuredNumericEditor.prototype.doIsValueValid = function(dom){
        var invalidList=[],
        invalidListObj = {};

        var txtNewValue = dom.find('#txtNewValue').val();

        if(!$.isNumeric(txtNewValue)){
            invalidListObj = {};
            invalidListObj.id="txtNewValue";
            invalidListObj.errorMsg=lex.get('pointValue.format.error');
            invalidList.push(invalidListObj);

            return invalidList;
        }

        var that = this,
            helper = that.$getHelper(),
            result = helper.parse(dom.find('#txtNewValue').val());

        if (isNaN(result)) {
          return result;
        }

        //convert from user's preferred unit system to real number
        return convertUnitFrom(result.valueOf(), that.$getUnits(), that.$getUnitConversion())
          .then(function (converted) {
              var newValueField = helper.toType(converted);
              if(newValueField!=="")
              {
                if(!$.isNumeric(newValueField)){
                  invalidListObj = {};
                  invalidListObj.id="txtNewValue";
                  invalidListObj.errorMsg=lex.get('pointValue.format.error');
                  invalidList.push(invalidListObj);
                }
                else if(!that.isWithinRange())
                {
                  var minString = min;
                  if(minString === null)
                  {
                    minString = "-Infinity";
                  }

                  var maxString = max;
                  if(maxString === null)
                  {
                    maxString = "+Infinity";
                  }

                  invalidListObj = {};
                  invalidListObj.id="txtNewValue";
                  invalidListObj.errorMsg=lex.get('point.range.message') + minString + " - " + maxString + "].";
                  invalidList.push(invalidListObj);
                }
                else
                {
                  dom.find('#txtNewValue').removeClass("errorMsg");
                }
              }
              else
              {
                invalidListObj = {};
                invalidListObj.id="txtNewValue";
                invalidListObj.errorMsg=lex.get('pointValue.format.error');
                invalidList.push(invalidListObj);
              }
              return invalidList;
        });
    };

    SecuredNumericEditor.prototype.doRead = function(){
      var that = this,
      dom = this.jq(),
      secureDouble = that.value(),
      helper = that.$getHelper(),
      result = helper.parse(getPointNewValue(dom).val());

      if (isNaN(result)) {
        return result;
      }

      //convert from user's preferred unit system to real number
      return convertUnitFrom(result.valueOf(), that.$getUnits(), that.$getUnitConversion())
        .then(function (converted) {
          var convertedValue =  helper.toType(converted);
          return secureDouble.setNewValue(convertedValue).then(function(){ return secureDouble; });
        });

    };

    SecuredNumericEditor.prototype.doSave = function (secureDouble,params) {
      var dom = this.jq();
      secureDouble = this.doSaveValue(secureDouble,dom);
      return secureDouble;
    };

    function getPointNewValue(dom) {
      return dom.find(".newValue");
    }

    return SecuredNumericEditor;
  });