/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Balaji Gite
 */

/**
 * SecuredBooleanOverrideEditor manages functionality like primary, secondary authentication ,Change new value and maintain the secure point history.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredBooleanOverrideEditor
 */
define(['baja!','baja!electronicSignature:SecureNumericOverride,control:Override,control:NumericOverride',
        'lex!baja',
        'jquery',
        'Promise',
        'dialogs',
        'lex!electronicSignature',
        'bajaux/Widget',
        'underscore',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredNumericOverrideTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'bajaux/events',
        "nmodule/webEditors/rc/fe/fe",
        'nmodule/webEditors/rc/fe/baja/OverrideEditor',
        'nmodule/webEditors/rc/fe/baja/util/numberUtils',
        'nmodule/webEditors/rc/fe/baja/util/FloatingPointHelper',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
    		 baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         electronicSignatureLex,
         Widget,
         _,
         subscriberMixIn,
         template,
         loader,
         BaseEditor,
         SecuredParentEditor,
         events,
         fe,
         OverrideEditor,
         numberUtils,
         FloatingPointHelper) {
  'use strict';
     var convertUnitTo = numberUtils.convertUnitTo,
         convertUnitFrom = numberUtils.convertUnitFrom,
         getDisplayUnits = numberUtils.getDisplayUnits,
         DEFAULT_PROPERTIES = {
             forceSign: false,
             precision: 2,
             showSeparators: false
         },
         lex = electronicSignatureLex[0],
         min, max;

  /**
   * A field editor for working with booleans.
   *
   * @private
   * @class
   * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
   * @alias module:nmodule/electronicSignature/rc/fe/baja/SecuredNumericOverrideEditor
   */
  var SecuredNumericOverrideEditor = function (params) {
    BaseEditor.call(this, $.extend({
      keyName: 'SecuredNumericOverrideEditor'
    }, params));
    subscriberMixIn(this);
  };

  SecuredNumericOverrideEditor.isAuthenticatedUser = false;
  SecuredNumericOverrideEditor.componentFE = null;
  SecuredNumericOverrideEditor.prototype = Object.create(SecuredParentEditor.prototype);
  SecuredNumericOverrideEditor.prototype.constructor = SecuredNumericOverrideEditor;


  SecuredNumericOverrideEditor.prototype.doInitialize = function (dom) {
      var jsonObject={pointName:lex.get('fe.pointName.label'),
          overrideValue:lex.get('editor.override.val'),
          overrideDuration:lex.get('editor.override.duration'),
          reason:lex.get('fe.reason.label'),
          otherReason:lex.get('fe.otherReason.label'),
          authLevel1:lex.get('fe.authorization.level1.label'),
          authLevel2:lex.get('fe.authorization.level2.label'),
          userName:lex.get('fe.userName.label'),
          password:lex.get('fe.password.label'),
          comment:lex.get('fe.comment.label'),
          sign:lex.get('fe.sign.button'),
          sign2:lex.get('fe.sign2.button'),
          remote:lex.get('fe.remote.button'),
          chooseUser:lex.get('remote.chooseUser.label'),
          local:lex.get('remote.local.button'),
          sendRequest:lex.get('remote.sendRequest.button'),
          acknowledgement:lex.get('eSignature.ack.label'),
          label1:lex.get('eSignature.ack.label1'),
          label2:lex.get('eSignature.ack.label2'),
          label3:lex.get('eSignature.ack.label3'),
          label4:lex.get('eSignature.ack.label4'),
          label5:lex.get('eSignature.ack.label5'),
          label6:lex.get('eSignature.ack.label6'),
          label7:lex.get('eSignature.ack.label7')

      };
      dom.html(template({jsonObject:jsonObject}));
    this.doInitializeContent(dom);
  };

  SecuredNumericOverrideEditor.prototype.$getUnits = function () {
    return this.properties().getValue('units');
  };

  SecuredNumericOverrideEditor.prototype.$getHelper = function () {
    var that = this,
        helper = that.$helper;

    if (!helper) {
      var value = that.value(),
          ctor = (value !== null) && value.constructor;

      helper = new FloatingPointHelper(ctor || baja.Double,
        $.extend({}, DEFAULT_PROPERTIES, that.properties().toValueMap()));

      that.$helper = helper;
    }

    return helper;
  };

  SecuredNumericOverrideEditor.prototype.$getUnitConversion = function () {
    return this.properties().getValue('unitConversion');
  };

  SecuredNumericOverrideEditor.prototype.loadOverrideFE = function(dom, override)
  {
    var overrideDom = $("#overrideValue",dom);
    return fe.buildFor({
      dom: overrideDom,
      value: override,
      type: OverrideEditor,
      formFactor: "mini"
    }).then(function(overrideEditor){
      if($(overrideDom).find(".col-display").length>0)
      {
        $(overrideDom).find(".col-display").hide();
        $(overrideDom).find(".key-value").hide();
        $('.easNewValue', dom).find("#nvLabel").html("Duration:");
      }
      overrideEditor.jq().on(events.MODIFY_EVENT, function(){
          return overrideEditor.read().then(function(override){
            setTimeout(function(){ // jshint ignore:line
               $('.js-dialog-button-ok').attr('disabled', true);
            },100);
          });
       });
      return overrideEditor;
    });
  };


  SecuredNumericOverrideEditor.prototype.doLoad = function (secureNumericOverride,params) {

    var widget = this, dom = widget.jq();
    SecuredNumericOverrideEditor.componentFE = secureNumericOverride;
    var result = this.doLoadValue(dom,secureNumericOverride);
      if(result === "NOT_CONNECTED")
          return Promise.resolve("");
    var value = secureNumericOverride.getValue();
    //dom.find("#txtNewValue").val(secureNumericOverride.getValue());

      min =  secureNumericOverride.getSecureSlotFacets().$map.get("min").$val;
      max =  secureNumericOverride.getSecureSlotFacets().$map.get("max").$val;

      if(min === undefined){
          min =  secureNumericOverride.getSecureSlotFacets().$map.get("min");
          max =  secureNumericOverride.getSecureSlotFacets().$map.get("max");
      }

    if(min === null || !$.isNumeric(min))
    {
      min = null;
    }
    else
    {
      min = parseFloat(min);
    }

    if(max === null || !$.isNumeric(max))
    {
      max = null;
    }
    else
    {
      max = parseFloat(max);
    }

    var promise = widget.loadOverrideFE(dom,secureNumericOverride.getOverride());
    var overrideEditor;
    return Promise.resolve(promise).then(function(fe){
      overrideEditor = fe;
      return overrideEditor.setReadonly(false);
    }).then(function(){
      overrideEditor.$builder.getEditorFor('duration').setModified(true);
      overrideEditor.setModified(true);
      return convertUnitTo(value, widget.$getUnits(), widget.$getUnitConversion());
    }).then(function (converted) {
      var str = widget.$getHelper().format(value.make(converted.valueOf()));
      return str;
    }).then(function(formattedValue){
       dom.find("#txtNewValue").val(formattedValue);
       return widget.validateSecondaryUsers(secureNumericOverride);
    });
  };

  SecuredNumericOverrideEditor.prototype.isWithinRange = function(newValueField)
  {
      var val = newValueField;
      if(!$.isNumeric(val))
      {
        return false;
      }
      if(min !== null && min !== undefined && val < min)
      {
        return false;
      }
      if(max !== null && max !== undefined && val > max)
      {
        return false;
      }
      return true;
  };


  SecuredNumericOverrideEditor.prototype.doIsValueValid = function(dom)
  {
  	 var invalidList=[],
     invalidListObj = {};

      var txtNewValue = dom.find('#txtNewValue').val();
      if(!$.isNumeric(txtNewValue)){
          invalidListObj = {};
          invalidListObj.id="txtNewValue";
          invalidListObj.errorMsg=lex.get('pointValue.format.error');
          invalidList.push(invalidListObj);

          return invalidList;
      }

      var that = this,
         helper = that.$getHelper(),
         result = helper.parse(dom.find('#txtNewValue').val());

     if (isNaN(result)) {
       return result;
     }

     return convertUnitFrom(result.valueOf(), that.$getUnits(), that.$getUnitConversion())
       .then(function (converted) {
           var newValueField = helper.toType(converted);
           if(newValueField!=="")
           {
             if(!$.isNumeric(newValueField))
             {
               invalidListObj = {};
               invalidListObj.id="txtNewValue";
               invalidListObj.errorMsg=lex.get('pointValue.format.error');
               invalidList.push(invalidListObj);
             }
             else if(!that.isWithinRange(newValueField))
             {
               var minString = min;
               if(minString === null)
               {
                 minString = "-Infinity";
               }

               var maxString = max;
               if(maxString === null)
               {
                 maxString = "+Infinity";
               }

               invalidListObj = {};
               invalidListObj.id="txtNewValue";
               invalidListObj.errorMsg=lex.get('point.range.message') + minString + " - " + maxString + "].";
               invalidList.push(invalidListObj);
             }
             else
             {
               dom.find('#txtNewValue').removeClass("errorMsg");
             }
           }
           else
           {
             invalidListObj = {};
             invalidListObj.id="txtNewValue";
             invalidListObj.errorMsg=lex.get('pointValue.format.error');
             invalidList.push(invalidListObj);
           }

           var overrideWidget = $("#overrideValue", dom).data('widget');
           var promise = new Promise(function(resolve, reject){
             overrideWidget.save().then(function(){
               overrideWidget.doRead().then(function(newValueField){
                 try
                 {
                   baja.$("baja:RelTime").make(overrideWidget.$builder.$dataSource.$map.get("duration").$val.$ms);
                 }
                 catch(err)
                 {
                   // No logging as overrideWidget.$builder.$dataSource.$map becomes null when duration is permanent.
                 }
                 resolve(invalidList);
               });
             },function(err){
                if(err)
                {
                    invalidListObj = {};
                    invalidListObj.id="staticParent";
                    invalidListObj.errorMsg=err;
                    invalidList.push(invalidListObj);
                }
                else
                {
                  dom.find('#staticParent').removeClass("errorMsg");
                }
                resolve(invalidList);
             });
           });
           return promise;
       });
  };

  SecuredNumericOverrideEditor.prototype.doRead = function(){
      var that = this,
      dom = this.jq(),
      secureDouble = that.value(),
      helper = that.$getHelper(),
      result = helper.parse(dom.find("#txtNewValue").val());

      if (isNaN(result)) {
          return result;
      }

      //convert from user's preferred unit system to real number
      return convertUnitFrom(result.valueOf(), that.$getUnits(), that.$getUnitConversion()).then(function (converted) {
        var convertedValue =  helper.toType(converted);
        var FE = $("#overrideValue",dom).data('widget');
        FE.setModified(true);
        FE.$builder.getEditorFor('duration').setModified(true);

        return FE.read().then(function(overrideValue){
          var securedNumericOverride = that.value();
          securedNumericOverride.setValue(convertedValue);
          if(!_.isEmpty(overrideValue.$map))
          {
            var duration = baja.$("baja:RelTime").make(overrideValue.$map.duration.$ms);
            var overrideVal = baja.$("control:NumericOverride",{ duration : duration, value: convertedValue });
            return securedNumericOverride.setOverride(overrideVal).then(function()
            {
              return securedNumericOverride;
            });
          }
          else
          {
            return securedNumericOverride.getOverride().setValue(convertedValue).then(function(){
              return securedNumericOverride;
            });
          }
        });
      });
  };

  SecuredNumericOverrideEditor.prototype.doSave = function (secureNumericOverride,params) {
  	var dom = this.jq();
    secureNumericOverride = this.doSaveValue(secureNumericOverride,dom);
    return secureNumericOverride;
  };
  
  return SecuredNumericOverrideEditor;
});

