/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

/**
 * SecureParentEditor manages the  reusable code for the child FE.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecureParentEditor
 */
define(['baja!', 'baja!electronicSignature:SecureDouble,electronicSignature:SecureVirtualAction',
    'lex!baja',
    'jquery',
    'Promise',
    'dialogs',
    'lex!electronicSignature',
    'bajaux/Widget',
    'underscore',
    'bajaux/mixin/subscriberMixIn',
    'nmodule/electronicSignature/rc/fe/baja/loader',
    'nmodule/webEditors/rc/fe/baja/BaseEditor',
    'nmodule/electronicSignature/rc/fe/baja/Util',
    'css!nmodule/electronicSignature/rc/css/widgets'], function (baja,
     types,
     lexs,
     jQuery,
     Promise,
     dialogs,
     electronicSignatureLex,
     Widget,
     _,
     subscriberMixIn,
     loader,
     BaseEditor,
     Util)
    {
    'use strict';


    var lex = electronicSignatureLex[0];
    var $ = jQuery;
    var _facets = null,
        _SecondaryAuthenticatorRoles = "", actionName = null, component = null, isRemoteRequest = null,
        isValidManager = false,
        isValidUserSecondary = false,
        bajaUser = baja.getUserName(),
        legalTextTemplate = "",
        customerName = "";

    var SecureParentEditor = function ()
    {
        var that = this;
        subscriberMixIn(that);
    };

    SecureParentEditor.prototype = Object.create(BaseEditor.prototype);
    SecureParentEditor.prototype.constructor = SecureParentEditor;
    SecureParentEditor.isAuthenticatedUser = false;
    SecureParentEditor.componentFE = null;
    SecureParentEditor.prototype.doSaveValue = function (newComp, dom)
    {
        return savePointValues(newComp, dom);
    };

    /*
      Below method will get overridden in the child FE to implemnt the child FE related validations
      @param dom
    */
    SecureParentEditor.prototype.doIsValueValid = function (dom)
    {
        return null;
    };

    /*
      Calling from checkSecondarySign and checkPrimarySign for Data validation for Reason , Newvalue etc
      @param dom
      @param authenticationType
    */
    SecureParentEditor.prototype.doIsDataValid = function (dom, authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        //var regxNumericAndDecimalValue = new RegExp(/^[0-9]\d*(\.\d+)?$/,"g");
        dom.find('#staticParent').removeClass("errorMsg");

        if (authenticationType === "primary")
        {
            var reasonSelectField = dom.find('#idReason').val();
            if (dom.find('#username').val() === null || dom.find('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = lex.get('parent.editor.userName.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                dom.find('#username').removeClass("errorMsg");
            }
            if (dom.find('#password').val() === null || dom.find('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg =lex.get('parent.editor.password.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                dom.find('#password').removeClass("errorMsg");
            }
            if (reasonSelectField === "Other")
            {
                var otherReasonTextBoxValue = dom.find("#idTextarea").val();
                if (otherReasonTextBoxValue === "" || otherReasonTextBoxValue.trim().length === 0)
                {
                    invalidListObj = {};
                    invalidListObj.id = "idTextarea";
                    invalidListObj.errorMsg = lex.get('eSignature.reason.error');
                    invalidList.push(invalidListObj);
                }
                else
                {
                    dom.find('#idTextarea').removeClass("errorMsg");
                }
            }


            var validationErrorMessages = this.doIsValueValid(dom);
            return Promise.resolve(validationErrorMessages).then(function (messages)
            {
                if (_.size(messages) > 0)
                {
                    _.each(messages, function (value)
                    {
                        invalidList.push(value);
                    });
                }
                return invalidList;
            }, function (err)
            {
                if (err)
                {
                    invalidListObj = {};
                    invalidListObj.id = "staticParent";
                    invalidListObj.errorMsg = err;
                    invalidList.push(invalidListObj);
                }
                else
                {
                    dom.find('#staticParent').removeClass("errorMsg");
                }
                return invalidList;
            });
        }
        if (authenticationType === "secondary")
        {

            if (dom.find('#secondaryUsername').val() === null || dom.find('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg = lex.get('parent.editor.userName.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                dom.find('#secondaryUsername').removeClass("errorMsg");
            }
            if (dom.find('#secondaryPassword').val() === null || dom.find('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = lex.get('parent.editor.password.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                dom.find('#secondaryPassword').removeClass("errorMsg");
            }

            if (SecureParentEditor.componentFE.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null && SecureParentEditor.componentFE.has("isRejected") && SecureParentEditor.componentFE.get("isRejected").toString() === "true")
            {
                var rejectionComments = dom.find("#secondaryComments").val();
                if (rejectionComments === "")
                {
                    invalidListObj = {};
                    invalidListObj.id = "secondaryComments";
                    invalidListObj.errorMsg = lex.get('text.error.rejectionComments.NotNull');
                    invalidList.push(invalidListObj);
                }
                else
                {
                    dom.find('#secondaryComments').removeClass("errorMsg");
                }
            }
            return invalidList;
        }
    };

    /*
      Calling from btnSecondarySign button click handler to implement secondary authentication
      @param credentials
      @param dom
    */
    SecureParentEditor.prototype.checkSecondarySign = function (credentials, dom)
    {
        SecureParentEditor.isAuthenticatedUser = false;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title: "Authentication Error",
                content: lex.get('level2.authentication.message')
            });
        }
        var validation = this.doIsDataValid(dom, "secondary");
        Promise.resolve(validation).then(function (validationSummary)
        {
            if (validationSummary !== null && validationSummary.length > 0)
            {
                showValidationSummary(dom, validationSummary);
            }
            else
            {
                var pointOrd = Util.getFormatedPointOrd(SecureParentEditor.componentFE, _facets),
                    pointComp,
                    isValidManager = false;
                if (pointOrd !== "")
                {
                    Util.getComponentByOrd(pointOrd).then(function (comp)
                    {
                        pointComp = comp;
                        dom.find(".btnSecondarySign").html(lex.get('fe.signing.button'));
                        dom.find(".btnSecondarySign").prop("disabled", true);

                        return Util.authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                    }).then(function (authenticatedComp)
                    {
                        dom.find(".btnSecondarySign").html(lex.get('fe.sign.button'));
                        dom.find(".btnSecondarySign").prop("disabled", false);

                        isValidManager = authenticatedComp;
                        if (authenticatedComp)
                        {
                            return Util.authenticateSecondaryUser(pointComp, credentials.username);
                        }
                        else
                        {
                            dialogs.showOk({
                                title: lex.get('authentication.error.title'),
                                content: lex.get('authentication.error.message')
                            });
                        }
                    }).then(function (secondaryUserDetails)
                    {
                        //secondaryUserDetails.loadSlots();
                        return secondaryUserDetails;
                    }).then(function (secondaryUserDetails)
                    {
                        var managerRoles = _SecondaryAuthenticatorRoles.split(",");
                        isValidUserSecondary = false;
                        if (secondaryUserDetails.get("roles") !== null)
                        {
                            var secondaryUsersRoles = secondaryUserDetails.get("roles").split(",");
                            _.each(managerRoles, function (value)
                            {
                                var roleExistList = _.filter(secondaryUsersRoles, function (role)
                                {
                                    return role === value;
                                });
                                if (_.size(roleExistList) > 0)
                                {
                                    isValidUserSecondary = true;

                                }
                            });
                            if (!isValidUserSecondary)
                            {
                                isValidManager = false;
                                return dialogs.showOk({
                                    title: lex.get('authentication.error.title'),
                                    content: lex.get('secUser.role.error')
                                });
                            }
                            else
                            {
                                var permissions = secondaryUserDetails.get("permission");
                                if (permissions !== null)
                                {
                                    var slotName = _facets.$map.get("SlotName").toString();
                                    //if (!((pointComp.getFlags(pointComp.getSlot(slotName)) === baja.Flags.OPERATOR && permissions.hasOperatorRead() && permissions.hasOperatorInvoke()) || (pointComp.getFlags(pointComp.getSlot(slotName)) !== baja.Flags.OPERATOR && permissions.hasAdminRead() && permissions.hasAdminInvoke())))
                                    if (!(((pointComp.getFlags(pointComp.getSlot(slotName)) & baja.Flags.OPERATOR) && permissions.hasOperatorRead() && permissions.hasOperatorInvoke()) || (!(pointComp.getFlags(pointComp.getSlot(slotName)) & baja.Flags.OPERATOR) && permissions.hasAdminRead() && permissions.hasAdminInvoke())))
                                    {
                                        isValidManager = false;
                                        return dialogs.showOk({
                                            title: lex.get('permission.error.title'),
                                            content:lex.get('secUser.permission.error.message')
                                        });
                                    }
                                }
                            }
                        }
                        else
                        {
                            isValidUserSecondary = true;
                            //isValidManager=true;
                        }
                        if (isValidManager && isValidUserSecondary)
                        {
                            $('.js-dialog-button-ok').attr('disabled', false);
                            $('.btnRemote').attr('disabled', true);
                            SecureParentEditor.isAuthenticatedUser = true;
                            dom.find("#secondaryComments").attr('disabled', true);
                            dom.find("#secondaryUsername").prop("disabled", true);
                            dom.find(".btnSecondarySign").html(lex.get('fe.signed.button'));
                            /*return dialogs.showOk({
                                title: lex.get('auth.success.title'),
                                content: lex.get('auth.success.msg')
                            });*/
                        }
                    })
                        .catch(function (error)
                        {
                            dom.find(".btnSecondarySign").html(lex.get('fe.sign.button'));
                            dom.find(".btnSecondarySign").prop("disabled", false);
                            console.log(error); // jshint ignore:line
                        });
                }
                else
                {
                    return dialogs.showOk({
                        title:  lex.get('authentication.error.title'),
                        content: lex.get('authentication.error.message')
                    });
                }
            }
        }, function (validationSummary)
        {
            if (validationSummary !== null && validationSummary.length > 0)
            {
                showValidationSummary(dom, validationSummary);
            }
        });
    };

    /*
      Calling from btnPrimarySign button click handler to implement primary authentication
      @param credentials
      @param dom
    */
    SecureParentEditor.prototype.checkPrimarySign = function (credentials, dom)
    {
        var that = this;
        bajaUser = credentials.username;
        SecureParentEditor.isAuthenticatedUser = false;
        var validation = this.doIsDataValid(dom, "primary");
        Promise.resolve(validation).then(function (validationSummary)
        {
            if (validationSummary !== null && validationSummary.length > 0)
            {
                showValidationSummary(dom, validationSummary);
            }
            else
            {
                var pointOrd = Util.getFormatedPointOrd(SecureParentEditor.componentFE, _facets);
                if (pointOrd !== "")
                {
                    Util.getComponentByOrd(pointOrd).then(function (comp)
                    {
                        component = comp;
                        dom.find("#btnPrimarySign").html(lex.get('fe.signing.button'));
                        dom.find("#btnPrimarySign").prop("disabled", true);
                        return Util.authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                    }).then(function (authenticatedComp)
                    {
                        dom.find("#btnPrimarySign").html(lex.get('fe.sign.button'));
                        dom.find("#btnPrimarySign").prop("disabled", false);
                        if (authenticatedComp)
                        {
                            return Util.authorizeUser(component, baja.$("baja:String").make(credentials.username),
                                baja.$("baja:String").make(_facets.$map.get("SlotName").toString())).then(function(authorizationComp){
                                if(authorizationComp.get("isAuthenticatedUser"))
                                {
                                    if(_facets.$map.get("isSecuredPrimary")){
                                        SecureParentEditor.isAuthenticatedUser = true;
                                        $('.js-dialog-button-ok').attr('disabled', false);
                                        dom.find("#topPane").find("input").prop("disabled", true);
                                        dom.find("#topPane").find("textarea").prop("disabled", true);
                                        dom.find("#topPane").find("select").prop("disabled", true);
                                        dom.find("#topPane").find("button").prop("disabled", true);
                                        dom.find("#btnPrimarySign").html(lex.get('fe.signed.button'));
                                    }
                                    else {
                                        that.initializeSecondaryUsers(dom, dom.find('#username').val().toString(), "secondaryUsername").then(
                                            function (result) {
                                                if (result === "No_Secondary_Users_Found") {
                                                    /* Show Error Message To User*/
                                                    let formattedMessage = lex.get('eSignature.secondaryAuthenticators.configure.error');
                                                    return dialogs.showOk({
                                                        title: lex.get('authentication.error.title'),
                                                        content: formattedMessage
                                                    });
                                                } else {
                                                    dom.find("#primarySignatureProperty").find("#username").prop("disabled", true);
                                                    SecureParentEditor.isAuthenticatedUser = true;
                                                        $('.js-dialog-button-ok').attr('disabled', true);
                                                        dom.find("#txtNewValue").attr("disabled", true);
                                                        dom.find("#idReason").attr("disabled", true);
                                                        dom.find("#idTextarea").attr("disabled", true);
                                                        try {
                                                            dom.find('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
                                                        } catch (e) {
                                                            dom.find('.nav-tabs a[href="#easPrimarySignature"]').parent().removeClass("active");
                                                            dom.find('#easSecondarySignature').removeClass("fade");
                                                            dom.find('.nav-tabs a[href="#easSecondarySignature"]').parent().addClass("active");
                                                            dom.find('#easPrimarySignature').hide();
                                                            dom.find('#easSecondarySignature').show();
                                                        }
                                                        dom.find("#topPane").find("input").prop("disabled", true);
                                                        dom.find("#topPane").find("textarea").prop("disabled", true);
                                                        dom.find("#topPane").find("select").prop("disabled", true);
                                                        dom.find("#topPane").find("button").prop("disabled", true);
                                                        that.initializeSecondaryUsers(dom, dom.find('#username').val().toString(), "secondaryUsername")
                                                            .then(function () {
                                                                changeLegalText(dom.find("#secondaryUsername").val(), dom);
                                                            });
                                                        dom.find("#btnPrimarySign").html(lex.get('fe.signed.button'));
                                                    }
                                            }
                                        );
                                    }

                                }
                                else
                                {
                                    let primaryUserName = credentials.username;
                                    let actionName = _facets.$map.get("SlotName").toString();
                                    let formattedMessage = `User [${primaryUserName}] does not have permission to perform the action ${actionName}.`;
                                    return dialogs.showOk({
                                        title: lex.get('authentication.error.title'),
                                        content: formattedMessage
                                    });
                                }
                            });
                        }
                        else
                        {
                            return dialogs.showOk({
                                title: lex.get('authentication.error.title'),
                                content: lex.get('authentication.error.message')
                            });
                        }
                    }).catch(function (error)
                        {
                            dom.find("#btnPrimarySign").html(lex.get('fe.sign.button'));
                            dom.find("#btnPrimarySign").prop("disabled", false);
                            console.log(error); // jshint ignore:line
                        });
                }
            }
        }, function (validationSummary)
        {
            if (validationSummary !== null && validationSummary.length > 0)
            {
                showValidationSummary(dom, validationSummary);
            }
        });
    };

    /*
      Below method is used initialize the dom content of child FE and initialize the event handlers
      @param dom
    */
    SecureParentEditor.prototype.doInitializeContent = function (dom)
    {
        var that = this;
        SecureParentEditor.isAuthenticatedUser = false;
        dom.find(".primaryTab").click(function ()
        {
            return false;
        });

        dom.find(".secondaryTab").click(function ()
        {
            return false;
        });

        dom.find(".esLocal").click(function ()
        {
            $('.js-dialog-button-ok').attr('disabled', true);
            if (isValidManager && isValidUserSecondary)
            {
                $('.js-dialog-button-ok').attr('disabled', false);
            }
            else
            {
                $('.js-dialog-button-ok').attr('disabled', true);
            }
            SecureParentEditor.componentFE.setIsSecondaryRemote(false);
            return that.initializeSecondaryUsers(dom, dom.find('#username').val().toString(),"secondaryUsername")
                .then(function ()
                {
                    dom.find(".secondarySignatureProperty").show();
                    dom.find(".hideProperty").hide();
                    changeLegalText(dom.find("#secondaryUsername").val(),dom);
                });
        });

        dom.find("#btnPrimarySign").click(function ()
        {
            var credentials = {};
            credentials.username = dom.find('#username').val();
            credentials.password = dom.find('#password').val();
            that.checkPrimarySign(credentials, dom);
        });

        dom.find(".btnSecondarySign").click(function ()
        {
            var credentials = {};
            credentials.username = dom.find('#secondaryUsername').val();
            credentials.primaryUserName = dom.find('#username').val();
            credentials.password = dom.find('#secondaryPassword').val();
            that.checkSecondarySign(credentials, dom);
        });

        dom.find(".btnRemote").click(function ()
        {
            $('.js-dialog-button-ok').attr('disabled', true);
            return that.initializeSecondaryUsers(dom, dom.find('#username').val().toString(),"idChoose")
                .then(function ()
                {
                    dom.find(".secondarySignatureProperty").hide();
                    dom.find(".hideProperty").show();
                    changeLegalText(dom.find("#idChoose").val(),dom);
                });
        });
        dom.find(".esSend").click(function ()
        {
            //var user = dom.find("#idChoose").val();
            SecureParentEditor.componentFE.setIsSecondaryRemote(true);
            $('.js-dialog-button-ok').attr('disabled', false);
            dialogs.showOk({
                title: lex.get('remote.request.title'),
                content: lex.get('remote.request.success.message') + " " + dom.find("#idChoose").val()
            });
        });
    };

    SecureParentEditor.prototype.initializeSecondaryUsers = function (dom, primaryUser, userSelectBoxId)
    {
        return Util.fetchSecondaryUser(component, primaryUser, actionName)
            .then(function (comp)
            {
                if (!SecureParentEditor.componentFE.getSecureSlotFacets().$map.get("isSecuredPrimary") && (comp.get("secUser") == null || comp.get("secUser").toString() === "" || comp.get("secUser").toString().length === 0))
                {
                    return Promise.resolve("No_Secondary_Users_Found");
                }
                else
                {
                    var secondaryUsers = comp.get("secUser").toString();
                    _SecondaryAuthenticatorRoles = comp.get("secAuthRole").toString();
                    var secondaryUsersList = secondaryUsers.split(",");
                    var usersHtml = "";
                    _.each(secondaryUsersList, function (value, index)
                    {
                        if (index + 1 !== secondaryUsersList.length)
                        {
                            usersHtml += "<option value=" + value + ">" + value + "</option>";
                        }
                    });
                    dom.find("#" + userSelectBoxId).html(usersHtml);
                }
            });
    };

    SecureParentEditor.prototype.isValidUser = function ()
    {
        return SecureParentEditor.isAuthenticatedUser;
    };


    SecureParentEditor.prototype.doLayout = function ()
    {
        var that = this;
        var result = BaseEditor.prototype.doLayout.apply(that, arguments);
        return Promise.resolve(result).then(function ()
        {
            setCustomerName(that);
        });

    };

    function setCustomerName(that)
    {
        try
        {
            if (that.value() !== null)
            {
                // jshint ignore:line
                $('.js-dialog-button-ok').attr('disabled', true);
                customerName = that.value().getSecureSlotFacets().$map.get("customerName").toString();
                var legaltextParagraph = that.jq().find("#legaltext");
                if(legalTextTemplate === "")
                {
                    legalTextTemplate = legaltextParagraph.html();
                }
                var html = legalTextTemplate;
                html = html.replace("customerName", customerName);
                if (that.value().getSecureSlotFacets().$map.get("IsRemoteRequest") === null)
                {
                    html = html.replace("userAccountName", bajaUser);
                }
                else
                {
                    html = html.replace("userAccountName", baja.getUserName());
                }
                legaltextParagraph.empty();
                legaltextParagraph.html(html);
            }
            else
            {
                setTimeout(function () // jshint ignore:line
                {
                    setCustomerName(that);
                }, 150);
            }

        }
        catch (err)
        {

        }
    }

    /*
     It will load the initial values of child FE at load time e.g pointName, newValue, reason etc
      @param dom
      @para point
    */
    SecureParentEditor.prototype.doLoadValue = function (dom, point)
    {
        isValidManager = false;
        isValidUserSecondary = false;
        if(!point.getConnected())
        {
            dom.empty();
            dom.html("<h3>"+lex.get('fox.connection.msg')+"</h3>");
            setTimeout(function (){
                $('.js-dialog-button-ok').attr('disabled', true);
            },500);
            return "NOT_CONNECTED";
        }
         if(point.getSecureSlotFacets().$map.get("isSecuredSecondary") == true){
             dom.find("#btnPrimarySign").text(lex.get('fe.sign1.button'));
         }
        SecureParentEditor.componentFE = point;
        _facets = point.getSecureSlotFacets();
      //  dom.find("#txtPointName").val(point.getSecureSlotFacets().$map.get("Name"));
        dom.find("#txtPointName").val(point.getSecureSlotFacets().$map.get("componentDisplayName"));

        bajaUser = baja.getUserName();
        dom.find("#username").val(bajaUser);
        actionName = point.getSecureSlotFacets().$map.get("SlotName");
        /*var reasonsList = point.getSecureSlotFacets().$map.get("reasons");
        var reasonshtml = "";
        var reasonFound = false;
        if (reasonsList !== "" && reasonsList !== null)
        {
            var tempReasonList = null;
            if (reasonsList.indexOf(",") > -1)
            {
                tempReasonList = reasonsList.split(",");
            }
            else
            {
                tempReasonList = [reasonsList];
            }
            _.each(tempReasonList, function (value)
            {
                if (value === point.get("reasonForChange").toString() && point.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
                {
                    reasonFound = true;
                    reasonshtml += "<option value='" + value + "' selected>" + value + "</option>";
                }
                else
                {
                    reasonshtml += "<option value='" + value + "'>" + value + "</option>";
                }
            });
            reasonshtml += "<option value='Other'>Other</option>";
            dom.find("#idReason").html(reasonshtml);

        }
        else
        {
            reasonshtml = "<option value='Other'>Other</option>";
            dom.find("#idReason").html(reasonshtml);
        }

        if (dom.find("#idReason").val() === "Other")
        {
            dom.find("#idTextarea").attr("disabled", false);
        }
        if (reasonFound === false && point.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
        {
            dom.find("#idReason").val("Other");
            dom.find("#idTextarea").val(point.get("reasonForChange").toString());
        }*/

        dom.find("#idReason").change(function ()
        {
            if (dom.find("#idReason").val() === "Other")
            {
                dom.find("#idTextarea").attr("disabled", false);
            }
            else
            {
                dom.find('#idTextarea').val("");
                dom.find("#idTextarea").attr("disabled", true);
                dom.find('#idTextarea').removeClass("errorMsg");
            }
        });

        dom.ready(function(){
            dom.find("#username").change(function(){
                changeLegalText(dom.find("#username").val(),dom);
            });
        });

        if (point.getSecureSlotFacets().$map.get("isSecuredPrimary") !== null)
        {
            dom.find(".secondaryTab").hide();
        }

        if (point.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
        {
            isRemoteRequest = true;

            bajaUser = point.get("userName").toString();
            dom.find("#username").val(bajaUser);
            dom.find("#username").attr("disabled", true);

            dom.find("#secondaryUsername").attr("disabled", true);
            dom.find("#secondaryUsername").val(baja.getUserName());

            dom.find("#txtNewValue").attr("disabled", true);
            dom.find("#idReason").attr("disabled", true);
            dom.find("#idTextarea").attr("disabled", true);
            // Switch to secondary user's tab after this widget is loaded into browser DOM.
            // Timeout has to be done as .tab function was not working if invoked before adding this widget to browser DOM.
            setTimeout(function ()
            {
                // jshint ignore:line
                showSecondary(dom);
            }, 500);
        }
        //customerName = point.getSecureSlotFacets().$map.get("customerName").toString();
        setTimeout(function ()
        {
            changeLegalText(baja.getUserName(),dom);
        }, 300);

        dom.find("#primaryComment").val(point.get("primaryComments").toString());
        dom.find("#secondaryComments").val(point.get("secondaryComments").toString());
    };

    function showSecondary(dom){
        dom.find('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
        dom.find('#easPrimarySignature').removeClass('active')
        dom.find('#easPrimarySignature').removeClass('in')

        dom.find('#easSecondarySignature').addClass('active')
        dom.find('#easSecondarySignature').addClass('in')

        dom.find("#topPane").find("input").prop("disabled", true);
        dom.find("#topPane").find("textarea").prop("disabled", true);
        dom.find("#topPane").find("select").prop("disabled", true);
        dom.find("#topPane").find("button").prop("disabled", true);
        dom.find('.btnRemote').hide();
        if(dom.find('.secondarySignatureProperty')[0].clientHeight ==0){
            setTimeout(function ()
            {
                // jshint ignore:line
                showSecondary(dom);
            }, 500);
        }
    }

    // Comment added to check in the change.
    function getReasonValue(dom)
    {
        if (dom.find("#idReason").val() === "Other")
        {
            return dom.find("#idTextarea").val();
        }
        else
        {
            return dom.find("#idReason").val();
        }
    }

    function savePointValues(newComp, dom)
    {
        var OldComp = SecureParentEditor.componentFE,
            username = "",
            manager = "",
            pointName = dom.find("#txtPointName").val();
        if (!(OldComp.getSecureSlotFacets().$map.$map.isSecuredSecondary))
        {
            username = dom.find("#username").val();
            newComp.setPrimaryPassword(btoa(dom.find('#password').val()));

        }
        else
        {
            username = dom.find("#username").val();
            manager = dom.find("#secondaryUsername").val();
            newComp.setPrimaryPassword(btoa(dom.find('#password').val()));
            newComp.setSecondaryPassword(btoa(dom.find('#secondaryPassword').val()));
        }
        if (newComp.getUserName() !== null)
        {
            newComp.setUserName(username);
        }
        if (newComp.getManager() !== null)
        {
            newComp.setManager(manager);
        }
        if (newComp.getReasonForChange() !== null)
        {
            newComp.setReasonForChange(getReasonValue(dom));
        }
        if (newComp.getPointName() !== null)
        {
            newComp.setPointName(pointName);
        }
        if (newComp.getSlot("statusValue") !== null)
        {
            newComp.setStatusValue(true);
        }
        if (SecureParentEditor.isAuthenticatedUser)
        {
            if (newComp.getIsValidUser() !== null)
            {
                newComp.setIsValidUser((SecureParentEditor.isAuthenticatedUser).toString());
            }
            /*TODO : Remote implementation remaining*/
        }
        else
        {
            if (newComp.getIsValidUser() !== null)
            {
                newComp.setIsValidUser((SecureParentEditor.isAuthenticatedUser).toString());
            }
            dialogs.showOk({
                title: lex.get('error.massages.title'),
                content: lex.get('user.notValid.msg')
            }).ok(function ()
            {
                return newComp;
            });
            return newComp;
        }

        //Adding code for remote authentication
        // IsSecondaryRemote set to true when user click on remote and set to false when user click on local
        if (OldComp.getIsSecondaryRemote() === true)
        {
            // isRemoteRequest get set when user approve or reject from wb view
            if (isRemoteRequest === true)
            {
                if (newComp.getIsSecondaryRemote !== null)
                {
                    newComp.setIsSecondaryRemote(true);
                }
                if (newComp.getisSecondaryApproved !== null)
                {
                    newComp.setIsSecondaryApproved(true);
                }
                if (newComp.getProcessSecondary !== null)
                {
                    newComp.setProcessSecondary(true);
                }
                if (newComp.getIsPending !== null)
                {
                    newComp.setIsPending(true);
                }
                //TODO add comments when remote view is implemented in HTML5
                // setComments(commentEditorS.getText());
                if (SecureParentEditor.componentFE.has("isRejected") && SecureParentEditor.componentFE.get("isRejected").toString() === "true")
                {
                    newComp.setRejectedComments(dom.find("#secondaryComments").val());
                }
            }
            else
            {

                if (newComp.getIsSecondaryRemote !== null)
                {
                    newComp.setIsSecondaryRemote(true);
                }
                if (newComp.getIsSecondaryApproved !== null)
                {
                    newComp.setIsSecondaryApproved(false);
                }
                if (newComp.getProcessSecondary !== null)
                {
                    newComp.setProcessSecondary(false);
                }
                if (newComp.getIsPending !== null)
                {
                    newComp.setIsPending(true);
                }

                if (newComp.getActionName !== null)
                {
                    newComp.setActionName(actionName);
                }
                if (newComp.getManager !== null)
                {
                    newComp.setManager(dom.find("#idChoose").val().toString());
                }
                if (newComp.getRequestTime !== null)
                {
                    newComp.setRequestTime(baja.AbsTime.now());
                }
                //v.add("feType", this.getType().getTypeSpec());
            }
        }

        newComp.set({slot:"primaryComments",value :dom.find("#primaryComment").val()});
        newComp.set({slot:"secondaryComments",value:dom.find("#secondaryComments").val()});
        return newComp;
    }

    function showValidationSummary(dom, validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            dom.find("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: lex.get('error.massages.title'),
            content: errorMessage
        });
    }

    SecureParentEditor.prototype.fetchSecondaryUsers = function ()
    {
        var pointOrd = Util.getFormatedPointOrd(SecureParentEditor.componentFE, _facets);
        if (pointOrd !== "")
        {
            return Util.getComponentByOrd(pointOrd).then(function (comp)
            {
                var actionName = _facets.$map.get("SlotName").toString();
                return Util.fetchSecondaryUser(comp, bajaUser, actionName);
            }).then(function (secondaryUsers)
            {
                if (secondaryUsers.get("secUser") && secondaryUsers.get("secUser").toString() !== "")
                {
                    _SecondaryAuthenticatorRoles = secondaryUsers.get("secAuthRole").toString();
                    return Promise.resolve("");
                }
                else
                {
                    if (secondaryUsers.$map.get("secAuthRole").$val === "")
                    {
                        return Promise.resolve(lex.get('eSignature.secondaryAuthenticators.configure.error'));
                    }
                    else
                    {
                        // Please check the roles for primary/secondary authenticator,
                        // or check the permissions for the secondary authenticator user.
                        return Promise.resolve(lex.get('eSignature.secondaryAuthenticators.permission.error'));
                    }
                }
            });
        }
        else
        {
            return Promise.resolve(lex.get('eSignature.component.notFound.error'));
        }
    };

    SecureParentEditor.prototype.validateSecondaryUsers = function (actionParameter)
    {
        var dom = this.jq();
        var that = this;
        if (window.location.protocol !== "https:")
        {
             dom.empty();
             dom.html("<h3>"+lex.get('secure.connection.msg')+"</h3>");
             return Promise.resolve("");
        }

        if(actionParameter.get("isProxyCall")!==null
            && actionParameter.get("isProxyCall") == true
            && actionParameter.getSecureSlotFacets().$map.get("invokedFromProxy")!==null)
        {
            if(actionParameter.getSecureSlotFacets().$map.get("proxyConnType") === "fox:")
            {
                dom.empty();
                dom.html("<h3>"+lex.get('secure.connection.msg')+"</h3>");
                return Promise.resolve("");
            }
        }

          if(actionParameter.getSecureSlotFacets().$map.get("isLicensed")!==null
          && actionParameter.getSecureSlotFacets().$map.get("isLicensed") == false)
          {
            dom.empty();
            dom.html("<h3>"+lex.get('eSignature.license.error')+"</h3>");
            return Promise.resolve("");
          }

          if(actionParameter.getSecureSlotFacets().$map.get("withinCountRange")!==null && actionParameter.getSecureSlotFacets().$map.get("withinCountRange") === false)
          {
            dom.empty();
            dom.html("<h3>"+lex.get('eSignature.license.maximum.pointCountExceeded')+"</h3>");
            return Promise.resolve("");
          }

        if(actionParameter.getParent()!==null && actionParameter.getParent().getType().is("electronicSignature:SecondaryRemoteAuthentication"))
        {
            dom.empty();
            dom.html("<h3>"+lex.get('remote.auth.view.msg')+"</h3>");
            return Promise.resolve("");
        }

        return that.initializeControlPoint().then(function ()
        {
            // fetch reasons and show them in ui.
            var actionArg = {
                slot: "call",
                value: 'getreasons'
                //value: baja.$("baja:String").make("getreasons")
            };
            return component.invoke({
                slot: "call",
                value: baja.$("baja:String").make("getreasons")
            }).then(function(reasons){

                var reasonsList = reasons.$map.get("reasons").$val;
                var reasonshtml = "";
                var reasonFound = false;
                if (reasonsList !== "" && reasonsList !== null)
                {
                    var tempReasonList = null;
                    //Fix for issue PROSVC-9652-ESignature doesn't handle commas in Reasons well
                    if (reasonsList.indexOf(";;;") > -1)
                    {
                        tempReasonList = reasonsList.split(";;;");
                    }
                    else
                    {
                        tempReasonList = [reasonsList];
                    }
                    _.each(tempReasonList, function (value)
                    {
                        if (value === actionParameter.get("reasonForChange").toString() && actionParameter.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
                        {
                            reasonFound = true;
                            reasonshtml += "<option value='" + value + "' selected>" + value + "</option>";
                        }
                        else
                        {
                            reasonshtml += "<option value='" + value + "'>" + value + "</option>";
                        }
                    });
                    reasonshtml += "<option value='Other'>Other</option>";
                    dom.find("#idReason").html(reasonshtml);

                }
                else
                {
                    reasonshtml = "<option value='Other'>Other</option>";
                    dom.find("#idReason").html(reasonshtml);
                }

                if (dom.find("#idReason").val() === "Other")
                {
                    dom.find("#idTextarea").attr("disabled", false);
                }
                if (reasonFound === false && actionParameter.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
                {
                    dom.find("#idReason").val("Other");
                    dom.find("#idTextarea").val(actionParameter.get("reasonForChange").toString());
                }

            // initialize secondary users and build the ui accordingly.
            return that.initializeSecondaryUsers(dom, dom.find('#username').val().toString(),"secondaryUsername")
                .then(function ()
                {
                    if (actionParameter.getSecureSlotFacets().$map.get("isSecuredSecondary") !== null)
                    {
                        return that.fetchSecondaryUsers().then(function (result)
                        {
                            if (result !== "")
                            {
                                dom.empty();
                                dom.html("<h3>" + result + "</h3>");
                            }
                            else if (actionParameter.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
                            {
                                dom.find("#secondaryUsername").val(baja.getUserName());
                                changeLegalText(dom.find("#secondaryUsername").val(),dom);
                            }

                            dom.find("#secondaryUsername").change(function(){
                                changeLegalText(dom.find("#secondaryUsername").val(),dom);
                            });

                            dom.find("#idChoose").change(function(){
                                changeLegalText(dom.find("#idChoose").val(),dom);
                            });

                            if (actionParameter.getSecureSlotFacets().$map.get("IsRemoteRequest") !== null)
                            {
                                changeLegalText(dom.find("#username").val(),dom);
                            }

                            return Promise.resolve("");
                        }, function (err)
                        {
                            console.log(err); // jshint ignore:line
                            dom.empty();
                            dom.html("<h3>"+lex.get('eSignature.secondaryAuthenticators.fetch.error')+"</h3>");
                            return Promise.resolve("");
                        });
                    }
                    else
                    {
                        changeLegalText(bajaUser, dom);
                        return Promise.resolve("");
                    }

                });
            });
        });
    };

    SecureParentEditor.prototype.initializeControlPoint = function ()
    {

        var pointOrd = Util.getFormatedPointOrd(SecureParentEditor.componentFE, _facets);
        if (pointOrd !== "")
        {
            return Util.getComponentByOrd(pointOrd).then(function (comp)
            {
                component = comp;
                return component.loadSlots();
            }).then(function(){
                return component;
            });
        }
        return Promise.resolve(null);
    };


    function changeLegalText(userName, dom)
    {
        var pointOrd = Util.getFormatedPointOrd(SecureParentEditor.componentFE, _facets);
        if (pointOrd !== "")
        {
            Util.getComponentByOrd(pointOrd).then(function (comp) {
                component = comp;
                return component.invoke({
                    slot: "call",
                    value: baja.$("baja:String").make("getLegalText," + customerName.toString() + "," + userName)
                }).then(function (component){
                    var legalText = component.get("legalText").toString();
                    overrideLegalText(userName,dom,legalText);
                });
            });
        }
    }

    function overrideLegalText(userName, dom, legalText)
    {
        legalText = Util.wordWrapAck(legalText,70);
        var legalTextParagraph = dom.find("#legaltext");
        legalTextParagraph.empty();
        legalTextParagraph.html( lex.get('eSignature.ack.label') +"\n" + legalText);
    }

    return SecureParentEditor;
});