function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * API Status: **Private**
 * @module nmodule/webEditors/rc/wb/menu/componentMenuAgent
 */
define(['baja!', 'lex!', 'bajaux/commands/CommandGroup', 'Promise', 'underscore',
    'nmodule/js/rc/asyncUtils/asyncUtils',
    'nmodule/webEditors/rc/wb/commands/transfer/TransferCommand',
    'nmodule/webEditors/rc/wb/menu/menuUtils',
    'nmodule/webEditors/rc/wb/mixin/mixinUtils',
    'lex!electronicSignature'], function (baja, lex, CommandGroup,
    Promise, _, asyncUtils, TransferCommand, menuUtils, mixinUtils,
    electronicSignatureLex) {
    'use strict';

    var doRequire = asyncUtils.doRequire,
        ifAllowedByProfile = menuUtils.ifAllowedByProfile,
        makeActionsGroup = menuUtils.makeActionsGroup,
        makeNewGroup = menuUtils.makeNewGroup,
        makeViewsGroup = menuUtils.makeViewsGroup,
        hasMixin = mixinUtils.hasMixin,
        eSignLex = _slicedToArray(electronicSignatureLex, 1)[0];

    function isParented(comp) {
        return !!comp.getPropertyInParent();
    }
    /**
     * Check to see if the component is frozen (either unparented, or a frozen
     * slot).
     *
     * @inner
     * @param {baja.Complex} comp
     * @returns {Boolean}
     */


    function isFrozen(comp) {
        var prop = comp.getPropertyInParent();
        return !prop || prop.isFrozen();
    }

    function isESignature(comp) {
        if(comp.getType().$typeSpec.equals("electronicSignature:SecuredDashboardConfiguration")){
            return true;
        }
        return false;
    }

    function isReadonly(comp) {
        var space = comp.getComponentSpace();
        return space && space.isSpaceReadonly();
    }
    /**
     * Asynchronously require the appropriate command modules and instantiate
     * the `CommandGroup`. Assign the appropriate enabled/disabled status for
     * the different commands depending on the write/invoke permissions on this
     * component.
     *
     * @inner
     * @param {*} owner
     * @param {baja.Component} comp
     * @returns {Promise}
     */

    function makeGroup(owner, comp) {
        return doRequire(['nmodule/webEditors/rc/transform/ExportCommand',
            'nmodule/webEditors/rc/wb/commands/AddSlotCommand',
            'nmodule/electronicSignature/rc/fe/baja/SecuredRenameCommand',
            'nmodule/webEditors/rc/wb/commands/LinkMarkCommand',
            'nmodule/webEditors/rc/wb/commands/LinkFromCommand',
            'nmodule/webEditors/rc/wb/commands/LinkToCommand',
            'nmodule/webEditors/rc/wb/commands/ReorderSlotsCommand',
            'nmodule/electronicSignature/rc/fe/baja/SecuredSetDisplayNameCmd',
            'nmodule/webEditors/rc/wb/commands/CompositeEditorCommand',
            'nmodule/webEditors/rc/wb/menu/Separator',
            'nmodule/electronicSignature/rc/fe/baja/SecuredDeleteCommand']).then(function (_ref) {
            var _ref2 = _slicedToArray(_ref, 11),
                ExportCommand = _ref2[0],
                AddSlotCommand = _ref2[1],
                RenameCommand = _ref2[2],
                LinkMarkCommand = _ref2[3],
                LinkFromCommand = _ref2[4],
                LinkToCommand = _ref2[5],
                ReorderSlotsCommand = _ref2[6],
                SetDisplayNameCommand = _ref2[7],
                CompositeEditorCommand = _ref2[8],
                Separator = _ref2[9],
                DeleteCommand = _ref2[10];

            var group = new CommandGroup(),
                newGroup,
                deleteCmd,
                renameCmd,
                setDisplayNameCmd,
                reorderCmd,
                compositeCmd,
                exportCmd;

            if (!isReadonly(comp)) {
                var frozen = isFrozen(comp),
                    ESignature = isESignature(comp),
                    parented = isParented(comp),
                    hasAdminWrite = comp.getPermissions().hasAdminWrite();
                newGroup = makeNewGroup(comp, new AddSlotCommand(comp));
                setDisplayNameCmd = new SetDisplayNameCommand(comp);
                reorderCmd = new ReorderSlotsCommand(comp);
                compositeCmd = new CompositeEditorCommand(comp);
                exportCmd = new ExportCommand(comp);
                setDisplayNameCmd.setEnabled(hasAdminWrite && parented && !ESignature);
                reorderCmd.setEnabled(hasAdminWrite && !ESignature);
                compositeCmd.setEnabled(hasAdminWrite && parented && !ESignature);
                exportCmd.setEnabled(hasAdminWrite && !ESignature);

                if (hasMixin(owner, 'transfer')) {
                    var subject = [comp];

                    if (owner.isTransferOpSupported('delete', subject)) {
                        deleteCmd = new DeleteCommand(comp);
                        deleteCmd.setEnabled(hasAdminWrite && !frozen && !ESignature);
                    }

                    if (owner.isTransferOpSupported('rename', subject)) {
                        renameCmd = new RenameCommand(comp);
                        renameCmd.setEnabled(hasAdminWrite && !frozen && !ESignature);
                    }
                } else {
                    renameCmd = new RenameCommand(comp);
                    renameCmd.setEnabled(hasAdminWrite && !frozen);
                }
            }

            var makeEditTagsCommmand = _.once(function (component, enabled) {
                // Edit Tags should not be not available for AceComponents
                if (!componentMenuAgent.$supportsEditTagsCommand(component)) {
                    return Promise.resolve(null);
                }

                return asyncUtils.doRequire([//prevent JsBuildVerificationTests failures
                    //nmodule/tagdictionary/rc/commands/TagManagerCommand
                    'nmodule'.toLowerCase() + '/tagdictionary/rc/commands/TagManagerCommand', //nmodule/tagdictionary/rc/util/taggingUtil
                    'nmodule'.toLowerCase() + '/tagdictionary/rc/util/taggingUtil'], {
                    deps: menuUtils.useTagDictionaryMinified() && ['nmodule'.toLowerCase() + '/tagdictionary/rc/tagdictionary.built.min']
                }).then(function (_ref3) {
                    var _ref4 = _slicedToArray(_ref3, 2),
                        TagManagerCommand = _ref4[0],
                        taggingUtil = _ref4[1];

                    return taggingUtil.getTagDictionaryService().then(function (tagDictionaryService) {
                        // only add the TagManagerCommand if there is a TagDictionaryService
                        if (!tagDictionaryService) {
                            return null;
                        }

                        var cmd = new TagManagerCommand(component);
                        cmd.setEnabled(enabled);
                        return cmd;
                    });
                })["catch"](function (ignore) {
                    return null;
                });
            }); //TODO: TransferWidget.isDeleteEnabled()?


            return Promise.all([ifAllowedByProfile('viewSelection', function () {
                return makeViewsGroup(comp.getNavOrd());
            }), makeActionsGroup(comp), new Separator(), newGroup, new Separator(), makeEditTagsCommmand(comp, hasAdminWrite), new Separator(), deleteCmd, new Separator(), LinkMarkCommand.make(comp), LinkFromCommand.make(comp), LinkToCommand.make(comp), new Separator(), renameCmd, setDisplayNameCmd, reorderCmd, compositeCmd, new Separator(), exportCmd]).then(function (allKids) {
                var actionGroup = allKids[1];
                var callAction = actionGroup.flatten().
                filter(x => x.getDisplayNameFormat().toLowerCase() ===eSignLex.get("call").toLowerCase());
                if (callAction !== null && callAction.length > 0)
                {
                    actionGroup.remove(callAction[0]);
                }
                group.add.apply(group, _.compact(allKids));
                if(ESignature){
                    allKids[1].$kids.splice(0);
                }
                return group;
            });
        });
    }
    /**
     * Create a command group that can be used to construct a context menu. Note
     * that the component, if mounted, will be leased at this time so any
     * dynamic actions can be included in the menu.
     *
     * The value may also be a `NavContainer`, e.g., a nav node corresponding to
     * a component inside a .bog file inside the station's file space, that
     * BajaScript has not fully resolved yet. In this case, the `NavContainer`'s
     * nav ORD will be resolved and the menu will be generated from the result.
     *
     * @alias module:nmodule/webEditors/rc/wb/menu/componentMenuAgent
     * @param {*} owner if the owner has the `TransferSupport` mixin, then
     * transfer commands will be included
     * @param {baja.Component|baja.NavContainer} comp
     * @returns {Promise} promise to be resolved with a `CommandGroup`
     * for the given component
     * @see module:nmodule/webEditors/rc/wb/mixin/TransferSupport
     */


    var componentMenuAgent = function componentMenuAgent(owner, comp) {
        if (comp instanceof baja.NavContainer) {
            return comp.getNavOrd().get({
                lease: true
            }).then(function (comp) {
                return makeGroup(owner, comp);
            });
        }

        return Promise.resolve(comp.isMounted() && comp.lease()).then(function () {
            return makeGroup(owner, comp);
        });
    };
    /**
     * @private
     * @param {baja.Component} comp
     */


    componentMenuAgent.$supportsEditTagsCommand = function (comp) {
        return !baja.hasType(comp, 'ace:AceComponent') && !baja.hasType(comp, 'baja:VirtualComponent');
    };

    return componentMenuAgent;
});
