/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

define(['baja!',
        'baja!electronicSignature:SecuredNumericWritable,electronicSignature:SecuredBooleanWritable,electronicSignature:SecuredEnumWritable,electronicSignature:SecuredStringWritable,electronicSignature:SecuredNumericPoint,electronicSignature:SecuredBooleanPoint,electronicSignature:SecuredEnumPoint,electronicSignature:SecuredStringPoint',
        'lex!webEditors',
        'jquery',
        'dialogs',
        'lex!electronicSignature',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/wb/commands/ComponentEditorCommand',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/electronicSignature/rc/fe/baja/Util',
        'nmodule/webEditors/rc/fe/feDialogs',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         dialogs,
         electronicSignatureLex,
         Promise,
         _,
         ComponentEditorCommand,
         template,
         loader,
         Util,
         feDialogs
         ) {

  'use strict';
  var lex = electronicSignatureLex[0];
  var webEditorsLex = lexs[0];
  /* _SecondaryAuthenticatorRoles="",
  isValidUserSecondary=false; */

////////////////////////////////////////////////////////////////
// Exports
////////////////////////////////////////////////////////////////

  /**
   * A command for renaming slots on an editor's `Component` value.
   *
   * @private
   * @class
   * @extends module:webEditors/rc/wb/commands/ComponentEditorCommand
   * @alias module:nmodule/webEditors/rc/wb/commands/SecuredRenameCommand
   * @param {module:bajaux/Widget} ed the `Widget` to bind this `Command` to
   * @param {baja.Component} the component on which to rename slots
   * @throws {Error} if no `Widget` provided
   */
  var SecuredRenameCommand = function SecuredRenameCommand(component) {
    var that = this;

    ComponentEditorCommand.call(that, {
      module: 'webEditors',
      lex: 'commands.rename'
    }, component);
  };
  SecuredRenameCommand.prototype = Object.create(ComponentEditorCommand.prototype);
  SecuredRenameCommand.prototype.constructor = SecuredRenameCommand;

  /**
   * Make sure we have admin write permissions and we're not renaming a
   * frozen slot.
   *
   * @param {baja.Component} comp
   * @param {baja.Slot} slot
   * @returns {Boolean} true if I can rename this slot
   */
  SecuredRenameCommand.prototype.canPerformCommand = function (comp, slot) {
    return comp.getPermissions().hasAdminWrite() && !slot.isFrozen();
  };

  //TODO: support find/replace to renames
  /**
   * Shows a dialog for the component's currently configured slot name,
   * and renames the slot after the user clicks OK.
   *
   * @param {baja.Component} comp
   * @param {Array.<baja.Slot>} slots the slots to rename (*Note*: only the
   * first given slot will be renamed)
   * @returns {Promise}
   */
  SecuredRenameCommand.prototype.performCommand = function (comp, slots, params) {
    var slot = slots[0];

    if (!slot) {
      return Promise.reject(lex.get('rename.slot.required'));
    }
    var oldName = slot.getName();

    if(slot.getType().is(baja.$("electronicSignature:SecuredNumericWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredNumericPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringPoint").getType())){

      slots[0].$val.getActionParameterDefault(slots[0].$val.getSlot("call")).then(function(actionPoint){
          var primarySignLabel = lex.get('fe.sign1.button');
          if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
          {
              primarySignLabel = lex.get('fe.sign.button');
          }
          var jsonObject={
              authLevel1:lex.get('fe.authorization.level1.label'),
              authLevel2:lex.get('fe.authorization.level2.label'),
              userName:lex.get('fe.userName.label'),
              password:lex.get('fe.password.label'),
              comment:lex.get('fe.comment.label'),
              sign:primarySignLabel,
              sign2:lex.get('fe.sign2.button'),
              acknowledgement:lex.get('eSignature.ack.label'),
              label1:lex.get('eSignature.ack.label1'),
              label2:lex.get('eSignature.ack.label2'),
              label3:lex.get('eSignature.ack.label3'),
              label4:lex.get('eSignature.ack.label4'),
              label5:lex.get('eSignature.ack.label5'),
              label6:lex.get('eSignature.ack.label6'),
              label7:lex.get('eSignature.ack.label7')
          };
           dialogs.showOkCancel({title:lex.get('authenticate.user.title'),content:template({jsonObject:jsonObject})}).ok(function(val){
              return feDialogs.showFor({
                title: webEditorsLex.get('commands.rename.description'),
                value: baja.SlotPath.unescape(oldName)
              })
                .then(function (newName) {
                  if (newName === null) {
                    return;
                  }

                  newName = baja.SlotPath.escape(newName);
                  if (newName && (newName !== oldName)) {
                    //TODO: is this a bajascript bug? display name does not follow the renamed slot
                    return comp.rename({ slot: oldName, newName: newName }).then(function(){
                    var dashboardHandle=actionPoint.getSecureSlotFacets().$map.$map.dashboardHandle;
                     getDashBoardComponent(dashboardHandle).then(function(dashBoardComponent){
                      var primaryComments=val.jq().find("#primaryComment").val();
                      actionPoint.setActionName("rename");
                      if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                      {
                       actionPoint.setPrimaryComments(primaryComments);
                      }else
                      {
                        var secondaryComments=val.jq().find("#secondaryComments").val();
                        var managerName= val.jq().find("#secondaryUsername").val();
                        actionPoint.setPrimaryComments(primaryComments);
                        actionPoint.setSecondaryComments(secondaryComments);
                        actionPoint.setManager(managerName);

                      }
                       actionPoint.setNewValue(newName);
                       var comp= baja.$("baja:Component", {
                          actionPoint: actionPoint,
                       });
                     dashBoardComponent.invoke({slot:"updateHistory", value: comp});

                     });

                    });
                  }
                });
           });
           Util.loadInitialEventsAndData(actionPoint,"rename");
      }).catch(function (err) {
        return dialogs.showOk({
           title: "Error",
           content:lex.get('editor.primary.user.validation')
        });
      });
    }
    else{
      return feDialogs.showFor({
        title: webEditorsLex.get('commands.rename.description'),
        value: baja.SlotPath.unescape(oldName)
      })
      .then(function (newName) {
        if (newName === null) {
          return;
        }

        newName = baja.SlotPath.escape(newName);
        if (newName && (newName !== oldName)) {
          //TODO: is this a bajascript bug? display name does not follow the renamed slot
          return comp.rename({ slot: oldName, newName: newName });
        }
      });
    }
  };
  function getDashBoardComponent(handle) {
    var ord="station:|h:"+handle;
    return baja.Ord.make(ord).get({lease: true});
  }


  return SecuredRenameCommand;
});