function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * API Status: **Private**
 * @module nmodule/webEditors/rc/wb/commands/RenameCommand
 */
define(['baja!',
        'baja!electronicSignature:SecuredNumericWritable,electronicSignature:SecuredBooleanWritable,electronicSignature:SecuredEnumWritable,electronicSignature:SecuredStringWritable,electronicSignature:SecuredNumericPoint,electronicSignature:SecuredBooleanPoint,electronicSignature:SecuredEnumPoint,electronicSignature:SecuredStringPoint',
        'lex!webEditors',
        'jquery',
        'dialogs',
        'lex!electronicSignature',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/wb/commands/ComponentEditorCommand',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/electronicSignature/rc/fe/baja/Util',
        'nmodule/webEditors/rc/fe/feDialogs',
        'nmodule/webEditors/rc/fe/baja/util/compUtils',
        'css!nmodule/electronicSignature/rc/css/widgets'],
    function (baja,
       types,
       lexs,
       $,
       dialogs,
       electronicSignatureLex,
       Promise,
       _,
       ComponentEditorCommand,
       template,
       loader,
       Util,
       feDialogs,
       compUtils) {
    'use strict';

    var webEditorsLex = _slicedToArray(lexs, 1)[0],
        lex = _slicedToArray(electronicSignatureLex, 1)[0];

    var getDisplayName = compUtils.getDisplayName;
    ////////////////////////////////////////////////////////////////
    // Exports
    ////////////////////////////////////////////////////////////////

    /**
     * A command for renaming slots on an editor's `Component` value.
     *
     * @class
     * @extends module:nmodule/webEditors/rc/wb/commands/ComponentEditorCommand
     * @alias module:nmodule/webEditors/rc/wb/commands/RenameCommand
     * @param {baja.Component} component the component on which to rename slots
     * @throws {Error} if no `Widget` provided
     */

    var SecuredRenameCommand = function SecuredRenameCommand(component) {
        var that = this;
        ComponentEditorCommand.call(that, {
            module: 'webEditors',
            lex: 'commands.rename'
        }, component);
    };

    SecuredRenameCommand.prototype = Object.create(ComponentEditorCommand.prototype);
    SecuredRenameCommand.prototype.constructor = SecuredRenameCommand;
    /**
     * Make sure we have admin write permissions and we're not renaming a
     * frozen slot.
     *
     * @param {baja.Component} comp
     * @param {baja.Slot} slot
     * @returns {Boolean} true if I can rename this slot
     */

    SecuredRenameCommand.prototype.canPerformCommand = function (comp, slot) {
        return isRenamable(comp, slot);
    }; //TODO: support find/replace to renames

    /**
     * Shows a dialog for the component's currently configured slot name,
     * and renames the slot after the user clicks OK.
     *
     * @param {baja.Component} comp
     * @param {Array.<baja.Slot>} slots the slots to rename (*Note*: only the
     * first given slot will be renamed)
     * @returns {Promise}
     */



    SecuredRenameCommand.prototype.performCommand = function (comp, slots) {
        var _this = this;

        return getSlotToRename(slots).then(function (slot) {
            var oldName = slot.getName();
            if(isSecuredPoint(slot))
            {
                slot.$val.getActionParameterDefault(slot.$val.getSlot("call")).then(function(actionPoint){
                    var primarySignLabel = lex.get('fe.sign1.button');
                    if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                    {
                        primarySignLabel = lex.get('fe.sign.button');
                    }
                    var jsonObject={
                        authLevel1:lex.get('fe.authorization.level1.label'),
                        authLevel2:lex.get('fe.authorization.level2.label'),
                        userName:lex.get('fe.userName.label'),
                        password:lex.get('fe.password.label'),
                        comment:lex.get('fe.comment.label'),
                        sign:primarySignLabel,
                        sign2:lex.get('fe.sign2.button'),
                        acknowledgement:lex.get('eSignature.ack.label'),
                        label1:lex.get('eSignature.ack.label1'),
                        label2:lex.get('eSignature.ack.label2'),
                        label3:lex.get('eSignature.ack.label3'),
                        label4:lex.get('eSignature.ack.label4'),
                        label5:lex.get('eSignature.ack.label5'),
                        label6:lex.get('eSignature.ack.label6'),
                        label7:lex.get('eSignature.ack.label7')
                    };
                    dialogs.showOkCancel({title:lex.get('authenticate.user.title'),content:template({jsonObject:jsonObject})}).ok(function(val){
                        return _this.$promptForNewName(oldName).then(validateNewName).then(function (newName) {
                            return _this.$doRename(comp, oldName, newName).then(function(){
                                var dashboardHandle=actionPoint.getSecureSlotFacets().$map.$map.dashboardHandle;
                                getDashBoardComponent(dashboardHandle).then(function(dashBoardComponent){
                                    var primaryComments=val.jq().find("#primaryComment").val();
                                    actionPoint.setActionName("rename");
                                    if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                                    {
                                        actionPoint.setPrimaryComments(primaryComments);
                                    }else
                                    {
                                        var secondaryComments=val.jq().find("#secondaryComments").val();
                                        var managerName= val.jq().find("#secondaryUsername").val();
                                        actionPoint.setPrimaryComments(primaryComments);
                                        actionPoint.setSecondaryComments(secondaryComments);
                                        actionPoint.setManager(managerName);

                                    }
                                    actionPoint.setNewValue(newName);
                                    var comp= baja.$("baja:Component", {
                                        actionPoint: actionPoint,
                                    });
                                    dashBoardComponent.invoke({slot:"updateHistory", value: comp});

                                });

                            });
                        });
                    });
                    Util.loadInitialEventsAndData(actionPoint,"rename");
                }).catch(function (err) {
                    return dialogs.showOk({
                        title: "Error",
                        content:lex.get('editor.primary.user.validation')
                    });
                });
            }
            else
            {
                return _this.$promptForNewName(oldName).then(validateNewName).then(function (newName) {
                    return _this.$doRename(comp, oldName, newName);
                });
            }
        })["catch"](feDialogs.error);
    };

    /**
     * @private
     * @param {object} params params passed to invoke() or undoable()
     * @returns {Promise.<{ comp: baja.Component, oldName: string, newName: string }>}
     */
    SecuredRenameCommand.prototype.$setupRename = function (params) {
        var _this3 = this;

        return this.processParams(params).then(function (_ref2) {
            var comp = _ref2.comp,
                slots = _ref2.slots;
            return getSlotToRename(slots).then(function (slot) {
                var oldName = slot.getName();
                return _this3.$promptForNewName(oldName).then(validateNewName).then(function (newName) {
                    return {
                        comp: comp,
                        oldName: oldName,
                        newName: newName
                    };
                })["catch"](function (err) {
                    feDialogs.error(err);
                    return {};
                });
            });
        });
    };

    /**
     * @private
     * @param {baja.Component }comp
     * @param {string} oldName
     * @param {string} newName
     * @returns {Promise}
     */
    SecuredRenameCommand.prototype.$doRename = function (comp, oldName, newName) {
        if (newName && newName !== oldName) {
            return comp.rename({
                slot: oldName,
                newName: newName
            });
        } else {
            return Promise.resolve();
        }
    };

    /**
     * @private
     * @returns {Promise.<string|null>}
     */
    SecuredRenameCommand.prototype.$promptForNewName = function (oldName) {
        return feDialogs.showFor({
            title: webEditorsLex.get('commands.rename.description'),
            value: baja.SlotPath.unescape(oldName),
            formFactor: 'mini'
        });
    };

    function validateNewName(newName) {
        if (newName === null) {
            return null;
        }

        if (newName.trim() === '') {
            throw new Error(webEditorsLex.get('enterName'));
        }

        return baja.SlotPath.escape(newName.trim());
    }

    function getSlotToRename(slots) {
        var slot = slots[0];

        if (!slot) {
            return Promise.reject(new Error('slot to rename required'));
        } else {
            return Promise.resolve(slot);
        }
    }

    function isRenamable(comp, slot) {
        slot = comp && slot && comp.getSlot(slot);
        return !!(slot && comp.getPermissions().hasAdminWrite() && !slot.isFrozen());
    }

    function getDashBoardComponent(handle) {
            var ord="station:|h:"+handle;
            return baja.Ord.make(ord).get({lease: true});
        }

    function isSecuredPoint(slot)
    {
        return slot.getType().is(baja.$("electronicSignature:SecuredNumericWritable").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredBooleanWritable").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredEnumWritable").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredStringWritable").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredNumericPoint").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredBooleanPoint").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredEnumPoint").getType()) ||
            slot.getType().is(baja.$("electronicSignature:SecuredStringPoint").getType());
    }

    return SecuredRenameCommand;
});
