
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * API Status: **Private**
 * @module nmodule/webEditors/rc/wb/commands/SetDisplayNameCommand
 */
define(['baja!',
        'baja!electronicSignature:SecuredNumericWritable,electronicSignature:SecuredBooleanWritable,electronicSignature:SecuredEnumWritable,electronicSignature:SecuredStringWritable,electronicSignature:SecuredNumericPoint,electronicSignature:SecuredBooleanPoint,electronicSignature:SecuredEnumPoint,electronicSignature:SecuredStringPoint',
        'jquery',
        'dialogs',
        'lex!electronicSignature',
        'Promise',
        'underscore',
        'lex!webEditors',
        'nmodule/webEditors/rc/fe/feDialogs',
        'nmodule/webEditors/rc/fe/baja/util/compUtils',
        'nmodule/webEditors/rc/wb/commands/ComponentEditorCommand',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/electronicSignature/rc/fe/baja/Util',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
        baja,
        types,
        $,
        dialogs,
        electronicSignatureLex,
        Promise,
        _,
        lexs,
        feDialogs,
        compUtils,
        ComponentEditorCommand,
        template,
        loader,
        Util) {
    'use strict';

    var webEditorsLex = _slicedToArray(lexs, 1)[0],
        lex = _slicedToArray(electronicSignatureLex, 1)[0];

    var getDisplayName = compUtils.getDisplayName;
    var _baja$Flags = baja.Flags,
        HIDDEN = _baja$Flags.HIDDEN,
        READONLY = _baja$Flags.READONLY,
        OPERATOR = _baja$Flags.OPERATOR;
    var DISPLAY_NAMES_FLAGS = HIDDEN | READONLY | OPERATOR;

    /**
     * A command for setting the display name on a slot on an editor's loaded
     * `Component` value.
     *
     * @class
     * @extends module:nmodule/webEditors/rc/wb/commands/ComponentEditorCommand
     * @alias module:nmodule/webEditors/rc/wb/commands/SetDisplayNameCommand
     * @param {baja.Component} component the component on which to set slot
     * display names
     * @throws {Error} if no `Widget` provided
     */
    var SecuredSetDisplayNameCmd = function SecuredSetDisplayNameCmd(component) {
        var _this = this;

        ComponentEditorCommand.call(this, {
            module: 'webEditors',
            lex: 'commands.setDisplayName',
        }, component);
    };

    SecuredSetDisplayNameCmd.prototype = Object.create(ComponentEditorCommand.prototype);
    SecuredSetDisplayNameCmd.prototype.constructor = SecuredSetDisplayNameCmd;

    /**
     * Make sure we have admin write permissions.
     *
     * @param {baja.Component} comp
     * @returns {Boolean} true if I can set slot display names on this component
     */
    SecuredSetDisplayNameCmd.prototype.canPerformCommand = function (comp) {
        return hasAdminWrite(comp);
    }; //TODO: support search/replace for multiple display names

    /**
     * Set the display names for the given slots.
     *
     * @param {baja.Component} comp
     * @param {Array.<baja.Slot>} slots the slots for which to set display names
     * (note: only the first slot will actually get its display name set)
     * @returns {Promise} promise to be resolved after the user as entered
     * a new display name and it has been set on the component slot
     */


    function isSecuredPoint(slot) {
        return slot.getType().is(baja.$("electronicSignature:SecuredNumericWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringWritable").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredNumericPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredBooleanPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredEnumPoint").getType()) || slot.getType().is(baja.$("electronicSignature:SecuredStringPoint").getType());
    }

    SecuredSetDisplayNameCmd.prototype.performCommand = function (comp, slots) {
        return loadComponentSlots(comp).then(function(comp){
            var slot = slots[0];
            if (!slot) {
                return Promise.reject(new Error(lex.get('rename.slot.required')));
            }
            if(isSecuredPoint(slot)){
                var oldDisplayName = comp.getDisplayName(slot);
                slot.$val.getActionParameterDefault(slot.$val.getSlot("call")).then(function(actionPoint){
                    var primarySignLabel = lex.get('fe.sign1.button');
                    if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                    {
                        primarySignLabel = lex.get('fe.sign.button');
                    }
                    var jsonObject={
                        authLevel1:lex.get('fe.authorization.level1.label'),
                        authLevel2:lex.get('fe.authorization.level2.label'),
                        userName:lex.get('fe.userName.label'),
                        password:lex.get('fe.password.label'),
                        comment:lex.get('fe.comment.label'),
                        sign:primarySignLabel,
                        sign2:lex.get('fe.sign2.button'),
                        acknowledgement:lex.get('eSignature.ack.label'),
                        label1:lex.get('eSignature.ack.label1'),
                        label2:lex.get('eSignature.ack.label2'),
                        label3:lex.get('eSignature.ack.label3'),
                        label4:lex.get('eSignature.ack.label4'),
                        label5:lex.get('eSignature.ack.label5'),
                        label6:lex.get('eSignature.ack.label6'),
                        label7:lex.get('eSignature.ack.label7')
                    };
                    dialogs.showOkCancel({title:lex.get('authenticate.user.title'),content:template({jsonObject:jsonObject})}).ok(function(val){
                        return SecuredSetDisplayNameCmd.$setupRename(comp, slots).then(function (_ref) {
                            var comp = _ref.comp,
                                slot = _ref.slot,
                                displayName = _ref.displayName;

                            if (displayName !== null) {
                                return SecuredSetDisplayNameCmd.setDisplayName(comp, slot, displayName).then(function(){
                                    var dashboardHandle=actionPoint.getSecureSlotFacets().$map.$map.dashboardHandle;
                                    return getDashBoardComponent(dashboardHandle).then(function(dashBoardComponent){
                                        var primaryComments=val.jq().find("#primaryComment").val();
                                        actionPoint.setActionName("setDisplayName");
                                        if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
                                        {
                                            actionPoint.setPrimaryComments(primaryComments);
                                        }else
                                        {
                                            var secondaryComments=val.jq().find("#secondaryComments").val();
                                            var managerName= val.jq().find("#secondaryUsername").val();
                                            actionPoint.setPrimaryComments(primaryComments);
                                            actionPoint.setSecondaryComments(secondaryComments);
                                            actionPoint.setManager(managerName);
                                        }
                                        actionPoint.setNewValue(displayName);
                                        actionPoint.setOldValue(oldDisplayName);
                                        var comp= baja.$("baja:Component", {
                                            actionPoint: actionPoint,
                                        });
                                        return dashBoardComponent.invoke({slot:"updateHistory", value: comp});
                                    });
                                });
                            }
                        });
                    });
                    Util.loadInitialEventsAndData(actionPoint,"displayName");
                }).catch(function (err) {
                    return dialogs.showOk({
                        title: "Error",
                        content:lex.get('editor.primary.user.validation')
                    });
                });
            }
            else{
                return SecuredSetDisplayNameCmd.$setupRename(comp, slots).then(function (_ref) {
                    var comp = _ref.comp,
                        slot = _ref.slot,
                        displayName = _ref.displayName;

                    if (displayName !== null) {
                        return SecuredSetDisplayNameCmd.setDisplayName(comp, slot, displayName);
                    }
                });
            }
        });
    };

    /**
     * @private
     * @param {baja.Component} comp
     * @param {Array.<baja.Slot>} slots
     * @returns {Promise.<{ comp: baja.Component, slot: baja.Slot, displayName: string, oldDisplayName: string }>} displayName
     * will be null if user canceled prompt
     */
    SecuredSetDisplayNameCmd.$setupRename = function (comp, slots) {
        var _slots = _slicedToArray(slots, 1),
            slot = _slots[0];

        if (!slot) {
            return Promise.reject(new Error('slot to rename required'));
        }

        var oldDisplayName = comp.getDisplayName(slot);
        return SecuredSetDisplayNameCmd.$promptForNewDisplayName(oldDisplayName).then(function (displayName) {
            if (displayName === oldDisplayName) {
                displayName = null;
            }

            return {
                comp: comp,
                slot: slot,
                displayName: displayName,
                oldDisplayName: oldDisplayName
            };
        });
    };

    /**
     * @private
     * @param {string} oldDisplayName
     * @returns {Promise.<string|null>}
     */
    SecuredSetDisplayNameCmd.$promptForNewDisplayName = function (oldDisplayName) {
        return feDialogs.showFor({
            title: webEditorsLex.get('commands.setDisplayName.description'),
            value: oldDisplayName,
            formFactor: 'mini'
        });
    };

    /**
     * Performs the display name set by adding it to the component's
     * `displayNames` slot. If an empty string is given, the display name will
     * be removed.
     *
     * @param {baja.Component} comp
     * @param {String|baja.Slot} slot
     * @param {String} newDisplayName
     * @returns {Promise}
     */
    SecuredSetDisplayNameCmd.setDisplayName = function (comp, slot, newDisplayName) {
        var displayNames = comp.get('displayNames');
        var method;
        var arg;

        if (displayNames) {
            method = 'set';
            arg = displayNames.toObject();
        } else {
            method = 'add';
            arg = {};
        }

        if (newDisplayName) {
            arg[slot] = newDisplayName;
        } else {
            delete arg[slot];
        }

        return comp[method]({
            slot: 'displayNames',
            value: baja.NameMap.make(arg),
            flags: DISPLAY_NAMES_FLAGS //ignored in set()

        });
    };

    function hasAdminWrite(comp) {
        return comp.getPermissions().hasAdminWrite();
    }

    function loadComponentSlots(comp){
        return new Promise(function(fullfil,reject){
            comp.lease().then(function(){
                comp.loadSlots({
                    ok:function(){
                        fullfil(comp);
                    },
                    fail:function(){
                        reject(comp);
                    }
                });
            });
        });
    }

    function getDashBoardComponent(handle) {
        var ord="station:|h:"+handle;
        return baja.Ord.make(ord).get({lease: true});
    }

    return SecuredSetDisplayNameCmd;
});
