/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

/**
 * SecureStringEditor manages functionality like primary, secondary authentication ,Change new value and maintain the secure point history.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/SecuredStringEditor
 */
define(['baja!','baja!electronicSignature:SecureString',
        'lex!baja',
        'jquery',
        'Promise',
        'dialogs',
        'lex!electronicSignature',
        'bajaux/Widget',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredStringTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         electronicSignatureLex,
         Widget,
         subscriberMixIn,
         template,
         loader,
         BaseEditor,
         SecureParentEditor) {

      'use strict';
     var lex = electronicSignatureLex[0];
      /**
       * A field editor for working with strings.
       *
       * @private
       * @class
       * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
       * @alias module:nmodule/electronicSignature/rc/fe/baja/SecuredStringEditor
       */
      var SecuredStringEditor = function SecuredStringEditor(params) {
        BaseEditor.call(this, $.extend({
          keyName: 'SecuredStringEditor'
        }, params));
        subscriberMixIn(this);
      };

      SecuredStringEditor.prototype = Object.create(SecureParentEditor.prototype);

      SecuredStringEditor.prototype.constructor = SecuredStringEditor;
      SecuredStringEditor.componentFE = null;

      SecuredStringEditor.prototype.doInitialize = function (dom) {
          var jsonObject={pointName:lex.get('fe.pointName.label'),
              newValue:lex.get('fe.newValue.label'),
              reason:lex.get('fe.reason.label'),
              otherReason:lex.get('fe.otherReason.label'),
              authLevel1:lex.get('fe.authorization.level1.label'),
              authLevel2:lex.get('fe.authorization.level2.label'),
              userName:lex.get('fe.userName.label'),
              password:lex.get('fe.password.label'),
              comment:lex.get('fe.comment.label'),
              sign:lex.get('fe.sign.button'),
              sign2:lex.get('fe.sign2.button'),
              remote:lex.get('fe.remote.button'),
              chooseUser:lex.get('remote.chooseUser.label'),
              local:lex.get('remote.local.button'),
              sendRequest:lex.get('remote.sendRequest.button'),
              acknowledgement:lex.get('eSignature.ack.label'),
              label1:lex.get('eSignature.ack.label1'),
              label2:lex.get('eSignature.ack.label2'),
              label3:lex.get('eSignature.ack.label3'),
              label4:lex.get('eSignature.ack.label4'),
              label5:lex.get('eSignature.ack.label5'),
              label6:lex.get('eSignature.ack.label6'),
              label7:lex.get('eSignature.ack.label7')

          };
          dom.html(template({jsonObject:jsonObject}));
          this.doInitializeContent(dom);
      };

      SecuredStringEditor.prototype.doLoad = function(secureString,params){
          var widget = this,
          dom = widget.jq();
          SecuredStringEditor.componentFE = secureString;
          var result = this.doLoadValue(dom,secureString);
          if(result === "NOT_CONNECTED")
              return Promise.resolve("");
          if(secureString.getSecureSlotFacets().$map.get("isMultiline") === "true"){
          	var input    = dom.find("#txtNewValue")[0],
            textarea = document.createElement('textarea');  // jshint ignore:line
		        textarea.id    = input.id;
		        textarea.className = "fixControlWidth";
		        textarea.value = secureString.getNewValue();
		        input.parentNode.replaceChild(textarea, input);
          }else{
          	dom.find("#txtNewValue").val(secureString.getNewValue());
          }
          
          return widget.validateSecondaryUsers(secureString);

      };

    /*  SecuredStringEditor.prototype.doIsValueValid = function(dom){
          var invalidList=[];
          //invalidListObj = {};

          var newValueField = dom.find('#txtNewValue')[0].value;
          {
            dom.find('#txtNewValue').removeClass("errorMsg");
          }
          return invalidList;
      };*/

      SecuredStringEditor.prototype.doRead = function(){
        var that = this;
        var dom = this.jq();
        var secureString = that.value();
        return secureString.setNewValue(getPointNewValue(dom)).then(function(){return secureString;});
      };

      SecuredStringEditor.prototype.doSave = function (secureString,params) {

        var dom = this.jq();
        secureString = this.doSaveValue(secureString,dom);
        return secureString;

      };

      function getPointNewValue(dom) {
        return dom.find("#txtNewValue")[0].value;
      }

      return SecuredStringEditor;

   });