
define(['baja!', 'baja!baja:Role', 'bajaux/Widget', 'jquery', 'Promise', 'underscore', 'bajaux/events',  'nmodule/webEditors/rc/fe/CompositeEditor', 'nmodule/webEditors/rc/fe/baja/util/spaceUtils', 'nmodule/webEditors/rc/fe/baja/util/subscriberUtils', 'nmodule/webEditors/rc/fe/baja/util/userUtils', 'nmodule/webEditors/rc/fe/config/CompositeBuilder'], function (baja, types, Widget, $, Promise, _,events, CompositeEditor, spaceUtils, subscriberUtils, userUtils, CompositeBuilder) {

    'use strict';
    var MODIFY_EVENT = events.MODIFY_EVENT;
    var resolveService = spaceUtils.resolveService;
    var leaseWhenNeeded = subscriberUtils.leaseWhenNeeded;
    var resolveCurrentUser = userUtils.resolveCurrentUser;

    var getRoleService = leaseWhenNeeded(_.partial(resolveService, 'niagaraDriver:NiagaraNetwork'));

    /**
     * Editor for editing available roles on a user.
     *
     * @class
     * @alias module:nmodule/webEditors/rc/wb/util/StationListEditor
     * @extends module:nmodule/webEditors/rc/fe/CompositeEditor
     */
    var StationListEditor = function StationListEditor(params) {
        CompositeEditor.apply(this, arguments);
    };
    StationListEditor.prototype = Object.create(CompositeEditor.prototype);
    StationListEditor.prototype.constructor = StationListEditor;

    /**
     * @private
     * @param {baja.Component} [base]
     * @returns {Promise.<baja.Component>}
     */
    StationListEditor.prototype.$getRoleService = function (base) {
        return getRoleService(base);
    };

    /**
     * @private
     * @param {baja.Component} base
     * @returns {Promise.<baja.Component>}
     */
    StationListEditor.prototype.$getCurrentUser = function (base) {
        return resolveCurrentUser(base).catch(function () {
            return null;
        });
    };

    /**
     * Resolve the roles that are available to add to a user.
     *
     * @private
     * @returns {Promise.<Array.<String>>} promise to be resolved with an array
     * of role names
     */
    StationListEditor.prototype.$getAvailableRoles = function () {
        var _this = this;

        var that = this;

        return this.getOrdBase().then(function (base) {
            return Promise.all([_this.$getRoleService(base), _this.$getCurrentUser(base)]);
        }).then(function (_ref) {
            var _ref2 = _slicedToArray(_ref, 2),
                roleService = _ref2[0],
                user = _ref2[1];

            var roleNames = _.map(roleService.getSlots().is('niagaraDriver:NiagaraStation').toArray(), String);
            var rolesList=[];
            if(roleNames.length>0){
                rolesList.push("All");
            }
            for (let i = 0; i < roleNames.length ; i++) {
                rolesList.push(roleNames[i]);
            }
            return rolesList;
           /* return _.filter(roleNames, function (role) {
                return _.contains(allowedRoles, role) || _.contains(loadedRoles, role);
            });*/
        }).catch(function () {
            return [];
        });
    };

    StationListEditor.prototype.$getRolesContainerElement = function () {
        return this.jq().children('.rolesContainer');
    };

    /**
     * Add "StationListEditor" class.
     *
     * @param {JQuery} dom
     */
    StationListEditor.prototype.doInitialize = function (dom) {
        var that = this;
        dom.addClass('StationListEditor');
        dom.append('<div class="rolesContainer"/>');

        dom.on(MODIFY_EVENT, '.editor', function (e, ed) {
            that.setModified(true, ed);
            var checkBoxList=[];
            $( ".rolesContainer" ).find( ":checkbox" ).each(function(){
                checkBoxList.push(this);
            });
            var checkBox = _.first(ed.$getCheckbox());
            if(e.target.innerText==="All"){
                if(checkBox.checked){
                    for (let i = 0; i < checkBoxList.length; i++) {
                        checkBoxList[i].checked=true;
                    }
                }
                else{
                    for (let i = 0; i < checkBoxList.length; i++) {
                        checkBoxList[i].checked=false;
                    }
                }
            }
            else{
                that.checkedDuplicateControl(checkBoxList,e.target.innerText,checkBox.checked);
                if(that.isAllSelected(checkBoxList)){
                    for (let i = 0; i < checkBoxList.length; i++) {
                        checkBoxList[i].checked=true;
                    }
                }
                else {
                    var list = that.getListOfAll(checkBoxList);
                    for (let i = 0; i < list.length; i++) {
                        list[i].checked=false;
                    }
                }
            }
            return false;
        });
    };

    StationListEditor.prototype.checkedDuplicateControl = function (checkBoxList, name,value) {
        var that=this;
        for (let i = 0; i < checkBoxList.length; i++) {
            if(that.compaireName(checkBoxList[i],name)){
                checkBoxList[i].checked = value;
            }
        }
    }

    StationListEditor.prototype.compaireName = function (checkBox, name) {
       return checkBox.parentElement.className.indexOf(name)>-1;
    }
    StationListEditor.prototype.getListOtherThanAll = function (checkBoxList) {
        var list = [];
        var that = this;
        for (let i = 0; i < checkBoxList.length; i++) {
            if(!that.compaireName(checkBoxList[i],"All")){
                list.push(checkBoxList[i]);
            }
        }
        return list;
    }
    StationListEditor.prototype.getListOfAll = function (checkBoxList) {
        var list = [];
        var that = this;
        for (let i = 0; i < checkBoxList.length; i++) {
            if(that.compaireName(checkBoxList[i],"All")){
                list.push(checkBoxList[i]);
            }
        }
        return list;
    }



    StationListEditor.prototype.isAllSelected = function (checkBoxList) {
        var isSelected = true;
        var listOtherThanAll = this.getListOtherThanAll(checkBoxList);
        for (let i = 0; i < listOtherThanAll.length; i++) {
            isSelected = listOtherThanAll[i].checked;
            if(!isSelected){
                return false;
            }
        }
        return true;
    }

    /**
     * The builder for a `StationListEditor` will create a boolean editor for each
     * available role.
     *
     * @returns {module:nmodule/webEditors/rc/fe/config/CompositeBuilder}
     */
    StationListEditor.prototype.makeBuilder = function () {
        var that = this,
            builder = new CompositeBuilder();

        builder.getKeys = function () {
            return that.$getAvailableRoles().then(function (roleNames) {
                return roleNames;
            });
        };

        builder.getValueFor = function (key) {
            return _.contains(this.getDataSource().split(','), key);
        };

        builder.getConfigFor = function (key) {
            return { properties: { trueText: key, falseText: key }, formFactor: 'mini' };
        };

        builder.getDomFor = function () {
            return $('<div/>').appendTo(that.$getRolesContainerElement());
        };

        return builder;
    };

    /**
     * Read what roles are selected and join them together into a string.
     *
     * @returns {Promise} promise to be resolved with a comma-separated
     * string containing what roles are selected
     */
    StationListEditor.prototype.doRead = function () {
        var builder = this.getBuilder();
        return Promise.join(builder.getKeys(), builder.readAll()).spread(function (roleNames, checks) {
            return _.filter(roleNames, function (roleName, i) {
                return checks[i];
            }).join(',');
        });
    };

    /**
     * Remove "StationListEditor" class.
     *
     * @returns {Promise}
     */
    StationListEditor.prototype.doDestroy = function () {
        this.jq().removeClass('StationListEditor');
        return CompositeEditor.prototype.doDestroy.apply(this, arguments);
    };

    return StationListEditor;
});
