/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Swapnil Dhamal
 */

const SECONDARY = "Secondary";
const PRIMARY = "Primary";
const NORMAL = "Normal";
const primaryButtonSelector = "#btnPrimarySign";

define(['baja!',
    'jquery',
    'dialogs',
    'lex!electronicSignature',
    'Promise',
    'underscore',
    "nmodule/js/rc/csrf/csrfUtil",
    "baja!baja:JobState"], function (baja,
                             $,
                             dialogs,
                             electronicSignatureLex,
                             Promise,
                             _, csrfUtil, types)
{
    'use strict';
    /*Private properties*/
    var isAuthenticatedUser = false,
        _SecondaryAuthenticatorRoles = "",
        isValidUserSecondary = false,
        component = null;
    var lex = electronicSignatureLex[0];
    var legalTextTemplate ="";
    var customerName ="";
    var resultList = new Map(), alarmAuthenticationLevel ="";
    var alarmPoint = new Map();
    var nonAuthenticatedAlarms = "", primaryComment = "";
    var normalAlarmPoints = new Set();


    function getResultList (){
        return resultList;
    }

    function getAlarmAuthenticationLevel(){
        return alarmAuthenticationLevel;
    }
    function initializeControlPoint(actionPoint, _facets)
    {
        var pointOrd = getFormatedPointOrdPrivate(actionPoint, _facets);
        if (pointOrd !== "")
        {
            return getComponentByOrdPrivate(pointOrd).then(function (comp)
            {
                component = comp;
                return component;
            });
        }
        return Promise.resolve(null);
    }

    function getFormatedPointOrdPrivate(component, facets)
    {
        var pointOrd = "";
        if (!component.getIsProxyCall() || component.getIsSecondaryRemote())
        {
            if (component.getSlot("remoteJace")!=null)
            {
                var componentHandle = component.get("proxyComponentHandle").toString();
                if (componentHandle !== "" && componentHandle !== null)
                {
                    pointOrd = "station:|h:" + componentHandle.toString();
                }
            }
            else
            {
                var componentHandle = facets.$map.get("handle");
                if (componentHandle !== "" && componentHandle !== null)
                {
                    pointOrd = "station:|h:" + componentHandle.toString();
                }
            }
        }
        else if (component.getSlot("remoteJace")!=null)
        {
            var componentHandle = component.get("proxyComponentHandle").toString();
            if (componentHandle !== "" && componentHandle !== null)
            {
                pointOrd = "station:|h:" + componentHandle.toString();
            }
        }
        else
        {
            var componentSlotPath = component.getPointOrdProxyDisplay();
            if (componentSlotPath !== "" && componentSlotPath !== null)
            {
                pointOrd = "station:|" + componentSlotPath.toString();
            }
        }
        return pointOrd;
    }

    function getComponentByOrdPrivate(ord)
    {
        return baja.Ord.make(ord).get({lease: true});
    }

    function authorizeUserPrivate(component, primaryUser, actionName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("authorization," + primaryUser.toString() + "," + actionName.toString())
        });
    }

    function authenticateUserPrivate(component, credentials)
    {
        return new Promise(function (resolve, reject) {
            component.invoke({
                slot: "call",
                value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
            }).then(function (authenticatedComp)
            {
                if(authenticatedComp.getSlot("authenticationJobOrd")===null)
                {
                    resolve(authenticatedComp.get("isValidUser"));
                }
                else
                {
                    let getJobStatus = ()=>{
                        component.invoke({
                            slot: "call",
                            value: baja.$("baja:String").make("LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                        }).then(function(returnComponent){
                            if(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("running"))
                            {
                                //schedule the timeout.
                                setTimeout(function (){
                                    getJobStatus();
                                },1000);
                            }
                            else
                            {
                                resolve(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("success"));
                            }
                        });
                    };
                    setTimeout(function (){
                        getJobStatus();
                    },1000);
                }
            }).catch(function(err){
                console.log("Error occurred while authenticating the user.", err);
                reject();
            });
        });
        /*return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
        });*/
    }

    function authenticateSecondaryUserPrivate(component, userName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUserDetails," + userName.toString())
        });
    }

    function showValidationSummaryPrivate(validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            $("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: lex.get('error.massages.title'),
            content: errorMessage
        });
    }

    function doIsDataValidPrivate(authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        if (authenticationType === "primary")
        {
            if ($('#username').val() === null || $('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = lex.get('parent.editor.userName.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#username').removeClass("errorMsg");
            }
            if ($('#password').val() === null || $('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg = lex.get('parent.editor.password.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#password').removeClass("errorMsg");
            }
            return invalidList;
        }
        if (authenticationType === "secondary")
        {

            if ($('#secondaryUsername').val() === null || $('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg = lex.get('parent.editor.userName.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryUsername').removeClass("errorMsg");
            }
            if ($('#secondaryPassword').val() === null || $('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = lex.get('parent.editor.password.mandatory');
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryPassword').removeClass("errorMsg");
            }
            return invalidList;
        }
    }

    function changeLegalTextPrivate(userName,pointComponent)
    {

        var pointOrd = "station:|h:" + pointComponent.getHandle().toString();

        if (pointOrd !== "")
        {

            pointComponent.getCustomer().loadSlots().then(function(){
                getComponentByOrdPrivate(pointOrd).then(function (comp) {
                    component = comp;
                    return component.invoke({
                        slot: "call",
                        value: baja.$("baja:String").make("getLegalText," + pointComponent.getCustomer().getCustomerName() + "," + userName)
                    }).then(function (component){
                        var legalText = component.get("legalText").toString();
                        overrideLegalText(userName,legalText);
                    });
                });
            });
        }
    }


    function changeLegalTextPrivateNew(userName,pointComponent)
    {
        var pointOrd = getFormatedPointOrdPrivate(pointComponent, pointComponent.getSecureSlotFacets());

        if (pointOrd !== "")
        {

            getComponentByOrdPrivate(pointOrd).then(function(){
                getComponentByOrdPrivate(pointOrd).then(function (comp) {
                    component = comp;
                    return component.invoke({
                        slot: "call",
                        value: baja.$("baja:String").make("getLegalText," + pointComponent.getSecureSlotFacets().$map.get("customerName").toString() + "," + userName)
                    }).then(function (component){
                        var legalText = component.get("legalText").toString();
                        overrideLegalText(userName,legalText);
                    });
                });
            });
        }
    }

    function overrideLegalText(userName, legalText)
    {
        legalText = wordWrapPrivate(legalText,70);
        var legalTextParagraph = $(document).find("#legaltext");
        legalTextParagraph.empty();
        legalTextParagraph.html(lex.get('eSignature.ack.label')+"\n" + legalText);
    }

    function wordWrapPrivate(str, maxWidth) {
        var newLineStr = "\n", res = '';
        while (str.length > maxWidth) {
            var found = false;
            // Inserts new line at first whitespace of the line
            for (var i = maxWidth - 1; i >= 0; i--) {
                if (testWhite(str.charAt(i))) {
                    res = res + [str.slice(0, i), newLineStr].join('');
                    str = str.slice(i + 1);
                    found = true;
                    break;
                }
            }
            // Inserts new line at maxWidth position, the word is too long to wrap
            if (!found) {
                res += [str.slice(0, maxWidth), newLineStr].join('');
                str = str.slice(maxWidth);
            }

        }

        return res + str;
    }

    function testWhite(x) {
        var white = new RegExp(/^\s$/);
        return white.test(x.charAt(0));
    }

    function checkPrimarySignPrivate(credentials, pointComponent, _facets)
    {
        isAuthenticatedUser = false;
        var that = this,
            validation = that.doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            that.showValidationSummary(validation);
        }
        else
        {
            var pointOrd = that.getFormatedPointOrd(pointComponent, _facets);
            if (pointOrd !== "")
            {
                this.getComponentByOrd(pointOrd).then(function (comp)
                {
                    $("#btnPrimarySign").html(lex.get('fe.signing.button'));
                    $("#btnPrimarySign").prop("disabled", true);
                    return that.authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp)
                {
                    $("#btnPrimarySign").html(lex.get('fe.sign.button'));
                    $("#btnPrimarySign").prop("disabled", false);
                    if (authenticatedComp)
                    {
                        isAuthenticatedUser = true;
                        if (_facets.$map.get("isSecuredSecondary"))
                        {
                            $('.js-dialog-button-ok').attr('disabled', true);
                            $('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
                            _SecondaryAuthenticatorRoles = _facets.$map.get("secondaryAuthenticatorRoles").toString();
                            changeLegalTextPrivateNew($("#secondaryUsername").val(),pointComponent);
                            $("#btnPrimarySign").html(lex.get('fe.signed.button'));
                        }
                        if (_facets.$map.get("isSecuredPrimary"))
                        {
                            $('.js-dialog-button-ok').attr('disabled', false);
                            $("#btnPrimarySign").html(lex.get('fe.signed.button'));
                            return dialogs.showOk({
                                title: lex.get('auth.success.title'),
                                content: lex.get('auth.success.msg')
                            });
                        }
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: lex.get('error.massages.title'),
                            content: lex.get('authentication.error.message')
                        });
                    }
                });
            }
        }
    }

    function checkSecondarySignPrivate(credentials, pointComponent, _facets, nativeActionName)
    {
        isAuthenticatedUser = false;
        var that = this;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title:  lex.get('authentication.error.title'),
                content: lex.get('level2.authentication.message')
            });
        }
        var validation = that.doIsDataValid("secondary");
        if (validation !== null && validation.length > 0)
        {
            that.showValidationSummary(validation);
        }
        else
        {
            var pointOrd = that.getFormatedPointOrd(pointComponent, _facets),
                pointComp,
                isValidManager = false;
            if (pointOrd !== "")
            {
                that.getComponentByOrd(pointOrd).then(function (comp)
                {
                    pointComp = comp;
                    $(".btnSecondarySign").html(lex.get('fe.signing.button'));
                    $(".btnSecondarySign").prop("disabled", true);

                    return that.authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp)
                {
                    $(".btnSecondarySign").html(lex.get('fe.sign.button'));
                    $(".btnSecondarySign").prop("disabled", false);
                    isValidManager = authenticatedComp;
                    if (authenticatedComp)
                    {
                        return that.authenticateSecondaryUser(pointComp, credentials.username);
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: lex.get('authentication.error.title'),
                            content: lex.get('authentication.error.message')
                        });
                    }
                }).then(function (secondaryUserDetails)
                {
                    //secondaryUserDetails.loadSlots();
                    return secondaryUserDetails;
                }).then(function (secondaryUserDetails)
                {
                    if (isValidManager === true)
                    {
                        var managerRoles = _SecondaryAuthenticatorRoles.split(",");
                        isValidUserSecondary = false;
                        if (secondaryUserDetails.get("roles") !== null)
                        {
                            var secondaryUsersRoles = secondaryUserDetails.get("roles").split(",");
                            _.each(managerRoles, function (value)
                            {
                                var roleExistList = _.filter(secondaryUsersRoles, function (role)
                                {
                                    return role === value;
                                });
                                if (_.size(roleExistList) > 0)
                                {
                                    isValidUserSecondary = true;
                                }
                            });
                            if (!isValidUserSecondary)
                            {
                                isValidManager = false;
                                return dialogs.showOk({
                                    title: lex.get('authentication.error.title'),
                                    content: lex.get('secUser.role.error')
                                });
                            }
                            else
                            {
                                var permissions = secondaryUserDetails.get("permission");
                                if (permissions !== null)
                                {
                                    if (nativeActionName !== undefined)
                                    {
                                        //if (!(permissions.hasAdminRead() && permissions.hasAdminWrite())) {
                                        if (!(permissions.hasAdminWrite() && permissions.hasOperatorInvoke()))
                                        {
                                            isValidManager = false;
                                            return dialogs.showOk({
                                                title: lex.get('permission.error.title'),
                                                content: lex.get('secUser.permission.error.message')
                                            });
                                        }
                                    }
                                    else
                                    {
                                        var slotName = _facets.$map.get("SlotName").toString();
                                        if (!((pointComp.getFlags(pointComp.getSlot(slotName)) === baja.Flags.OPERATOR && permissions.hasOperatorRead() && permissions.hasOperatorInvoke()) || (pointComp.getFlags(pointComp.getSlot(slotName)) !== baja.Flags.OPERATOR && permissions.hasAdminRead() && permissions.hasAdminInvoke())))
                                        {
                                            isValidManager = false;
                                            return dialogs.showOk({
                                                title: lex.get('permission.error.title'),
                                                content: lex.get('secUser.permission.error.message')
                                            });
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            isValidUserSecondary = true;
                            //isValidManager=true;
                        }
                        if (isValidManager && isValidUserSecondary)
                        {
                            $('.js-dialog-button-ok').attr('disabled', false);
                            isAuthenticatedUser = true;
                            $("#secondaryComments").attr('disabled', true);
                            $("#secondaryUsername").prop("disabled", true);

                            $(".btnSecondarySign").html(lex.get('fe.signed.button'));

                            /*return dialogs.showOk({
                                title: lex.get('auth.success.title'),
                                content: lex.get('auth.success.msg'),
                            });*/
                        }
                    }
                })
                    .catch(function (error)
                    {
                        console.log(error); // jshint ignore:line
                    });
            }
            else
            {
                return dialogs.showOk({
                    title: lex.get('authentication.error.title'),
                    content: lex.get('authentication.error.message')
                });
            }
        }
    }

    function loadInitialEventsAndDataPrivate(actionPoint, nativeActionName)
    {
        var that = this;
        $('.js-dialog-button-ok').attr('disabled', true);
        $("#username").val(actionPoint.getSecureSlotFacets().$map.get("loginUserName"));
        if(actionPoint.getSecureSlotFacets().$map.get("isSecuredSecondary") == true)
        {
            $("#btnPrimarySign").text(lex.get('fe.sign1.button'));
        }
        $("#btnPrimarySign").click(function ()
        {
            var credentials = {};
            credentials.username = $('#username').val();
            credentials.password = $('#password').val();
            that.checkPrimarySign(credentials, actionPoint, actionPoint.getSecureSlotFacets());
        });

        $(".btnSecondarySign").click(function ()
        {
            var credentials = {};
            credentials.username = $('#secondaryUsername').val();
            credentials.primaryUserName = $('#username').val();
            credentials.password = $('#secondaryPassword').val();
            that.checkSecondarySign(credentials, actionPoint, actionPoint.getSecureSlotFacets(), nativeActionName);
        });
        $(".primaryTab").click(function ()
        {
            return false;
        });

        $(".secondaryTab").click(function ()
        {
            return false;
        });

        if (actionPoint.getSecureSlotFacets().$map.get("isSecuredPrimary") !== null)
        {
            $(".secondaryTab").hide();
        }
        try
        {
            // jshint ignore:line
            customerName = actionPoint.getSecureSlotFacets().$map.get("customerName").toString();
            changeLegalTextPrivateNew(actionPoint.getSecureSlotFacets().$map.get("loginUserName"),actionPoint);

            if(actionPoint.getSecureSlotFacets().$map.get("isSecuredSecondary")!==null)
            {
                return initializeControlPoint(actionPoint, actionPoint.getSecureSlotFacets()).then(function ()
                {
                    var actionName = nativeActionName;
                    if(actionName === undefined || actionName === null || actionName ==="")
                    {
                        actionName = "delete";
                    }
                    return fetchSecondaryUserPrivate(component, baja.getUserName(), actionName).then(function (secondaryUsers)
                    {
                        if (secondaryUsers.get("secUser") && secondaryUsers.get("secUser").toString() !== "")
                        {
                            //_SecondaryAuthenticatorRoles = secondaryUsers.get("secAuthRole").toString();
                            var secondaryUsersList = secondaryUsers.get("secUser").toString().split(",");
                            var usersHtml = "";
                            _.each(secondaryUsersList, function (value, index)
                            {
                                if (index + 1 !== secondaryUsersList.length)
                                {
                                    usersHtml += "<option value=" + value + ">" + value + "</option>";
                                }
                            });
                            $(document).find("#secondaryUsername").html(usersHtml);
                            $(document).find("#secondaryUsername").change(function(){
                                changeLegalTextPrivateNew($(document).find("#secondaryUsername").val(),actionPoint);
                            });
                            return Promise.resolve("");
                        }
                        else
                        {
                            $("#easSignatureTabs").parent().text(lex.get('primUser.privilege.error.pointchooser.message'));
                            $("#legaltext").hide();
                            /*return dialogs.showOk({
                               title: lex.get('custom.error.message'),
                               content: lex.get('eSignature.secondaryAuthenticators.configure.error')
                            });*/
                        }
                    });
                });
            }

        }
        catch (err)
        {
        }
    }

    function fetchSecondaryUserPrivate(component, currentUser, slotName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUsers," + currentUser + "," + slotName)
        });
    }

    var eSignature = null;
    function getESignatureComponentPrivate()
    {
        if(eSignature == null)
        {
            return $.ajax("/rpc/rpcGetESignHandle/type:electronicSignature:ProtectedConsoleRecipient", {
                method: "POST",
                dataType: "json",
                data: null,
                headers:{
                    'x-niagara-csrfToken': csrfUtil.getCsrfToken()
                }
            })
            .then(function (response) {
                return getComponentByOrdPrivate('station:|h:' +response.value);
            }).then(function(eSign){
                eSignature = eSign;
                return eSignature;
            });
        }
        return Promise.resolve(eSignature);
    }

    function getAlarmDetailsPrivate(selectedAlarmRecords,commandNameToSend,primaryUserComments,reason,secondaryUsername,secondaryComments) {
        var alarmRecordsDetails = [];
        reason            = reason === undefined ? "" : reason;
        secondaryUsername = secondaryUsername === undefined ? "" : secondaryUsername;
        secondaryComments = secondaryComments === undefined ? "" : secondaryComments;
        var primaryUsername = baja.getUserName();
        for (var i = 0; i < selectedAlarmRecords.length; i++) {
          var detailsOfAlarm = "";
          var sourceList =   selectedAlarmRecords[i].source.split("\n");
          if(sourceList.length > 1){
              detailsOfAlarm += "jaceOrd:" + sourceList[1] + ";;;";
          }
            detailsOfAlarm += "sourceOrd:" + sourceList[0] + ";;;toState:" + selectedAlarmRecords[i].alarmData.toState
                + ";;;fromState:" + selectedAlarmRecords[i].alarmData.fromState + ";;;status:" + selectedAlarmRecords[i].alarmData.status
                + ";;;presentValue:" + selectedAlarmRecords[i].alarmData.presentValue;
            alarmRecordsDetails.push(detailsOfAlarm);
        }
        return [
            {
                loginUserName: primaryUsername,
                ackNotes: primaryUserComments,
                commandName: commandNameToSend,
                alarmRecords: alarmRecordsDetails,
                reason: reason,
                secondaryAckNotes: secondaryComments,
                secondarySignerName: secondaryUsername
            }
        ];
    }

    function updateJaceOrdDetail(alarmRecord,jaceOrdDetails) {
        let source = alarmRecord.getSource().getOrds();
        let pointOrd = source[0];
        let arr = new Set();
        let jaceStationName;

        if (source.length > 1) { // if source.length >1, it means alarm is from JACE.
            jaceStationName = source[1].encodeToString().substring(0, source[1].encodeToString().lastIndexOf("/"));
        }
        else {
            jaceStationName = "Sup";
        }

        if (jaceOrdDetails.has(jaceStationName)) {
            let jacePointOrd = jaceOrdDetails.get(jaceStationName) + "," + pointOrd;
            jaceOrdDetails.set(jaceStationName, jacePointOrd);
        } else {
            jaceOrdDetails.set(jaceStationName, pointOrd);
        }

        let JACE_Point = jaceStationName + ";;;" + pointOrd;
        if (alarmPoint.get(JACE_Point) == null) {
            arr.add(alarmRecord);
        } else {
            arr = alarmPoint.get(JACE_Point);
            arr.add(alarmRecord);
        }
        alarmPoint.set(JACE_Point, arr);
        return jaceOrdDetails;
    }

    async function getPointAuthenticationLevel(jaceAndSupervisorPointOrd){
        let promises = [];

        jaceAndSupervisorPointOrd.forEach(function(value,key){
            promises.push( jaceUpdateForPoint(value,key));

            async function jaceUpdateForPoint(value,key) {
                let pointsOrdWithAuthenticationLevel;
                if (!(key === "Sup")) {
                    let pointOrd = [key + ";" + value];
                    pointsOrdWithAuthenticationLevel = await baja.rpc({
                        typeSpec: 'electronicSignature:ProtectedConsoleRecipient',
                        method: 'rpcCheckAuthenticationLevel',
                        args: pointOrd
                    });
                } else {
                    eSignature = await getESignatureComponentPrivate();
                    pointsOrdWithAuthenticationLevel = await eSignature.invoke({
                        slot: "pointAuthenticationInfo",
                        value: baja.$("baja:String").make(value.toString())
                    });
                }
                if (pointsOrdWithAuthenticationLevel == null || pointsOrdWithAuthenticationLevel === "") {
                    let pointOrds = value.encodeToString().split(",");
                    let pointJace = new Map();
                    pointOrds.forEach(function (ord) {
                        pointJace.set(ord, "false%%%Unable to fetch authentication level for point");
                    })
                    resultList.set(key, pointJace);
                } else {
                    let jaceAndPointOrds = pointsOrdWithAuthenticationLevel.split(";;;");
                    let points = jaceAndPointOrds[0].split(",");
                    points.forEach(function (point) {
                        let values = point.substring(point.lastIndexOf("-") + 1);
                        let pointJace = new Map();

                        if (values === SECONDARY) {
                            pointJace.set(point, SECONDARY);
                            alarmAuthenticationLevel = SECONDARY;
                        } else if (values === PRIMARY) {
                            pointJace.set(point, PRIMARY);
                            if (!(alarmAuthenticationLevel === SECONDARY)) {
                                alarmAuthenticationLevel = PRIMARY;
                            }
                        } else {
                            pointJace.set(point, NORMAL);
                            if ((alarmAuthenticationLevel !== SECONDARY) && (alarmAuthenticationLevel !== PRIMARY)) {
                                alarmAuthenticationLevel = NORMAL;
                            }
                        }

                        if (resultList.get(key) != null) {
                            let pointResult;
                            pointResult = resultList.get(key);
                            pointJace = new Map([...pointResult, ...pointJace]);
                        }

                        resultList.set(key, pointJace);
                    });
                }
            }
        });

        return  Promise.all(promises)
          .then(() => {
              return resultList;
          })
          .catch((e) => {
              console.log("resultL:ist ----- -",  resultList);
              console.log("error while fetching authentication level ",e);
          });

    }

    function getJacepointOrds(view){
        var selectedAlarmRecords =view.getSelectedAlarmRecords();
        var jaceOrdDetails = new Map();

        selectedAlarmRecords.forEach(function(alarmRecord){
            updateJaceOrdDetail(alarmRecord, jaceOrdDetails);
        });
        return jaceOrdDetails;
    }

    async function checkSecondarySignAndUpdateResultList(credentials, eSignature)
    {
        $('.js-dialog-button-ok').attr('disabled', true);
        isAuthenticatedUser = false;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title: lex.get('authentication.error.title'),
                content: lex.get('level2.authentication.message')
            });
        }
        var validation = doIsDataValid("secondary");
        if (validation !== null && validation.length > 0)
        {
            await  showValidationSummary(validation);
        }
        else
        {
            let isValidManager = false;
            $(document).find('.btnSecondarySign').html(lex.get('fe.signing.button'));
            $(document).find('.btnSecondarySign').prop("disabled", true);

            try {
                let atleastOneAlarmAuthenticated = await authenticateAndResultListUpdate(eSignature, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)),false);
                $(document).find('.btnSecondarySign').html(lex.get('fe.sign2.button'));
                $(document).find('.btnSecondarySign').prop("disabled", false);
                isValidManager = atleastOneAlarmAuthenticated;
                if (atleastOneAlarmAuthenticated)
                {
                    isAuthenticatedUser = true;
                    let atleastOneAlarmAuthorized = await authorizeUser(credentials,eSignature);
                    if(atleastOneAlarmAuthorized){
                        $('.js-dialog-button-ok').attr('disabled', false);
                        return dialogs.showOk({
                            title: lex.get('auth.success.title'),
                            content: lex.get('auth.success.msg')
                        });
                    }
                    else    {
                        return dialogs.showOk({
                            title: lex.get("authorization.error.title.browser"),
                            content: lex.get("authorization.error.message.browser")
                        });
                    }
                }
                else
                {
                    let isAnyPrimaryAuthenticatedAlarm =  successAlarmRecordsForjaces().length;
                    if(isAnyPrimaryAuthenticatedAlarm >0){
                        $('.js-dialog-button-ok').attr('disabled', false);
                        /*** 2nd Level Authentication failed for Following records.
                        record1
                        record2
                        record3
                        Press Ok to clear alarm records which passes single level authentication or
                        doesn't require authentication.
                         */
                        const msg = "Press OK to clear successfully authenticated records" ;
                        return dialogs.showOk({
                            title: lex.get("authentication.error.title"),
                            content:  '<div><div style = "background-color: #ff0000 ; color : white ; padding : 5px ">'+ nonAuthenticatedAlarms +'</div><br/><b>'+msg+'</b></div>'
                        });
                    }
                    return dialogs.showOk({
                        title: lex.get("authentication.error.title"),
                        content: lex.get("authentication.error.message")
                    });
                }
            }
            catch (e) {
                $(document).find(".btnSecondarySign").html(lex.get('fe.sign.button'));
                $(document).find(".btnSecondarySign").prop("disabled", false);
                console.log(e); // jshint ignore:line
            }
        }
    }

    function doIsDataValid(authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        if (authenticationType === "primary")
        {
            if ($('#username').val() === null || $('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = lex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#username').removeClass("errorMsg");
            }
            if ($('#password').val() === null || $('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg = lex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#password').removeClass("errorMsg");
            }
            return invalidList;
        }
        if (authenticationType === "secondary")
        {

            if ($('#secondaryUsername').val() === null || $('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg = lex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryUsername').removeClass("errorMsg");
            }
            if ($('#secondaryPassword').val() === null || $('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = lex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryPassword').removeClass("errorMsg");
            }
            return invalidList;
        }
    }

    async function checkPrimarySignAndUpdateResultList(credentials, eSignature,resultListUpdate)
    {
        resultList = resultListUpdate;
        isAuthenticatedUser = false;
        let validation = doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            await showValidationSummary(validation);
        }
        else
        {
            primaryComment =$('#primaryComment').val();
            $(document).find(primaryButtonSelector).html(lex.get('fe.signing.button'));
            $(document).find(primaryButtonSelector).prop("disabled", true);

            try {
                let anyOneAlarmSourceIsAuthenticated = await authenticateAndResultListUpdate(eSignature,baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)),true)
                $(document).find(primaryButtonSelector).html(lex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if (anyOneAlarmSourceIsAuthenticated)
                {
                    isAuthenticatedUser = true;
                    if(alarmAuthenticationLevel === PRIMARY) {
                        $('.js-dialog-button-ok').attr('disabled', false);
                        $(primaryButtonSelector).html(lex.get("fe.signed.button"));
                    }
                    else {
                        $('.js-dialog-button-ok').attr('disabled', true);

                        fetchSecondaryUser(eSignature, credentials.username, "call").then(function (secondaryUsers) {
                            if (secondaryUsers.get("secUser") && secondaryUsers.get("secUser").toString() !== "") {
                                var secondaryUsersList = secondaryUsers.get("secUser").toString().split(",");
                                _SecondaryAuthenticatorRoles = secondaryUsers.get("secAuthRole").toString();
                                var usersHtml = "";
                                _.each(secondaryUsersList, function (value, index) {
                                    if (index + 1 !== secondaryUsersList.length) {
                                        usersHtml += "<option value=" + value + ">" + value + "</option>";
                                    }
                                });
                                $(document).find("#secondaryUsername").html(usersHtml);
                            } else {
                                return dialogs.showOk({
                                    title: lex.get("custom.error.message"),
                                    content: lex.get("eSignature.secondaryAuthenticators.configure.l2Error")
                                });
                            }
                            $('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
                        });
                    }
                }
                else
                {
                    $("#password").addClass("errorMsg");
                    return dialogs.showOk({
                        title: lex.get("error.massages.title"),
                        content: lex.get("authentication.error.message")
                    });
                }
            }
            catch (e) {
                console.log(e);
                $(document).find(primaryButtonSelector).html(lex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if(error.javaClass ==="javax.baja.security.PermissionException"){
                    return dialogs.showOk({
                        title: lex.get('permission.error.title'),
                        content:lex.get('primUser.permission.error.message')
                    });
                }
            }
        }
    }

    function fetchSecondaryUser(component, currentUser, slotName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUsers," + currentUser + "," + slotName)
        });
    }

    function authenticateAndResultListUpdate(eSignature,credentials,isPrimary){
        var isValid = false;
        const promises = [];

        resultList.forEach(function (value,key){
            promises.push(authCheckForJace(value,key));
        });

        async function authCheckForJace(value,key) {
            isValid = false;
            let isAuthenticated = false;
            try {
                if (!(key === "Sup"))
                    isAuthenticated = await authenticateUserJace(credentials, key);
                else
                    isAuthenticated = await authenticateUser(eSignature, credentials);
            }
            catch (e) {

            }
            return updateResultList(value, key,isAuthenticated);
        }


        function updateResultList(mapOfPointsAndLevels,stationName,isAuthenticated) {
            let updatedPrimaryStatus = "";
            let updatedSecondaryStatus = "";

            if (isPrimary) {
                if (isAuthenticated) {
                    updatedPrimaryStatus = "Primary%%%true"
                    updatedSecondaryStatus = "Secondary%%%Pretrue";
                    isValid = true;
                }
                else
                {
                    updatedPrimaryStatus = "Primary%%%false%%%Authentication"
                    updatedSecondaryStatus = "Secondary%%%false%%%Authentication";
                }
            }
            else
            {
                if (isAuthenticated)
                    updatedSecondaryStatus = "Secondary%%%true";
                else
                    updatedSecondaryStatus = "Secondary%%%Secfalse%%%Authentication";
            }
            if (isPrimary) {
                mapOfPointsAndLevels.forEach(function (authStatus, pointOrd) {
                    if (authStatus.includes(SECONDARY)) {
                        authStatus = updatedSecondaryStatus;
                    } else if (authStatus.includes(PRIMARY)) {
                        authStatus = updatedPrimaryStatus;
                    }
                    mapOfPointsAndLevels.set(pointOrd, authStatus);
                });
            } else {
                mapOfPointsAndLevels.forEach(function (authStatus, pointOrd) {
                    if ((isAuthenticated &&
                        ((authStatus.includes(SECONDARY) && authStatus.includes("Pretrue"))
                          || authStatus.includes("Secfalse")) )
                      ||( !isAuthenticated && (authStatus.includes(SECONDARY) && authStatus.includes("Pretrue")))) {
                        authStatus = updatedSecondaryStatus;
                    }
                    mapOfPointsAndLevels.set(pointOrd, authStatus);
                });
            }
            resultList.set(stationName, mapOfPointsAndLevels);
            return isAuthenticated;
        }


        return  Promise.all(promises)
          .then((results) => {
              return  _.some(results, function(resultOfIndividualStation)
              { return resultOfIndividualStation === true; });
          })
          .catch((e) => {
              console.log("authenticateAndResultListUpdate:"+e);
          });
    }

    function authenticateUserJace( credentials,jace)
    {
        return new Promise(function (resolve, reject) {
            baja.rpc({
                typeSpec: 'electronicSignature:ProtectedConsoleRecipient',
                method: "rpcJaceAuthenticateForAlarms",
                args:[jace+";;;"+"authenticateUser," + credentials.toString()]
            }).then(function (authenticatedComp)
            {
                if(authenticatedComp == null){
                    resolve(false) ;
                }
                else {
                    if (authenticatedComp.authenticationJobOrd === undefined || authenticatedComp.authenticationJobOrd === null) {
                        if(authenticatedComp.isValidUser === "true"){
                            resolve(true);
                        }
                        resolve(false);
                    } else {
                        let getJobStatus = () => {
                            baja.rpc({
                                typeSpec: 'electronicSignature:ProtectedConsoleRecipient',
                                method: "rpcJaceAuthenticateForAlarms",
                                args: [jace + ";;;" + "LDAPAuthentication," + authenticatedComp.authenticationJobOrd]
                            }).then(function (returnComponent) {
                                if (returnComponent.jobStatus === "Running") {
                                    //schedule the timeout.
                                    setTimeout(function () {
                                        getJobStatus();
                                    }, 1000);
                                } else {
                                    if(returnComponent.jobStatus === "Success"){
                                        resolve(true);
                                    }
                                    resolve(false);
                                }
                            });
                        };
                        setTimeout(function () {
                            getJobStatus();
                        }, 1000);
                    }
                }
            }).catch(function(err){
                console.log("Error occurred while authenticating the user.", err);
                resolve(false);
            });
        }).catch(function(err){
            console.log("Error occurred while authenticating the user.", err);
            resolve(false);
        });
    }

    async function authenticateUser(eSignature, credentials)
    {
        return new Promise(async function (resolve, reject) {
            try {
                let authenticatedComp = await eSignature.invoke({
                    slot: "call",
                    value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
                });

                if(authenticatedComp == null){
                    resolve(false);
                }
                else {
                    if (authenticatedComp.getSlot("authenticationJobOrd") === null) {
                        resolve(authenticatedComp.get("isValidUser"));
                    } else {
                        let getJobStatus = async () => {
                            try {
                                let returnComponent = await eSignature.invoke({
                                    slot: "call",
                                    value: baja.$("baja:String").make("LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                                })
                                if (returnComponent.get("jobStatus") === baja.$("baja:JobState").get("running")) {
                                    //schedule the timeout.
                                    setTimeout(function () {
                                        getJobStatus();
                                    }, 1000);
                                } else {
                                    resolve(returnComponent.get("jobStatus") === baja.$("baja:JobState").get("success"));
                                }
                            }
                            catch (e)
                            {
                                console.log(e);
                                resolve(false);
                            }
                        };
                        setTimeout(function () {
                            getJobStatus();
                        }, 1000);
                    }
                }
            }
            catch(err){
                console.log("Error occurred while authenticating the user.", err);
                resolve(false);
            }
        });
    }

    function updateListForAuthorization(result,pointOrdAndLevel,jace){
        var isValid = false;
        var pointAndResult = result.toString().split("%%%");
        pointAndResult.forEach(function(results){
            var pointResult = results.toString().split(";;;");
            if(pointResult[1].includes("true")){
                pointOrdAndLevel.set(pointResult[0],"Secondary%%%true");
                isValid = true;
            }
            else{
                pointOrdAndLevel.set(pointResult[0],"Secondary%%%false%%%Authorization");
            }
        })
        resultList.set(jace,pointOrdAndLevel);
        return isValid;
    }

    function authorizeUser(credentials,eSignature){
        var promises = [], primaryCheck = false;

        resultList.forEach(function(pointOrdAndLevel,jace){
            promises.push(authorizationCheck());

            function authorizationCheck() {
                if (pointOrdAndLevel.values().next().value.toString() === "false%%%Unable to fetch authentication level for point") {
                    return new Promise((resolve) => resolve(false));
                } else {
                    var pointOrd = "";
                    pointOrdAndLevel.forEach(function (result, point) {
                        if (result.includes(SECONDARY) && result.includes("true")) {
                            if (pointOrd === "") {
                                pointOrd = point;
                            } else {
                                pointOrd += "%%%" + point;
                            }
                        }

                        if(result.includes(PRIMARY) && result.includes("true")){
                            primaryCheck = true;
                        }
                    });
                    pointOrd = pointOrd + ";;;" + credentials.primaryUserName + ";;;" + credentials.username + ";;;" + jace;
                    if (!(jace === "Sup")) {
                        return new Promise((resolve) => {
                            baja.rpc({
                                typeSpec: 'electronicSignature:ProtectedConsoleRecipient',
                                method: 'rpcCheckAuthorizationOfPoints',
                                args: [pointOrd]
                            }).then(function (result) {
                                if(result == null || result.toString() === "false"){
                                    resolve(primaryCheck);
                                }
                                else {
                                    resolve(updateListForAuthorization(result, pointOrdAndLevel, jace));
                                }
                            })
                        })
                    } else {
                        return new Promise((resolve) =>{
                            eSignature.invoke({
                                slot: "getAlarmPermissionForJacePoints",
                                value: baja.$("baja:String").make(pointOrd)
                            }).then(function (result)
                            {
                                if(result == null || result.toString() === "false"){
                                    resolve(primaryCheck);
                                }
                                else {
                                    resolve(updateListForAuthorization(result, pointOrdAndLevel, jace));
                                }
                            })
                        })
                    }
                }
            }
        })

        return  Promise.all(promises)
          .then((results) => {
              if(results.includes(true)){
                  return true;
              }
              else
                  return false;
          })
          .catch((e) => {
              console.log("err");
          });
    }

    function showValidationSummary(validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            $("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: lex.get("error.massages.title"),
            content: errorMessage
        });
    }

    function successAlarmRecordsForjaces(){
        let authenticatedAlarmRecords = [];
        resultList.forEach(function(pointResult,jaces){
            pointResult.forEach(function(result,point){
                if(result.toString().includes("true")|| result.toString().includes(NORMAL)){
                    let alarm = alarmPoint.get(jaces.toString()+";;;"+point.toString().substring(0,point.toString().lastIndexOf("-")));
                    authenticatedAlarmRecords = authenticatedAlarmRecords.concat(Array.from(alarm));
                    if(result.toString().includes(NORMAL)){
                        normalAlarmPoints.add(point.toString().substring(0,point.toString().lastIndexOf("-")))
                    }
                }
                else{
                    let msg = "Current LoggedIn station";
                    if(!(jaces.toString() === "Sup")) {
                        msg = jaces.toString().substring(jaces.toString().lastIndexOf("/")+1) ;
                    }

                    msg = msg + " :- " + point.substring(0,point.toString().lastIndexOf("/"))+" :-- "+  errorMessages(result);
                    if(nonAuthenticatedAlarms === "") {
                        nonAuthenticatedAlarms = msg;
                    }
                    else{
                        nonAuthenticatedAlarms += "<br/>" + msg;
                    }
                }
            })
        })
        return authenticatedAlarmRecords;
    }

    function errorMessages (result){
        if(result === "false%%%Unable to fetch authentication level for point"){
            return "Unable to fetch authentication level for point";
        }
        else if(result === "Secondary%%%Secfalse%%%Authentication"){
            return "Secondary authentication failed";
        }
        else if(result === "Primary%%%false%%%Authentication"){
            return "Primary authentication failed";
        }
        else if(result === "Secondary%%%false%%%Authentication"){
            return "Secondary authentication failed";
        }
        else
            return "Authorization Failed";
    }

    function clearAuthenticationRelatedParameters()
    {
        if(resultList !== undefined && alarmPoint !== undefined ) {
            resultList.clear();
            alarmPoint.clear();
            alarmAuthenticationLevel = "";
            nonAuthenticatedAlarms = "";
        }
    }



    /*Public methods*/
    return {
        getFormatedPointOrd: getFormatedPointOrdPrivate,
        getComponentByOrd: getComponentByOrdPrivate,
        authenticateUser: authenticateUserPrivate,
        authorizeUser: authorizeUserPrivate,
        authenticateSecondaryUser: authenticateSecondaryUserPrivate,
        showValidationSummary: showValidationSummaryPrivate,
        doIsDataValid: doIsDataValidPrivate,
        checkPrimarySign: checkPrimarySignPrivate,
        checkSecondarySign: checkSecondarySignPrivate,
        loadInitialEventsAndData: loadInitialEventsAndDataPrivate,
        fetchSecondaryUser: fetchSecondaryUserPrivate,
        getESignatureComponent: getESignatureComponentPrivate,
        getAlarmDetails:getAlarmDetailsPrivate,
        changeLegalText: changeLegalTextPrivate,
        wordWrapAck: wordWrapPrivate,
        updateJaceOrdDetail:updateJaceOrdDetail,
        getResultList:getResultList,
        getPointAuthenticationLevel:getPointAuthenticationLevel,
        getAlarmAuthenticationLevel:getAlarmAuthenticationLevel,
        checkSecondarySignAndUpdateResultList:checkSecondarySignAndUpdateResultList,
        checkPrimarySignAndUpdateResultList:checkPrimarySignAndUpdateResultList,
        successAlarmRecordsForjaces:successAlarmRecordsForjaces,
        clearAuthenticationRelatedParameters:clearAuthenticationRelatedParameters
    };

});
