/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * ValidationStringEditor manages functionality to manage secure point list.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/ValidationStringEditor
 */
'use strict';

define(['baja!',
    'lex!baja',
    'jquery',
    'Promise',
    'lex!electronicSignature',
    'nmodule/electronicSignature/rc/fe/baja/loader',
    'nmodule/webEditors/rc/fe/baja/StringEditor'],
    function (baja,
              lexs,
              $,
              Promise,
              electronicSignatureLex,
              bootstrap,
              StringEditor)
{
    var lex = electronicSignatureLex[0];

    function isSecurePropertyValid(widget, val)
    {
        if(widget.getComplex()!== null && widget.getComplex()!== undefined &&
            widget.getComplex().getType().toString() === "electronicSignature:Customer")
        {
            if(widget.getSlot()!==null && widget.getSlot()!== undefined)
            {
                if(widget.getSlot().getName() === "customerPhoneNumber")
                {
                    if(val !== null && val !== "")
                    {
                        var digitCount = 0;
                        for(var i = 0;i < val.length; i++)
                        {
                            if(!isNaN( parseInt(val.charAt(i),10)))
                            {
                                digitCount++;
                            }
                        }
                        //if(/^\+(?:[0-9]-?){1,3}[0-9]{10}$/.test(val) === false)
                        if(!(digitCount>=10 && digitCount<=18))
                        {
                            throw new Error(lex.get('customer.phone.validation.msg'));
                        }
                    }
                }

                if(widget.getSlot().getName() === "contactPersonPhoneNumber")
                {
                    if(val !== null && val !== "")
                    {
                        var digitCount = 0;
                        for(var i = 0;i < val.length; i++)
                        {
                            if(!isNaN( parseInt(val.charAt(i),10)))
                            {
                                digitCount++;
                            }
                        }
                        //if(/^\+(?:[0-9]-?){1,3}[0-9]{10}$/.test(val) === false)
                        if(!(digitCount>=10 && digitCount<=18))
                        {
                            throw new Error(lex.get('contact.phone.validation.msg'));
                        }
                    }
                }

                if(widget.getSlot().getName() === "address")
                {
                    if(val !== null && val !== undefined && val !== "" && val.length > 200)
                    {
                        throw new Error(lex.get('address.length.validation.msg'));
                    }
                }

                if(widget.getSlot().getName() === "customerName")
                {
                    if(val !== null && val !== undefined && val !== "" && val.length > 100)
                    {
                        throw new Error(lex.get('customerName.length.validation.msg'));
                    }
                    else if(val === null || val === undefined || val === "" || val.length === 0)
                    {
                        throw new Error(lex.get('customerName.blank.validation.msg'));
                    }
                }

                if(widget.getSlot().getName() === "contactPersonName")
                {
                    if(val !== null && val !== undefined && val !== "" && val.length > 100)
                    {
                        throw new Error(lex.get('contactPersonName.length.validation.msg'));
                    }
                    else if(val === null || val === undefined || val === "" || val.length === 0)
                    {
                        throw new Error(lex.get('contactPersonName.blank.validation.msg'));
                    }
                }
            }
        }

        if(widget.getComplex()!== null && widget.getComplex()!== undefined &&
            widget.getComplex().getType().toString() === "electronicSignature:Zone")
        {
            if(widget.getSlot()!==null && widget.getSlot()!== undefined)
            {
                if(widget.getSlot().getName() === "zoneName")
                {
                    if(val !== null && val !== undefined && val !== "" && val.length > 50)
                    {
                        throw new Error(lex.get('mgr.validation.zoneName.max.length'));
                    }
                    else if(val === null || val === undefined || val === "" || val.length === 0)
                    {
                        throw new Error(lex.get('mgr.validation.zoneName.min.length'));
                    }
                }
                if(widget.getSlot().getName() === "zoneLocation")
                {
                    if(val !== null && val !== undefined && val !== "" && val.length > 100)
                    {
                        throw new Error(lex.get('mgr.validation.zoneLocation.max.length'));
                    }
                    else if(val === null || val === undefined || val === "" || val.length === 0)
                    {
                        throw new Error(lex.get('mgr.validation.zoneLocation.min.length'));
                    }
                }
            }
        }
    }

    /**
     * A field editor for working with secured point list.
     *
     * @private
     * @class
     * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
     * @alias module:nmodule/electronicSignature/rc/fe/baja/ValidationStringEditor
     */
    var ValidationStringEditor = function ValidationStringEditor(params)
    {
        StringEditor.call(this, $.extend({
            keyName: 'ValidationStringEditor'
        }, params));

        this.validators().add(function (val) {
            isSecurePropertyValid(this, val);
        });
    };
    ValidationStringEditor.prototype = Object.create(StringEditor.prototype);
    ValidationStringEditor.prototype.constructor = ValidationStringEditor;



    return ValidationStringEditor;

});