/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

define(['baja!',
        'lex!webEditors',
        'dialogs',
        'lex!electronicSignature',
        'jquery',
        'Promise',
        'bajaux/events',
        'bajaux/Widget',
        'bajaux/commands/Command',
        'bajaux/util/CommandButton',
        'nmodule/webEditors/rc/fe/fe',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
/*    'nmodule/electronicSignature/rc/wb/mgr/commands/ZoneEditCommand',
    'nmodule/electronicSignature/rc/wb/mgr/commands/ZoneAddCommand',*/
        'nmodule/webEditors/rc/fe/baja/util/typeUtils'], function (
         baja,
         lexs,
         dialogs,
         electronicSignatureLex,
         $,
         Promise,
         events,
         Widget,
         Command,
         CommandButton,
         fe,
         BaseEditor,
/*         ZoneEditCommand,
         ZoneAddCommand,*/
         typeUtils) {

  'use strict';
  var lex = electronicSignatureLex[0];
console.log("Inside manager dialog");
  var LOAD_EVENT = events.LOAD_EVENT,
      MODIFY_EVENT = events.MODIFY_EVENT,

      DEFAULT_DELAY = 200,
      ENTER_KEY = 13,

      VALUE_READY_EVENT = BaseEditor.VALUE_READY_EVENT;

  /**
   * Functions for showing field editors in modal dialogs. Useful for prompting
   * the user to enter values, edit individual slots, and fire actions.
   *
   * @exports nmodule/webEditors/rc/fe/feDialogs
   */
  var feDialogs = {};


  /**
   * Build the editor in the given dialog.
   *
   * As the editor is created, initialized and loaded, progress events with
   * those same names will be passed to the given progress handler. This way,
   * someone using `feDialogs.showFor` can get callbacks for the actual editor
   * instance as it is created, and add event handlers on it, for instance.
   *
   * @inner
   * @param {Object} params fe params
   * @param {jQuery} contentDiv
   * @param {Function} progress
   * @returns {*}
   */
  function buildEditor(params, contentDiv, progress) {
    var parent = contentDiv.parent();

    contentDiv.detach();

    return fe.params(params)
      .then(function (feParams) {
        var Ctor = feParams.getWidgetConstructor(),
            ed = new Ctor(feParams);

        progress('created', ed);

        return ed.initialize(contentDiv)
          .then(function () {
            progress('initialized', ed);
            return ed.load(feParams.getValueToLoad());
          })
          .then(function () {
            progress('loaded', ed);
            contentDiv.prependTo(parent);
            return ed;
          });
      });
  }

  function saveAndDestroy(ed) {
    return ed.save()
      .catch(function (err) {
     /*  feDialogs.error(err);
        throw err; //failed to validate - keep dialog open
    */   var msg = err instanceof Error ? err.message : String(err);
          if(!msg.includes(lex.get('slot.already.used')))
          {
           feDialogs.error(err);
           throw err; //failed to validate - keep dialog open
          }
   })
    .then(function () {
        return ed.read();
      })
      .then(function (value) {
        return ed.destroy()
          .then(function () {
            return value;
          });
      });
  }

  /**
   * Shows a field editor in a dialog.
   *
   * When the user clicks OK, the editor will be saved, committing any changes.
   * The value that the user entered will be read from the editor and used to
   * resolve the promise.
   *
   * @param {Object} params params to be passed to `fe.buildFor()`. Any `dom`
   * parameter will be ignored and replaced with the content div of the dialog
   * shown.
   * @param {String} [params.title] title for the dialog
   * @param {Number} [params.delay=200] delay in ms to wait before showing a
   * loading spinner. The spinner will disappear when the field editor has
   * finished initializing and loading.
   * @param {Function} [params.progressCallback] pass a progress callback to
   * receive notifications as the editor being shown goes through the stages
   * of its life cycle (`created`, `initialized`, `loaded`), as well as whenever
   * the editor is validated (`invalid`, `valid`).
   * @returns {Promise} promise to be resolved when the user has entered
   * a value into the field editor and clicked OK, or rejected if the field
   * could not be read. The promise will be resolved with the value that the
   * user entered (or `null` if Cancel was clicked).
   *
   * @example
   *   feDialogs.showFor({
   *     value: 'enter a string here (max 50 chars)',
   *     facets: { max: 50 },
   *     progressCallback: function (msg, arg) {
   *       switch(msg) {
   *       case 'created':     return console.log('editor created', arg);
   *       case 'initialized': return console.log('editor initialized', arg.jq());
   *       case 'loaded':      return console.log('editor loaded', arg.value());
   *       case 'invalid':     return console.log('validation error', arg);
   *       case 'valid':       return console.log('value is ok');
   *       }
   *     }
   *   })
   *   .then(function (str) {
   *     if (str === null) {
   *       console.log('you clicked cancel');
   *     } else {
   *       console.log('you entered: ' + str);
   *     }
   *   });
   */
  feDialogs.showFor = function showFor(params) {
    params = baja.objectify(params, 'value');

    var progress = params.progressCallback || $.noop;

    return new Promise(function (resolve, reject) {
      dialogs.showOkCancel({
        delay: params.delay || DEFAULT_DELAY,
        title: params.title,
        content: function (dlg, content) {
          var contentDiv = $('<div/>').appendTo(content);

          contentDiv.on(LOAD_EVENT + ' ' + MODIFY_EVENT, function (e, ed) {
            ed.validate()
              .then(function () {
                dlg.buttonJq('ok').attr('title', '');
                dlg.enableButton("ok");
                progress('valid');
              }, function (err) {
                dlg.buttonJq('ok').attr('title', String(err));
                dlg.disableButton("ok");
                progress('invalid', err);
              });
          });

          contentDiv.on('keyup', function (e) {
            if (e.which === ENTER_KEY) {
              contentDiv.data('widget').validate().then(function () {
                dlg.click('ok');
              });
            }
          });

          return buildEditor(params, contentDiv, progress)
            .then(function (ed) {
              contentDiv.on(VALUE_READY_EVENT, function (e, value) {
                ed.destroy().then(function () {
                  dlg.close();
                  resolve(value);
                });
              });

              dlg
                .ok(function () {
                   /* if(typeof  ZoneEditCommand !== 'undefined' && ZoneEditCommand.zoneEditor!==null){
                        ZoneEditCommand.zoneEditor = null;
                    }
                    if(typeof  ZoneAddCommand !== 'undefined' && ZoneAddCommand.zoneEditor!==null){
                        ZoneAddCommand.zoneEditor = null;
                    }*/
             /*   var invalidList= params.command(ed);
                if(invalidList.length>0)
                {
             dialogs.showOk({
                         title: "Error Messages",
                         content:invalidList
                       });
                return false;
                }*/
                if(params.command !== undefined)
                {
               return params.command(ed).then(function(invalidList){
                  if(invalidList.length>0)
                        {
                     dialogs.showOk({
                                 title: lex.get('error.massages.title'),
                                 content:invalidList
                               });
                       throw  lex.get('custom.error.message');
                       // return false;
                        }
                }).then(function(){
                return saveAndDestroy(ed).then(resolve);
                });
                }else
                {
                 return saveAndDestroy(ed).then(resolve);
                }
                })
                .cancel(function () {
                 /*   if(typeof  ZoneEditCommand !== 'undefined' && ZoneEditCommand.zoneEditor!==null){
                        ZoneEditCommand.zoneEditor = null;
                    }
                    if(typeof  ZoneAddCommand !== 'undefined' && ZoneAddCommand.zoneEditor!==null){
                        ZoneAddCommand.zoneEditor = null;
                    }*/
                  return ed.destroy()
                    .then(function () {
                      resolve(null);
                    });
                });

              //TODO: dialogs.js needs callback to let me know when dialog is shown.
              //for now, run it async so it runs after the promise resolves,
              //when the dialog DOM is visible and focus requests are valid.
              baja.runAsync(function () {
                return ed.requestFocus && ed.requestFocus();
              });
            })
            .catch(reject);
        }
      });
    });
  };


  /**
   * Show details about an error.
   *
   * @param {Error|*} err
   * @returns {Promise}
   */
  feDialogs.error = function (err) {
     return  dialogs.showOk({
       title: lex.get('error.massages.title'),
       content:err
     });
    };

  return feDialogs;
});
