/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * API Status: **Private**
 * @module nmodule/history/rc/DatabaseMaintenance
 */
define([
    'baja!',
    'baja!history:History,history:HistorySpace',
    'lex!history,electronicSignature',
    'jquery',
    'underscore',
    'Promise',
    'dialogs',
    'bajaux/events',
    'nmodule/webEditors/rc/fe/fe',
    'nmodule/webEditors/rc/fe/baja/BaseEditor',
    'nmodule/webEditors/rc/wb/tree/NavTree',
    'nmodule/webEditors/rc/wb/tree/TreeNode',
    'nmodule/webEditors/rc/wb/tree/BajaNavTreeNode',
    'nmodule/webEditors/rc/util/htmlUtils',
    'bajaux/commands/Command',
    'bajaux/util/CommandButton',
    'nmodule/history/rc/maintenance',
   // 'hbs!nmodule/history/rc/templates/view',
   'hbs!nmodule/electronicSignature/rc/wb/template/view2',
    'hbs!nmodule/history/rc/templates/dialog',
    'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
    'nmodule/electronicSignature/rc/fe/baja/Util',
    'nmodule/electronicSignature/rc/jsFiles/bootstrap.min',
    'nmodule/electronicSignature/rc/wb/mgr/ProtectedMaintenance',
    'css!nmodule/electronicSignature/rc/css/widgets',
    'css!nmodule/history/rc/history'


], function (baja, types, lexs, $, _, Promise, dialogs, events, fe, BaseEditor,
             NavTree, TreeNode, BajaNavTreeNode, htmlUtils,
             Command, CommandButton,
             maintenance,

             viewTemplate, dialogTemplate,authenticationTemplate,Util,bootsrtap,ProtectedMaintenance) {

    'use strict';

    var lex = lexs[0],
        electronicSignatureLex = lexs[1],
        escapeHtml = htmlUtils.escapeHtml,
        DATABASE_MAINTENANCE_CLASS = 'dbMaintenance',
        HISTORY_NAV_TREE_CLASS = 'historyTree',
        HISTORY_NAV_TREE_SELECTOR = '.' + HISTORY_NAV_TREE_CLASS,
        HISTORY_LIST_SELECTOR = '.historyList > ul',
        DATE_EDITOR_CLASS = 'dateEditor',
        DATE_EDITOR_SELECTOR = '.' + DATE_EDITOR_CLASS,
        RUN_COMMAND_BUTTON_CLASS = 'run',
        RUN_COMMAND_BUTTON_SELECTOR = '.' + RUN_COMMAND_BUTTON_CLASS,

        SELECT_ALL_HISTORIES_MSG = lex.get('history.db.selectAllDisplayName'),
        SELECT_ALL_HISTORIES_ICON = lex.get('history.db.selectAllIcon'),
        DESELECT_ALL_HISTORIES_MSG = lex.get('history.db.deselectAllDisplayName'),
        DESELECT_ALL_HISTORIES_ICON = lex.get('history.db.deselectAllIcon'),

        CLEAR_OLD_HISTORIES_LABEL = lex.get('history.db.clearOld'),
        CLEAR_ALL_HISTORIES_LABEL = lex.get('history.db.clearAll'),
        DELETE_ALL_HISTORIES_LABEL = lex.get('history.db.deleteAll'),

        HISTORY_SELECTED_CLASS = 'history-selected',
        HISTORY_SELECTED_SELECTOR = '.' + HISTORY_SELECTED_CLASS,
        DELETE_ICON = baja.Icon.make(lex.getSafe('history.db.deleteIcon')).getImageUris()[0],
        DELETE_ENTRY_COMMAND_CLASS = 'deleteEntryCmd',

        HISTORY_REGEX = /history:\/[^\/]+\/.+/,
        HISTORY_DEVICE_REGEX = /history:\/[^\/].+/,
        GROUP_REGEX = /.*\/\/.*\/.*/,
        GROUP_HISTORY_REGEX = /.*\/\/.*\/\/.*/,
        DEFAULT_REGEX = /.*\/\/\//,
        WARNING_ICON = lex.getSafe('history.db.warningIcon'),

        CLEAR_ALL_OP_COMPLETE_TITLE = lex.get('history.db.dialog.clearAll.title'),
        CLEAR_OLD_COMPLETE_TITLE = lex.get('history.db.dialog.clearOld.title'),
        DELETE_ALL_COMPLETE_TITLE = lex.get('history.db.dialog.delete.title'),
        INVALID_TIME_TITLE = lex.get('history.db.dialog.invalidTime.title'),
        SELECT_HISTORY_MSG = lex.getSafe('history.db.selectMessage'),
        CONFIRM_CLEAR_MSG = lex.getSafe('history.db.dialog.confirmClear'),
        CONFIRM_DELETE_MSG = lex.getSafe('history.db.dialog.confirmDelete'),
        COMMAND_SUCCESS_MSG = lex.getSafe('history.db.successMessage'),
        COMMAND_ERROR_MSG = lex.getSafe('history.db.errorMessage'),
        OPERATIONS = {
            clearOld: 'clearOld',
            clearAll: 'clearAll',
            deleteAll: 'deleteAll'
        },
        secureDashboardConfiguration=null,
        isAuthenticatedUser = false,
        _SecondaryAuthenticatorRoles = "",
        isValidUserSecondary = false,
        selectedHistoryOrds = null,
        legalTextTemplate ="",
        customerName ="";

    /**
     * DatabaseMaintenance allows for running maintenance operations on
     * histories (clearing old histories, deleting histories etc.)
     *
     * @class
     * @extends module:nmodule/webEditors/rc/fe/baja/BaseEditor
     * @alias module:nmodule/history/rc/DatabaseMaintenance
     */
    var ProtectedDatabaseMaintenance = function ProtectedDatabaseMaintenance() {
        BaseEditor.call(this, {moduleName: 'electronicSignature', keyName: 'ProtectedDatabaseMaintance'});
    };

    //extend and set up prototype chain
    ProtectedDatabaseMaintenance.prototype = Object.create(BaseEditor.prototype);
    ProtectedDatabaseMaintenance.prototype.constructor = ProtectedDatabaseMaintenance;


    function initCommandGroup(view) {
        var list = view.$getList();

        function selectAllHistories() {
            return view.$getHistories().then(function (histories) {
                appendNodesToList(histories, list);
            }).catch(function(err){
                dialogs.showOk({
                    content: COMMAND_ERROR_MSG + "<br/>(" + escapeHtml(err) + ")"
                });
            });
        }

        function deselectAllHistories() {
            list.empty();
            view.jq().find(HISTORY_SELECTED_SELECTOR).removeClass(HISTORY_SELECTED_CLASS);
        }

        var cmdGroup = view.getCommandGroup(),
            selectAllCmd = new Command({
                displayName: SELECT_ALL_HISTORIES_MSG,
                description: SELECT_ALL_HISTORIES_MSG,
                icon: SELECT_ALL_HISTORIES_ICON,
                func: selectAllHistories
            }),

            deselectallCmd = new Command({
                displayName: DESELECT_ALL_HISTORIES_MSG,
                description: DESELECT_ALL_HISTORIES_MSG,
                icon: DESELECT_ALL_HISTORIES_ICON,
                func: deselectAllHistories
            });

        cmdGroup.add(selectAllCmd);
        cmdGroup.add(deselectallCmd);
    }

    function initNavTreeEditor(view) {
        return baja.Ord.make('history:').get().then(function (space) {
            return fe.buildFor({
                dom: $(HISTORY_NAV_TREE_SELECTOR, view.jq()),
                value: new BajaNavTreeNode(space),
                type: NavTree
            }).then(function (tree) {

                function getParent(tree) {
                    var parent = tree.jq().parent().parent().data('widget');
                    return parent instanceof NavTree && parent || getParent(parent);
                }

                tree.expand();

                //register event handler for double-clicking a tree node
                tree.jq().on(NavTree.ACTIVATED_EVENT, null, view, nodeDblClicked);

                //register batch select handler
                var anchor, //stores the anchor node in a shift + click operation
                    node;//stores the currently selected node,

                tree.jq().on(NavTree.SELECTED_EVENT, function (e, root, modified) { //single click
                    node = root; //cache the clicked node for use in 'mouseup'
                }).on('mouseup', function (e) { //batch selection using shift + click
                    if (e.shiftKey && anchor && node && anchor !== node) {
                        var anchorParent = getParent(anchor),
                            nodeParent = getParent(node);

                        //only permit batch selection for nodes with a common parent
                        if (anchorParent && anchorParent === nodeParent) {
                            var kids = anchorParent.$getKids(),
                                anchorIndex, nodeIndex, i,
                                startIndex, endIndex;

                            for (i = 0; i < kids.length; i++) {

                                //locate the anchor index
                                if (kids[i] === anchor) {
                                    anchorIndex = i;
                                }

                                //locate the node index
                                else if (kids[i] === node) {
                                    nodeIndex = i;
                                }

                                //break if both indices have been found
                                if (anchorIndex && nodeIndex) {
                                    break;
                                }
                            }

                            startIndex = anchorIndex < nodeIndex ? anchorIndex : nodeIndex;
                            endIndex = anchorIndex > nodeIndex ? anchorIndex : nodeIndex;
                            _.each(kids.slice(startIndex, endIndex + 1), function (kid) {
                                kid.setSelected(true, true);
                            });
                        }
                    }

                    else {
                        anchor = node;
                    }
                });

                //arm drag/drop handler
                var list = view.$getList();
                list.on('dragover', function () {
                    return false;
                }).on('drop', function (e) {
                    getNodes(tree, NavTree.prototype.isSelected).then(function (nodes) {
                        appendNodesToList(nodes, list);
                    });
                    return false;
                });
            });
        });
    }

    function nodeDblClicked(event, node) {
        var view = event.data;
        appendNodesToList([node], view.$getList());
    }

    //node must be an instance of NavTree
    function isValid(node, list) {
        //a node is valid if it or one of it's ancestor's isn't already selected and
        //is either a history, history device, group or group history
        return !(isDefault(node) || ancestorSelected(node) || inList(node, list)) &&
            (isHistory(node) || isHistoryDevice(node) || isGroup(node) || isGroupHistory(node));
    }

    function getSelectedOrds(list){
        return list.children().map(function () {
            return $(this).data('ord');
        }).get();
    }

    function inList(node, list){
        var srcOrd = node.value().value().getNavOrd().relativizeToSession().toString().substring('history:'.length),
            targetOrds = getSelectedOrds(list),
            lastDblSlash = srcOrd.lastIndexOf('//');

        //get portion of ord after last double slash (for history groups)
        if(lastDblSlash !== -1){
            srcOrd = srcOrd.substring(lastDblSlash + 1);
        }

        return _.some(_.map(targetOrds, function(ord){
            return ord.toString().substring('history:'.length);
        }), function(targetOrd){
            lastDblSlash = targetOrd.lastIndexOf('//');
            if(lastDblSlash !== -1){
                targetOrd = targetOrd.substring(lastDblSlash + 1);
            }
            return srcOrd === targetOrd;
        });
    }

    function ancestorSelected(node){
        return node.jq().closest(HISTORY_SELECTED_SELECTOR).length;
    }

    function isDefault(node){
        var ord = node.value().value().getNavOrd().toString();
        return DEFAULT_REGEX.test(ord);
    }

    function isHistory(node) {
        var ord = node.value().value().getNavOrd().toString();
        return HISTORY_REGEX.test(ord);
    }

    function isHistoryDevice(node) {
        var ord = node.value().value().getNavOrd().toString();
        return HISTORY_DEVICE_REGEX.test(ord);
    }

    function isGroup(node){
        var ord = node.value().value().getNavOrd().toString();
        return GROUP_REGEX.test(ord);
    }

    function isGroupHistory(node) {
        var ord = node.value().value().getNavOrd().toString();
        return GROUP_HISTORY_REGEX.test(ord);
    }

    function getHistories(root) {
        return getNodes(root, isHistory);
    }

    function getHistoryGroups(root) {
        return getNodes(root, isGroupHistory);
    }

    function getHistoryDevices(root) {
        return getNodes(root, isHistoryDevice);
    }

    //DFS search of nodes from the specified root satisfying the filter
    function getNodes(root, filter) {
        if (filter.call(root, root)) {
            return Promise.resolve([root]);
        }

        return root.$setLoaded(true).then(function () {
            var kids = root.$getKids(),
                kidLoadingPromises = _.map(kids, function (kid) {
                    return kid.$setLoaded(true);
                });
            return Promise.all(kidLoadingPromises).then(function () {
                var kidRetrievalPromises = _.map(kids, function (kid) {
                    return getNodes(kid, filter);
                });
                return Promise.all(kidRetrievalPromises);
            });
        }).then(_.flatten);
    }

    //nodes must be NavTrees
    function appendNodesToList(nodes, list) {
        var parent = list.parent();
        list.detach();
        _.each(nodes, function (node) {
            if (isValid(node, list)) {

                var treeNode = node.value(),
                    navNode = treeNode.value(),
                    navOrd = navNode.getNavOrd().relativizeToSession(),
                    text = baja.SlotPath.unescape(navOrd.toString().substring('history:'.length)),
                    listItem;


                return secureDashboardConfiguration.invoke({
                    slot:"getHistorySpacePermission",
                    value:baja.$("baja:String").make(navOrd.toString())
                }).then(function(hasAdminReadWritePermission){
                    if(hasAdminReadWritePermission){
                        node.jq().addClass(HISTORY_SELECTED_CLASS);

                        //append list item
                        listItem = $("<li><div class='historyId'>" + text + "</div></li>")
                            .data('ord', navOrd)
                            .appendTo(list);

                        //append delete img
                        $('<img>').attr('src', DELETE_ICON)
                            .addClass(DELETE_ENTRY_COMMAND_CLASS)
                            .prependTo(listItem)
                            .click(function () {
                                //unmark the selected node
                                node.jq().removeClass(HISTORY_SELECTED_CLASS);
                                listItem.remove();
                            });
                    }
                    else{
                        return dialogs.showOk({
                            title: electronicSignatureLex.get('error.title'),
                            content: electronicSignatureLex.get("user.permission.missing")
                        });
                    }
                });
            }
        });

        parent.append(list);
    }


    function initDateEditor(view) {
        var today = new Date(),
            year  = today.getFullYear() - 5,
            month = today.getMonth(),
            day   = today.getDate();

        return fe.buildFor({
            dom: $(DATE_EDITOR_SELECTOR, view.jq()),
            value: baja.AbsTime.make({jsDate: new Date(year, month, day)}), //NCCB-14801
            formFactor: 'mini'
        }).then(function (editor) {
            editor.jq().on(events.MODIFY_EVENT, function () {
                return false;
            });
        });
    }

    function initRunCommand(view) {
        return fe.buildFor({
            dom: $(RUN_COMMAND_BUTTON_SELECTOR, view.jq()),
            type: CommandButton,
            value: new Command({
                displayName: lex.get('history.db.run.displayName'),
                description: lex.get('history.db.run.description'),
                func: function () {
                    return runMaintenance(view);
                }
            }),
            formFactor: 'mini'
        });
    }

    function runMaintenance(view) {

        function makeDialogContent(date) {
            var obj = {
                    title: CONFIRM_CLEAR_MSG,
                    completionTitle: CLEAR_OLD_COMPLETE_TITLE
                },
                warningIcon = baja.Icon.make(WARNING_ICON).getImageUris()[0],
                op = view.$getSelectedOperation(),
                content = {icon: warningIcon},
                ords = view.$getSelectedOrds(),
                args = [ords];
                args.push(secureDashboardConfiguration);

            if (!ords.length) {
                obj.title = electronicSignatureLex.get('no.record.selected');
                obj.content = SELECT_HISTORY_MSG;
            }

            else {
                switch (op) {
                    case OPERATIONS.clearOld:
                        obj.completionTitle = CLEAR_OLD_COMPLETE_TITLE;
                        //content.time = date.getJsDate();
                        content.operation = lex.getSafe({ key: "confirmClear.operation.clearOld", args: [ date.getJsDate() ] });
                        args.push(date);
                        break;
                    case OPERATIONS.clearAll:
                        obj.completionTitle = CLEAR_ALL_OP_COMPLETE_TITLE;
                        content.operation = lex.getSafe("confirmClear.operation.clearAll");
                        break;
                    case OPERATIONS.deleteAll:
                        obj.title = CONFIRM_DELETE_MSG;
                        obj.completionTitle = DELETE_ALL_COMPLETE_TITLE;
                        content.delete = 'delete';
                        content.operation = lex.getSafe("confirmClear.operation.delete");
                        break;
                }
                content.confirm = lex.getSafe("confirmClear.confirm");
                content.question = lex.getSafe("confirmClear.question");

                obj.content = dialogTemplate(content);
                obj.func = function () {
                    return ProtectedMaintenance[op].apply(null, args).then(function () {
                        return Promise.resolve(op === OPERATIONS.deleteAll &&
                            view.$getDeselectAllCommand().invoke());
                    }).then(function () {
                        dialogs.showOk({
                            title: obj.completionTitle,
                            content: COMMAND_SUCCESS_MSG
                        });
                    }).catch(function (err) {
                        dialogs.showOk({
                            title: obj.completionTitle,
                            content: COMMAND_ERROR_MSG + "<br/>(" + escapeHtml(err) + ")"
                        });
                    });
                };
            }
            return obj;
        }

        return view.$getDateEditor().read().then(function (date) {
            var obj = makeDialogContent(date);
            if (!obj.func) {
                dialogs.showOk({
                    title: obj.title,
                    content: obj.content
                });
            }
            else {
                loadComponentSlots(secureDashboardConfiguration).then(function(secureDashboardConfigurationLeased){
                    var primaryUsername = baja.getUserName();
                    fetchSecondaryUser(secureDashboardConfigurationLeased,primaryUsername,"call").then(function(secondaryUsers){
                        var selectedHistoryOrdData = obj.args;
                        if (secondaryUsers.get("secUser") && secondaryUsers.get("secUser").toString() !== "")
                        {
                            var uniqueSecondaryUser = _.uniq(secondaryUsers.get("secUser").toString().split(","));
                            var jsonObject={
                                authLevel1:electronicSignatureLex.get('fe.authorization.level1.label'),
                                authLevel2:electronicSignatureLex.get('fe.authorization.level2.label'),
                                userName:electronicSignatureLex.get('fe.userName.label'),
                                password:electronicSignatureLex.get('fe.password.label'),
                                comment:electronicSignatureLex.get('fe.comment.label'),
                                sign:electronicSignatureLex.get('fe.sign1.button'),
                                sign2:electronicSignatureLex.get('fe.sign2.button'),
                                acknowledgement:electronicSignatureLex.get('eSignature.ack.label'),
                                label1:electronicSignatureLex.get('eSignature.ack.label1'),
                                label2:electronicSignatureLex.get('eSignature.ack.label2'),
                                label3:electronicSignatureLex.get('eSignature.ack.label3'),
                                label4:electronicSignatureLex.get('eSignature.ack.label4'),
                                label5:electronicSignatureLex.get('eSignature.ack.label5'),
                                label6:electronicSignatureLex.get('eSignature.ack.label6'),
                                label7:electronicSignatureLex.get('eSignature.ack.label7')
                            };
                            dialogs.showOkCancel({title:electronicSignatureLex.get("authenticate.user.title"),content:authenticationTemplate({jsonObject:jsonObject})}).ok(function(val){
                                var dlg = dialogs.showYesNo({
                                    title: obj.title,
                                    content: obj.content,
                                    yes: function(){
                                        dlg.close();
                                        dialogs.showLoading(0, obj.func());
                                    }
                                });
                            });

                            var secondaryUsersList = uniqueSecondaryUser;
                            _SecondaryAuthenticatorRoles = secondaryUsers.get("secAuthRole").toString();
                            var usersHtml = "";
                            _.each(secondaryUsersList, function (value, index)
                            {
                                if (index + 1 !== secondaryUsersList.length)
                                {
                                    usersHtml += "<option value=" + value + ">" + value + "</option>";
                                }
                            });
                            $(document).find("#secondaryUsername").html(usersHtml);
                            $(document).find("#secondaryUsername").change(function(){
                                Util.changeLegalText($(document).find("#secondaryUsername").val(),secureDashboardConfigurationLeased);
                            });
                            $(document).find("#username").val(primaryUsername);
                            $(document).find('.js-dialog-button-ok').attr('disabled', true);

                            $(document).find("#secondaryUsername").change(function(){
                                $(document).find('.js-dialog-button-ok').attr('disabled', true);
                            });

                            customerName = secureDashboardConfigurationLeased.getCustomer().getCustomerName();
                            Util.changeLegalText(primaryUsername,secureDashboardConfigurationLeased);

                            $(document).find("#btnPrimarySign").click(function (){
                                var credentials = {};
                                credentials.username = $('#username').val();
                                credentials.password = $('#password').val();
                                checkPrimarySign(credentials, secureDashboardConfigurationLeased);
                            });

                            $(document).find(".primaryTab").click(function (e){
                                e.preventDefault();
                            });

                            $(document).find(".secondaryTab").click(function (e){
                                e.preventDefault();
                            });
                            $(document).find(".btnSecondarySign").click(function (){
                                var list = $(".historyList > ul");
                                var targetOrds = getSelectedOrds(list).toString();
                                var historyDetailsComp = baja.$("baja:Component");
                                historyDetailsComp.add({
                                    slot:"userName",
                                    value:$('#secondaryUsername').val()
                                });

                                historyDetailsComp.add({
                                    slot:"targetOrds",
                                    value:targetOrds
                                });

                                secureDashboardConfigurationLeased.invoke({
                                    slot: "getSecondaryHistorySpacePermission",
                                    value: historyDetailsComp
                                }).then(function(responce){
                                    var jsonConvertedResponce = JSON.parse(responce);
                                    if(_.size(jsonConvertedResponce)>0)
                                    {
                                        var errorMessageHtml="<b>"+electronicSignatureLex.get('secondary.permission.missing')+"</b></br></br>";
                                        _.each(jsonConvertedResponce,function(value,index){
                                            errorMessageHtml += "<b>" + (index+1) + "</b>" + ". " + value.name  + " -  " + value.ord + "</br>";
                                        });
                                        return dialogs.showOk({
                                            title: electronicSignatureLex.get('error.title'),
                                            content: errorMessageHtml
                                        });
                                    }
                                    else{
                                        var credentials = {};
                                        credentials.username = $('#secondaryUsername').val();
                                        credentials.primaryUserName = $('#username').val();
                                        credentials.password = $('#secondaryPassword').val();
                                        checkSecondarySign(credentials, secureDashboardConfigurationLeased);
                                    }
                                });
                            });
                        }
                        else
                        {
                            return dialogs.showOk({
                                title: electronicSignatureLex.get('error.title'),
                                content: electronicSignatureLex.get('secondary.users.missing')
                            });
                        }

                    });
                });
            }
        }).catch(function(err){
            dialogs.showOk({
                title: INVALID_TIME_TITLE,
                content: err
            });
        });
    }

    function authenticateSecondaryUser(component, userName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUserDetails," + userName.toString())
        });
    }

    function checkSecondarySign(credentials, pointComponent)
    {
        $('.js-dialog-button-ok').attr('disabled', true);
        isAuthenticatedUser = false;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title: electronicSignatureLex.get('authentication.error.title'),
                content: electronicSignatureLex.get('level2.authentication.message')
            });
        }
        var validation = doIsDataValid("secondary");
        if (validation !== null && validation.length > 0)
        {
            showValidationSummary(validation);
        }
        else
        {
            var pointOrd = getFormatedPointOrd(pointComponent),
                pointComp,
                isValidManager = false;
            if (pointOrd !== "")
            {
                getComponentByOrd(pointOrd).then(function (comp)
                {
                    pointComp = comp;
                    $(".btnSecondarySign").html(electronicSignatureLex.get('fe.signing.button'));
                    $(".btnSecondarySign").prop("disabled", true);
                    return authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp){
                    $(".btnSecondarySign").html(electronicSignatureLex.get('fe.sign.button'));
                    $(".btnSecondarySign").prop("disabled", false);
                    isValidManager = authenticatedComp;
                    if (authenticatedComp)
                    {
                        $('.js-dialog-button-ok').attr('disabled', false);
                        //isAuthenticatedUser = true;
                        return dialogs.showOk({
                            title: electronicSignatureLex.get('auth.success.title'),
                            content: electronicSignatureLex.get('auth.success.msg')
                        });
                        //return authenticateSecondaryUser(pointComp, credentials.username);
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: electronicSignatureLex.get("authentication.error.title"),
                            content: electronicSignatureLex.get("authentication.error.message")
                        });
                    }
                }).catch(function (error)
                    {
                        $(".btnSecondarySign").html(electronicSignatureLex.get('fe.sign.button'));
                        $(".btnSecondarySign").prop("disabled", false);
                        console.log(error); // jshint ignore:line
                    });
            }
            else
            {
                return dialogs.showOk({
                    title: electronicSignatureLex.get("authentication.error.title"),
                    content: electronicSignatureLex.get("authentication.error.message")
                });
            }
        }
    }

    function showValidationSummary(validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            $("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: electronicSignatureLex.get("error.title"),
            content: errorMessage
        });
    }

    function getFormatedPointOrd(component)
    {
        var pointOrd = "";

        pointOrd = "station:|h:" + component.getHandle().toString();

        return pointOrd;
    }

    function getComponentByOrd(ord)
    {
        return baja.Ord.make(ord).get({lease: true});
    }

    function authenticateUser(component, credentials)
    {
        return new Promise(function (resolve, reject) {
            component.invoke({
                slot: "call",
                value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
            }).then(function (authenticatedComp)
            {
                if(authenticatedComp.getSlot("authenticationJobOrd")===null)
                {
                    resolve(authenticatedComp.get("isValidUser"));
                }
                else
                {
                    let getJobStatus = ()=>{
                        component.invoke({
                            slot: "call",
                            value: baja.$("baja:String").make("LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                        }).then(function(returnComponent){
                            if(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("running"))
                            {
                                //schedule the timeout.
                                setTimeout(function (){
                                    getJobStatus();
                                },1000);
                            }
                            else
                            {
                                resolve(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("success"));
                            }
                        });
                    };
                    setTimeout(function (){
                        getJobStatus();
                    },1000);
                }
            }).catch(function(err){
                console.log("Error occurred while authenticating the user.", err);
                reject();
            });
        });
        /*return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
        });*/
    }


    function checkPrimarySign(credentials, pointComponent)
    {
        isAuthenticatedUser = false;
        var that = this,
            validation = doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            showValidationSummary(validation);
        }
        else
        {
            var pointOrd = getFormatedPointOrd(pointComponent);
            if (pointOrd !== "")
            {
                getComponentByOrd(pointOrd).then(function (comp)
                {
                    $("#btnPrimarySign").html(electronicSignatureLex.get('fe.signing.button'));
                    $("#btnPrimarySign").prop("disabled", true);
                    return authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp)
                {
                    $("#btnPrimarySign").html(electronicSignatureLex.get('fe.sign.button'));
                    $("#btnPrimarySign").prop("disabled", false);
                    if (authenticatedComp)
                    {
                        isAuthenticatedUser = true;
                        $('.js-dialog-button-ok').attr('disabled', true);
                        $('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
                        Util.changeLegalText($("#secondaryUsername").val(),pointComponent);
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: electronicSignatureLex.get("authentication.error.title"),
                            content: electronicSignatureLex.get("authentication.error.message")
                        });
                    }
                    Util.changeLegalText($(document).find("#secondaryUsername").val(),pointComponent);
                }).catch(function (error)
                {
                    $("#btnPrimarySign").html(electronicSignatureLex.get('fe.sign.button'));
                    $("#btnPrimarySign").prop("disabled", false);
                    console.log(error); // jshint ignore:line
                });
            }
        }
    }

    function doIsDataValid(authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        if (authenticationType === "primary")
        {
            if ($('#username').val() === null || $('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#username').removeClass("errorMsg");
            }
            if ($('#password').val() === null || $('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#password').removeClass("errorMsg");
            }
            return invalidList;
        }
        if (authenticationType === "secondary")
        {

            if ($('#secondaryUsername').val() === null || $('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryUsername').removeClass("errorMsg");
            }
            if ($('#secondaryPassword').val() === null || $('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryPassword').removeClass("errorMsg");
            }
            return invalidList;
        }
    }

    function resize(view) {
        var containerHeight = $('.bajaux-container').outerHeight(true),
            maintenanceDivHeight = $('.maintenance2', view.jq()).outerHeight(true),
            toolbarHeight = $('.bajaux-toolbar-outer').outerHeight(true),
            historyChooserDivHeight = containerHeight - maintenanceDivHeight - toolbarHeight - 30;
        $('.historyChooser', view.jq()).outerHeight(historyChooserDivHeight);
    }

    function loadComponentSlots(comp){
        return new Promise(function(fullfil,reject){
            comp.lease().then(function(){
                comp.loadSlots({
                    ok:function(){
                        fullfil(comp);
                    },
                    fail:function(){
                        reject(comp);
                    }
                });
            });
        });
    }

    /**
     * Initializes the DOM with the HistorySpace tree, history selection list and options
     * for running maintenance
     * @param dom
     * @returns {Promise} Promise resolved once initializtion is complete
     */
    ProtectedDatabaseMaintenance.prototype.doInitialize = function (dom) {
        var that = this;

        dom.addClass(DATABASE_MAINTENANCE_CLASS)
            .html(viewTemplate({
                clearOldId: this.generateId(),
                clearAllId: this.generateId(),
                deleteAllId: this.generateId(),
                clearOld: CLEAR_OLD_HISTORIES_LABEL,
                clearOldValue: OPERATIONS.clearOld,
                clearAll: CLEAR_ALL_HISTORIES_LABEL,
                clearAllValue: OPERATIONS.clearAll,
                deleteAll: DELETE_ALL_HISTORIES_LABEL,
                deleteAllValue: OPERATIONS.deleteAll
            }));

        this.$selectOperation(OPERATIONS.clearOld);

        /*global window*/
        $(window).resize(function () {
            resize(that);
        });

        initCommandGroup(this);

        return Promise.join(
            initNavTreeEditor(this),
            initDateEditor(this),
            initRunCommand(this)
        );
    };

    /**
     * Resizes the view once it's been initialized
     */
    ProtectedDatabaseMaintenance.prototype.doLoad = function (secureDashboardConfigComp) {
        var that = this;
        return loadComponentSlots(secureDashboardConfigComp).then(function(secureDashboardConfigCompLeased){
            secureDashboardConfiguration=secureDashboardConfigCompLeased;
            resize(that);
        });
    };

    /**
     * Destroys all child editors and clears the history selection list
     * @param params
     * @returns {Promise}
     */
    ProtectedDatabaseMaintenance.prototype.doDestroy = function (params) {
        this.$getList().empty();
        return this.getChildEditors().destroyAll();
    };

    /**
     * Returns a reference to the HistorySpace NavTree editor
     * @private
     * @returns {NavTree}
     */
    ProtectedDatabaseMaintenance.prototype.$getNavTree = function () {
        return this.jq().find(HISTORY_NAV_TREE_SELECTOR).data('widget');
    };

    /**
     * Returns an array of all NavTree editors representing HistoryGroup nodes
     * @private
     * @returns {Promise} Promise resolved with array of NavTree nodes
     */
    ProtectedDatabaseMaintenance.prototype.$getHistoryGroups = function () {
        return getHistoryGroups(this.$getNavTree());
    };

    /**
     * Returns an array of all NavTree editors representing HistoryDevice nodes
     * @private
     * @returns {Promise} Promise resolved with array of NavTree nodes
     */
    ProtectedDatabaseMaintenance.prototype.$getHistoryDevices = function () {
        return getHistoryDevices(this.$getNavTree());
    };

    /**
     * Returns an array of all NavTree editors representing History nodes
     * @private
     * @returns {Promise} Promise resolved with array of NavTree nodes
     */
    ProtectedDatabaseMaintenance.prototype.$getHistories = function () {
        return getHistories(this.$getNavTree());
    };

    /**
     * Returns a reference to the list containing the histories marked for maintenance
     * @private
     * @returns {jQuery}
     */
    ProtectedDatabaseMaintenance.prototype.$getList = function () {
        return this.jq().find(HISTORY_LIST_SELECTOR);
    };

    /**
     * Returns a reference to the CompositeAbsTimeEditor for selecting dates
     * @private
     * @returns {CompositeAbsTimeEditor}
     */
    ProtectedDatabaseMaintenance.prototype.$getDateEditor = function () {
        return this.jq().find(DATE_EDITOR_SELECTOR).data('widget');
    };

    /**
     * Returns a reference to the CommandButton editor for running maintenance
     * @private
     * @returns {CommandButton}
     */
    ProtectedDatabaseMaintenance.prototype.$getRunCommandButton = function () {
        return this.jq().find(RUN_COMMAND_BUTTON_SELECTOR).data('widget');
    };

    /**
     * Returns a reference to the Command for selecting all histories in the dataase*
     * @private
     * @returns {Command}
     */
    ProtectedDatabaseMaintenance.prototype.$getSelectAllCommand = function () {
        return this.getCommandGroup().get(0);
    };

    /**
     * Returns a reference to the Command for deselecting all histories from the selection list
     * @private
     * @returns {Command}
     */
    ProtectedDatabaseMaintenance.prototype.$getDeselectAllCommand = function () {
        return this.getCommandGroup().get(1);
    };

    /**
     * Returns the value for the currently selected maintenance operation
     * @private
     * @returns {String} Value for current maintenance operation
     */
    ProtectedDatabaseMaintenance.prototype.$getSelectedOperation = function () {
        return this.jq().find('input[type=radio]:checked').val();
    };

    /**
     * Selects the maintenance operation to run
     * @private
     * @param operation One of 'clearOld', 'clearAll', or 'deleteAll'
     */
    ProtectedDatabaseMaintenance.prototype.$selectOperation = function (operation) {
        this.jq().find('input[type=radio]:checked').prop('checked', false);
        this.jq().find('input[type=radio][value=' + operation + ']').prop('checked', true);
    };

    /**
     * Returns an arrray of baja.Ord for the currently selected histories marked for maintenance
     * @private
     * @returns {Array} Array of baja.Ord
     */
    ProtectedDatabaseMaintenance.prototype.$getSelectedOrds = function () {
        return getSelectedOrds(this.$getList());
    };

    function fetchSecondaryUser(component, currentUser, slotName)
    {
        var selectedHistoryList = $(".historyList > ul"),
            getSelectedHistoryOrds = getSelectedOrds(selectedHistoryList).toString();

        var historyDetailsComp = baja.$("baja:Component");
        historyDetailsComp.add({
            slot:"currentUser",
            value:currentUser
        });

        historyDetailsComp.add({
            slot:"componentSlotName",
            value:slotName
        });

        historyDetailsComp.add({
            slot:"targetOrds",
            value:getSelectedHistoryOrds
        });
        return component.invoke({
            slot: "getSecondaryUserDetailsForHistory",
            value: historyDetailsComp
        });
    }

    return ProtectedDatabaseMaintenance;
});
