/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * A module defining `ReasonConfigurationManager`.
 *
 * @private
 * @module nmodule/caverion/rc/fe/ReasonConfigurationManager
 */
define(['baja!',

       'nmodule/webEditors/rc/fe/baja/BaseEditor',
       'jquery',
       'Promise',
       'underscore',
       'nmodule/electronicSignature/rc/wb/mgr/model/ConfigSetModel',
       'nmodule/webEditors/rc/fe/fe',
       'nmodule/webEditors/rc/wb/mgr/Manager',
        'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
       'nmodule/webEditors/rc/wb/table/model/source/ContainerComponentSource',
       'nmodule/webEditors/rc/wb/mgr/model/columns/PropertyMgrColumn',
       'nmodule/webEditors/rc/wb/mgr/model/columns/NameMgrColumn',
       'nmodule/webEditors/rc/wb/table/model/Column',
       'nmodule/webEditors/rc/fe/baja/util/DepthSubscriber',
       'bajaux/mixin/subscriberMixIn',
       'nmodule/webEditors/rc/fe/baja/util/typeUtils',
       'nmodule/webEditors/rc/fe/registry/StationRegistry',
       'nmodule/electronicSignature/rc/wb/mgr/commands/ReasonEditCommand',
       'nmodule/electronicSignature/rc/wb/mgr/commands/AddReasonCommand',
       'nmodule/electronicSignature/rc/wb/mgr/commands/AddReasonRemoveCommand',
        'nmodule/webEditors/rc/wb/mgr/model/mgrAgents',
       'baja!baja:User,web:WebProfileConfig,electronicSignature:Reasons,electronicSignature:ReasonSet'
       ], function (
        baja,
        BaseEditor,
        $,
        Promise,
        _,
        ConfigSetModel,
        fe,
        Manager,
        MgrTypeInfo,
        ContainerComponentSource,
        PropertyMgrColumn,
        NameMgrColumn,
        Column,
        DepthSubscriber,
        subscribable,
        typeUtils,
        StationRegistry,
        ReasonEditCommand,
        AddReasonCommand,
        AddReasonRemoveCommand,
        mgrAgents,
        types
        ) {
        'use strict';
  var REASON_TYPE = baja.lt('electronicSignature:Reasons'),
  EDITABLE = Column.flags.EDITABLE;
 /* UNSEEN = Column.flags.UNSEEN,
  READONLY=Column.flags.READONLY;*/
  function toColumn(slot, params) {
    params = params || {};
    var Ctor = params.ctor || PropertyMgrColumn,
    flags = params.flags || EDITABLE;
    return new Ctor(slot, { type: REASON_TYPE, flags: flags });

}

  var COLUMNS = [
    new NameMgrColumn({ flags: EDITABLE }),
    //toColumn('reasonID'),
    toColumn('reasonForChange'),
    toColumn('reasonSetName')
     ];
  /**
   * Description of your widget.
   *
   * @class
   * @extends module:bajaux/Widget
   * @alias module:nmodule/caverion/rc/ReasonConfigurationManager
   */
  var ReasonConfigurationManager = function ReasonConfigurationManager(params) {
    Manager.call(this, _.extend({}, {
      moduleName: 'electronicSignature', keyName: 'ReasonConfigurationManager'
    }, params));
    this.$subscriber = new DepthSubscriber(3);
      this.getCommandGroup().add(
         new AddReasonCommand(this),
         new ReasonEditCommand(this),
         new AddReasonRemoveCommand(this)
      );
  subscribable(this);
  };

  //extend and set up prototype chain
  ReasonConfigurationManager.prototype = Object.create(Manager.prototype);
  ReasonConfigurationManager.prototype.constructor = ReasonConfigurationManager;
  ReasonConfigurationManager.prototype.doInitialize = function (dom) {

    var that = this,
    args = arguments;
   // preloadTypes();
  /*   dom.on('dblclick', '.mainTable tr', function (e) {
         that.$getEditCommand().invoke();
       });*/
   return Manager.prototype.doInitialize.apply(that, args);
  };

    /**
     * Create a `UserManagerMgrModel` instance with the appropriate columns.
     *
     * @param {baja.Component} userService
     * @returns {Promise.<module:nmodule/webEditors/rc/wb/util/user/UserManagerMgrModel>}
     */
    ReasonConfigurationManager.prototype.makeModel = function (securedDashboardConfiguration) {
        return Promise.all([
            MgrTypeInfo.make([ REASON_TYPE ]),
            mgrAgents.getMixInColumns(REASON_TYPE)
        ])
        .spread(function (newTypes, mixinCols) {
            return new ConfigSetModel({
                columns: COLUMNS.concat(mixinCols),
                componentSource: new ContainerComponentSource({
                    container: securedDashboardConfiguration,
                    filter: [ 'electronicSignature:Reasons' ]
                }),
                newTypes: newTypes
            });
        });
    };

    ReasonConfigurationManager.prototype.doLoad = function (securedDashboardConfiguration) {
        var that = this;
        return Manager.prototype.doLoad.apply(that, arguments);
    };

    /*
    ReasonConfigurationManager.prototype.load = function (securedDashboardConfiguration) {
      var that = this, newTypes;
      return MgrTypeInfo.make([ 'electronicSignature:Reasons' ])
        .then(function (typeInfos) {
          newTypes = typeInfos;
          var model = new ConfigSetModel({
          columns: COLUMNS,
          componentSource: new ContainerComponentSource({
            container: securedDashboardConfiguration,
            filter: [ 'electronicSignature:Reasons' ]
          }),
          newTypes: newTypes
          });
          return Manager.prototype.load.call(that,model);
        });
    };*/

      return ReasonConfigurationManager;
});
