/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * A module defining `ReasonSetConfigurationManager`.
 *
 * @private
 * @module nmodule/caverion/rc/fe/ReasonSetConfigurationManager
 */
define(['baja!',

       'nmodule/webEditors/rc/fe/baja/BaseEditor',
       'jquery',
       'Promise',
       'underscore',
       'nmodule/electronicSignature/rc/wb/mgr/model/ConfigSetModel',
       'nmodule/webEditors/rc/fe/fe',
       'nmodule/webEditors/rc/wb/mgr/Manager',
       'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
       'nmodule/webEditors/rc/wb/table/model/source/ContainerComponentSource',
       'nmodule/webEditors/rc/wb/mgr/model/columns/PropertyMgrColumn',
       'nmodule/webEditors/rc/wb/mgr/model/columns/NameMgrColumn',
       'nmodule/webEditors/rc/wb/table/model/Column',
       'nmodule/webEditors/rc/fe/baja/util/DepthSubscriber',
       'bajaux/mixin/subscriberMixIn',
       'nmodule/webEditors/rc/fe/baja/util/typeUtils',
       'nmodule/webEditors/rc/fe/registry/StationRegistry',
       'nmodule/electronicSignature/rc/wb/mgr/commands/ReasonSetEditCommand',
       'nmodule/electronicSignature/rc/wb/mgr/commands/AddReasonSetCommand',
       'nmodule/electronicSignature/rc/wb/mgr/commands/AddReasonRemoveCommand',
        'nmodule/webEditors/rc/wb/mgr/model/mgrAgents',
       'baja!baja:User,web:WebProfileConfig,electronicSignature:ReasonSet'
       ], function (
        baja,
        BaseEditor,
        $,
        Promise,
        _,
        ConfigSetModel,
        fe,
        Manager,
        MgrTypeInfo,
        ContainerComponentSource,
        PropertyMgrColumn,
        NameMgrColumn,
        Column,
        DepthSubscriber,
        subscribable,
        typeUtils,
        StationRegistry,
        ReasonSetEditCommand,
        AddReasonSetCommand,
        AddReasonRemoveCommand,
        mgrAgents,
        types
        ) {
        'use strict';
  var REASON_TYPE = baja.lt('electronicSignature:ReasonSet'),
  EDITABLE = Column.flags.EDITABLE;
/*  UNSEEN = Column.flags.UNSEEN,
  READONLY=Column.flags.READONLY;*/
  function toColumn(slot, params) {
    params = params || {};
    var Ctor = params.ctor || PropertyMgrColumn,
    flags = params.flags || EDITABLE;
    return new Ctor(slot, { type: REASON_TYPE, flags: flags });

}

  var COLUMNS = [
    new NameMgrColumn({ flags: EDITABLE }),
    //toColumn('reasonSetID'),
    toColumn('reasonCollection'),
     ];
  /**
   * Description of your widget.
   *
   * @class
   * @extends module:bajaux/Widget
   * @alias module:nmodule/caverion/rc/ReasonSetConfigurationManager
   */
  var ReasonSetConfigurationManager = function ReasonSetConfigurationManager(params) {
    Manager.call(this, _.extend({}, {
      moduleName: 'electronicSignature', keyName: 'ReasonSetConfigurationManager'
    }, params));
    this.$subscriber = new DepthSubscriber(3);
    this.getCommandGroup().add(
    new AddReasonSetCommand(this),
    new ReasonSetEditCommand(this),
    new AddReasonRemoveCommand(this)
    );
  subscribable(this);
  };

    function getSelectedRows(manager) {
        return manager.getMainTable().getSelectedRows();
    }

/*  function isMultiSelection(manager) {
    return getSelectedRows(manager).length > 1;
  }*/
  //extend and set up prototype chain
  ReasonSetConfigurationManager.prototype = Object.create(Manager.prototype);
  ReasonSetConfigurationManager.prototype.constructor = ReasonSetConfigurationManager;
    /**
     * Returns array of currently selected subjects for context menu
     * @override
     * @returns {Array}
     */
    ReasonSetConfigurationManager.prototype.getSubject = function () {
      return _.map(getSelectedRows(this), function (row) {
        return row.getSubject();
      });
    };
  ReasonSetConfigurationManager.prototype.doInitialize = function (dom) {

    var that = this,
    args = arguments;
   // preloadTypes();

    dom.on('dblclick', '.mainTable tr', function (e) {
         var subject = that.getSubject();
         console.log(subject); // jshint ignore:line
         var ord = baja.Ord.make(subject[0].getNavOrd().toString().substring(subject[0].getNavOrd().toString().indexOf("station:")) + "|view:electronicSignature:ReasonManager");
         niagara.env.hyperlink(ord); // jshint ignore:line
         //document.location.href = document.location.href.substring(0,document.location.href.indexOf("station:")) + subject[0].getNavOrd().toString().substring(subject[0].getNavOrd().toString().indexOf("station:"));
       });
     return Manager.prototype.doInitialize.apply(that, args);
  };

    /**
     * Create a `UserManagerMgrModel` instance with the appropriate columns.
     *
     * @param {baja.Component} userService
     * @returns {Promise.<module:nmodule/webEditors/rc/wb/util/user/UserManagerMgrModel>}
     */
    ReasonSetConfigurationManager.prototype.makeModel = function (securedDashboardConfiguration) {
        return Promise.all([
            MgrTypeInfo.make([ REASON_TYPE ]),
            mgrAgents.getMixInColumns(REASON_TYPE)
        ])
        .spread(function (newTypes, mixinCols) {
            return new ConfigSetModel({
                columns: COLUMNS.concat(mixinCols),
                componentSource: new ContainerComponentSource({
                    container: securedDashboardConfiguration,
                    filter: [ 'electronicSignature:ReasonSet' ]
                }),
                newTypes: newTypes
            });
        });
    };

    ReasonSetConfigurationManager.prototype.doLoad = function (securedDashboardConfiguration) {
        var that = this;
        return Manager.prototype.doLoad.apply(that, arguments);
    };

/*  ReasonSetConfigurationManager.prototype.load = function (securedDashboardConfiguration) {
    var that = this, newTypes;
    return MgrTypeInfo.make([ 'electronicSignature:ReasonSet' ])
      .then(function (typeInfos) {
        newTypes = typeInfos;
       var model = new ConfigSetModel({
        columns: COLUMNS,
        componentSource: new ContainerComponentSource({
          container: securedDashboardConfiguration,
          filter: [ 'electronicSignature:ReasonSet' ]
        }),
        newTypes: newTypes
      });
      return Manager.prototype.load.call(that,model);
    });
  };*/

      return ReasonSetConfigurationManager;
});
