/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * @private
 * @module nmodule/alarm/rc/console/commands/AcknowledgeCommand
 */
define([
    'baja!',
    'lex!alarm,electronicSignature',
    'Promise',
    'underscore',
    'dialogs',
    'bajaux/commands/Command',
    'nmodule/alarm/rc/console/commands/alarmConsoleCommandMixIn',
    'nmodule/electronicSignature/rc/wb/mgr/ProtectedAlarmMaintance',
    'hbs!nmodule/electronicSignature/rc/fe/template/ProtectedAlarmConsole',
    'jquery',
    'nmodule/electronicSignature/rc/fe/baja/Util'
], function (baja,
             lexs,
             Promise,
             _,
             dialogs,
             Command,
             alarmConsoleCommandMixIn,
             ProtectedAlarmMaintance,
             authenticationTemplate,
             $,
             Util) {
    'use strict';

    var lex              = lexs[0],
        isAuthenticatedUser = false,
        electronicSignatureLex = lexs[1],
        ERROR_MSG_PREFIX = lex.get('alarm.console.acknowledge.insufficient.permissions');
    var eSignature = null, recipient = null;
    var resultList = new Map(), alarmAuthenticationLevel ="";
    var alarmPoint = new Map();
    var nonAuthenticatedAlarms = "", primaryComment = "";
    var normalAlarmPoints = new Set();

    function getSelectedSubjects(table) {
        return _.invoke(table.getSelectedRows(), 'getSubject');
    }

    function makeAckParams(view, params) {
        var ackParams    = {},
            table        = view.$getAlarmTable(),
            singleSource = view.isSingleSourceView(),
            mostRecent   = params && params.mostRecent,
            details      = params && params.details,
            subject      = details && params.subject;

        if (subject) {
            ackParams.ids = [subject.uuid];
        }
        else if (singleSource) {
            ackParams.ids = _.pluck(getSelectedSubjects(table), 'uuid');
        }
        else {
            ackParams.srcs = _.pluck(getSelectedSubjects(table), 'source');
        }
        if (mostRecent) {
            ackParams.mostRecent = mostRecent;
        }
        return ackParams;
    }

    function reportError(failureList) {
        if (_.isArray(failureList) && failureList.length) {
            var errorMsg = ERROR_MSG_PREFIX + "<br/><br/>" + failureList.join("<br/>");
            dialogs.showOk(errorMsg);
        }
        return failureList;
    }

    function assignEventHandlerForSingleAuthentication(primaryUsername, eSignature, reasonHtml)
    {
        $("#username").val(primaryUsername);
        $("#reason").html(reasonHtml);

        $('.js-dialog-button-ok').attr('disabled', true);

        $('#secondTab').hide();

        $("#secondaryUsername").change(function(){
            $('.js-dialog-button-ok').attr('disabled', true);
        });

        $('#reason').change(function(){
            if($('#reason').val() === "Other"){
                $('#idTextarea').attr('disabled',false);
            }
            else{
                $('#idTextarea').attr('disabled',true);
            }
        });

        $(primaryButtonSelector).click(function (){
            var credentials = {};
            credentials.username = $('#username').val();
            credentials.password = $('#password').val();
            checkPrimarySignForUser(credentials, eSignature);
        });

        $(".primaryTab").click(function (e){
            e.preventDefault();
        });

        Util.changeLegalText(primaryUsername,eSignature);
    }

    function assigneEventHandlerForDualAuthentication(primaryUsername, eSignature, authenticationLevel,params, reasonHtml)
    {
        $("#username").val(primaryUsername);
        $("#reason").html(reasonHtml);
        $('.js-dialog-button-ok').attr('disabled', true);

        if (authenticationLevel === PRIMARY) {
            $('#secondTab').hide();
        }

        $("#secondaryUsername").change(function () {
            $('.js-dialog-button-ok').attr('disabled', true);
        });

        $(primaryButtonSelector).click(function () {
            var credentials = {};
            credentials.username = $('#username').val();
            credentials.password = $('#password').val();
            checkPrimarySign(credentials, params.recipient, eSignature);
        });

        $(".primaryTab").click(function (e) {
            e.preventDefault();
        });

        $(".secondaryTab").click(function (e) {
            e.preventDefault();
        });

        $('#reason').change(function(){
            if($('#reason').val() === "Other"){
                $('#idTextarea').attr('disabled',false);
            }
            else{
                $('#idTextarea').attr('disabled',true);
            }
        });

        $("#btnSecondarySign").click(function () {
            var credentials = {};
            credentials.username = $('#secondaryUsername').val();
            credentials.primaryUserName = $('#username').val();
            credentials.password = $('#secondaryPassword').val();
            checkSecondarySign(credentials, params.recipient, eSignature);
        })

        Util.changeLegalText(primaryUsername, eSignature);

        $(document).ready(function () {
            $(document).find("#secondaryUsername").change(function () {
                Util.changeLegalText($(document).find("#secondaryUsername").val(), eSignature);
            });
        });

    }

    function clearAuthenticationRelatedParameters()
    {
        resultList.clear();
        alarmPoint.clear();
        alarmAuthenticationLevel = "";
        nonAuthenticatedAlarms = "";
    }

    async function acknowledge(view, params) {
        var viewModel = view.$getViewModel(),
            support   = viewModel.$getSupport(),
            singleSrc = viewModel.$isSingleSourceModel(),
            options   = support.getAlarmConsoleOptions(),
            notesCmd  = view.$notesCmd,
            ackParams = makeAckParams(view, params),
            notesOnAck;

        options = singleSrc ? options.singleSourceView : options.multiSourceView;
        notesOnAck = options && options.notesRequiredOnAck;

        if (_.isEmpty(ackParams.srcs) && _.isEmpty(ackParams.ids)) {
            return Promise.resolve();
        }

        eSignature = await Util.getESignatureComponent();
        let primaryUsername = baja.getUserName(),
            lexiconJson = getLexiconFortemplate();
        let selectedAlarmRecords;
        let alarmReasonSets = await eSignature.getAlarmReasons();
        var alarmReasonSetList = alarmReasonSets.split(",");
        var tempHtml ="<option value='' selected>--Select--</option>";


        _.each(alarmReasonSetList,function(value,key){
            if(value!=="All") {
                tempHtml += "<option value=\"" + value + "\">" + value + "</option>";
            }
        });

        tempHtml +="<option value='Other'>Other</option>";

        if(eSignature.getUseSingleAuthenticationForAlarm())
        {
            let result = dialogs.showOkCancel({title:electronicSignatureLex.get("authenticate.user.title"),
                content:authenticationTemplate({jsonObject:lexiconJson})})
                .ok(async function(val){
                    selectedAlarmRecords = view.getSelectedAlarmRecords();
                    return await checkAndAckAlarm();
                });
            assignEventHandlerForSingleAuthentication(primaryUsername, eSignature,tempHtml);
            return result;
        }
        else
        {
            let jacesAndPointOrds = getJacepointOrds(view);
            await getPointAuthenticationLevel(jacesAndPointOrds, params.recipient);
            if (alarmAuthenticationLevel === "") {
                selectedAlarmRecords = successAlarmRecordsForjaces();
                return  await checkAndAckAlarm();
            } else {
                let result = dialogs.showOkCancel({
                    title: electronicSignatureLex.get("authenticate.user.title"),
                    content: authenticationTemplate({jsonObject: lexiconJson})
                }).ok(async function () {
                    selectedAlarmRecords = successAlarmRecordsForjaces();
                    await checkAndAckAlarm();
                    let msg = nonAuthenticatedAlarms === "" ?"Alarm Acknowledge":nonAuthenticatedAlarms;
                    clearAuthenticationRelatedParameters();
                    return dialogs.showOk({
                        title: "Acknowledge",
                        content: msg
                    });
                }).cancel(function () {
                    clearAuthenticationRelatedParameters();
                });
                assigneEventHandlerForDualAuthentication(primaryUsername,eSignature, alarmAuthenticationLevel,params,tempHtml);
                return result;
            }
        }

        function updateAckParamSrcs(){
            let srcs = [], ids = [];
            selectedAlarmRecords.forEach(function(selectedAlarm){
                srcs.push(selectedAlarm.source);
                ids.push(selectedAlarm.uuid)
            });
            ackParams.srcs= srcs;
            ackParams.ids = ids;
        }

        function acknowledgeAlarms(userNotes) {
            resultList.clear();
            alarmPoint.clear();
            alarmAuthenticationLevel = "";

            //let primaryUserComments = "";

            /*if (isAuthenticatedUser && (primaryComment !== null || primaryComment !== "")) {
                primaryUserComments = primaryComment;
                ackParams.notes = (userNotes) ? (userNotes + "\n" + primaryComment) : primaryComment;
            }*/

            var priComment =primaryComment;
            if(priComment!==""){
                if(userNotes !== ""){
                    userNotes +="\n";
                }
                userNotes += priComment;
            }
            var secComment = $("#secondaryComments").val();
            if(secComment!==""){
                userNotes +="\n";
                userNotes += secComment;
            }
            ackParams.notes = userNotes ;

            var reason ;

            if($("#reason").val() === '' || $("#reason").val() === 'Other'){
                reason = $("#idTextarea").val();
            }
            else{
                reason = $("#reason").val();
            }

            var secondaryUsername = $("#secondaryUsername").val();
            var secondaryComments = $("#secondaryComments").val();

            var alarmDataToSend = Util.getAlarmDetails(removeNormalPointAlarmToSaveHistory(selectedAlarmRecords), "Acknowledge", priComment,reason,secondaryUsername,secondaryComments);
            updateAckParamSrcs();

            return Promise.resolve(dialogs.showLoading(0,
                viewModel.ackAlarms(ackParams).then(reportError))).then(function () {
                return baja.rpc({
                    typeSpec: 'electronicSignature:ProtectedConsoleRecipient',
                    method: 'rpcAddHistoryForAlarm',
                    args: alarmDataToSend
                });
            });
        }

         function checkAndAckAlarm() {
            if (notesOnAck) {
                return notesCmd.invoke({callThrough: false})
                    .then(function (notes) {
                        if (notes) {
                            acknowledgeAlarms(notes);
                        }
                    });
            }
            else {
                acknowledgeAlarms("");
            }
        }
    }


    /**
     * Acknowledges all selected alarms or just the most recent.
     *
     * @constructor
     * @param {module:nmodule/alarm/rc/console/AlarmConsole} view The alarm console view.
     * @param {Object} params
     * @param {Boolean} params.mostRecent If true then only the most recent alarm is
     * @param {Object} params.recipient The Protected Console Recipient object
     * acknowledged.
     * @extends {module:bajaux/commands/Command}
     * @mixes {module:nmodule/alarm/rc/console/commands/alarmConsoleCommandMixIn}
     */
    var AcknowledgeCommand = function (view, params) {
        Command.apply(this, [{
            module: 'alarm',
            lex: params && params.mostRecent ? 'commands.acknowledgeRecent' : 'commands.acknowledge',
            func: function (ackParams) {
                return acknowledge(view, _.extend({}, params, ackParams));
            }
        }]);

        alarmConsoleCommandMixIn(this);

        this.$view = view;
        this.$mostRecent = params && params.mostRecent;
        this.recipient = params.recipient;
    };

    AcknowledgeCommand.prototype = Object.create(Command.prototype);
    AcknowledgeCommand.prototype.constructor = AcknowledgeCommand;

    AcknowledgeCommand.prototype.selectionChanged = function () {
        this.setEnabled(!this.$view.isSelectionEmpty() && (!this.$mostRecent || !this.$view.isSingleSourceView()));
    };

    AcknowledgeCommand.prototype.isShownInMenu = function () {
        return !this.$mostRecent || !this.$view.isSingleSourceView();
    };

    AcknowledgeCommand.prototype.isShownInCommandBar = function () {
        return !this.$mostRecent;
    };


    function getFormatedPointOrd(component)
    {
        var pointOrd = "";

        pointOrd = "station:|h:" + component.getHandle().toString();

        return pointOrd;
    }

    function showValidationSummary(validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            $("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: electronicSignatureLex.get("error.massages.title"),
            content: errorMessage
        });
    }

    function getComponentByOrd(ord)
    {
        return baja.Ord.make(ord).get({lease: true});
    }

    function updateListForAuthorization(result,pointOrdAndLevel,jace){
        var isValid = false;
        var pointAndResult = result.toString().split("%%%");
        pointAndResult.forEach(function(results){
            var pointResult = results.toString().split(";;;");
            if(pointResult[1].includes("true")){
                pointOrdAndLevel.set(pointResult[0],"Secondary%%%true");
                isValid = true;
            }
            else{
                pointOrdAndLevel.set(pointResult[0],"Secondary%%%false%%%Authorization");
            }
        })
        resultList.set(jace,pointOrdAndLevel);
        return isValid;
    }

    function authorizeUser(credentials,recipient,eSignature){
        var promises = [], primaryCheck = false;

        resultList.forEach(function(pointOrdAndLevel,jace){
            promises.push(authorizationCheck());

            function authorizationCheck() {
                if (pointOrdAndLevel.values().next().value.toString() === "false%%%Unable to fetch authentication level for point") {
                    return new Promise((resolve) => resolve(false));
                } else {
                    var pointOrd = "";
                    pointOrdAndLevel.forEach(function (result, point) {
                        if (result.includes(SECONDARY) && result.includes("true")) {
                            if (pointOrd === "") {
                                pointOrd = point;
                            } else {
                                pointOrd += "%%%" + point;
                            }
                        }

                        if(result.includes(PRIMARY) && result.includes("true")){
                            primaryCheck = true;
                        }
                    });
                    pointOrd = pointOrd + ";;;" + credentials.primaryUserName + ";;;" + credentials.username + ";;;" + jace;
                    if (!(jace === "Sup")) {
                        return new Promise((resolve) => {
                            recipient.invoke({
                                slot: "checkAuthorizationOfPoints",
                                value: baja.$("baja:String").make(pointOrd)
                            }).then(function (result) {
                                if(result == null || result.toString() === "false"){
                                    resolve(primaryCheck);
                                }
                                else {
                                    resolve(updateListForAuthorization(result, pointOrdAndLevel, jace));
                                }
                            })
                        })
                    } else {
                        return new Promise((resolve) =>{
                            eSignature.invoke({
                                slot: "getAlarmPermissionForJacePoints",
                                value: baja.$("baja:String").make(pointOrd)
                            }).then(function (result)
                            {
                                if(result == null || result.toString() === "false"){
                                    resolve(primaryCheck);
                                }
                                else {
                                    resolve(updateListForAuthorization(result, pointOrdAndLevel, jace));
                                }
                            })
                        })
                    }
                }
            }
        })

        return  Promise.all(promises)
            .then((results) => {
                if(results.includes(true)){
                    return true;
                }
                else
                    return false;
            })
            .catch((e) => {
                console.log("err");
            });
    }

    function authenticateUserJace(recipient, credentials,jace)
    {
        return new Promise(function (resolve, reject) {
            recipient.invoke({
                slot: "jaceAuthenticateForAlarms",
                value: baja.$("baja:String").make(jace+";;;"+"authenticateUser," + credentials.toString())
            }).then(function (authenticatedComp)
            {
                if(authenticatedComp == null){
                    resolve(false) ;
                }
                else {
                    if (authenticatedComp.getSlot("authenticationJobOrd") === null) {
                        resolve(authenticatedComp.get("isValidUser"));
                    } else {
                        let getJobStatus = () => {
                            recipient.invoke({
                                slot: "jaceAuthenticateForAlarms",
                                value: baja.$("baja:String").make(jace + ";;;" + "LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                            }).then(function (returnComponent) {
                                if (returnComponent.get("jobStatus") === baja.$("baja:JobState").get("running")) {
                                    //schedule the timeout.
                                    setTimeout(function () {
                                        getJobStatus();
                                    }, 1000);
                                } else {
                                    resolve(returnComponent.get("jobStatus") === baja.$("baja:JobState").get("success"));
                                }
                            });
                        };
                        setTimeout(function () {
                            getJobStatus();
                        }, 1000);
                    }
                }
            }).catch(function(err){
                console.log("Error occurred while authenticating the user.", err);
                resolve(false);
            });
        });
    }


    async function authenticateUser(eSignature, credentials)
    {
        return new Promise(async function (resolve, reject) {
            try {
                let authenticatedComp = await eSignature.invoke({
                    slot: "call",
                    value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
                });

                if(authenticatedComp == null){
                    resolve(false);
                }
                else {
                    if (authenticatedComp.getSlot("authenticationJobOrd") === null) {
                        resolve(authenticatedComp.get("isValidUser"));
                    } else {
                        let getJobStatus = async () => {
                            try {
                                let returnComponent = await eSignature.invoke({
                                    slot: "call",
                                    value: baja.$("baja:String").make("LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                                })
                                if (returnComponent.get("jobStatus") === baja.$("baja:JobState").get("running")) {
                                    //schedule the timeout.
                                    setTimeout(function () {
                                        getJobStatus();
                                    }, 1000);
                                } else {
                                    resolve(returnComponent.get("jobStatus") === baja.$("baja:JobState").get("success"));
                                }
                            }
                            catch (e)
                            {
                                console.log(e);
                                resolve(false);
                            }
                        };
                        setTimeout(function () {
                            getJobStatus();
                        }, 1000);
                    }
                }
            }
            catch(err){
                console.log("Error occurred while authenticating the user.", err);
                resolve(false);
            }
        });
    }

    function authenticateAndResultListUpdate(eSignature,recipient,credentials,isPrimary){
        var isValid = false;
        const promises = [];

        resultList.forEach(function (value,key){
            promises.push(authCheckForJace(value,key));
        });

        async function authCheckForJace(value,key) {
            isValid = false;
            let isAuthenticated = false;
            try {
                if (!(key === "Sup"))
                    isAuthenticated = await authenticateUserJace(recipient, credentials, key);
                else
                    isAuthenticated = await authenticateUser(eSignature, credentials);
            }
            catch (e) {

            }
            return updateResultList(value, key,isAuthenticated);
        }


        function updateResultList(mapOfPointsAndLevels,stationName,isAuthenticated) {
            let updatedPrimaryStatus = "";
            let updatedSecondaryStatus = "";

            if (isPrimary) {
                if (isAuthenticated) {
                    updatedPrimaryStatus = "Primary%%%true"
                    updatedSecondaryStatus = "Secondary%%%Pretrue";
                    isValid = true;
                }
                else
                {
                    updatedPrimaryStatus = "Primary%%%false%%%Authentication"
                    updatedSecondaryStatus = "Secondary%%%false%%%Authentication";
                }
            }
            else
            {
                if (isAuthenticated)
                    updatedSecondaryStatus = "Secondary%%%true";
                else
                    updatedSecondaryStatus = "Secondary%%%Secfalse%%%Authentication";
            }
            if (isPrimary) {
                mapOfPointsAndLevels.forEach(function (authStatus, pointOrd) {
                    if (authStatus.includes(SECONDARY)) {
                        authStatus = updatedSecondaryStatus;
                    } else if (authStatus.includes(PRIMARY)) {
                        authStatus = updatedPrimaryStatus;
                    }
                    mapOfPointsAndLevels.set(pointOrd, authStatus);
                });
            } else {
                mapOfPointsAndLevels.forEach(function (authStatus, pointOrd) {
                    if ((isAuthenticated &&
                            ((authStatus.includes(SECONDARY) && authStatus.includes("Pretrue"))
                                || authStatus.includes("Secfalse")) )
                        ||( !isAuthenticated && (authStatus.includes(SECONDARY) && authStatus.includes("Pretrue")))) {
                        authStatus = updatedSecondaryStatus;
                    }
                    mapOfPointsAndLevels.set(pointOrd, authStatus);
                });
            }
            resultList.set(stationName, mapOfPointsAndLevels);
            return isAuthenticated;
        }


        return  Promise.all(promises)
            .then((results) => {
                return  _.some(results, function(resultOfIndividualStation)
                { return resultOfIndividualStation === true; });
            })
            .catch((e) => {
                console.log("authenticateAndResultListUpdate:"+e);
            });
    }


    async function checkPrimarySign(credentials,alarmRecipient, eSignature)
    {
        isAuthenticatedUser = false;
        let validation = doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            await showValidationSummary(validation);
        }
        else
        {
            primaryComment =$('#primaryComment').val();
            $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.signing.button'));
            $(document).find(primaryButtonSelector).prop("disabled", true);

            try {
                let anyOneAlarmSourceIsAuthenticated = await authenticateAndResultListUpdate(eSignature,alarmRecipient,baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)),true)
                $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if (anyOneAlarmSourceIsAuthenticated)
                {
                    isAuthenticatedUser = true;
                    if(alarmAuthenticationLevel === PRIMARY) {
                        $('.js-dialog-button-ok').attr('disabled', false);
                        $(primaryButtonSelector).html(electronicSignatureLex.get("fe.signed.button"));
                    }
                    else {
                        $('.js-dialog-button-ok').attr('disabled', true);
                        $('.nav-tabs a[href="#easSecondarySignature"]').tab('show');
                    }
                }
                else
                {
                    $("#password").addClass("errorMsg");
                    return dialogs.showOk({
                        title: electronicSignatureLex.get("error.massages.title"),
                        content: electronicSignatureLex.get("authentication.error.message")
                    });
                }
            }
            catch (e) {
                console.log(e);
                $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if(error.javaClass ==="javax.baja.security.PermissionException"){
                    return dialogs.showOk({
                        title: electronicSignatureLex.get('permission.error.title'),
                        content:electronicSignatureLex.get('primUser.permission.error.message')
                    });
                }
            }
        }
    }

    async function checkSecondarySign(credentials,recipient, eSignature)
    {
        $('.js-dialog-button-ok').attr('disabled', true);
        isAuthenticatedUser = false;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title: electronicSignatureLex.get('authentication.error.title'),
                content: electronicSignatureLex.get('level2.authentication.message')
            });
        }
        var validation = doIsDataValid("secondary");
        if (validation !== null && validation.length > 0)
        {
            await  showValidationSummary(validation);
        }
        else
        {
            let isValidManager = false;
            $(document).find('#btnSecondarySign').html(electronicSignatureLex.get('fe.signing.button'));
            $(document).find('#btnSecondarySign').prop("disabled", true);

            try {
                let atleastOneAlarmAuthenticated = await authenticateAndResultListUpdate(eSignature,recipient, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)),false);
                $(document).find('#btnSecondarySign').html(electronicSignatureLex.get('fe.sign2.button'));
                $(document).find('#btnSecondarySign').prop("disabled", false);
                isValidManager = atleastOneAlarmAuthenticated;
                if (atleastOneAlarmAuthenticated)
                {
                    isAuthenticatedUser = true;
                    let atleastOneAlarmAuthorized = await authorizeUser(credentials,recipient,eSignature);
                    if(atleastOneAlarmAuthorized){
                        $('.js-dialog-button-ok').attr('disabled', false);
                        return dialogs.showOk({
                            title: electronicSignatureLex.get('auth.success.title'),
                            content: electronicSignatureLex.get('auth.success.msg')
                        });
                    }
                    else    {
                        return dialogs.showOk({
                            title: electronicSignatureLex.get("authorization.error.title.browser"),
                            content: electronicSignatureLex.get("authorization.error.message.browser")
                        });
                    }
                }
                else
                {
                    let isAnyPrimaryAuthenticatedAlarm =  successAlarmRecordsForjaces().length;
                    if(isAnyPrimaryAuthenticatedAlarm >0){
                        $('.js-dialog-button-ok').attr('disabled', false);
                        /* 2nd Level Authentication failed for Following records.
                        record1
                        record2
                        record3
                        Press Ok to Acknowledge alarm records which passes single level authentication or
                        doesn't required authentication.
                         */
                        const msg = "\n  Press Ok on Authentication Dialog Box to Acknowledge alarm records which passes single level authentication or doesn't required authentication" ;
                        return dialogs.showOk({
                            title: electronicSignatureLex.get("authentication.error.title"),
                            content: nonAuthenticatedAlarms + msg
                        });
                    }
                    return dialogs.showOk({
                        title: electronicSignatureLex.get("authentication.error.title"),
                        content: electronicSignatureLex.get("authentication.error.message")
                    });
                }
            }
            catch (e) {
                $(document).find(".btnSecondarySign").html(electronicSignatureLex.get('fe.sign.button'));
                $(document).find(".btnSecondarySign").prop("disabled", false);
                console.log(e); // jshint ignore:line
            }
        }
    }

    async function checkPrimarySignForUser(credentials, eSignature)
    {
        isAuthenticatedUser = false;
        var validation = doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            await showValidationSummary(validation);
        }
        else {
            try {
                primaryComment = $('#primaryComment').val();
                $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.signing.button'));
                $(document).find(primaryButtonSelector).prop("disabled", true);
                let authenticatedComp = await authenticateUser(eSignature, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));

                $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if (authenticatedComp) {
                    isAuthenticatedUser = true;
                    $('.js-dialog-button-ok').attr('disabled', false);
                    $(primaryButtonSelector).html(electronicSignatureLex.get("fe.signed.button"));
                } else {
                    $("#password").addClass("errorMsg");
                    return dialogs.showOk({
                        title: electronicSignatureLex.get("error.massages.title"),
                        content: electronicSignatureLex.get("authentication.error.message")
                    });
                }
            } catch (error) {
                $(document).find(primaryButtonSelector).html(electronicSignatureLex.get('fe.sign.button'));
                $(document).find(primaryButtonSelector).prop("disabled", false);
                if (error.javaClass === "javax.baja.security.PermissionException") {
                    return dialogs.showOk({
                        title: electronicSignatureLex.get('permission.error.title'),
                        content: electronicSignatureLex.get('primUser.permission.error.message')
                    });
                }
            }
        }
    }

    function doIsDataValid(authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        if (authenticationType === "primary")
        {
            if ($('#username').val() === null || $('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#username').removeClass("errorMsg");
            }
            if ($('#password').val() === null || $('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#password').removeClass("errorMsg");
            }
            if ($('#primaryComment').val() === null || $('#primaryComment').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "primaryComment";
                invalidListObj.errorMsg = "Notes are mandatory";
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#primaryComment').removeClass("errorMsg");
            }

            if ($('#reason').val() === '' || $('#reason').val() === 'Other')
            {
                if($('#idTextarea').val() === '') {
                    invalidListObj = {};
                    invalidListObj.id = "reason";
                    invalidListObj.errorMsg = "Reasons are mandatory";
                    invalidList.push(invalidListObj);
                }
            }
            else
            {
                $('#reason').removeClass("errorMsg");
            }

            return invalidList;
        }
        if (authenticationType === "secondary")
        {

            if ($('#secondaryUsername').val() === null || $('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg =  electronicSignatureLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryUsername').removeClass("errorMsg");
            }
            if ($('#secondaryPassword').val() === null || $('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = electronicSignatureLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryPassword').removeClass("errorMsg");
            }
            return invalidList;
        }
    }



    function errorMessages (result){
        if(result === "false%%%Unable to fetch authentication level for point"){
            return "Unable to fetch authentication level for point";
        }
        else if(result === "Secondary%%%Secfalse%%%Authentication"){
            return "Secondary authentication failed";
        }
        else if(result === "Primary%%%false%%%Authentication"){
            return "Primary authentication failed";
        }
        else if(result === "Secondary%%%false%%%Authentication"){
            return "Secondary authentication failed";
        }
        else
            return "Authorization Failed";
    }

    function removeNormalPointAlarmToSaveHistory(selectedAlarmRecords){
        var noNormalPointRecordsList= [];
        for(var i = 0 ; i<selectedAlarmRecords.length; i++){
            var sourceList =   selectedAlarmRecords[i].source.split("\n");
            if(normalAlarmPoints.has(sourceList[0])){
                continue;
            }
            noNormalPointRecordsList.push(selectedAlarmRecords[i]);
        }
        return noNormalPointRecordsList;
    }

    function successAlarmRecordsForjaces(){
        let authenticatedAlarmRecords = [];
        resultList.forEach(function(pointResult,jaces){
            pointResult.forEach(function(result,point){
                if(result.toString().includes("true")|| result.toString().includes(NORMAL)){
                    let alarm = alarmPoint.get(jaces.toString()+";;;"+point.toString().substring(0,point.toString().lastIndexOf("-")));
                    authenticatedAlarmRecords = authenticatedAlarmRecords.concat(Array.from(alarm));
                    if(result.toString().includes(NORMAL)){
                        normalAlarmPoints.add(point.toString().substring(0,point.toString().lastIndexOf("-")))
                    }
                }
                else{
                    let msg = "Current LoggedIn station";
                    if(!(jaces.toString() === "Sup")) {
                        msg = jaces.toString().substring(jaces.toString().lastIndexOf("/")+1) ;
                    }

                    msg = msg + " :- " + point+"--"+  errorMessages(result);
                    if(nonAuthenticatedAlarms === "") {
                        nonAuthenticatedAlarms = msg;
                    }
                    else{
                        nonAuthenticatedAlarms += "\n" + msg;
                    }
                }
            })
        })
        return authenticatedAlarmRecords;
    }

    async function getPointAuthenticationLevel(jaceAndSupervisorPointOrd,recipient){
        let promises = [];

        jaceAndSupervisorPointOrd.forEach(function(value,key){
            promises.push( jaceUpdateForPoint(value,key));

            async function jaceUpdateForPoint(value,key) {
                let pointsOrdWithAuthenticationLevel;
                if (!(key === "Sup")) {
                    let pointOrd = key + ";" + value;
                    pointsOrdWithAuthenticationLevel = await recipient.invoke({
                        slot: "checkAuthenticationLevel",
                        value: baja.$("baja:String").make(pointOrd)
                    });
                } else {
                    pointsOrdWithAuthenticationLevel = await eSignature.invoke({
                        slot: "pointAuthenticationInfo",
                        value: baja.$("baja:String").make(value)
                    });
                }
                if ((pointsOrdWithAuthenticationLevel == null)) {
                    let pointOrds = value.split(",");
                    let pointJace = new Map();
                    pointOrds.forEach(function (ord) {
                        pointJace.set(ord, "false%%%Unable to fetch authentication level for point");
                    })
                    resultList.set(key, pointJace);
                } else {
                    let jaceAndPointOrds = pointsOrdWithAuthenticationLevel.split(";;;");
                    let points = jaceAndPointOrds[0].split(",");
                    points.forEach(function (point) {
                        let values = point.substring(point.lastIndexOf("-") + 1);
                        let pointJace = new Map();

                        if (values === SECONDARY) {
                            pointJace.set(point, SECONDARY);
                            alarmAuthenticationLevel = SECONDARY;
                        } else if (values === PRIMARY) {
                            pointJace.set(point, PRIMARY);
                            if (!(alarmAuthenticationLevel === SECONDARY)) {
                                alarmAuthenticationLevel = PRIMARY;
                            }
                        } else {
                            pointJace.set(point, NORMAL);
                        }

                        if (resultList.get(key) != null) {
                            let pointResult;
                            pointResult = resultList.get(key);
                            pointJace = new Map([...pointResult, ...pointJace]);
                        }

                        resultList.set(key, pointJace);
                    });
                }
            }
        });

        return  Promise.all(promises)
            .then(() => {
                return resultList;
            })
            .catch((e) => {
                console.log("error while fetching authentication level ",e);
            });

    }

    function updateJaceOrdDetail(alarmRecord, jaceOrdDetails) {
        let source = alarmRecord.source.split("\n");
        let pointOrd = source[0];
        let arr = new Set();
        let jaceStationName;

        if (source.length > 1) { // if source.length >1, it means alarm is from JACE.
            jaceStationName = source[1].substring(0, source[1].lastIndexOf("/"));
        }
        else {
            jaceStationName = "Sup";
        }

        if (jaceOrdDetails.has(jaceStationName)) {
            let jacePointOrd = jaceOrdDetails.get(jaceStationName) + "," + pointOrd;
            jaceOrdDetails.set(jaceStationName, jacePointOrd);
        } else {
            jaceOrdDetails.set(jaceStationName, pointOrd);
        }

        let JACE_Point = jaceStationName + ";;;" + pointOrd;
        if (alarmPoint.get(JACE_Point) == null) {
            arr.add(alarmRecord);
        } else {
            arr = alarmPoint.get(JACE_Point);
            arr.add(alarmRecord);
        }
        alarmPoint.set(JACE_Point, arr);
        return jaceOrdDetails;
    }


    function getJacepointOrds(view){
        var selectedAlarmRecords =view.getSelectedAlarmRecords();
        var jaceOrdDetails = new Map();

        selectedAlarmRecords.forEach(function(alarmRecord){
            updateJaceOrdDetail(alarmRecord, jaceOrdDetails);
        });
        return jaceOrdDetails;
    }


    function getLexiconFortemplate(){
        var jsonObject={
            authLevel1:electronicSignatureLex.get('fe.authorization.level1.label'),
            authLevel2:electronicSignatureLex.get('fe.authorization.level2.label'),
            userName:electronicSignatureLex.get('fe.userName.label'),
            password:electronicSignatureLex.get('fe.password.label'),
            notes:electronicSignatureLex.get('fe.sign.button.Alarm.Notes'),
            sign:electronicSignatureLex.get('fe.sign.button.Alarm.Ack'),
            sign2:electronicSignatureLex.get('fe.sign2.button'),
            reason:electronicSignatureLex.get('fe.reason.label'),
            otherReason:electronicSignatureLex.get('fe.otherReason.label'),
            acknowledgement:electronicSignatureLex.get('eSignature.ack.label'),
            label1:electronicSignatureLex.get('eSignature.ack.label1'),
            label2:electronicSignatureLex.get('eSignature.ack.label2'),
            label3:electronicSignatureLex.get('eSignature.ack.label3'),
            label4:electronicSignatureLex.get('eSignature.ack.label4'),
            label5:electronicSignatureLex.get('eSignature.ack.label5'),
            label6:electronicSignatureLex.get('eSignature.ack.label6'),
            label7:electronicSignatureLex.get('eSignature.ack.label7')
        };
        return jsonObject;
    }

    return AcknowledgeCommand;
});
