/**
  * @copyright 2015 Tridium, Inc. All Rights Reserved.
  * @author Smita Rajge
  */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/commands/AddReasonCommand
 */
define(['baja!',
      'lex!webEditors,electronicSignature',
      'bajaux/commands/Command',
      'Promise',
      'underscore',
      'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
      'nmodule/js/rc/asyncUtils/asyncUtils',

      'nmodule/webEditors/rc/fe/feDialogs',
      'nmodule/webEditors/rc/fe/baja/util/typeUtils',
      'nmodule/webEditors/rc/wb/PropertySheet',
      'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
      'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
      'dialogs',
      'nmodule/electronicSignature/rc/wb/dialog/ManagerDialog',
      'nmodule/electronicSignature/rc/wb/mgr/editor/ReasonEditor',
     'baja!electronicSignature:ReasonSet,electronicSignature:Reasons',
     'nmodule/webEditors/rc/wb/mgr/model/columns/TypeMgrColumn'
      ], function (
       baja,
       lexs,
       Command,
       Promise,
       _,
       MgrTypeInfo,
       asyncUtils,

       feDialogs,
       typeUtils,
       PropertySheet,
       BatchComponentEditor,
       MgrModel,
       dialogs,
       ManagerDialog,
       ReasonEditor,
       types,TypeMgrColumn) {

  'use strict';

  // var lex = lexs[0],

    var webEditorsLex = lexs[0], lex = lexs[1], mapAsync = asyncUtils.mapAsync;


 function getReasonSet(handle) {
     var ord='station:|h:'+handle;
       return baja.Ord.make(ord).get({lease: true});
  }
/*  var replaceAt=function(str,index, replacement) {
    return str.substr(0, index) + replacement+ str.substr(index + replacement.length);
  };*/

  function zeroFill( number, width )
  {
    width -= number.toString().length;
    if ( width > 0 )
    {
      return new Array( width + (/\./.test( number ) ? 2 : 1) ).join( '0' ) + number;
    }
    return number + ""; // always return a string
  }

  /**
   * Returns a reason ID that is in specific sequence.
   * Like AA001, AA002,.. AA999
   * @param reasonSet
   * @return
   */
 /*function getNextReasonID(reasonSet)
  {
   if(reasonSet.getSlot("ReasonID")==null)
      {
       var reasonID1 = reasonSet.getReasonSetID().substring(0,2) + "001";
         reasonSet.add({
           slot: "ReasonID" ,
           value:baja.$("baja:String").make(reasonID1),
           flags: baja.Flags.READONLY,
        });
        return reasonID1;
      }
      else
      {
        var reasonID = reasonSet.getSlot("ReasonID").$val.toString();
        var r = reasonID;
        var valuePresent=false;
        reasonSet.getSlots().is("electronicSignature:Reasons").each(function(reason){
        if(reason.$val.getReasonID()==r)
         {
         valuePresent=true;
         }
        });
        if(valuePresent==false)
        {
        return reasonID;
        }
       var firstPart= reasonID.substring(0,2);
       var secondPart= parseInt(reasonID.substring(2))+1;
       var x=zeroFill(secondPart,3);
       var mergedString=firstPart+x;

       mergedString=baja.$("baja:String").make(mergedString);
       reasonSet.set({
             slot: "ReasonID" ,
             value:mergedString
          });
        return mergedString;
      }
  }
*/

function getNextReasonID(reasonSet)
  {
   if(reasonSet.getSlot("ReasonID") === null)
      {
        var reasonID1 = reasonSet.getReasonSetID().substring(0,2) + "001";
        return reasonSet.add({
           slot: "ReasonID" ,
           value:baja.$("baja:String").make(reasonID1),
           flags: baja.Flags.READONLY,
        }).then(function(){
          return reasonSet.loadSlots().then(function(){
             return reasonSet.getSlot("ReasonID").$val;
           });
        });
     //   return reasonID1;
      }
      else
      {
        var reasonID = reasonSet.getSlot("ReasonID").$val.toString();
        var r = reasonID;
        var valuePresent=false;
        reasonSet.getSlots().is("electronicSignature:Reasons").each(function(reason){
        if(reason.$val.getReasonID() === r)
         {
         valuePresent=true;
         }
        });
        if(valuePresent === false)
        {
         return Promise.resolve(reasonID);
        //return reasonID;
        }
       var firstPart= reasonID.substring(0,2);
       var secondPart= parseInt(reasonID.substring(2))+1;
       var x=zeroFill(secondPart,3);
       var mergedString=firstPart+x;

       mergedString=baja.$("baja:String").make(mergedString);
        return reasonSet.set({
           slot: "ReasonID" ,
           value:mergedString
          }).then(function(){
          return reasonSet.loadSlots().then(function(){
             return reasonSet.getSlot("ReasonID").$val;
          });
        });
        //return mergedString;
      }
  }
  ////////////////////////////////////////////////////////////////
  // Support functions
  ////////////////////////////////////////////////////////////////

  /**
  * To query the user for what type of new components to create, and how
  * many, a PropertySheet will be shown in a dialog. This component contains
  * the necessary fields to load into that PropertySheet.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel the
  * model we're adding new instances to
  * @returns {baja.Component}
  */
  function makeComponent(newTypes) {
    var newTypesEnum = baja.DynamicEnum.make({
        range: baja.EnumRange.make({
          ordinals: _.map(newTypes, function (type, i) { return i; }),
          tags: _.map(_.invoke(newTypes, 'getDisplayName'), baja.SlotPath.escape)
        })
      }),
      comp = baja.$('baja:Component', {
        newTypes: newTypesEnum,
        count: baja.Integer.make(1),
        displayNames: baja.NameMap.make({
          newTypes: webEditorsLex.get('commands.mgr.add.type'),
          count: webEditorsLex.get('commands.mgr.add.count')
        })
      });

    comp.setFacets({
      slot: 'count',
      facets: baja.Facets.make({ min: 1, max: 100 })
    });

    comp.setFlags({slot: 'displayNames', flags: baja.Flags.HIDDEN});

    return comp;
  }

  /**
  * When loading up the `BatchComponentEditor`, a secondary `MgrModel` will be
  * constructed solely for the components being edited. After editing the
  * components, they will be applied back to the original `MgrModel`.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel the
  * `MgrModel` to which we're adding components
  * @param {Object} params
  * @param {String} params.typeSpec the type of the components we're creating
  * @param {Number} params.count how many new components to create
  * @returns {Promise} promise to be resolved with a new
  * `MgrModel` containing the new instances to edit
  */
  function makeSubMgrModel(mgrModel, params) {


      var origContainer = mgrModel.getComponentSource().getContainer(),
          source = baja.$('baja:Folder'),
          typeInfo = params.typeInfo,
          uniqueName = origContainer.getUniqueName(typeInfo.toSlotName()),
          count = params.count,
          slotNames = [],
          slotName,
          slotNum = uniqueName.match(/\d*$/)[0] || 0,
          i;

      uniqueName = uniqueName.replace(/\d*$/, '');

      for (i = 0; i < count; i++) {
        while (origContainer.has(slotName = uniqueName + (slotNum || ''))) {
          slotNum++;
        }
        slotNames.push(slotName);
        slotNum++;
      }

      return getReasonSet(mgrModel.$componentSource.$container.$handle).then(function(reasonSetComp){
         return getNextReasonID(reasonSetComp).then(function(comp){
             var reasonId=comp;
               return mapAsync(slotNames, function (slotName) {
                return Promise.resolve(mgrModel.newInstance(typeInfo))
                  .then(function (instance) {
                    instance.setReasonID(reasonId);
                    instance.setReasonSetName(reasonSetComp.getReasonCollection());
                    return source.add({ slot: slotName, value: instance });
                  });
              });
            });
      }).then(function () {
        var newTypes = mgrModel.getNewTypes(), model;

        model = new MgrModel({
          componentSource: source,
          columns: mgrModel.getColumns()
        });

        _.each(model.getRows(), function (row) {
          TypeMgrColumn.setAvailableTypes(row, newTypes);
        });

        var obj={
                  mod:new MgrModel({
                   componentSource: source,
                   columns: mgrModel.getColumns()})
                };
        obj.reasonSet=mgrModel.$componentSource.$container.$handle;
        return obj;
      });
  }

  /**
  * Present the user with a prompt asking what type of components to add and
  * how many.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel
  * @returns {Promise} promise to be resolved with an object with
  * `typeSpec` and `count` properties, or `null` if the prompt was canceled
  */
  function promptForParams(mgrModel) {

      var newTypes = mgrModel.getNewTypes(), comp;

      MgrTypeInfo.markDuplicates(newTypes);
      comp = makeComponent(newTypes);
      var val={
        typeInfo: newTypes[comp.get('newTypes').getOrdinal()],
        count: comp.get('count').valueOf()
      };
     return Promise.resolve(val);
  }

  function promptForNewComponents(mgrModel, params,ordBase,that) {
    return makeSubMgrModel(mgrModel, params)
    .then(function (model) {
      return ManagerDialog.showFor({ value: model.mod, type: ReasonEditor, progressCallback: function (msg, arg) {
      switch(msg) {

         case 'loaded': return that.startValidation(arg.jq());
         }},reasonSetHandle:model.reasonSet,com:"Add"});
    })
    .then(function (model) {
      return model && _.map(model.getRows(), function (row) {
        return row.getSubject();
      });
    });
  }

  ////////////////////////////////////////////////////////////////
  // AddReasonCommand
  ////////////////////////////////////////////////////////////////

  /**
  * Command for adding new components to a Manager view.
  *
  * @private
  * @class
  * @alias module:nmodule/webEditors/rc/wb/mgr/commands/AddReasonCommand
  * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
  */
  var AddReasonCommand = function AddReasonCommand(manager) {
  var that=this;
  Command.call(this, {
   displayName:lex.get("commands.addReason.label"),
     /**
     * Add new components to the Manager view. First shows a dialog asking
     * for the type of component to add and how many, then a manager editor
     * allowing the components to be configured before adding.
     *
     * @alias module:nmodule/webEditors/rc/wb/mgr/comands/AddReasonCommand#invoke
     * @returns {Promise}
     */
  /*  func: function () {
      var mgrModel = manager.value();

      return promptForParams(mgrModel)
        .then(function (params) {
          return params && promptForNewComponents(mgrModel, params);
        })
        .then(function (comps) {
          return comps && mgrModel.addInstances(comps);
        })
        .catch(feDialogs.error);
    }*/
          func: function () {
            var mgrModel = manager.getModel();

            return manager.getOrdBase()
              .then(function (ordBase) {
                /*  setTimeout(function(){
                  that.startValidation();
                 }, 450);*/
                return promptForParams(mgrModel, ordBase)
                  .then(function (params) {
                    return params && promptForNewComponents(mgrModel, params, ordBase,that);
                  })
                  .then(function (comps) {
                    return comps && mgrModel.addInstances(comps);
                  });
              })
              .catch(feDialogs.error);
          }
  });
  };

  AddReasonCommand.prototype = Object.create(Command.prototype);
  AddReasonCommand.prototype.constructor = AddReasonCommand;
 AddReasonCommand.prototype.startValidation = function(dom)
  {

  jQuery(jQuery(dom).find("tr.key-reasonSetName").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line
  jQuery(jQuery(dom).find("tr.key-reasonID").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line
  // jQuery(jQuery("tr.key-reasonID").find("input[type='text']")[0]).attr('readonly', true);
    /*jQuery(jQuery("tr.key-reasonSetName").find("input[type='text']")[0]).attr('readonly', true);
    jQuery(jQuery("tr.key-reasonID").find("input[type='text']")[0]).attr('readonly', true);*/
  };


  return AddReasonCommand;
  });
