/**
  * @copyright 2015 Tridium, Inc. All Rights Reserved.
  * @author Smita Rajge
  */

  /**
  * @private
  * @module nmodule/webEditors/rc/wb/mgr/commands/AddReasonSetCommand
  */
define(['baja!',
      'lex!webEditors,electronicSignature',
      'bajaux/commands/Command',
      'Promise',
      'underscore',
      'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
      'nmodule/js/rc/asyncUtils/asyncUtils',
      'nmodule/webEditors/rc/fe/feDialogs',
      'nmodule/webEditors/rc/fe/baja/util/typeUtils',
      'nmodule/webEditors/rc/wb/PropertySheet',
      'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
      'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
      'dialogs',
      'nmodule/electronicSignature/rc/wb/dialog/ManagerDialog',
      'baja!electronicSignature:ReasonSet,electronicSignature:Reasons',
      'nmodule/webEditors/rc/wb/mgr/model/columns/TypeMgrColumn'], function (
       baja,
       lexs,
       Command,
       Promise,
       _,
       MgrTypeInfo,
       asyncUtils,
       feDialogs,
       typeUtils,
       PropertySheet,
       BatchComponentEditor,
       MgrModel,
       dialogs,
       ManagerDialog,
       types,
       TypeMgrColumn) {

  'use strict';

  // var lex = lexs[0],
  var webEditorsLex = lexs[0], lex = lexs[1], mapAsync = asyncUtils.mapAsync;
  var securedDashboardComp;


 /* function getSecuredDashBoardConfiguration() {
    return baja.Ord.make('station:|slot:/ElectronicSignature').get({lease: true});
  }*/
  var handleOrd;
   function getSecuredDashBoardConfiguration(handleOrd) {
        return baja.Ord.make(handleOrd).get({lease: true});
    }


  var replaceAt=function(str,index, replacement) {
    return str.substr(0, index) + replacement+ str.substr(index + replacement.length);
  };
  /**
   * Returns a reason set ID that is in specific sequence.
   * Like AA000, AB000,.. AZ000, BA000
   * @param config
   * @return
   */
var getNextReasonSetID = function(config)
  {
        if(config.getSlot("ReasonSetID") === null)
        {
          return config.add({
           slot: "ReasonSetID" ,
           value:baja.$("baja:String").make("AA000"),
           flags: baja.Flags.READONLY,
          }).then(function(){
            return config.loadSlots().then(function(){
               return config.getSlot("ReasonSetID").$val;
             });
          });
        }
        else
        {
          var ReasonSetID = config.getSlot("ReasonSetID").$val;
          var valuePresent=false;
          config.getSlots().is("electronicSignature:ReasonSet").each(function(reasonSet){
          if(reasonSet.$val.getReasonSetID() === ReasonSetID)
           {
           valuePresent=true;
           }
          });
          if(valuePresent === false)
          {
           return Promise.resolve(ReasonSetID);
          }
          var c = ReasonSetID.charAt(1);
           var nextChar,reasonSetVal;
          if(c !== 'Z')
          {
           nextChar= String.fromCharCode(ReasonSetID.charCodeAt(1)+1);
           ReasonSetID=replaceAt(ReasonSetID,1, nextChar);

            reasonSetVal=baja.$("baja:String").make(ReasonSetID);
           return config.set({
             slot: "ReasonSetID" ,
             value:reasonSetVal
           }).then(function(){
            return config.loadSlots().then(function(){
               return config.getSlot("ReasonSetID").$val;
             });
           });

          }
          else
          {
            nextChar= String.fromCharCode(ReasonSetID.charCodeAt(0)+1);
            nextChar= nextChar+"A000";
            reasonSetVal=baja.$("baja:String").make(nextChar);
          return  config.set({
               slot: "ReasonSetID" ,
               value:reasonSetVal
            }).then(function(){
              return config.loadSlots().then(function(){
                 return config.getSlot("ReasonSetID").$val;
               });
            });
          }
        }
  };

  ////////////////////////////////////////////////////////////////
  // Support functions
  ////////////////////////////////////////////////////////////////

  /**
  * To query the user for what type of new components to create, and how
  * many, a PropertySheet will be shown in a dialog. This component contains
  * the necessary fields to load into that PropertySheet.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel the
  * model we're adding new instances to
  * @returns {baja.Component}
  */
  function makeComponent(newTypes) {
    var newTypesEnum = baja.DynamicEnum.make({
        range: baja.EnumRange.make({
          ordinals: _.map(newTypes, function (type, i) { return i; }),
          tags: _.map(_.invoke(newTypes, 'getDisplayName'), baja.SlotPath.escape)
        })
      }),
      comp = baja.$('baja:Component', {
        newTypes: newTypesEnum,
        count: baja.Integer.make(1),
        displayNames: baja.NameMap.make({
          newTypes: webEditorsLex.get('commands.mgr.add.type'),
          count: webEditorsLex.get('commands.mgr.add.count')
        })
      });

    comp.setFacets({
      slot: 'count',
      facets: baja.Facets.make({ min: 1, max: 100 })
    });

    comp.setFlags({slot: 'displayNames', flags: baja.Flags.HIDDEN});

    return comp;
  }

  /**
  * When loading up the `BatchComponentEditor`, a secondary `MgrModel` will be
  * constructed solely for the components being edited. After editing the
  * components, they will be applied back to the original `MgrModel`.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel the
  * `MgrModel` to which we're adding components
  * @param {Object} params
  * @param {String} params.typeSpec the type of the components we're creating
  * @param {Number} params.count how many new components to create
  * @returns {Promise} promise to be resolved with a new
  * `MgrModel` containing the new instances to edit
  */
  function makeSubMgrModel(mgrModel, params) {

      var origContainer = mgrModel.getComponentSource().getContainer(),
          source = baja.$('baja:Folder'),
          typeInfo = params.typeInfo,
          uniqueName = origContainer.getUniqueName(typeInfo.toSlotName()),
          count = params.count,
          slotNames = [],
          slotName,
          slotNum = uniqueName.match(/\d*$/)[0] || 0,
          i;

      uniqueName = uniqueName.replace(/\d*$/, '');

      for (i = 0; i < count; i++) {
        while (origContainer.has(slotName = uniqueName + (slotNum || ''))) {
          slotNum++;
        }
        slotNames.push(slotName);
        slotNum++;
      }
      handleOrd= mgrModel.$componentSource.$container.getOrdInSession();
      return getSecuredDashBoardConfiguration(handleOrd).then(function(comp){
        securedDashboardComp=comp;
         return getNextReasonSetID(comp).then(function(comp){
             var reasnSetId=comp;
               return mapAsync(slotNames, function (slotName) {
                return Promise.resolve(mgrModel.newInstance(typeInfo))
                  .then(function (instance) {
                   instance.setReasonSetID(reasnSetId);
                    return source.add({ slot: slotName, value: instance });
                  });
              });
            });
      }).then(function () {
        var newTypes = mgrModel.getNewTypes(),
            model;

        model = new MgrModel({
          componentSource: source,
          columns: mgrModel.getColumns()
        });

        _.each(model.getRows(), function (row) {
          TypeMgrColumn.setAvailableTypes(row, newTypes);
        });

        return model;
      });
  }

  /**
  * Present the user with a prompt asking what type of components to add and
  * how many.
  *
  * @inner
  * @param {module:nmodule/webEditors/rc/wb/mgr/model/MgrModel} mgrModel
  * @returns {Promise} promise to be resolved with an object with
  * `typeSpec` and `count` properties, or `null` if the prompt was canceled
  */
  function promptForParams(mgrModel) {
    var newTypes = mgrModel.getNewTypes(), comp;

    MgrTypeInfo.markDuplicates(newTypes);
    comp = makeComponent(newTypes);
    var val={
      typeInfo: newTypes[comp.get('newTypes').getOrdinal()],
      count: comp.get('count').valueOf()
    };
   return Promise.resolve(val);
  }

  var validate=function(ed)
  {
    var errorMessage="";
    var index = 0;
      return new Promise( function(fulfill,reject){
         return getSecuredDashBoardConfiguration(handleOrd).then(function(securedDashboardComp){
           if(ed.$value.$rows[0].$data["proposed.__name"]!== undefined && ed.$value.$rows[0].$data["proposed.__name"] === "")
           {
           index = index +1;
           errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get("specify.name.message")+"</br>";
           }
           if(ed.$value.$rows[0].$data["proposed.__name"]!== undefined && securedDashboardComp.getSlot(baja.SlotPath.escape(ed.$value.$rows[0].$data["proposed.__name"])) !== null)
           {
             index = index +1;
             errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get("reasonSet.name.validation")+"</br>";
           }
           if(ed.$value.$rows[0].$data["proposed.reasonCollection"] === undefined || !(ed.$value.$rows[0].$data["proposed.reasonCollection"].length>0 && ed.$value.$rows[0].$data["proposed.reasonCollection"].length<50))
           {
             index = index +1;
             errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get("mgr.validation.reasonCollection.length")+"</br>";
           }
           fulfill(errorMessage);
          },
         function(err){
         reject(err);
         });
      });
  };
  function promptForNewComponents(mgrModel, params,ordBase, that) {
    return makeSubMgrModel(mgrModel, params)
    .then(function (model) {
      return ManagerDialog.showFor({ value: model, progressCallback: function (msg, arg) {
         switch(msg) {
            case 'loaded': return that.startValidation(arg.jq());
           }}, type: BatchComponentEditor,command:validate
      });
    })
    .then(function (model) {
      return model && _.map(model.getRows(), function (row) {
        return row.getSubject();
      });
    });
  }

  ////////////////////////////////////////////////////////////////
  // AddReasonSetCommand
  ////////////////////////////////////////////////////////////////

  /**
  * Command for adding new components to a Manager view.
  *
  * @private
  * @class
  * @alias module:nmodule/webEditors/rc/wb/mgr/commands/AddReasonSetCommand
  * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
  */
  var AddReasonSetCommand = function AddReasonSetCommand(manager) {
  var that=this;
  Command.call(this, {
   displayName:lex.get("commands.addReasonSet.label"),
     /**
     * Add new components to the Manager view. First shows a dialog asking
     * for the type of component to add and how many, then a manager editor
     * allowing the components to be configured before adding.
     *
     * @alias module:nmodule/webEditors/rc/wb/mgr/comands/AddReasonSetCommand#invoke
     * @returns {Promise}
     */
    func: function () {
      var mgrModel = manager.getModel();
      return manager.getOrdBase()
        .then(function (ordBase) {
          return promptForParams(mgrModel, ordBase).then(function (params) {
            return params && promptForNewComponents(mgrModel, params,ordBase, that);
          })
          .then(function (comps) {
            return comps && mgrModel.addInstances(comps);
          });
        }).catch(feDialogs.error);
    }
  });
  };


  AddReasonSetCommand.prototype = Object.create(Command.prototype);
  AddReasonSetCommand.prototype.constructor = AddReasonSetCommand;
 AddReasonSetCommand.prototype.startValidation = function(dom)
  {
   jQuery(jQuery(dom).find("tr.key-reasonSetID").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line
       //  jQuery(jQuery("tr.key-reasonSetID").find("input[type='text']")[0]).attr('readonly', true);
  };
  return AddReasonSetCommand;
  });
