/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
 */
define(['baja!',
        'underscore',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/fe/feDialogs',
        'lex!electronicSignature',
        'nmodule/electronicSignature/rc/wb/mgr/editor/CustomerEditor',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/table/model/source/ArrayComponentSource'], function (
         baja,
         _,
         Command,
         feDialogs,
         electronicSignatureLex,
         CustomerEditor,
         MgrModel,
         ArrayComponentSource) {

  'use strict';
  var lex = electronicSignatureLex[0];
  function getSelection(manager) {
    //return manager.$getMainTable().$getSelection();
      return manager.getMainTable().$getSelection();
  }

  function makeSubMgrModel(manager) {
      var mgrModel = manager.getModel(),
      selectedRows = manager.getMainTable().getSelectedRows(),
        comps = _.map(selectedRows, function (row) { return row.getSubject(); });

    return new MgrModel({
      componentSource: new ArrayComponentSource(comps),
      columns: mgrModel.getColumns()
    });
  }

  /**
   * Command for editing existing components in a Manager view.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var EditCommand = function EditCommand(manager) {
    var that = this;

    Command.call(that, {
      module: 'webEditors',
      lex: 'commands.mgr.edit',
      enabled: false,

      /**
       * Takes the components that are currently selected in the manager's
       * main table, and allows for editing them using a `BatchComponentEditor`.
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand#invoke
       * @returns {Promise}
       */
      func: function () {
        var subModel = makeSubMgrModel(manager);

        return manager.getOrdBase()
          .then(function (ordBase) {
            /*setTimeout(function(){
              that.startValidation();
            }, 250);*/
            return feDialogs.showFor({
              value: subModel,
              properties: { ordBase: ordBase },
              type: CustomerEditor
            }).then(function(editor){
              that.startValidation();
              return editor;
            });
          });
      }
    });

    manager.on('loaded', function () {
      var selection = getSelection(manager);
      selection.on('changed', function () {
        that.setEnabled(!selection.isEmpty());
      });
    });
  };
  EditCommand.prototype = Object.create(Command.prototype);
  EditCommand.prototype.constructor = EditCommand;

  EditCommand.prototype.setEnabled = function (enabled) {
    return Command.prototype.setEnabled.call(this, !baja.isOffline() && enabled);
  };

  EditCommand.prototype.startValidation = function()
  {
    jQuery(jQuery("tr.key-customerName").find("input[type='text']")[0]).attr("maxLength",100); // jshint ignore:line
    jQuery(jQuery("tr.key-customerName").find("input[type='text']")[0]).attr("maxLength",100); // jshint ignore:line
    jQuery(jQuery("tr.key-contactPersonName").find("input[type='text']")[0]).attr("maxLength",100); // jshint ignore:line
    jQuery(jQuery("tr.key-address").find("input[type='text']")[0]).attr("maxLength",200); // jshint ignore:line
    jQuery(jQuery("tr.key-customerPhoneNumber").find("input[type='text']")[0]).attr("maxLength",200); // jshint ignore:line

    jQuery("tr.key-__name").hide(); // jshint ignore:line
  };

  return EditCommand;
});
