/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Smita Rajge
 */

define(['baja!',
        'underscore',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/table/model/source/ArrayComponentSource',
        'nmodule/electronicSignature/rc/wb/dialog/ManagerDialog',
        'nmodule/electronicSignature/rc/wb/mgr/editor/ReasonEditor',
        'nmodule/webEditors/rc/fe/feDialogs',
        'lex!electronicSignature',
        'baja!electronicSignature:ReasonSet,electronicSignature:Reasons'], function (
         baja,
         _,
         Command,
         BatchComponentEditor,
         MgrModel,
         ArrayComponentSource,
         ManagerDialog,
         ReasonEditor,
         feDialogs, electronicSignatureLex,types) {

  'use strict';
  var lex = electronicSignatureLex[0];
  function getSelection(manager) {
    return manager.getMainTable().$getSelection();
  }

  function makeSubMgrModel(manager) {
      var mgrModel = manager.getModel(),
          selectedRows = manager.getMainTable().getSelectedRows(),
        comps = _.map(selectedRows, function (row) { return row.getSubject(); });

    return new MgrModel({
      componentSource: new ArrayComponentSource(comps),
      columns: mgrModel.getColumns()
    });
  }

  /**
   * Command for editing existing components in a Manager view.
   *
   * @private
   * @class
   * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var ReasonEditCommand = function ReasonEditCommand(manager) {
    var that = this;
    Command.call(that, {
      module: 'webEditors',
      lex: 'commands.mgr.edit',
      enabled: false,

      /**
       * Takes the components that are currently selected in the manager's
       * main table, and allows for editing them using a `BatchComponentEditor`.
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand#invoke
       * @returns {Promise}
       */
             func: function () {
               var mgrModel = manager.getModel(),
                   subModel = makeSubMgrModel(manager);

               return manager.getOrdBase()
                 .then(function (ordBase) {
                 /*  setTimeout(function(){
                     that.startValidation();
                   }, 250);*/
                   return ManagerDialog.showFor({
                     value: subModel,
                     properties: { ordBase: ordBase },
                     type: ReasonEditor,
                     progressCallback: function (msg, arg) {
                     switch(msg) {
                        case 'loaded':      return that.startValidation(arg.jq());
                      }
                      },
                      reasonSetHandle:mgrModel.$componentSource.$container.$handle,
                      com:"Edit"
                   });
                 });
             }
    });

    manager.on('loaded', function () {
      var selection = getSelection(manager);
      selection.on('changed', function () {
        that.setEnabled(!selection.isEmpty());
      });
    });
  };

  ReasonEditCommand.prototype.setEnabled = function (enabled) {
    return Command.prototype.setEnabled.call(this, !baja.isOffline() && enabled);
  };
  ReasonEditCommand.prototype = Object.create(Command.prototype);
  ReasonEditCommand.prototype.constructor = ReasonEditCommand;
  ReasonEditCommand.prototype.startValidation = function(dom)
  {
  /*  jQuery(jQuery("tr.key-reasonSetName").find("input[type='text']")[0]).attr('readonly', true);
    jQuery(jQuery("tr.key-reasonID").find("input[type='text']")[0]).attr('readonly', true);
*/
    jQuery(jQuery(dom).find("tr.key-reasonSetName").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line
     jQuery(jQuery(dom).find("tr.key-reasonID").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line

   };
  return ReasonEditCommand;
});
