/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Smita Rajge
 */

define(['baja!',
        'underscore',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/table/model/source/ArrayComponentSource',
        'nmodule/electronicSignature/rc/wb/dialog/ManagerDialog',
        'lex!electronicSignature',
        'baja!electronicSignature:ReasonSet,electronicSignature:Reasons'], function (
         baja,
         _,
         Command,
         BatchComponentEditor,
         MgrModel,
         ArrayComponentSource,
         ManagerDialog,electronicSignatureLex, types) {

  'use strict';
  var lex = electronicSignatureLex[0];
 function getSecuredDashBoardConfiguration() {
    return baja.Ord.make('station:|slot:/ElectronicSignature').get({lease: true});
 }

  function getSelection(manager) {
    return manager.getMainTable().$getSelection();
  }

  function makeSubMgrModel(manager) {
      var mgrModel = manager.getModel(),
          selectedRows = manager.getMainTable().getSelectedRows(),
        comps = _.map(selectedRows, function (row) { return row.getSubject(); });

    return new MgrModel({
      componentSource: new ArrayComponentSource(comps),
      columns: mgrModel.getColumns()
    });
  }

  function hasDuplicates(myArray) {
    return !(myArray.length === _.uniq(myArray).length);  // jshint ignore:line
  }


  var validate=function(ed){
    var errorMessage="";
    var index = 0;
    var invalidData=false;
    return new Promise( function(fulfill,reject){ // jshint ignore:line
      return getSecuredDashBoardConfiguration().then(function(securedDashboardComp){
        var reasonSetPresent=false;
        var nameEmpty=false;
        var testArray=[];
        ed.$value.$rows.forEach(function(row,index) {
          if(row.$data["proposed.__name"] === undefined)
          {
           testArray.push(ed.$value.$rows[index].$subject.$propInParent.$displayName);
          }else  if((row.$data["proposed.__name"] !== ""))
          {
          testArray.push(row.$data["proposed.__name"]);
          }
        });
          var duplicates=hasDuplicates(testArray);
        if(duplicates){
            index = index +1;
            errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get('reasonSet.duplicate.name')+"</br>";
        }
         ed.$value.$rows.forEach(function(row,index) {
           if(row.$data["proposed.__name"]!== undefined && row.$data["proposed.__name"] === "")
             {
             nameEmpty=true;
             }
           if(row.$data["proposed.__name"]!== undefined && securedDashboardComp.getSlot(baja.SlotPath.escape(row.$data["proposed.__name"])) !== null)
             {
                var comp=securedDashboardComp.getSlot(baja.SlotPath.escape(row.$data["proposed.__name"]));

               if(ed.$value.$componentSource.$container[index].$handle !== comp.$val.$handle)
               {
                   reasonSetPresent=true;
               }
             }
         });
         if(nameEmpty === true)
            {
              index = index +1;
              errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get('specify.name.message')+"</br>";
            }
         if(reasonSetPresent === true)
           {
           index = index +1;
           errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get('reasonSet.name.validation')+"</br>";
           }

        ed.$value.$rows.forEach(function(row) {
           if(row.$data["proposed.reasonCollection"]!==undefined &&
           !(row.$data["proposed.reasonCollection"].length>0 && row.$data["proposed.reasonCollection"].length<50))
            {
             invalidData=true;
            }
        });
        if(invalidData === true)
        {
        index = index +1;
        errorMessage+= "<b>"+ index +"</b>" + ". "+lex.get('mgr.validation.reasonCollection.length')+"</br>";
        }
        fulfill(errorMessage);
      },
      function(err){
        reject(err);
      });
    });
   // return errorMessage;
  };
  /**
   * Command for editing existing components in a Manager view.
   *
   * @private
   * @class
   * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var ReasonSetEditCommand = function ReasonSetEditCommand(manager) {
    var that = this;
    Command.call(that, {
      module: 'webEditors',
      lex: 'commands.mgr.edit',
      enabled: false,

      /**
       * Takes the components that are currently selected in the manager's
       * main table, and allows for editing them using a `BatchComponentEditor`.
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand#invoke
       * @returns {Promise}
       */
      func: function () {
        var subModel = makeSubMgrModel(manager);
            /*setTimeout(function(){
                   that.startValidation();
                 }, 250);*/
        return ManagerDialog.showFor({
          value: subModel,
          type: BatchComponentEditor,
          progressCallback: function (msg, arg) {
           switch(msg) {
              case 'loaded':      return that.startValidation(arg.jq());
              }},
           command:validate
           });
      }
    });

    manager.on('loaded', function () {
      var selection = getSelection(manager);
      selection.on('changed', function () {
        that.setEnabled(!selection.isEmpty());
      });
    });
  };

  ReasonSetEditCommand.prototype.setEnabled = function (enabled) {
    return Command.prototype.setEnabled.call(this, !baja.isOffline() && enabled);
  };
  ReasonSetEditCommand.prototype = Object.create(Command.prototype);
  ReasonSetEditCommand.prototype.constructor = ReasonSetEditCommand;
 ReasonSetEditCommand.prototype.startValidation = function(dom)
  {
 //   jQuery(jQuery("tr.key-reasonSetID").find("input[type='text']")[0]).attr('readonly', true);
 jQuery(jQuery(dom).find("tr.key-reasonSetID").find("input[type='text']")[0]).attr('readonly', true); // jshint ignore:line

  };
  return ReasonSetEditCommand;
});
