/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * API Status: **Private**
 * @module nmodule/alarm/rc/db/editors/RunCommand
 */
define([ 'baja!',
    'baja!alarm:ArchiveAlarmProvider',
    'jquery',
    'underscore',
    'Promise',
    'nmodule/webEditors/rc/fe/fe',
    'lex!alarm,electronicSignature',
    'dialogs',
    'nmodule/alarm/rc/Alarm',
    'nmodule/alarm/rc/console/baja/BajaDataChannel',
    'nmodule/electronicSignature/rc/fe/baja/Util',
    'bajaux/commands/Command',
    'bajaux/util/CommandButton',
    'hbs!nmodule/alarm/rc/db/templates/dialog',
    'hbs!nmodule/electronicSignature/rc/fe/template/SecuredRunCommandTemplate',
    'nmodule/electronicSignature/rc/jsFiles/bootstrap.min',
    'css!nmodule/alarm/rc/db/templates/view',
    'css!nmodule/electronicSignature/rc/css/widgets'
], function (baja, types, $, _, Promise, fe, lexs, dialogs, Alarm, BajaDataChannel, Util,Command, CommandButton, dialogTemplate, authenticationTemplate,
             bootsrtap) {

    "use strict";

    var lex                             = lexs[0],
        eSignLex                        = lexs[1],
        WARNING_ICON                    = lex.getSafe('alarm.db.warningIcon'),
        RUN_MAINTENANCE_BUTTON_SELECTOR = '.run',
        ALARM_TABLE_SELECTOR            = '.alarmTable',
        RUN_CMD_LABEL                   = lex.get('command.runMaintenance.label'),
        CONFIRM_CLEAR_MSG               = lex.get('confirmClear.title'),
        COMMAND_SUCCESS_MSG             = lex.getSafe('alarm.db.successMessage'),
        COMMAND_ERROR_MSG               = lex.getSafe('alarm.db.errorMessage'),
        INVALID_TIME_TITLE              = lex.get('alarm.db.invalidTime'),
        NO_RECORD_SELECTED              = lex.getSafe('alarm.noRecordSelected'),
        CANNOT_BE_UNDONE                = lex.get('cannotBeUndone'),
        CONTINUE                        = lex.get('continue'),
        DELAY                           = 1500,
        warningIcon                     = baja.Icon.make(WARNING_ICON).getImageUris()[0],
        operations                      = {
            'clearOld': { op: Alarm.clearOldRecords, arg: null },
            'clearSelected': { op: Alarm.clearRecords, arg: null },
            'clearAllBeforeSelected': { op: Alarm.clearOldRecords, arg: null },
            'clearAll': { op: Alarm.clearAllRecords, arg: null }
        },
        operation,
        selectedRows,
        secureDashboardConfigObj          = null,
        isAuthenticatedUser = false,
        _SecondaryAuthenticatorRoles = "",
        isValidUserSecondary = false,
        legalTextTemplate ="",
        customerName ="",
        jsonObject={
        authLevel1:eSignLex.get('fe.authorization.level1.label'),
        authLevel2:eSignLex.get('fe.authorization.level2.label'),
        userName:eSignLex.get('fe.userName.label'),
        password:eSignLex.get('fe.password.label'),
        comment:eSignLex.get('fe.comment.label'),
        sign:eSignLex.get('fe.sign1.button'),
        sign2:eSignLex.get('fe.sign2.button'),
        acknowledgement:eSignLex.get('eSignature.ack.label'),
        label1:eSignLex.get('eSignature.ack.label1'),
        label2:eSignLex.get('eSignature.ack.label2'),
        label3:eSignLex.get('eSignature.ack.label3'),
        label4:eSignLex.get('eSignature.ack.label4'),
        label5:eSignLex.get('eSignature.ack.label5'),
        label6:eSignLex.get('eSignature.ack.label6'),
        label7:eSignLex.get('eSignature.ack.label7')
    };

    var RunCommand = function () {
    };

    RunCommand.makeFor = function (view, isAlarmArchiveSpace) {
        return fe.buildFor({    
            dom: $(RUN_MAINTENANCE_BUTTON_SELECTOR, view.jq()),
            type: CommandButton,
            value: new Command({
                displayName: RUN_CMD_LABEL,
                func: function () {
                    return new Promise(function () {
                        /*jshint -W030 */
                        var key          = $("input[type='radio']:checked", view.jq()).val(),
                            operation    = operations[key],
                            uuids,
                            table        = $(ALARM_TABLE_SELECTOR, view.jq()).data('widget'),
                            model        = table.getModel(),
                            selectedRows = table.$getSelection().getSelectedElements(model.getRows()),
                            allRows = model.getRows(),
                            dialogContent;

                        commitOperation();

                        function createArgs() {
                            var args = [];

                            //push the function arg if the current op has one
                            if (operation.arg) {
                                args.push(operation.arg);
                            }

                            //push the callback object
                            args.push({
                                ok: function () {
                                    dialogs.showOk({
                                        content: COMMAND_SUCCESS_MSG,
                                        ok: function () {
                                            //NCCB-16175: Clear any selected rows
                                            table.$getSelection().clear();

                                            //update the table
                                            if (view.pager) {
                                                view.pager.update();
                                            }
                                        }
                                    });
                                },

                                fail: function () {
                                    dialogs.showOk({ content: COMMAND_ERROR_MSG });
                                }
                            });

                            args.push(isAlarmArchiveSpace);
                            return args;
                        }

                        async function clearRecords() {
                            await createUUIDForSelectedRecords();
                            var args = createArgs();

                            dialogs.showYesNo({
                                title: CONFIRM_CLEAR_MSG,
                                content: dialogContent,
                                yes: function () {
                                    dialogs.showLoading(DELAY, operation.op.apply(null, args));
                                    Util.clearAuthenticationRelatedParameters();
                                }
                            });
                        }

                        async function useDoubleAuthenticationToClearRecords(primaryUsername) {
                            if(key === "clearAllBeforeSelected"){
                                if(selectedRows.length > 1){
                                    dialogs.showOk({
                                        content: lex.getSafe('alarm.multipleRecords')
                                    });
                                    return;
                                }
                            }

                            if (operation.op === Alarm.clearOldRecords || operation.op === Alarm.clearAllRecords) {
                                await updateSelectedRecordListAccordingToOperationType();
                            }
                            var jaceOrdDetails = new Map();
                            if (selectedRows.length) {
                                selectedRows.forEach(function (row) {
                                    Util.updateJaceOrdDetail(row.getSubject(), jaceOrdDetails);
                                });
                            } else {
                                dialogs.showOk({ content: NO_RECORD_SELECTED });
                                return;
                            }
                            await Util.getPointAuthenticationLevel(jaceOrdDetails);

                            if (Util.getAlarmAuthenticationLevel() === NORMAL) {
                                await clearRecords();
                            } else {
                                dialogs.showOkCancel({ title: eSignLex.get("authenticate.user.title"), content: authenticationTemplate({ jsonObject: jsonObject }) }).ok(async function (val) {
                                    await clearRecords();
                                });


                                $(document).find("#secondaryUsername").change(function () {
                                    Util.changeLegalText($(document).find("#secondaryUsername").val(), secureDashboardConfigObj);
                                });
                                if (Util.getAlarmAuthenticationLevel() === PRIMARY) {
                                    $('#secondTab').hide();
                                }
                                $(document).find("#username").val(primaryUsername);
                                $(document).find('.js-dialog-button-ok').attr('disabled', true);

                                customerName = secureDashboardConfigObj.getCustomer().getCustomerName();
                                Util.changeLegalText(primaryUsername, secureDashboardConfigObj);

                                $(document).find("#btnPrimarySign").click(function () {
                                    var credentials = {};
                                    credentials.username = $('#username').val();
                                    credentials.password = $('#password').val();
                                    Util.checkPrimarySignAndUpdateResultList(credentials, secureDashboardConfigObj, Util.getResultList());
                                });

                                $(document).find(".btnSecondarySign").click(function () {
                                    var credentials = {};
                                    credentials.username = $('#secondaryUsername').val();
                                    credentials.primaryUserName = $('#username').val();
                                    credentials.password = $('#secondaryPassword').val();
                                    Util.checkSecondarySignAndUpdateResultList(credentials, secureDashboardConfigObj);
                                });

                                $(document).find(".primaryTab").click(function (e) {
                                    e.preventDefault();
                                });

                                $(document).find(".secondaryTab").click(function (e) {
                                    e.preventDefault();
                                });
                            }
                        }

                        async function useSingleAuthenticationToClearRecords(primaryUsername){
                            if (operation.op === Alarm.clearOldRecords) {
                                if (key === "clearAllBeforeSelected") {
                                    if (selectedRows.length) {
                                        if (selectedRows.length > 1) {
                                            dialogs.showOk({
                                                content: lex.getSafe('alarm.multipleRecords')
                                            });
                                            return;
                                        }
                                    } else {
                                        dialogs.showOk({ content: NO_RECORD_SELECTED });
                                        return;
                                    }
                                }
                            } else if (operation.op === Alarm.clearRecords) {
                                if (!selectedRows.length) {
                                    dialogs.showOk({ content: NO_RECORD_SELECTED });
                                    return;
                                }
                            }
                            dialogs.showOkCancel({title:eSignLex.get("authenticate.user.title"),content:authenticationTemplate({jsonObject:jsonObject})}).ok(function(val){
                                if (operation.op === Alarm.clearOldRecords) {
                                    clearOldRecords();
                                }
                                else if (operation.op === Alarm.clearRecords) {
                                    clearSelectedRecords();
                                }
                                else {
                                    clearAllRecords();
                                }

                                var args = createArgs();
                                dialogs.showYesNo({
                                    title: CONFIRM_CLEAR_MSG,
                                    content: dialogContent,
                                    yes: function () {
                                        dialogs.showLoading(DELAY, operation.op.apply(null, args));
                                    }
                                });
                            });

                            $(document).find("#username").val(primaryUsername);
                            $(document).find('.js-dialog-button-ok').attr('disabled', true);
                            $('#secondTab').hide();

                            customerName = secureDashboardConfigObj.getCustomer().getCustomerName();
                            Util.changeLegalText(primaryUsername,secureDashboardConfigObj);

                            $(document).find("#btnPrimarySign").click(function ()
                            {
                                var credentials = {};
                                credentials.username = $('#username').val();
                                credentials.password = $('#password').val();
                                checkPrimarySign(credentials, secureDashboardConfigObj);
                            });

                            $(document).find(".btnSecondarySign").click(function ()
                            {
                                var credentials = {};
                                credentials.username = $('#secondaryUsername').val();
                                credentials.primaryUserName = $('#username').val();
                                credentials.password = $('#secondaryPassword').val();
                                checkSecondarySign(credentials, secureDashboardConfigObj);
                            });

                            $(document).find(".primaryTab").click(function (e){
                                e.preventDefault();
                            });

                            $(document).find(".secondaryTab").click(function (e){
                                e.preventDefault();
                            });
                        }

                        async function commitOperation() {
                            Util.clearAuthenticationRelatedParameters();
                            var primaryUsername = baja.getUserName();
                            secureDashboardConfigObj.lease();
                            secureDashboardConfigObj.loadSlots();
                            var eSignatureComponent = await Util.getESignatureComponent();
                            var alarmClasses = getAlarmClasses(table, view);
                            return secureDashboardConfigObj.invoke({
                                slot: "getAlarmSpacePermission",
                                value: alarmClasses
                            }).then(async function (hasAdminReadWritePermission) {
                                if (hasAdminReadWritePermission) {
                                    if (eSignatureComponent.getUseSingleAuthenticationForDatabaseMaintenanceView()) {
                                        await useSingleAuthenticationToClearRecords(primaryUsername);
                                    } else {
                                        await useDoubleAuthenticationToClearRecords(primaryUsername);
                                    }
                                } else {
                                    return dialogs.showOk({
                                        title: eSignLex.get("custom.error.message"),
                                        content: eSignLex.get("eSignature.secondaryAuthenticators.configure.l3Error")
                                    });
                                }
                            });
                        }

                        function clearOldRecords() {
                            if (!view.dateEditor) {
                                return;
                            }

                            view.dateEditor.read().then(function (beforeTime) {
                                switch (key) {
                                    case "clearOld":
                                        operation.arg = beforeTime.encodeToString();
                                        dialogContent = dialogTemplate({
                                            icon: warningIcon,
                                            message: lex.get({ key: 'alarm.db.clearBeforeTimestamp', args: [ BajaDataChannel.createDateString(beforeTime.getJsDate()) ] }),
                                            cannotBeUndone: CANNOT_BE_UNDONE,
                                            continue: CONTINUE
                                        });
                                        break;

                                    case "clearAllBeforeSelected":
                                        if (table) {
                                            if (selectedRows.length) {

                                                if (selectedRows.length > 1) {
                                                    dialogs.showOk({
                                                        content: lex.getSafe('alarm.multipleRecords')
                                                    });
                                                    return;
                                                } else {
                                                    operation.arg = selectedRows[0].getSubject().get('timestamp').encodeToString();
                                                }
                                            } else {
                                                dialogs.showOk({ content: NO_RECORD_SELECTED });
                                                return;
                                            }

                                            dialogContent = dialogTemplate({
                                                icon: warningIcon,
                                                message: lex.get('alarm.db.clearOlderAlarms'),
                                                cannotBeUndone: CANNOT_BE_UNDONE,
                                                continue: CONTINUE
                                            });
                                        }
                                        break;

                                    default:
                                        operation.arg = null;
                                }

                            }).catch(function (err) {
                                dialogs.showOk({
                                    title: INVALID_TIME_TITLE,
                                    content: err
                                });
                            });
                        }

                        function clearSelectedRecords() {
                            uuids = [];

                            if (selectedRows.length) {
                                selectedRows.forEach(function (row) {
                                    uuids.push(row.getSubject().get('uuid').encodeToString());
                                });
                            } else {
                                dialogs.showOk({ content: NO_RECORD_SELECTED });
                                return;
                            }

                            operation.arg = uuids;

                            dialogContent = dialogTemplate({
                                icon: warningIcon,
                                message: lex.get('alarm.db.clearSelectedAlarms'),
                                cannotBeUndone: CANNOT_BE_UNDONE,
                                continue: CONTINUE
                            });

                        }

                        function clearAllRecords() {

                            if (!model.getRows().length) {
                                dialogs.showOk(lex.getSafe('alarm.db.isEmpty'));
                                return;
                            }

                            //clear all alarms
                            dialogContent = dialogTemplate({
                                icon: warningIcon,
                                message: lex.get('alarm.db.clearAllAlarms'),
                                cannotBeUndone: CANNOT_BE_UNDONE,
                                continue: CONTINUE
                            });
                        }

                        function updateSelectedRecordListAccordingToOperationType(){
                            var ListOfSelectedRecordsToClear = [] ;
                            switch (key) {
                                case "clearOld":
                                    return view.dateEditor.read().then(function (beforeTime) {
                                        allRows.forEach(function(row){
                                            if(row.getSubject().get('timestamp').getMillis() < beforeTime.getMillis()){
                                                ListOfSelectedRecordsToClear.push(row);
                                            }
                                        });
                                        dialogContent = dialogTemplate({
                                            icon: warningIcon,
                                            message: lex.get({ key: 'alarm.db.clearBeforeTimestamp', args: [BajaDataChannel.createDateString(beforeTime.getJsDate())] }),
                                            cannotBeUndone: CANNOT_BE_UNDONE,
                                            continue: CONTINUE
                                        });
                                        selectedRows = [...ListOfSelectedRecordsToClear];
                                    });
                                   /* break;*/

                                case "clearAllBeforeSelected":
                                    if (table) {
                                        if (selectedRows.length) {

                                            if (selectedRows.length > 1) {
                                                dialogs.showOk({
                                                    content: lex.getSafe('alarm.multipleRecords')
                                                });
                                                return;
                                            } else {
                                                allRows.forEach(function (row) {
                                                    if (row.getSubject().get('timestamp').getMillis() < selectedRows[ 0 ].getSubject().get('timestamp').getMillis()) {
                                                        ListOfSelectedRecordsToClear.push(row);
                                                    }
                                                })
                                            }
                                        }
                                        dialogContent = dialogTemplate({
                                            icon: warningIcon,
                                            message: lex.get('alarm.db.clearOlderAlarms'),
                                            cannotBeUndone: CANNOT_BE_UNDONE,
                                            continue: CONTINUE
                                        });
                                    }
                                    break;

                                case "clearAll":
                                    if (!model.getRows().length) {
                                        dialogs.showOk(lex.getSafe('alarm.db.isEmpty'));
                                        return;
                                    }
                                    //clear all alarms
                                    dialogContent = dialogTemplate({
                                        icon: warningIcon,
                                        message: lex.get('alarm.db.clearAllAlarms'),
                                        cannotBeUndone: CANNOT_BE_UNDONE,
                                        continue: CONTINUE
                                    });
                                    ListOfSelectedRecordsToClear = [...allRows];
                            }
                            selectedRows = [...ListOfSelectedRecordsToClear];
                        }

                        async function createUUIDForSelectedRecords() {
                            uuids = [];
                            operation = operations["clearSelected"];
                            var authenticatedSelectedAlarms = Util.successAlarmRecordsForjaces();
                            if (authenticatedSelectedAlarms.length) {
                                authenticatedSelectedAlarms.forEach(function (row) {
                                    uuids.push(row.get('uuid').encodeToString());
                                });
                            }

                            //uuids.push(row.getSubject().get('uuid').encodeToString());

                            operation.arg = uuids;

                            if(key === "clearSelected") {
                                dialogContent = dialogTemplate({
                                    icon: warningIcon,
                                    message: lex.get('alarm.db.clearSelectedAlarms'),
                                    cannotBeUndone: CANNOT_BE_UNDONE,
                                    continue: CONTINUE
                                });
                            }

                        }



                    });
                }
            })
        });
    };

    RunCommand.setObj = function (dashboardObj) {
        secureDashboardConfigObj = dashboardObj;
    };

    function getAlarmClasses(table, view) {
        return "defaultAlarmClass";
    }

    function authenticateSecondaryUser(component, userName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUserDetails," + userName.toString())
        });
    }

    function checkSecondarySign(credentials, pointComponent)
    {
        isAuthenticatedUser = false;
        if ((credentials.username).toLowerCase() === (credentials.primaryUserName).toLowerCase())
        {
            return dialogs.showOk({
                title: eSignLex.get("authentication.error.title"),
                content: eSignLex.get("level2.authentication.message")
            });
        }
        var validation = doIsDataValid("secondary");
        if (validation !== null && validation.length > 0)
        {
            showValidationSummary(validation);
        }
        else
        {
            var pointOrd = getFormatedPointOrd(pointComponent),
                pointComp,
                isValidManager = false;
            if (pointOrd !== "")
            {
                getComponentByOrd(pointOrd).then(function (comp)
                {
                    pointComp = comp;
                    $(document).find(".btnSecondarySign").html(eSignLex.get('fe.signing.button'));
                    $(document).find(".btnSecondarySign").prop("disabled", true);
                    return authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp)
                {
                    $(document).find(".btnSecondarySign").html(eSignLex.get('fe.sign.button'));
                    $(document).find(".btnSecondarySign").prop("disabled", false);
                    isValidManager = authenticatedComp;
                    if (authenticatedComp)
                    {
                        return authenticateSecondaryUser(pointComp, credentials.username);
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: eSignLex.get("authentication.error.title"),
                            content: eSignLex.get("authentication.error.message")
                        });
                    }
                }).then(function (secondaryUserDetails)
                {
                    //secondaryUserDetails.loadSlots();
                    return secondaryUserDetails;
                }).then(function (secondaryUserDetails)
                {
                    if (isValidManager === true)
                    {
                        var managerRoles = _SecondaryAuthenticatorRoles.split(",");
                        isValidUserSecondary = false;
                        if (secondaryUserDetails.get("roles") !== null)
                        {
                            var secondaryUsersRoles = secondaryUserDetails.get("roles").split(",");
                            _.each(managerRoles, function (value)
                            {
                                var roleExistList = _.filter(secondaryUsersRoles, function (role)
                                {
                                    return role === value;
                                });
                                if (_.size(roleExistList) > 0)
                                {
                                    isValidUserSecondary = true;
                                }
                            });
                            if (!isValidUserSecondary)
                            {
                                isValidManager = false;
                                return dialogs.showOk({
                                    title: eSignLex.get("authentication.error.title"),
                                    content: eSignLex.get("secUser.role.error")
                                });
                            }
                        }
                        else
                        {
                            isValidUserSecondary = true;
                            //isValidManager=true;
                        }
                        if (isValidManager && isValidUserSecondary)
                        {
                            $('.js-dialog-button-ok').attr('disabled', false);
                            isAuthenticatedUser = true;
                            return dialogs.showOk({
                                title: eSignLex.get("auth.success.title"),
                                content: eSignLex.get("auth.success.msg")
                            });
                        }
                    }
                })
                    .catch(function (error)
                    {
                        $(document).find(".btnSecondarySign").html(eSignLex.get('fe.sign.button'));
                        $(document).find(".btnSecondarySign").prop("disabled", false);
                        console.log(error); // jshint ignore:line
                    });
            }
            else
            {
                return dialogs.showOk({
                    title: eSignLex.get("authentication.error.title"),
                    content: eSignLex.get("authentication.error.message")
                });
            }
        }
    }

    function showValidationSummary(validationSummary)
    {
        var errorMessage = "";
        var index = 0;
        for (var i = 0; i < validationSummary.length; i++)
        {
            $("#" + validationSummary[i].id).addClass("errorMsg");
            if (validationSummary[i].errorMsg !== "")
            {
                index = index + 1;
                errorMessage += "<b>" + index + "</b>" + ". " + validationSummary[i].errorMsg + "</br>";
            }
        }
        return dialogs.showOk({
            title: eSignLex.get("error.massages.title"),
            content: errorMessage
        });
    }

    function getFormatedPointOrd(component)
    {
        var pointOrd = "";

        pointOrd = "station:|h:" + component.getHandle().toString();

        return pointOrd;
    }

    function getComponentByOrd(ord)
    {
        return baja.Ord.make(ord).get({lease: true});
    }

    function authenticateUser(component, credentials)
    {
        return new Promise(function (resolve, reject) {
            component.invoke({
                slot: "call",
                value: baja.$("baja:String").make("authenticateUser," + credentials.toString())
            }).then(function (authenticatedComp)
            {
                if(authenticatedComp.getSlot("authenticationJobOrd")===null)
                {
                    resolve(authenticatedComp.get("isValidUser"));
                }
                else
                {
                    let getJobStatus = ()=>{
                        component.invoke({
                            slot: "call",
                            value: baja.$("baja:String").make("LDAPAuthentication," + authenticatedComp.get("authenticationJobOrd").toString())
                        }).then(function(returnComponent){
                            if(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("running"))
                            {
                                //schedule the timeout.
                                setTimeout(function (){
                                    getJobStatus();
                                },1000);
                            }
                            else
                            {
                                resolve(returnComponent.get("jobStatus")===baja.$("baja:JobState").get("success"));
                            }
                        });
                    };
                    setTimeout(function (){
                        getJobStatus();
                    },1000);
                }
            }).catch(function(err){
                console.log("Error occurred while authenticating the user.", err);
                reject();
            });
        });
    }


    function checkPrimarySign(credentials, pointComponent)
    {
        isAuthenticatedUser = false;
        var that = this,
            validation = doIsDataValid("primary");
        if (validation !== null && validation.length > 0)
        {
            showValidationSummary(validation);
        }
        else
        {
            var pointOrd = getFormatedPointOrd(pointComponent);
            if (pointOrd !== "")
            {
                getComponentByOrd(pointOrd).then(function (comp)
                {

                    $(document).find("#btnPrimarySign").html(eSignLex.get('fe.signing.button'));
                    $(document).find("#btnPrimarySign").prop("disabled", true);
                    return authenticateUser(comp, baja.$("baja:String").make(credentials.username + ";" + btoa(credentials.password)));
                }).then(function (authenticatedComp)
                {
                    $(document).find("#btnPrimarySign").html(eSignLex.get('fe.sign.button'));
                    $(document).find("#btnPrimarySign").prop("disabled", false);
                    if (authenticatedComp)
                    {
                        isAuthenticatedUser = true;
                        $('.js-dialog-button-ok').attr('disabled', false);
                        Util.changeLegalText($("#secondaryUsername").val(),pointComponent);
                    }
                    else
                    {
                        return dialogs.showOk({
                            title: eSignLex.get("error.massages.title"),
                            content: eSignLex.get("authentication.error.message")
                        });
                    }
                }).catch(function (error)
                {
                    $(document).find("#btnPrimarySign").html(eSignLex.get('fe.sign.button'));
                    $(document).find("#btnPrimarySign").prop("disabled", false);
                    console.log(error); // jshint ignore:line
                });
            }
        }
    }

    function doIsDataValid(authenticationType)
    {
        var invalidList = [];
        var invalidListObj = {};
        if (authenticationType === "primary")
        {
            if ($('#username').val() === null || $('#username').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "username";
                invalidListObj.errorMsg = eSignLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#username').removeClass("errorMsg");
            }
            if ($('#password').val() === null || $('#password').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "password";
                invalidListObj.errorMsg = eSignLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#password').removeClass("errorMsg");
            }
            return invalidList;
        }
        if (authenticationType === "secondary")
        {

            if ($('#secondaryUsername').val() === null || $('#secondaryUsername').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryUsername";
                invalidListObj.errorMsg = eSignLex.get("parent.editor.userName.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryUsername').removeClass("errorMsg");
            }
            if ($('#secondaryPassword').val() === null || $('#secondaryPassword').val() === "")
            {
                invalidListObj = {};
                invalidListObj.id = "secondaryPassword";
                invalidListObj.errorMsg = eSignLex.get("parent.editor.password.mandatory");
                invalidList.push(invalidListObj);
            }
            else
            {
                $('#secondaryPassword').removeClass("errorMsg");
            }
            return invalidList;
        }
    }

    function fetchSecondaryUser(component, currentUser, slotName)
    {
        return component.invoke({
            slot: "call",
            value: baja.$("baja:String").make("getSecondaryUsers," + currentUser + "," + slotName)
        });
    }

    return RunCommand;
});



