/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Balaji R Gite
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
 */
define(['baja!',
        'underscore',
        'jquery',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/fe/feDialogs',
        'lex!electronicSignature,webEditors',
        'nmodule/electronicSignature/rc/wb/dialog/ManagerDialog',
        'nmodule/electronicSignature/rc/wb/mgr/editor/ZoneEditor',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/table/model/source/ArrayComponentSource',
        'baja!electronicSignature:SecuredNumericWritable,electronicSignature:Zone',
        'nmodule/webEditors/rc/wb/mgr/model/columns/TypeMgrColumn',
        'nmodule/webEditors/rc/wb/mgr/MgrTypeInfo',
        'nmodule/js/rc/asyncUtils/asyncUtils',
        ], function (
         baja,
         _,
         $,
         Command,
         feDialogs,
         electronicSignatureLex,
         ManagerDialog,
         ZoneEditor,
         MgrModel,
         ArrayComponentSource,
         types,
         TypeMgrColumn,MgrTypeInfo,asyncUtils) {

  'use strict';
  var lex = electronicSignatureLex[0],webEditorsLex = electronicSignatureLex[1], mapAsync = asyncUtils.mapAsync;
    console.log("Inside Zone add comp");
  function getSelection(manager) {
    return manager.getMainTable().$getSelection();
  }
    function makeComponent(newTypes) {
        var newTypesEnum = baja.DynamicEnum.make({
                range: baja.EnumRange.make({
                    ordinals: _.map(newTypes, function (type, i) { return i; }),
                    tags: _.map(_.invoke(newTypes, 'getDisplayName'), baja.SlotPath.escape)
                })
            }),
            comp = baja.$('baja:Component', {
                newTypes: newTypesEnum,
                count: baja.Integer.make(1),
                displayNames: baja.NameMap.make({
                    newTypes: webEditorsLex.get('commands.mgr.add.type'),
                    count: webEditorsLex.get('commands.mgr.add.count')
                })
            });

        comp.setFacets({
            slot: 'count',
            facets: baja.Facets.make({ min: 1, max: 100 })
        });

        comp.setFlags({slot: 'displayNames', flags: baja.Flags.HIDDEN});

        return comp;
    }

    function promptForParams(mgrModel) {

        var newTypes = mgrModel.getNewTypes(), comp;

        MgrTypeInfo.markDuplicates(newTypes);
        comp = makeComponent(newTypes);
        var val={
            typeInfo: newTypes[comp.get('newTypes').getOrdinal()],
            count: comp.get('count').valueOf()
        };
        return Promise.resolve(val);
    }

    function getZone(handle) {
        var ord='station:|h:'+handle;
        return baja.Ord.make(ord).get({lease: true});
    }

    function makeSubMgrModel(mgrModel, params) {


        var origContainer = mgrModel.getComponentSource().getContainer(),
            source = baja.$('electronicSignature:Zone'),
            typeInfo = params.typeInfo,
            uniqueName = origContainer.getUniqueName(typeInfo.toSlotName()),
            count = params.count,
            slotNames = [],
            slotName,
            slotNum = uniqueName.match(/\d*$/)[0] || 0,
            i;

        uniqueName = uniqueName.replace(/\d*$/, '');

        for (i = 0; i < count; i++) {
            while (origContainer.has(slotName = uniqueName + (slotNum || ''))) {
                slotNum++;
            }
            slotNames.push(slotName);
            slotNum++;
        }

        return getZone(mgrModel.$componentSource.$container.$handle).then(function(customer){
                return mapAsync(slotNames, function (slotName) {
                    return Promise.resolve(mgrModel.newInstance(typeInfo))
                        .then(function (instance) {
                            return source.add({ slot: slotName, value: instance });
                        });
                });
        }).then(function () {
            var newTypes = mgrModel.getNewTypes(), model;

            model = new MgrModel({
                componentSource: source,
                columns: mgrModel.getColumns()
            });

            _.each(model.getRows(), function (row) {
                TypeMgrColumn.setAvailableTypes(row, newTypes);
            });

            var obj={
                mod:new MgrModel({
                    componentSource: source,
                    columns: mgrModel.getColumns()})
            };
            return obj;
        });
    }

  /**
   * Command for editing existing components in a Manager view.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var AddCommand = function AddCommand(manager) {
    var that = this;
    AddCommand.zoneManager=manager;
    Command.call(that, {
      module: 'webEditors',
        displayName:lex.get("commands.addZone.label"),
      enabled: true,

      /**
       * Takes the components that are currently selected in the manager's
       * main table, and allows for editing them using a `BatchComponentEditor`.
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/AddCommand#invoke
       * @returns {Promise}
       */
      func: function () {
          var mgrModel = manager.getModel();

          return manager.getOrdBase()
              .then(function (ordBase) {
                  /*  setTimeout(function(){
                    that.startValidation();
                   }, 450);*/
                  return promptForParams(mgrModel, ordBase)
                      .then(function (params) {
                          return promptForNewComponents(mgrModel, params, ordBase,that);
                      })
                      .then(function (comps) {
                          return comps && mgrModel.addInstances(comps);
                      });
              })
              .catch(feDialogs.error);
      }
    });

     manager.on('loaded', function () {

      var selection = getSelection(manager);
      selection.on('changed', function () {
        //that.setEnabled(!selection.isEmpty());
      });

    });
  }

    function promptForNewComponents(mgrModel, params,ordBase,that) {
        return makeSubMgrModel(mgrModel, params)
            .then(function (model) {
                return ManagerDialog.showFor({ value: model.mod, type: ZoneEditor, progressCallback: function (msg, arg) {
                        switch(msg) {
                            case 'created': return that.startValidation(arg);
                        }},reasonSetHandle:model.reasonSet,com:"Add"});
            })
            .then(function (model) {
                return model && _.map(model.getRows(), function (row) {
                    return row.getSubject();
                });
            });
    }

  AddCommand.prototype = Object.create(Command.prototype);
  AddCommand.prototype.constructor = AddCommand;
  AddCommand.zoneManager=null;
  AddCommand.zoneEditor=null;
  AddCommand.choosePointsBtn=null;


  AddCommand.prototype.startValidation = function(arg)
  {
      var dom=arg.jq();
      AddCommand.zoneEditor=arg;
     $(dom).find("tr.key-__name").hide();
  };


  return AddCommand;
});
