/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Balaji R Gite
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
 */
define(['baja!',
        'underscore',
        'jquery',
        'bajaux/commands/Command',
        'nmodule/webEditors/rc/fe/feDialogs',
        'lex!electronicSignature',
        'nmodule/electronicSignature/rc/wb/mgr/editor/ZoneEditor',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModel',
        'nmodule/webEditors/rc/wb/table/model/source/ArrayComponentSource',
        'baja!electronicSignature:SecuredNumericWritable',
    'nmodule/electronicSignature/rc/wb/mgr/commands/ZoneAddCommand'
        ], function (
         baja,
         _,
         $,
         Command,
         feDialogs,
         electronicSignatureLex,
         ZoneEditor,
         MgrModel,
         ArrayComponentSource,
         types,ZoneAddCommand) {

  'use strict';
  var lex = electronicSignatureLex[0];
  function getSelection(manager) {
    return manager.getMainTable().$getSelection();
  }

  function makeSubMgrModel(manager) {
      var mgrModel = manager.getModel(),
          selectedRows = manager.getMainTable().getSelectedRows(),
        comps = _.map(selectedRows, function (row) {
        	return row.getSubject();
        });

    return new MgrModel({
      componentSource: new ArrayComponentSource(comps),
      columns: mgrModel.getColumns()
    });
  }

  /**
   * Command for editing existing components in a Manager view.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/commands/CustomerEditCommand
   * @param {module:nmodule/webEditors/rc/wb/mgr/Manager} manager
   */
  var EditCommand = function EditCommand(manager) {
    var that = this;
    EditCommand.zoneManager=manager;
    Command.call(that, {
      module: 'webEditors',
      lex: 'commands.mgr.edit',
      enabled: false,

      /**
       * Takes the components that are currently selected in the manager's
       * main table, and allows for editing them using a `BatchComponentEditor`.
       * @alias module:nmodule/webEditors/rc/wb/mgr/commands/EditCommand#invoke
       * @returns {Promise}
       */
      func: function () {
        var subModel = makeSubMgrModel(manager);

          if(typeof  ZoneAddCommand !== 'undefined' && ZoneAddCommand.zoneEditor!==null){
              ZoneAddCommand.zoneEditor = null;
          }
        return manager.getOrdBase()
          .then(function (ordBase) {
             return feDialogs.showFor({
              value: subModel,
              properties: { ordBase: ordBase },
              type: ZoneEditor,
              progressCallback: function (msg, arg) {
                switch(msg) {
                   case 'loaded': return that.startValidation(arg);
                }
              }
            });
          });
      }
    });

     manager.on('loaded', function () {

      var selection = getSelection(manager);
      selection.on('changed', function () {
        that.setEnabled(!selection.isEmpty());
      });

    });
  };

  EditCommand.prototype = Object.create(Command.prototype);
  EditCommand.prototype.constructor = EditCommand;
  EditCommand.zoneManager=null;
  EditCommand.zoneEditor=null;
  EditCommand.choosePointsBtn=null;
  EditCommand.prototype.setEnabled = function (enabled) {
    return Command.prototype.setEnabled.call(this, !baja.isOffline() && enabled);
  };


  EditCommand.prototype.startValidation = function(arg)
  {
      var dom=arg.jq();
      EditCommand.zoneEditor=arg;
     $(dom).find("tr.key-__name").hide();
  };


  return EditCommand;
});
