/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Smita Rajge
 */

/**
 * CustomPointSelectorEditor manages functionality of making points secured and unsecured.
 *
 * @module nmodule/electronicSignature/rc/fe/baja/CustomPointSelectorEditor
 */
define(['baja!','baja!electronicSignature:SecureDouble',
        'lex!baja,electronicSignature',
        'jquery',
        'Promise',
        'dialogs',
        'bajaux/Widget',
        'underscore',
        'bajaux/mixin/subscriberMixIn',
        'hbs!nmodule/electronicSignature/rc/wb/template/PointSelectionTemplate',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'nmodule/webEditors/rc/fe/feDialogs',
        'nmodule/electronicSignature/rc/wb/mgr/editor/ReasonSetSelectorEditor',
        'nmodule/electronicSignature/rc/fe/baja/Util',
        'hbs!nmodule/electronicSignature/rc/fe/template/SecuredDeleteComponentTemplate',
        'nmodule/webEditors/rc/wb/tree/stationTree',
        'nmodule/electronicSignature/rc/wb/mgr/tree/pointChooserTree',
        'nmodule/electronicSignature/rc/wb/mgr/tree/CustomNavTree',
        "nmodule/webEditors/rc/wb/tree/NavTree",
        'nmodule/electronicSignature/rc/wb/dialog/ordChooserDialog',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         $,
         Promise,
         dialogs,
         Widget,
         _,
         subscriberMixIn,
         template,
         bootstrap,
         BaseEditor,
         SecureParentEditor,
         feDialogs,
         ReasonSetSelectorEditor,
         Util,
         securedDeleteComponentTemplate,
         stationTree,
         pointChooserTree,
         CustomNavTree,
         NavTree,
         ordChooserDialog) {

    'use strict';
    var lex = lexs[1], zone = null,
        jsonObjectForAuthentication={
              authLevel1:lex.get('fe.authorization.level1.label'),
              authLevel2:lex.get('fe.authorization.level2.label'),
              userName:lex.get('fe.userName.label'),
              password:lex.get('fe.password.label'),
              comment:lex.get('fe.comment.label'),
              sign:lex.get('fe.sign1.button'),
              sign2:lex.get('fe.sign2.button'),
              acknowledgement:lex.get('eSignature.ack.label'),
              label1:lex.get('eSignature.ack.label1'),
              label2:lex.get('eSignature.ack.label2'),
              label3:lex.get('eSignature.ack.label3'),
              label4:lex.get('eSignature.ack.label4'),
              label5:lex.get('eSignature.ack.label5'),
              label6:lex.get('eSignature.ack.label6'),
              label7:lex.get('eSignature.ack.label7')
        };
    // to:
    // from :
    let allPointListID ="#allPoints";
    let primaryPointListID = "#primaryPoints";
    let secondaryPointListID = "#secondaryPoints";
    let primaryPointsTargeted = true;

    var CustomPointSelectorEditor = function CustomPointSelectorEditor(params) {
      BaseEditor.call(this, $.extend({
        keyName: 'CustomPointSelectorEditor'
      }, params));
      subscriberMixIn(this);
  };

  CustomPointSelectorEditor.componentFE = null;
  CustomPointSelectorEditor.prototype = Object.create(BaseEditor.prototype);
  CustomPointSelectorEditor.prototype.constructor = CustomPointSelectorEditor;
  CustomPointSelectorEditor.zoneManager = null;
  CustomPointSelectorEditor.reasonPointMapHash=[];
  CustomPointSelectorEditor.editedReasonPointMap=[];
  CustomPointSelectorEditor.allReasonPointMapHash=[];
  CustomPointSelectorEditor.zonePrimaryPropertyList=[];
  CustomPointSelectorEditor.zoneSecondaryPropertyList=[];
  CustomPointSelectorEditor.pointHistoryDetails=[];
  CustomPointSelectorEditor.securedDashBoardConfigurationManager=null;
  CustomPointSelectorEditor.loggedInUserName="";

  function getSecuredPoint(pointOrd) {
    var ord="station:|"+pointOrd;
    return baja.Ord.make(ord).get({lease: true});
  }


  function loadComponentSlots(comp){
    return new Promise(function(fullfil,reject){
      comp.lease().then(function(){
        comp.loadSlots({
          ok:function(){
            fullfil(comp);
          },
          fail:function(){
            reject(comp);
          }
        });
      });
    });
  }

   /* Check if point already exist in all points,primary list,Secondary List.If not then only add the point selected from ord chooser to all points list.*/
  function isPointExistInList(dom,element) {
    var allOptions=dom.find(allPointListID)[0].options;
    var pointPresentInAllList=false,j=0;
    for(j=0;j<allOptions.length;j++)
    {
      if(allOptions[j].value === element.k)
      {
        pointPresentInAllList=true;
        break;
      }
    }
    var primOptions=dom.find(primaryPointListID)[0].options;
    var pointPresentInPrimList=false;

    for(j=0;j<primOptions.length;j++)
    {
      if(primOptions[j].value === element.k)
      {
        pointPresentInPrimList=true;
        break;
      }
    }
    var secOptions=dom.find(secondaryPointListID)[0].options;
    var pointPresentInSecList=false;
    for(j=0;j<secOptions.length;j++)
    {
      if(secOptions[j].value === element.k)
      {
        pointPresentInSecList=true;
        break;
      }
    }
    if(pointPresentInAllList || pointPresentInPrimList || pointPresentInSecList )
    {
    return true;
    }
    return false;
  }
   var kids=[];
   /*Get the kids of selected node to show in all points list */
  function getKidsArray(node){
      if((node.value().getType().$typeSpec === "control:NumericWritable" || node.value().getType().$typeSpec === "control:BooleanWritable" || node.value().getType().$typeSpec === "control:EnumWritable" || node.value().getType().$typeSpec === "control:StringWritable") || node.value().getType().$typeSpec === "control:NumericPoint" ||  node.value().getType().$typeSpec === "control:BooleanPoint" || node.value().getType().$typeSpec === "control:EnumPoint" || node.value().getType().$typeSpec === "control:StringPoint")
      {
        return loadComponentSlots(node.value()).then(function(point){
           var index=point.getNavOrd().toString().indexOf("slot:/");
           var key = point.getNavOrd().toString().substring(index);
           var val = key.substring(0,(key.lastIndexOf("/")+1)) + point.getDisplayName();
           kids.push({k : key, v: val});
           return Promise.resolve(kids);
        });
      }
      else{
        return node.$loadKids().then(function(lodedKids){
          if(lodedKids.length!== 0)
          {
            return loadMyKids(lodedKids);
          }
        }).then(function(){
        return Promise.resolve(kids);
        });
      }
   }

    /* This method loads kids */
  function loadMyKids(lodedKids)
   {
      var Promises = [];
      for(var i=0;i<lodedKids.length;i++)
      {
        Promises.push(getKidsArray(lodedKids[i]));
      }
      return Promise.all(Promises);
   }
   /* Shows the validation details.*/
  function showValidationSummary(validationSummary){
    var errorMessage="";
    var index = 0;
    for(var i=0;i<validationSummary.length;i++){
      if(i!==0)
      {
        if(validationSummary[i].errorMsg!=="")
        {
          index = index +1;
          errorMessage+= "<b>"+ index +"</b>" + ". "+validationSummary[i].errorMsg+"</br>";
        }
      }
      else
      {
        if(validationSummary[i].errorMsg!=="")
        {
          errorMessage+= validationSummary[i].errorMsg+"</br>";
        }
      }
    }
    return errorMessage;
  }

  CustomPointSelectorEditor.prototype.doInitialize = function (dom) {
    CustomPointSelectorEditor.zonePrimaryPropertyList=[];
    CustomPointSelectorEditor.zoneSecondaryPropertyList=[];
    CustomPointSelectorEditor.reasonPointMapHash=[];
    CustomPointSelectorEditor.allReasonPointMapHash=[];
    CustomPointSelectorEditor.editedReasonPointMap=[];
    CustomPointSelectorEditor.securedDashBoardConfigurationManager="";
    CustomPointSelectorEditor.pointHistoryDetails=[];
    let jsonObject={choosePoints:lex.get('choosePoints'),
      ordChooserLabel:lex.get('ordChooser.label'),
      editReason:lex.get('editReason.button.label'),
      clearUnsecuredPoints:lex.get('clearUnsecuredPoints.button.label'),
      selectAll:lex.get('selectAll.checkbox.label'),
      addPrimary:lex.get('add.prim.button.label'),
      addFromAllPointsToSecondary:lex.get('add.sec.button.label'),
      removePrimary:lex.get('remove.prim.button.label'),
      unsecuredPointLabel:lex.get('unsecured.point.label'),
      singleSignLabel:lex.get('singleSig.point.label'),
      addSecondaryLabel:lex.get('add.sec.button.label'),
      removeSecondaryLabel:lex.get('remove.sec.button.label'),
      secondaryToPrimaryLabel:lex.get('sec.to.prim.button.label'),
      secondaryLabel:lex.get('dualSig.point.label')
    };
    dom.html(template({jsonObject:jsonObject}));

    $(document).on('change', ':checkbox', function() { // jshint ignore:line
      if(this.id === "selectAll")
      {
        if (this.checked)
        {
          $('#allPoints option').prop('selected', true);
          $('#secondaryPoints option').prop('selected', false);
          $('#primaryPoints option').prop('selected', false);
          dom.find("#btnRemoveFromPrimaryList")[0].disabled=false;
          dom.find("#btnRemoveFromSecondaryList")[0].disabled=false;
        }else
        {
          $('#allPoints option').prop('selected', false);
        }
      }
    });

    dom.find('#allPoints').change(function() {
      $('#primaryPoints option').prop('selected', false);
      $('#secondaryPoints option').prop('selected', false);
      dom.find("#btnRemoveFromPrimaryList")[0].disabled=false;
      dom.find("#btnRemoveFromSecondaryList")[0].disabled=false;
    });

    dom.find('#primaryPoints').change(function() {
      $('#secondaryPoints option').prop('selected', false);
      $('#allPoints option').prop('selected', false);
      $('#selectAll').prop('checked', false);
      //This code checks if point is present in zoneSecondaryPropertyList.If yes disables btnRemoveFromPrimaryList
      dom.find("#btnRemoveFromSecondaryList")[0].disabled=false;
      var value = $(this).val();
      var index = CustomPointSelectorEditor.zoneSecondaryPropertyList.indexOf(value);
      if(index !== -1) {

      dom.find("#btnRemoveFromPrimaryList")[0].disabled=true;
      }else
       {
       dom.find("#btnRemoveFromPrimaryList")[0].disabled=false;
       }
    });

    dom.find('#secondaryPoints').change(function() {
      $('#allPoints option').prop('selected', false);
      $('#primaryPoints option').prop('selected', false);
      $('#selectAll').prop('checked', false);
      //This code checks if point is present in zonePrimaryPropertyList.If yes disables btnRemoveFromSecondaryList
      dom.find("#btnRemoveFromPrimaryList")[0].disabled=false;
      var value = $(this).val();
      var index = CustomPointSelectorEditor.zonePrimaryPropertyList.indexOf(value);
      if(index !== -1) {
        dom.find("#btnRemoveFromSecondaryList")[0].disabled=true;
      }else
      {
        dom.find("#btnRemoveFromSecondaryList")[0].disabled=false;
      }
    });
    /* Shows the reason set selector editor for editing the reasons of points.*/
    dom.find("#btnEditReason").click(async function() {
      const selectedPoints = await checkMinimumOnePointIsSelected(dom,
          [primaryPointListID, secondaryPointListID],
          "select.point.to.edit.reason", false);
      if(selectedPoints.length===0)
        return;

      let securedDashBoardConfigManager = await loadComponentSlots(CustomPointSelectorEditor.securedDashBoardConfigurationManager);
      var reasons="";
      let obj={};
      obj.selectedPoints=selectedPoints;
      obj.securedDashBoardConfigurationManager=CustomPointSelectorEditor.securedDashBoardConfigurationManager;
      let resolvedPoints = [];
      let errorPoints = [];
      let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints);
      let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
      if(shouldProceedEvenIfPartialPointsResolved === false)
        return;
      let permissionsCheckResult = await hasAdminWriteAndOperatorInvokePermissions(resolvedPoints, "editor.primary.user.validation");
      if(permissionsCheckResult !== 'NO_ERROR')
        return;

      //Below code checks if the reasons are available in editedReasonPointMap for selected point if yes then use them to show in reasonSelectorEditor.
      //If its not available in editedReasonPointMap then check if the reasons exist on point itself and use them to show in reasonSelectorEditor.
      if (resolvedPoints.length === 1)
      {
        let reasonsExistInEditedReasonPointMap = false;
        for (var i = 0; i < CustomPointSelectorEditor.editedReasonPointMap.length; i++)
        {
          if (selectedPoints[0] === CustomPointSelectorEditor.editedReasonPointMap[i].pointSlotPath)
          {
            reasonsExistInEditedReasonPointMap = true;
            reasons = CustomPointSelectorEditor.editedReasonPointMap[i].reasonHandles;
            break;
          }
        }
        if (!reasonsExistInEditedReasonPointMap)
        {
          //Check if point has reasons
          if (resolvedPoints[0].$map.$map.es_reasons != undefined)
          {
              reasons = resolvedPoints[0].$map.$map.es_reasons.$val;
          }
        }
      }

      obj.pointReasons=reasons;
      obj.pointChooserParentWidth = $("#pointChooserParent").width();
      let allPointsReasonSelection = await feDialogs.showFor({ value: obj, type: ReasonSetSelectorEditor });
      if(allPointsReasonSelection!==null)
      {
        updatePointAndReasonMapping(allPointsReasonSelection, CustomPointSelectorEditor.editedReasonPointMap, false);
      }
    });

    dom.find("#btnClearUnsecuredPointList").click( function() {
        dom.find(allPointListID).empty();
    });

    dom.find("#btnAddToPrimaryList").click(async function() {
      primaryPointsTargeted = true;
      await addSelectedPointsToPrimaryOrSecondaryList(dom);
    });

    dom.find("#btnAddFromAllPointsToSecondaryList").click(async function() {
      primaryPointsTargeted = false;
      await addSelectedPointsToPrimaryOrSecondaryList(dom);
    });

    dom.find("#btnRemoveFromPrimaryList").click(async function() {
        const selectedPoints = await checkMinimumOnePointIsSelected(dom, primaryPointListID,
            "points.not.selected.remove");
        if(selectedPoints.length===0)
            return;
        const allPointsAreAlreadyProcessed = checkWhetherAllPointsAreAlreadyProcessed(selectedPoints);

      if(allPointsAreAlreadyProcessed === true)
      {
          // to: #allPoints
          // from : #secondaryPoints
          removeSelectedPointsFromZonePrimaryList(selectedPoints);
          moveSelectedPointsFromSecondaryToAllPoints(selectedPoints, allPointListID, primaryPointListID, dom);
          removeSelectedPointsFromReasonMap(selectedPoints);
      }
      else
      {
          let resolvedPoints = [];
          let errorPoints = [];
          let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints);
          let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
          if(shouldProceedEvenIfPartialPointsResolved === false)
              return;
          let allPointsMovedFromSecondaryToPrimary = _.all(resolvedPoints, (point) => (point.getSlot("isSecuredSecondary") !== null));

          if(allPointsMovedFromSecondaryToPrimary===false)
          {
              let primaryPoint = _.find(resolvedPoints, (point)=> point.getSlot("isSecuredPrimary")!=null);
              let actionPoint = await primaryPoint.getActionParameterDefault(primaryPoint.getSlot("call"));
              let dlg = await authenticateUser(actionPoint);
              if(dlg!=null)
              {
                  console.log("User pressed ok.");
                  movePrimaryPointToAllPoints(dom, resolvedPoints, dlg)
              }
              return dlg;
          }
          return movePrimaryPointToAllPoints(dom, resolvedPoints, null);
        /* As points are secured before removing show authenticator editor */
      }
    });

    dom.find("#btnAddToSecondaryList").click(async function() {
        const selectedPoints = await checkMinimumOnePointIsSelected(dom, primaryPointListID,
            "select.point.add");

        if(selectedPoints.length===0)
            return;

        let resolvedPoints = [];
        let errorPoints = [];
        let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints)
        let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
        if(shouldProceedEvenIfPartialPointsResolved === false)
            return;
        let pointsToProcess = filterErrorneousPointsFromSelectedPoints(selectedPoints, resolvedPoints);
        let pointsWithoutPermissions = [];
        _.each(pointsToProcess,(point)=>
        {
            var permissions=point.niagaraPoint.getPermissions();
            if(permissions.hasAdminWrite() && permissions.hasOperatorInvoke())
            {
                var optionHtml = '<option value='+point.val +'>'+point.text+'</option>';
                dom.find(secondaryPointListID).append(optionHtml);
                dom.find(primaryPointListID).children('option[value="'+point.val+'"]').remove();
            }
            else
            {
                pointsWithoutPermissions.push(point);
            }
        });

        if(pointsWithoutPermissions.length>0)
        {
            dialogs.showOk({
                title: lex.get("error.title"),
                content:lex.get("primary.privileges.missing.short")
            });
        }
    });

    dom.find("#btnRemoveFromSecondaryList").click( async function() {
        const selectedPoints = await checkMinimumOnePointIsSelected(dom, secondaryPointListID,
            "points.not.selected.remove");

        if(selectedPoints.length===0)
            return;
        const allPointsAreAlreadyProcessed = checkWhetherAllPointsAreAlreadyProcessed(selectedPoints);

        if(allPointsAreAlreadyProcessed === true)
      {
          // to: #allPoints
          // from : #secondaryPoints
          removeSelectedPointsFromZoneSecondaryList(selectedPoints);
          moveSelectedPointsFromSecondaryToAllPoints(selectedPoints, allPointListID, secondaryPointListID, dom);
          removeSelectedPointsFromReasonMap(selectedPoints);
      }
      else
      {
          let resolvedPoints = [];
          let errorPoints = [];
          let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints)
          let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
          if(shouldProceedEvenIfPartialPointsResolved === false)
              return;
          let pointsToProcess = filterErrorneousPointsFromSelectedPoints(selectedPoints, resolvedPoints);
          // Ask for authentication and then move.
          // Here resolvedPoints[0] can be primary or secondary.
          // We need a point to be secondary as we want to show dual authentication.
          // So search a first secondary point in resolved points list and use that.
          // As we are moving from secondary section to primary, there needs to be dual authentication.
          let secondaryPoint = _.find(resolvedPoints, (point)=> point.getSlot("isSecuredSecondary")!=null);
          // If no secondary point is found in the selection then it means all of them are either primary or normal points.
          // so need to process them differently.
          if(secondaryPoint == null)
          {
              // check if there are any points that are primary or normal.
              // if some of them are primary then decide whether to show the primary
              return;
          }
          let actionPoint = await secondaryPoint.getActionParameterDefault(secondaryPoint.getSlot("call"));
          let dlg = await authenticateUser(actionPoint);
          if(dlg!=null)
          {
              console.log("User pressed ok.");
              console.log(dlg);
              moveSecondaryPointToAllPoints(dom,pointsToProcess, resolvedPoints, dlg);
          }
          return dlg;
         /* As points are secured before removing show authenticator editor */
      }
    });

    dom.find("#btnSecondaryToPrimary").click(async function() {
        const selectedPoints = await checkMinimumOnePointIsSelected(dom, secondaryPointListID,
            "points.not.selected.add.primary");
        if(selectedPoints.length===0)
            return;
        const allPointsAreAlreadyProcessed = checkWhetherAllPointsAreAlreadyProcessed(selectedPoints);

      if(allPointsAreAlreadyProcessed === true)
      {
          removeSelectedPointsFromZoneSecondaryList(selectedPoints);
          moveSelectedPointsFromSecondaryToAllPoints(selectedPoints, primaryPointListID, secondaryPointListID, dom);
      }
      else
      {
          let resolvedPoints = [];
          let errorPoints = [];
          let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints)
          let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
          if(shouldProceedEvenIfPartialPointsResolved === false)
              return;
          let allSelectedPointsArePrimary = _.all(resolvedPoints,(point)=>point.getSlot("isSecuredPrimary")!==null);

          let pointsToProcess = filterErrorneousPointsFromSelectedPoints(selectedPoints, resolvedPoints);
          if(allSelectedPointsArePrimary)
          {
              // Simply move from secondary to primary.
              return moveSelectedPointsFromSecondaryToAllPoints(pointsToProcess, primaryPointListID, secondaryPointListID, dom);
          }
          else
          {
              // Ask for authentication and then move.
              // Here resolvedPoints[0] can be primary or secondary.
              // We need a point to be secondary as we want to show dual authentication.
              // So search a first secondary point in resolved points list and use that.
              // As we are moving from secondary section to primary, there needs to be dual authentication.
              let secondaryPoint = _.find(resolvedPoints, (point)=> point.getSlot("isSecuredSecondary")!=null);
              let actionPoint = await secondaryPoint.getActionParameterDefault(secondaryPoint.getSlot("call"));
              let dlg = await authenticateUser(actionPoint);
              if(dlg!=null)
              {
                  console.log("User pressed ok.");
                  moveSelectedPointsFromSecondaryToAllPoints(pointsToProcess, primaryPointListID, secondaryPointListID, dom);
                  // Here, pointsToProcess contains primary as well as secondary points.
                  // "updateHistoryDetailsForSecToPrim" needs to happen for both type of points.
                  // but "CustomPointSelectorEditor.allReasonPointMapHash.push(pointObj);" needs to happen only for
                  // secondary points. So need to work on that.
                  _.each(pointsToProcess,(point)=>
                  {
                      if(point.niagaraPoint!=null && point.niagaraPoint.getSlot("isSecuredSecondary")!=null)
                      {
                          let reasons=point.niagaraPoint.getSlot("es_reasons");
                          let pointObj={};
                          pointObj.pointSlotPath=point.val;
                          pointObj.reasonHandles=reasons.$val;
                          CustomPointSelectorEditor.allReasonPointMapHash.push(pointObj);
                      }
                      updateHistoryDetailsForSecToPrim(point.val,dlg,actionPoint);
                  });
                  $('.js-dialog-button-cancel').attr('disabled', true);
              }
              return dlg;
          }
        /* As points are secured before removing show authenticator editor */
      }
    });

    dom.find("#ordChooser").click(function() {
      kids=[];
      var nodes;
      return pointChooserTree.makeComponentSpaceNode().then(function(node){
        return ordChooserDialog.showFor({
        value: node,
        type: CustomNavTree,
        });
      })
      .then(function(selectedNodes){
        nodes=selectedNodes;
        var selectedOrd="";
        if(selectedNodes !== undefined && selectedNodes.length>0)
        {
          var getKidsPromise = [],promise=null;
          for(var j=0;j<selectedNodes.length;j++)
          {  var index=selectedNodes[j].value().getNavOrd().toString().indexOf("slot:/");
             var ord=selectedNodes[j].value().getNavOrd().toString().substring(index);
             if(selectedOrd === "")
             {
               selectedOrd=ord;
             }else
             {
               selectedOrd=selectedOrd+","+ord;
             }
             promise = getKidsArray(selectedNodes[j]).then(function(arrayOfElement){
                return arrayOfElement;
             });
             //getKidsPromise.push(promise);
          }
          dom.find("#selectedOrd").val(selectedOrd);
          return Promise.all(promise);
        }
      })
      .then(function(arrayOfElement){
        /*var primaryList = _.map(dom.find(primaryPointListID)[0].options,function(slot,keys){ return slot.value; });
        var primaryList = _.map(dom.find(secondaryPointListID)[0].options,function(num,keys){ return num.value; });*/

        if(nodes !== undefined && nodes.length>0)
        {
          var addToAllPointsListHTML="";
          for(var i=0;i<arrayOfElement.length;i++)
          {
            var isPresent=isPointExistInList(dom,arrayOfElement[i]);
            if(!isPresent)
            {
              addToAllPointsListHTML+='<option value='+arrayOfElement[i].k+'>'+arrayOfElement[i].v+'</option>';
            }
          }
          dom.find(allPointListID).append(addToAllPointsListHTML);
        }
      });
    });

  };

  async function addSelectedPointsToPrimaryOrSecondaryList(dom)
  {
      const selectedPoints = await checkMinimumOnePointIsSelected(dom, allPointListID,
          "select.point.add");
      if(selectedPoints.length===0)
          return;

      let securedDashBoardConfigManager = await loadComponentSlots(CustomPointSelectorEditor.securedDashBoardConfigurationManager);
      if(await checkLicense(securedDashBoardConfigManager, selectedPoints, dom) === 'licensed')
      {
          let obj={};
          obj.selectedPoints=selectedPoints;
          obj.securedDashBoardConfigurationManager=securedDashBoardConfigManager;
          let resolvedPoints = [];
          let errorPoints = [];
          let values = await resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints);
          let shouldProceedEvenIfPartialPointsResolved = await confirmPartialOperation(errorPoints);
          if(shouldProceedEvenIfPartialPointsResolved === false)
              return;
          let permissionsCheckResult = await hasAdminWriteAndOperatorInvokePermissions(resolvedPoints, "primary.privileges.missing");
          if(permissionsCheckResult !== 'NO_ERROR')
              return;

          obj.pointChooserParentWidth = $("#pointChooserParent").width();
          let allPointsReasonSelection = await feDialogs.showFor({ value: obj, type: ReasonSetSelectorEditor });
          if(allPointsReasonSelection!==null)
          {
              let addToSecuredListHTML = updatePointAndReasonMapping(allPointsReasonSelection, CustomPointSelectorEditor.reasonPointMapHash, true);
              let authenticationID = (primaryPointsTargeted)?primaryPointListID:secondaryPointListID;
              dom.find(authenticationID).append(addToSecuredListHTML);
              dom.find(allPointListID).find('option:selected').remove();
          }
      }
  }

  function updatePointAndReasonMapping(userSelectedReasonsForPoints, existingReasonsForPoints, markPointsAsEdited)
  {
      let pointPresentInMap=false;
      let addToPrimaryListHTML="";
      for(var outerIndex=0;outerIndex<userSelectedReasonsForPoints.length;outerIndex++)
      {
          pointPresentInMap=false;
          for(var innerIndex=0;innerIndex<existingReasonsForPoints.length;innerIndex++)
          {
              if(userSelectedReasonsForPoints[outerIndex].pointSlotPath === existingReasonsForPoints[innerIndex].pointSlotPath)
              {
                  pointPresentInMap=true;
                  existingReasonsForPoints[innerIndex].reasonHandles=userSelectedReasonsForPoints[outerIndex].reasonHandles;
              }
          }

          if(pointPresentInMap === false)
          {
              let pointObj={};
              pointObj.pointSlotPath=userSelectedReasonsForPoints[outerIndex].pointSlotPath;
              pointObj.reasonHandles=userSelectedReasonsForPoints[outerIndex].reasonHandles;
              existingReasonsForPoints.push(pointObj);
          }

          if(markPointsAsEdited)
          {
              //Here we are adding this point obj to editedReasonPointMap so that when we add unsecured point in primary window with reasons and when user
              //click on edit reason selected reasons should be shown in list.We are always adding this object in below list considering that it will not be available in
              //editedReasonPointMap because when we click remove from primary or secondary we are removing entry from editedReasonPointMap.Which is not the case with reasonPointMapHash
              let reasonPointObj = {};
              reasonPointObj.pointSlotPath = userSelectedReasonsForPoints[outerIndex].pointSlotPath;
              reasonPointObj.reasonHandles = userSelectedReasonsForPoints[outerIndex].reasonHandles;
              CustomPointSelectorEditor.editedReasonPointMap.push(reasonPointObj);

              CustomPointSelectorEditor.allReasonPointMapHash = CustomPointSelectorEditor.reasonPointMapHash;
              addToPrimaryListHTML += '<option value=' + userSelectedReasonsForPoints[outerIndex].pointSlotPath + '>' + baja.SlotPath.unescape(userSelectedReasonsForPoints[outerIndex].pointDisplayPath) + '</option>';
          }
      }
      return addToPrimaryListHTML;
  }

  async function hasAdminWriteAndOperatorInvokePermissions(resolvedPoints, errorMessageLexicon)
  {
      let invalidList=[];
      for (var i=0;i<resolvedPoints.length;i++) {
          let per=resolvedPoints[i].getPermissions();
          if(!(per.hasAdminWrite() && per.hasOperatorInvoke())){
              if(invalidList.length===0){
                  let invalidListObj = {};
                  invalidListObj.errorMsg=lex.get(errorMessageLexicon);
                  invalidList.push(invalidListObj);
                  invalidListObj = {};
                  invalidListObj.errorMsg=resolvedPoints[i].getSlotPath();
                  invalidList.push(invalidListObj);
              }else
              {
                  let invalidListObj = {};
                  invalidListObj.errorMsg=resolvedPoints[i].getSlotPath();
                  invalidList.push(invalidListObj);
              }
          }
      }
      if(invalidList.length>0){
          let error=showValidationSummary(invalidList);
          return dialogs.showOk({
              title: lex.get("error.title"),
              content:error
          });
      }
      return Promise.resolve('NO_ERROR');
  }

  async function checkLicense(securedDashBoardConfigManager, selectedPoints, dom)
  {
      let maxPointCount=securedDashBoardConfigManager.getMaxPointCount();
      let primPointCount = dom.find($(primaryPointListID))[0].options.length;
      let secPointCount =dom.find($(secondaryPointListID))[0].options.length;
      let usedCount=primPointCount+secPointCount;
      let pointCanAdded=maxPointCount-usedCount;
      if(selectedPoints.length>pointCanAdded)
      {
          console.log("["+selectedPoints.length+
              "] Points can not be secured as they have exceeded the maximum point count["+
              maxPointCount+"] by ["+(selectedPoints.length-pointCanAdded)+"] points.");

          return dialogs.showOk({
              title: lex.get('error.title'),
              content:lex.get('max.pointCount.exceeded')
          });
      }
      return Promise.resolve('licensed');
  }

    function filterErrorneousPointsFromSelectedPoints(selectedPoints, resolvedPoints)
    {
        let pointsToProcess = _.toArray(_.filter(selectedPoints,(point)=>{
            return _.any(resolvedPoints,(resolved)=>resolved.getSlotPath().toString()===point.val);
        }));
        _.each(pointsToProcess, (point)=>{
            let actualPoint = _.find(resolvedPoints,(resolved)=>resolved.getSlotPath().toString()===point.val);
            if(actualPoint!=null)
                point.niagaraPoint = actualPoint;
        });
        return pointsToProcess;
    }
    async function authenticateUser(actionPoint)
    {
        let dlg = dialogs.showOkCancel(
            {title:lex.get("authenticate.user.title"),
                content:securedDeleteComponentTemplate({jsonObject:jsonObjectForAuthentication})});
        Util.loadInitialEventsAndData(actionPoint,"delete");

        const result = await dlg.promise();
        // If user has pressed yes then it means that we can go ahead and do the conversion.
        // Otherwise we should return from here.
        console.log("Result of the authentication.");
        console.log(result);
        if(result[1]!=='ok')
        {
            $('.js-dialog-button-ok').attr('disabled', false);
            return null;
        }
        return dlg;
    }

    async function confirmPartialOperation(errorPoints)
    {
        if(errorPoints.length>0)
        {
            //$('.js-dialog-button-ok').attr('disabled', false);
            var dlg = dialogs.showYesNo({
                title: lex.get("error.title"),
                content:"You do not have permission for some of the points. Do you still want to continue removal of remaining points?"//lex.get("primary.privileges.missing.short")
            });
            const result = await dlg.promise();
            // If user has pressed yes then it means that we can go ahead and do the conversion.
            // Otherwise we should return from here.
            console.log("Result of the confirming if the remaining points can be processed.");
            console.log(result);
            if(result[1]!=='yes')
                return false;
        }
        return true;
    }

    function resolveSelectedPoints(selectedPoints, resolvedPoints, errorPoints)
    {
        console.log("selected primary points array:");
        console.log(selectedPoints);
        const pointPromiseArray = _.toArray(_.map(selectedPoints,(pointOrd)=>getSecuredPoint(pointOrd.val)));

        return Promise.all(pointPromiseArray.map(function(promise) { return promise.reflect(); })).each(function(inspection){
            (inspection.isFulfilled())? resolvedPoints.push(inspection.value()): errorPoints.push(inspection.reason());
        });
    }

    function moveSelectedPointsFromSecondaryToAllPoints(selectedPoints, to, from, dom)
    {
        // to: #allPoints
        // from : #secondaryPoints
        _.each(selectedPoints, (point)=>{
            var optionHtml = '<option value='+point.val +'>'+point.text+'</option>';
            dom.find(to).append(optionHtml);
            dom.find(from).children('option[value="'+point.val+'"]').remove();
        });
    }

    function removeSelectedPointsFromReasonMap(selectedPoints)
    {
        CustomPointSelectorEditor.editedReasonPointMap = _.toArray(_.filter(CustomPointSelectorEditor.editedReasonPointMap,(val)=>{
            return _.any(selectedPoints, (point)=>point.val!==val.pointSlotPath);
        }));
    }

    function removeSelectedPointsFromZoneSecondaryList(selectedPoints)
    {
        CustomPointSelectorEditor.zoneSecondaryPropertyList = _.toArray(_.filter(CustomPointSelectorEditor.zoneSecondaryPropertyList,(val)=>{
            return _.any(selectedPoints, (point)=>point.val!==val);
        }));
    }

    function removeSelectedPointsFromZonePrimaryList(selectedPoints)
    {
        CustomPointSelectorEditor.zonePrimaryPropertyList = _.toArray(_.filter(CustomPointSelectorEditor.zonePrimaryPropertyList,(val)=>{
            return _.any(selectedPoints, (point)=>point.val!==val);
        }));
    }

    function checkWhetherAllPointsAreAlreadyProcessed(selectedPoints)
    {
        return _.all(selectedPoints,(selPoint)=>
        {
            return _.any(CustomPointSelectorEditor.reasonPointMapHash,
                (val)=>val.pointSlotPath===selPoint.val);
        });
    }

    async function checkMinimumOnePointIsSelected(dom, listName, messageIfNoPointSelected, addPointsFromAllTheSections)
    {
        let selectedPoints = [];
        let lists = listName;
        if(!Array.isArray(listName))
        {
            lists = [];
            lists.push(listName);
        }
        lists.forEach(list => {
            if(selectedPoints.length === 0 || (selectedPoints.length>0 && addPointsFromAllTheSections))
            {
                dom.find(list).find('option:selected').each(function(){
                    selectedPoints.push({val:$(this).val(), text : $(this).text()});
                });
            }
        });
        if(selectedPoints.length===0)
        {
            let dlg = dialogs.showOk({
                title: lex.get("error.title"),
                content:lex.get(messageIfNoPointSelected)
            });

            let result = await dlg.promise();
        }
        return selectedPoints;
    }

    function movePrimaryPointToAllPoints(dom, resolvedPoints, dialogDOM)
    {
        var processPoints = [];
        _.each(resolvedPoints, (point) =>
        {
            console.log("Moving each point");
            console.log(point);
            processPoints.push(point.getActionParameterDefault(point.getSlot("call")).then(function(actionPoint)
            {
                var selectedPrimaryPoints = point.getSlotPath().toString();
                var selectedPointDisplayOrd = selectedPrimaryPoints.substring(0,(selectedPrimaryPoints.lastIndexOf("/")+1))
                    + point.getDisplayName();
                // update the actionPoint(BSecureCall) with required values from values stored
                // in history map  for history logging at server side
                actionPoint = (point.getSlot("isSecuredSecondary")!=null)?updateActionPointForSecToPrim(selectedPrimaryPoints,actionPoint):
                    updateActionPoint(actionPoint,dialogDOM);

                removePrimaryPoint(dom,selectedPrimaryPoints,point,actionPoint, selectedPointDisplayOrd);
            }));
            //processPoints.push(movePoint);
        })
        return Promise.all(processPoints);
    }

    function moveSecondaryPointToAllPoints(dom, selectedPoints, resolvedPoints, dialogDOM)
    {
        removeSelectedPointsFromZoneSecondaryList(selectedPoints);
        removeSelectedPointsFromZonePrimaryList(selectedPoints);
        moveSelectedPointsFromSecondaryToAllPoints(selectedPoints, allPointListID, secondaryPointListID, dom);
        //Update the action point with required values which will be used to log history at sever side
        var processPoints = [];
        _.each(resolvedPoints, (point) =>
        {
            console.log("Moving each point");
            console.log(point);
            processPoints.push(point.getActionParameterDefault(point.getSlot("call")).then(function(actionPoint)
            {
                actionPoint=updateActionPoint(actionPoint,dialogDOM);
                return removeProperty(zone,actionPoint);
            }));
            //processPoints.push(movePoint);
        });
        return Promise.all(processPoints);
    }

  /* This method remove point from primary list and add to all points list */
  function removePrimaryPoint(dom,selectedPrimaryPoints,point,actionPoint, displayName)
  {
    var removedPrimaryPoint='<option value='+selectedPrimaryPoints+'>'+displayName+'</option>';
    dom.find(allPointListID).append(removedPrimaryPoint);
    var index = CustomPointSelectorEditor.zonePrimaryPropertyList.indexOf(selectedPrimaryPoints);
    if(index !== -1) {
      CustomPointSelectorEditor.zonePrimaryPropertyList.splice(index, 1);
    }
    dom.find(primaryPointListID).find('option[value="'+selectedPrimaryPoints+'"]').remove();

    removeProperty(zone,actionPoint).then(function(){
      //Disable the cancel button
      $('.js-dialog-button-cancel').attr('disabled', true);
      CustomPointSelectorEditor.securedDashBoardConfigurationManager.loadSlots().then(function(){
        CustomPointSelectorEditor.securedDashBoardConfigurationManager.lease().then(function(){
          var maxPointCount=CustomPointSelectorEditor.securedDashBoardConfigurationManager.getMaxPointCount();
          var usedPointCount=CustomPointSelectorEditor.securedDashBoardConfigurationManager.getUsedPointCount();
          if(usedPointCount<maxPointCount)
          {
           dom.find("#btnAddToPrimaryList")[0].disabled=false;
          }
        });
      });
      //This code removes the edited reasons from editedReasonPointMap if user click on remove from primary
      //If this is not done then if user edit reason of secured point then remove point from prim and then add that point with new reasons
       // reasons from editedReasonPointMap gets updated on point.
       for(var i=0;i<CustomPointSelectorEditor.editedReasonPointMap.length;i++)
       {
         if(selectedPrimaryPoints === CustomPointSelectorEditor.editedReasonPointMap[i].pointSlotPath)
         {
             CustomPointSelectorEditor.editedReasonPointMap.splice(i,1);
         }
       }
    });

  }
 /*This method gets the secondary users with secondary autheticated role and with proper permissions.*/
  function removeProperty(zoneComponent,point){
  return  zoneComponent.invoke({slot:"removeSecuredPoint", value: point});
   /* return zoneComponent.invoke({
      slot:"removeSecuredPoint",
      value:baja.$("baja:String").make(point)
    });*/
  }

  CustomPointSelectorEditor.prototype.doLoad = function(pointList,params){
    var widget = this, dom = widget.jq();
    if(pointList.zoneManager !== undefined && pointList.zoneManager !== null)
    {
        CustomPointSelectorEditor.zoneManager = pointList.zoneManager;
        zone = CustomPointSelectorEditor.zoneManager.getSubject()[0];
    }
    else if(pointList.zone !== undefined && pointList.zone !== null)
    {
        zone = pointList.zone;
    }
    var points =pointList.listOfPoints.split(",");
    var primaryHtmlList="",secondaryHtmlList="";
    var p = null;
    if(pointList!== null && pointList !== undefined && pointList !== "")
    {
        p = zone.invoke({
            slot: "getDisplayNames"
        });
    }
    else
    {
        p = Promise.resolve("");
    }
    p.then(function(displayNames){
        var dn  =displayNames.split(",");
        _.each(points, function(point, index){
            // if (point.endsWith("_P")) {
            if(point.lastIndexOf("_P")!== -1 && (point.lastIndexOf("_P") === (point.length - "_P".length))){
                point = point.substring(0, point.length - 2);
                var val = point.substring(0,(point.lastIndexOf("/")+1)) + dn[index];
                primaryHtmlList+='<option value='+point+'>'+baja.SlotPath.unescape(val)+'</option>';
                CustomPointSelectorEditor.zonePrimaryPropertyList.push(point);

            }
            else if (point.lastIndexOf("_S")!==-1 && (point.lastIndexOf("_S") === (point.length - "_S".length))){
                point = point.substring(0, point.length - 2);
                var val = point.substring(0,(point.lastIndexOf("/")+1)) + dn[index];
                secondaryHtmlList+='<option value='+point+'>'+baja.SlotPath.unescape(val)+'</option>';
                CustomPointSelectorEditor.zoneSecondaryPropertyList.push(point);
            }
        });
        dom.find(primaryPointListID).append(primaryHtmlList);
        dom.find(secondaryPointListID).append(secondaryHtmlList);
        CustomPointSelectorEditor.securedDashBoardConfigurationManager=zone.getParent().getParent();
    });
  };

  CustomPointSelectorEditor.prototype.doRead = function(secureDouble,params){
    var dom = this.jq();
    var securePointsWithFlagList= getSecurePointsWithFlag(dom);
    var value = "";
    if (securePointsWithFlagList !== null) {
      var itemCount = securePointsWithFlagList.length;
      for (var i = 0; i < itemCount; i++) {
        var temp = securePointsWithFlagList[i];
        if (i === itemCount - 1) {
          value = value + temp;
        } else {
          value = value + temp + ",";
        }
      }
    }
    var listOfPrimarySecondaryToAdd =addAndRemoveProperty(value);
    return listOfPrimarySecondaryToAdd;
 };
  function addAndRemoveProperty(value) {
    var primaryPropertyList = [];
    var secondaryPropertyList = [];
    var points = value.split(",");
    for (var i=0;i<points.length;i++) {
      // if (points[i].endsWith("_S")) {
      if ((points[i].lastIndexOf("_S") === (points[i].length - "_S".length))){
        secondaryPropertyList.push(points[i].substring(0, points[i].length - 2));
      }
      // if (points[i].endsWith("_P")) {
      if ((points[i].lastIndexOf("_P") === (points[i].length - "_P".length))){
        primaryPropertyList.push(points[i].substring(0, points[i].length - 2));
      }
    }
    var priAdd=evaluateAdditionListForPrimary(primaryPropertyList);
    var secAdd=evaluateAdditionListForSecondary(secondaryPropertyList);
    var priRem=evaluateRemovalListFromPrimary(primaryPropertyList);
    var secRem=evaluateRemovalListFromSecondary(secondaryPropertyList);
    var listOfPrimarySecondaryToAdd={};
    listOfPrimarySecondaryToAdd.priAdd=priAdd;
    listOfPrimarySecondaryToAdd.secAdd=secAdd;
    listOfPrimarySecondaryToAdd.priRem=priRem;
    listOfPrimarySecondaryToAdd.secRem=secRem;
    listOfPrimarySecondaryToAdd.pointList=value;
    listOfPrimarySecondaryToAdd.reasonPointMapHash=CustomPointSelectorEditor.allReasonPointMapHash;
    listOfPrimarySecondaryToAdd.editedReasonPointMap=CustomPointSelectorEditor.editedReasonPointMap;
    listOfPrimarySecondaryToAdd.pointHistoryDetails=CustomPointSelectorEditor.pointHistoryDetails;

    return listOfPrimarySecondaryToAdd;
  }
  function evaluateAdditionListForPrimary(primList){
    var temp=[];
    for (var i=0;i<primList.length;i++) {
      // if(!CustomPointSelectorEditor.zonePrimaryPropertyList.includes(primList[i])){
      if(CustomPointSelectorEditor.zonePrimaryPropertyList.indexOf(primList[i])=== -1){
        // indexOf(x) > -1
        temp.push(primList[i]);
      }
    }
    return temp;
  }
  function evaluateAdditionListForSecondary(secList){
    var temp= [];
    for (var i=0;i<secList.length;i++) {
       // if(!CustomPointSelectorEditor.zoneSecondaryPropertyList.includes(secList[i])){
       if(CustomPointSelectorEditor.zoneSecondaryPropertyList.indexOf(secList[i]) ===-1){
          temp.push(secList[i]);
       }
    }
    return temp;
  }
  function evaluateRemovalListFromPrimary(primList){
    var temp= [];
    for (var i=0;i<CustomPointSelectorEditor.zonePrimaryPropertyList.length;i++) {
      //if(!primList.includes(CustomPointSelectorEditor.zonePrimaryPropertyList[i])){
      if(primList.indexOf(CustomPointSelectorEditor.zonePrimaryPropertyList[i]) === -1){
        temp.push(CustomPointSelectorEditor.zonePrimaryPropertyList[i]);
      }
    }
    return temp;
  }
  function evaluateRemovalListFromSecondary(secList){
   var temp= [];
   for (var i=0;i<CustomPointSelectorEditor.zoneSecondaryPropertyList.length;i++) {
     //if(!secList.includes(CustomPointSelectorEditor.zoneSecondaryPropertyList[i])){
     if(secList.indexOf(CustomPointSelectorEditor.zoneSecondaryPropertyList[i]) === -1){
        temp.push(CustomPointSelectorEditor.zoneSecondaryPropertyList[i]);
     }
   }
   return temp;
  }
  /* Make list of primary and secondary points */
  function getSecurePointsWithFlag(dom) {
    var selectedPrimaryPoints = dom.find('#primaryPoints');
    var selectedSecondaryPoints = dom.find('#secondaryPoints');
    var securePointsWithFlagList = [],j=0;
    if (selectedPrimaryPoints[0].options.length > 0) {
      for(j=0;j<selectedPrimaryPoints[0].options.length;j++){
         securePointsWithFlagList.push(selectedPrimaryPoints[0].options[j].value + "_P");
      }
    }
    if (selectedSecondaryPoints[0].options.length > 0) {
      for(j=0;j<selectedSecondaryPoints[0].options.length;j++){
        securePointsWithFlagList.push(selectedSecondaryPoints[0].options[j].value + "_S");
      }
    }
    return securePointsWithFlagList;
  }
  /*Update the action point with required values which will be used to log history at sever side */
  function updateActionPoint(actionPoint,val)
  {
    var primaryComments=val.jq().find("#primaryComment").val();
    if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
    {
      actionPoint.setPrimaryComments(primaryComments);
    }
    else
    {
      var secondaryComments=val.jq().find("#secondaryComments").val();
      var managerName= val.jq().find("#secondaryUsername").val();
      actionPoint.setPrimaryComments(primaryComments);
      actionPoint.setSecondaryComments(secondaryComments);
      actionPoint.setManager(managerName);
    }
    return actionPoint;

  }
   /* This method updates the actionPoint(BSecurecall) for required details like prim comment,sec comments when user clicks on removeFromPrimary for secured secondary points so It will be used at server side for history logging */
  function updateActionPointForSecToPrim(selectedPoint,actionPoint)
  {
    for(var i=0;i<CustomPointSelectorEditor.pointHistoryDetails.length;i++)
    {
      if(selectedPoint === CustomPointSelectorEditor.pointHistoryDetails[i].pointSlotPath)
      {
         if(actionPoint.getSecureSlotFacets().$map.$map.isSecuredPrimary==true)
         {
           actionPoint.setPrimaryComments(CustomPointSelectorEditor.pointHistoryDetails[i].primComments);
         }else
         {
           actionPoint.setPrimaryComments(CustomPointSelectorEditor.pointHistoryDetails[i].primComments);
           actionPoint.setSecondaryComments(CustomPointSelectorEditor.pointHistoryDetails[i].secComments);
           actionPoint.setManager(CustomPointSelectorEditor.pointHistoryDetails[i].managerName);
         }
      }
    }

   return actionPoint;

  }
  /* This method updates the history details in history map when user clicks on btnSecondaryToPrimary so that it will be used when we invoke removeFromPrimary for secured secondary point
   Also same map is passed to pointlistEditor so that we can pass history details(sec comment,prim comments etc) for point added from secondaryToPrimary to server side*/
  function updateHistoryDetailsForSecToPrim(selectedPoint,val,actionPoint)
  {
    var pointPresentInHistoryMap=false;
    for(var i=0;i<CustomPointSelectorEditor.pointHistoryDetails.length;i++)
    {
      if(selectedPoint === CustomPointSelectorEditor.pointHistoryDetails[i].pointSlotPath)
      {
      pointPresentInHistoryMap=true;
      CustomPointSelectorEditor.pointHistoryDetails[i].primComments=val.jq().find("#primaryComment").val();
      CustomPointSelectorEditor.pointHistoryDetails[i].secComments=val.jq().find("#secondaryComments").val();
      CustomPointSelectorEditor.pointHistoryDetails[i].managerName=val.jq().find("#secondaryUsername").val();
      }
    }
    if(pointPresentInHistoryMap === false)
    {
      var pointObj={};
      pointObj.primComments=val.jq().find("#primaryComment").val();
      pointObj.secComments=val.jq().find("#secondaryComments").val();
      pointObj.managerName=val.jq().find("#secondaryUsername").val();
      pointObj.pointSlotPath= selectedPoint;
      CustomPointSelectorEditor.pointHistoryDetails.push(pointObj);
    }
  }


  return CustomPointSelectorEditor;
  });