/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/editor/CustomerEditor
 */
define(['baja!',
        'bajaux/events',
        'jquery',
        'Promise',
        'lex!electronicSignature',
        'underscore',
        'nmodule/js/rc/asyncUtils/asyncUtils',
        'nmodule/webEditors/rc/fe/fe',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/webEditors/rc/fe/baja/util/ComplexDiff',
        'nmodule/webEditors/rc/util/ListSelection',
        'nmodule/webEditors/rc/util/Switchboard',
        'nmodule/webEditors/rc/wb/mgr/MgrSheet',
        'nmodule/webEditors/rc/wb/mgr/model/MgrColumn',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModelCompositeBuilder',
        'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
        'nmodule/webEditors/rc/wb/table/Table'], function (
         baja,
         events,
         $,
         Promise,
         electronicSignatureLex,
         _,
         asyncUtils,
         fe,
         BaseEditor,
         ComplexDiff,
         ListSelection,
         Switchboard,
         MgrSheet,
         MgrColumn,
         MgrModelCompositeBuilder,
         BatchComponentEditor,
         Table) {

  'use strict';
  var lex = electronicSignatureLex[0];
  /**
   * Editor intended for use with a `MgrModel`, allowing multiple components
   * to be edited simultaneously.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/editor/CustomerEditor
   * @extends module:nmodule/webEditors/rc/wb/mgr/BatchComponentEditor
   */
  var CustomerEditor = function CustomerEditor() {
    var that = this,
        selection = that.$selection = new ListSelection();

    BaseEditor.apply(that, arguments);

    selection.select(0);

    selection.on('changed', function () {
      that.$onSelectionChanged();
    });

    new Switchboard(that)
      .allow('$onSelectionChanged')
      .oneAtATime().onRepeat().preempt();

    /**
     * A CustomerEditor will fail to validate under the following conditions:
     *
     * - phone number should be valid entry or blank.
     * @method module:nmodule/webEditors/rc/fe/baja/NumericEditor#validate
     */
    that.validators().add(function (val) {
      that.validateCustomer(val);
    });

  };

  CustomerEditor.prototype = Object.create(BatchComponentEditor.prototype);
  CustomerEditor.prototype.constructor = CustomerEditor;

  CustomerEditor.prototype.validateCustomer = function(val)
  {
    if( 'proposed.address' in val.$rows[0].$data)
    {
      var address = val.$rows[0].$data['proposed.address'];
      if(address === null || address.toString() === "")
      {
        throw lex.get('enter.address.message');
      }
      if(address !== null && address.toString().length > 200)
      {
        throw lex.get('address.length.validation.msg');
      }
    }

    if( 'proposed.customerName' in val.$rows[0].$data)
    {
      var customerName = val.$rows[0].$data['proposed.customerName'];
      if(customerName === null || customerName.toString() === "")
      {
        throw lex.get('customerName.blank.validation.msg');
      }
      if(customerName !== null && customerName.toString().length > 100)
      {
        throw lex.get('customerName.length.validation.msg');
      }
    }

    if( 'proposed.contactPersonName' in val.$rows[0].$data)
    {
      var contactPersonName = val.$rows[0].$data['proposed.contactPersonName'];
      if(contactPersonName === null || contactPersonName.toString() === "")
      {
        throw lex.get('contactPersonName.blank.validation.msg');
      }
      if(contactPersonName !== null && contactPersonName.toString().length > 100)
      {
        throw lex.get('contactPersonName.length.validation.msg');
      }
    }

    if( 'proposed.customerPhoneNumber' in val.$rows[0].$data)
    {
      var customerPhoneNumber = val.$rows[0].$data['proposed.customerPhoneNumber'];
      if(customerPhoneNumber !== null && customerPhoneNumber !== "")
      {
        var digitCount = 0;
        for(var i = 0;i < customerPhoneNumber.length; i++)
        {
          if(!isNaN( parseInt(customerPhoneNumber.charAt(i),10)))
          {
            digitCount++;
          }
        }

        //if(/[0-9]{10,18}/.test(customerPhoneNumber) === false)
        if(!(digitCount>=10 && digitCount<=18))
        {
          throw lex.get('customer.phone.validation.msg');
        }
      }
    }

    if( 'proposed.contactPersonPhoneNumber' in val.$rows[0].$data)
    {
      var contactPersonPhoneNumber = val.$rows[0].$data['proposed.contactPersonPhoneNumber'];
      if(contactPersonPhoneNumber !== null && contactPersonPhoneNumber !== "")
      {
        var digitCount = 0;
        for(var i = 0;i < contactPersonPhoneNumber.length; i++)
        {
          if(!isNaN( parseInt(contactPersonPhoneNumber.charAt(i),10)))
          {
            digitCount++;
          }
        }
        //if(/[0-9]{10,18}/.test(contactPersonPhoneNumber) === false)
        if(!(digitCount>=10 && digitCount<=18))
        {
          throw lex.get('contact.phone.validation.msg');
        }
      }
    }
  };

  return CustomerEditor;
});
