/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Smita Rajge
 */

/**
 * ReasonSetSelectorEditor manages Reason set and Reason selection
 *
 * @module nmodule/electronicSignature/rc/fe/baja/ReasonSetSelectorEditor
 */
define(['baja!','baja!electronicSignature:SecureDouble',
        'lex!baja',
        'lex!electronicSignature',
        'jquery',
        'Promise',
        'dialogs',
        'bajaux/Widget',
        'underscore',
        'bajaux/mixin/subscriberMixIn',
        'nmodule/electronicSignature/rc/fe/baja/loader',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/electronicSignature/rc/fe/baja/SecuredParentEditor',
        'hbs!nmodule/electronicSignature/rc/wb/template/ReasonSetSelectionTemplate',
        'css!nmodule/electronicSignature/rc/css/widgets'], function (
         baja,
         types,
         lexs,
         electronicSignatureLex,
         $,
         Promise,
         dialogs,
         Widget,
         _,
         subscriberMixIn,
         bootstrap,
         BaseEditor,
         SecureParentEditor,
         template) {

    'use strict';
    var lex = electronicSignatureLex[0];

    var ReasonSetSelectorEditor = function ReasonSetSelectorEditor(params) {
      BaseEditor.call(this, $.extend({
        keyName: 'ReasonSetSelectorEditor'
      }, params));
      subscriberMixIn(this);
    };

    ReasonSetSelectorEditor.componentFE = null;
    ReasonSetSelectorEditor.prototype = Object.create(BaseEditor.prototype);
    ReasonSetSelectorEditor.prototype.constructor = ReasonSetSelectorEditor;

    function updateSelectedReasonSet(dom)
    {
      var v=$(dom).find("input[type='checkbox']");
      for(var i=0;i<v.length;i++)
      {
        if (v[i].getAttribute("parent_reasonSetHandle")=== null)
        {
          var valChecked=true;
          var anyEntriesFound = false;
          for(var j=0;j<v.length;j++)
          {
            if (v[j].getAttribute("parent_reasonSetHandle")!== null)
            {
              if (v[i].getAttribute("reasonSetHandle") === v[j].getAttribute("parent_reasonSetHandle"))
              {
                anyEntriesFound = true;
                if(v[j].checked===false)
                {
                  valChecked=false;
                  break;
                }

              }
            }
          }
          if(valChecked===true && anyEntriesFound ===true)
          {
            v[i].checked=true;
          }
        }
      }
    }

   ReasonSetSelectorEditor.prototype.doInitialize = function (dom) {
    var jsonObject=
    {
        allPoints:lex.get('all.points'),
        reasonList:lex.get('reason.list')
    };
    dom.html(template({jsonObject:jsonObject}));

    $(".selectall").click(function () {
    $('.case').attr('checked', this.checked);
    });
    $(document).on('change', ':checkbox', function() { // jshint ignore:line
      var v=$(dom).find("input[type='checkbox']");
      var i=0;
      if(this.getAttribute("reasonSetHandle")!== null)
      {
         v= $(dom).find("input[type='checkbox']");
        for(i=0;i<v.length;i++)
        {
          if (v[i].getAttribute("parent_reasonSetHandle")!== null && v[i].getAttribute("parent_reasonSetHandle")=== this.getAttribute("reasonSetHandle"))
          {
             v[i].checked=this.checked;
          }
        }
      }	// 2. Check if Reason is checked/unchecked.
      else if (this.getAttribute("reasonHandle") !== null)
      {
        // 1.1 If checkbox is unchecked, then find its parent and uncheck it.
        if (!this.checked)
        {    v= $(dom).find("input[type='checkbox']");
          for(i=0;i<v.length;i++)
          {
            if (v[i].getAttribute("reasonSetHandle")!== null && v[i].getAttribute("reasonSetHandle") === this.getAttribute("parent_reasonSetHandle"))
            {
              v[i].checked=this.checked;
            }
          }

        }
        else
        {
          var allChildChecked=true;
           v= $(dom).find("input[type='checkbox']");
          for(i=0;i<v.length;i++)
          {
            if(v[i].getAttribute("parent_reasonSetHandle") !== null && v[i].getAttribute("parent_reasonSetHandle")===(this.getAttribute("parent_reasonSetHandle")))
            {
              if(!v[i].checked)
              {
                allChildChecked=false;
              }
            }
          }
          if (allChildChecked)
          {
            for(i=0;i<v.length;i++)
            {
              if (v[i].getAttribute("reasonSetHandle") !== null && v[i].getAttribute("reasonSetHandle")===(this.getAttribute("parent_reasonSetHandle")))
              {
                v[i].checked=true;
              }

            }
          }
        }

      }

    });
   };

  ReasonSetSelectorEditor.prototype.doLoad = function(obj){
    var reasons=obj.pointReasons;
    var pointReasons=[];
    if(reasons!==undefined)
    {
    pointReasons =reasons.split(",");
    }
    var widget = this,
    dom = widget.jq();
   // var reasonSets = [];
    var html = '';
    //working
   var securedDashBoardConfiguration=obj.securedDashBoardConfigurationManager;
    var selectedPointsHtml="";
    _.each(obj.selectedPoints, function(point, index){
      selectedPointsHtml+='<option value='+ point.val +'>' + point.text + '</option>';
    });
    //dom.find("#allPoints").append(selectedPointsHtml);
    dom.find("#allPointsReasonSelection").append(selectedPointsHtml);
    return securedDashBoardConfiguration.loadSlots().then(function(comp){
      return securedDashBoardConfiguration;
    }).then(function(comp){
      var reasonSetsPromise = [];
      securedDashBoardConfiguration.getSlots().is("electronicSignature:ReasonSet").each(function(reasonSet){
      reasonSetsPromise.push(reasonSet.$val.loadSlots());
      });
      return Promise.all(reasonSetsPromise);
    }).then(function(){
      var reasonsPromises = [];
      securedDashBoardConfiguration.getSlots().is("electronicSignature:ReasonSet").each(function(reasonSet){
      reasonSet.$val.getSlots().is("electronicSignature:Reasons").each(function(reason){
      reasonsPromises.push(reason.$val.loadSlots());
      });
      //reasonSetsPromise.push(reasonSet.$val.loadSlots());
      });
      return Promise.all(reasonsPromises);
    }).then(function(lodedReasons){
      var reasonSets = [];
      securedDashBoardConfiguration.getSlots().is("electronicSignature:ReasonSet").each(function(reasonSet){
        //logic
        var objReasonSet={};
        objReasonSet.name= reasonSet.$val.getReasonCollection();
        objReasonSet.handle=reasonSet.$val.getHandle();
        objReasonSet.slotPath= reasonSet.$val.getSlotPath();
        var reasons = [];
        reasonSet.$val.getSlots().is("electronicSignature:Reasons").each(function(reason){
          var reasonObj={};
          reasonObj.reasonName=reason.$val.getReasonForChange();
          reasonObj.reasonHandle=reason.$val.getHandle();
          reasons.push(reasonObj);
        });
        objReasonSet.reasons=reasons;
        reasonSets.push(objReasonSet);
      });
      _.each(reasonSets, function(reasonSet){
        var reasonSetID =  reasonSet.name;
        html+='<li><input type="checkbox" name="'+
        reasonSetID +'" id="'+reasonSetID+'" reasonSetHandle="'+reasonSet.handle+'"><label for=id="'+reasonSetID+'">'+reasonSetID +'</label></li><ul>';  //</li>';
        _.each(reasonSet.reasons, function(reason){
          // html+='<ul>';
          var reasonMatched=false;
          var reasonID = reasonSetID+ "." + reason.reasonName;
          for(var j=0;j<pointReasons.length;j++)
          {
            if(pointReasons[j]==reason.reasonHandle)
            {
              reasonMatched=true;
            }
          }
          if(reasonMatched)
          {
            html+='<li style="list-style-type: none;"><input type="checkbox" name="'+
            reasonID +'" checked id="'+reasonID+'"  parent_reasonSetHandle="'+reasonSet.handle+'" reasonHandle="'+reason.reasonHandle+'"><label for="'+reasonID+'">'+reason.reasonName +'</label></li>';
          }else
          {
          html+='<li style="list-style-type: none;"><input type="checkbox" name="'+
          reasonID +'"  id="'+reasonID+'"  parent_reasonSetHandle="'+reasonSet.handle+'" reasonHandle="'+reason.reasonHandle+'"><label for="'+reasonID+'">'+reason.reasonName +'</label></li>';
          }

        });
        html+='</ul></li>';
      });
       dom.find("#reasonSet").append(html);
       updateSelectedReasonSet(dom);
       dom.find("#reasonSelectionParent").css("min-width", obj.pointChooserParentWidth);

       setTimeout(function ()
       {
           var w = dom.find("#allPointsReasonSelection").parent().parent().width();
           if((w-160) < dom.find("#allPointsReasonSelection").parent().width())
           {
               dom.find("#allPointsReasonSelection").parent().css("width", w);
           }
           dom.find("#allPointsReasonSelection").parent().css("margin-right","-160px");
           setTimeout(function ()
           {
               var w = dom.find("#allPointsReasonSelection").parent().parent().width();
               if(w < dom.find("#allPointsReasonSelection").parent().width())
               {
                   dom.find("#allPointsReasonSelection").parent().css("width", w);
               }
           },500);
       },500);

    });
    // dom.find("#reasonSet").append(html);

  };

  ReasonSetSelectorEditor.prototype.doRead = function(secureDouble,params){
    var dom = this.jq();
    var allPointsReasonSelection=$(dom).find("#allPointsReasonSelection");
    var selectedPointsWithReasons=[];
    var v= $(dom).find("input[type='checkbox']");
    var selectedReasonString="";

    for(var i=0;i<v.length;i++)
    {
      if (v[i].getAttribute("reasonHandle") !== null && v[i].checked )
      {
        if(selectedReasonString === ""){
          selectedReasonString=v[i].getAttribute("reasonHandle");
        }else
        {
          selectedReasonString += "," + v[i].getAttribute("reasonHandle");
        }

      }
    }
    for(var j=0;j<allPointsReasonSelection[0].options.length;j++){
      var obj={};
      obj.pointSlotPath=allPointsReasonSelection[0].options[j].value;
      obj.pointDisplayPath=allPointsReasonSelection[0].options[j].innerHTML;
      obj.reasonHandles=selectedReasonString;
      selectedPointsWithReasons.push(obj);
    }

    return selectedPointsWithReasons;
  };

   return ReasonSetSelectorEditor;
  });