/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/mgr/editor/ZoneEditor
 */
define(['baja!',
        'lex!electronicSignature',
        'bajaux/events',
        'jquery',
        'Promise',
        'underscore',
        'nmodule/js/rc/asyncUtils/asyncUtils',
        'nmodule/webEditors/rc/fe/fe',
        'nmodule/webEditors/rc/fe/baja/BaseEditor',
        'nmodule/webEditors/rc/fe/baja/util/ComplexDiff',
        'nmodule/webEditors/rc/util/ListSelection',
        'nmodule/webEditors/rc/util/Switchboard',
        'nmodule/webEditors/rc/wb/mgr/MgrSheet',
        'nmodule/webEditors/rc/wb/mgr/model/MgrColumn',
        'nmodule/webEditors/rc/wb/mgr/model/MgrModelCompositeBuilder',
        'nmodule/webEditors/rc/wb/mgr/BatchComponentEditor',
        'nmodule/webEditors/rc/wb/table/Table',
        ], function (
         baja,
         electronicSignatureLex,
         events,
         $,
         Promise,
         _,
         asyncUtils,
         fe,
         BaseEditor,
         ComplexDiff,
         ListSelection,
         Switchboard,
         MgrSheet,
         MgrColumn,
         MgrModelCompositeBuilder,
         BatchComponentEditor,
         Table) {

    'use strict';
    var lex = electronicSignatureLex[0];

  /**
   * Editor intended for use with a `MgrModel`, allowing multiple components
   * to be edited simultaneously.
   *
   * @private
   * @class
   * @alias module:nmodule/electronicSignature/rc/wb/mgr/editor/ZoneEditor
   * @extends module:nmodule/webEditors/rc/wb/mgr/BatchComponentEditor
   */
  var ZoneEditor = function ZoneEditor() {
    var that = this,
        selection = that.$selection = new ListSelection();

    BaseEditor.apply(that, arguments);

    selection.select(0);

    selection.on('changed', function () {
      that.$onSelectionChanged();
    });

    new Switchboard(that)
      .allow('$onSelectionChanged')
      .oneAtATime().onRepeat().preempt();

    /**
     * ZoneEditoror will fail to validate under the following conditions:
     *
     * - phone number should be valid entry or blank.
     * @method module:nmodule/webEditors/rc/fe/baja/NumericEditor#validate
     */
    that.validators().add(function (val) {
      that.validateZone(val);
    });

  };

  ZoneEditor.prototype = Object.create(BatchComponentEditor.prototype);
  ZoneEditor.prototype.constructor = ZoneEditor;

  ZoneEditor.prototype.validateZone = function(val)
  {
      console.log("Inside validate Zone");
    if( 'proposed.zoneName' in val.$rows[0].$data)
    {
      var address = val.$rows[0].$data['proposed.zoneName'];
      if(address === null || address.toString() === "")
      {
        throw lex.get('mgr.validation.zoneName.min.length');
      }
      if(address !== null && address.toString().length > 50)
      {
        throw lex.get('mgr.validation.zoneName.max.length');
      }
    }

    if( 'proposed.zoneLocation' in val.$rows[0].$data)
    {
      var customerName = val.$rows[0].$data['proposed.zoneLocation'];
      if(customerName === null || customerName.toString() === "")
      {
        throw lex.get('mgr.validation.zoneLocation.min.length');
      }
      if(customerName !== null && customerName.toString().length > 50)
      {
        throw lex.get('mgr.validation.zoneLocation.max.length');
      }
    }
  };


  return ZoneEditor;
});
