/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Gunjan Moghe
 */

/**
 * @private
 * @module nmodule/electronicSignature/rc/wb/view/RemoteAuthenticationView
 */

define(['baja!',
  'baja!electronicSignature:RemoteRequestParameter,electronicSignature:SecondaryRemoteAuthentication,'+
  'baja:AbsTime,baja:Component,'+
  'control:Override,electronicSignature:SecureNumericOverride,electronicSignature:SecureEnumOverride,'+
  'electronicSignature:SecureStringOverride,electronicSignature:SecureAuto,electronicSignature:SecureBoolean,'+
  'electronicSignature:SecureActiveInactive,electronicSignature:SecureBooleanOverride,electronicSignature:SecureFacet,'+
  'electronicSignature:SecureEnum,electronicSignature:SecureDouble,electronicSignature:SecureString,'+
  'bql:DynamicTimeRange',
  'lex!baja',
  'lex!electronicSignature',
  'jquery', 'Promise', 'dialogs',
  'bajaux/Widget', 'underscore',
  'bajaux/events',
  'nmodule/webEditors/rc/fe/baja/BaseEditor',
  'nmodule/electronicSignature/rc/fe/baja/loader',
  'nmodule/bql/rc/fe/DynamicTimeRangeEditor',
  'bajaux/mixin/subscriberMixIn',
  "nmodule/webEditors/rc/fe/fe",
  'nmodule/electronicSignature/rc/fe/baja/Util',
  'hbs!nmodule/electronicSignature/rc/wb/view/template/RemoteAuthenticationView',
  'hbs!nmodule/electronicSignature/rc/wb/view/template/RemoteAuthenticationRequests',
  'hbs!nmodule/electronicSignature/rc/wb/view/template/RemoteAuthenticationHeader',
  'nmodule/electronicSignature/rc/fe/baja/SecuredActiveInactiveEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredAutoEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredBooleanEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredBooleanOverrideEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredEnumEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredEnumOverrideEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredFacetsEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredNumericEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredNumericOverrideEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredStringEditor',
  'nmodule/electronicSignature/rc/fe/baja/SecuredStringOverrideEditor',
  'nmodule/webEditors/rc/fe/feDialogs'
],
  function (
  baja, types, lexs, electronicSignatureLex,
  $, Promise, dialogs,
  Widget, _, events,
  BaseEditor, loader, DynamicTimeRangeEditor, subscriberMixIn,
  fe, Util,
  template, requestTemplate,
  headerTemplate,SecuredActiveInactiveEditor,
  SecuredAutoEditor, SecuredBooleanEditor,
  SecuredBooleanOverrideEditor, SecuredEnumEditor,
  SecuredEnumOverrideEditor, SecuredFacetsEditor,
  SecuredNumericEditor, SecuredNumericOverrideEditor,
  SecuredStringEditor, SecuredStringOverrideEditor,
  feDialogs)
  {
  'use strict';
  var lex = electronicSignatureLex[0];
  var autoRefreshTimer = null;

  var remoteAuthentication, timeRangeEditor, requestList, downJaces;
  var showProcessedRequest=true;
  var feMap = {};

  function fillFEMap()
  {
    feMap["electronicSignature:SecureBoolean"] =  SecuredBooleanEditor;
    feMap["electronicSignature:SecureAuto"] = SecuredAutoEditor;
    feMap["electronicSignature:SecureActiveInactive"] =  SecuredActiveInactiveEditor;
    feMap["electronicSignature:SecureBooleanOverride"] =  SecuredBooleanOverrideEditor;
    feMap["electronicSignature:SecureFacet"] =  SecuredFacetsEditor;
    feMap["electronicSignature:SecureEnum"] =  SecuredEnumEditor;
    feMap["electronicSignature:SecureEnumOverride"] = SecuredEnumOverrideEditor;
    feMap["electronicSignature:SecureDouble"] = SecuredNumericEditor;
    feMap["electronicSignature:SecureNumericOverride"] = SecuredNumericOverrideEditor;
    feMap["electronicSignature:SecureString"] = SecuredStringEditor;
    feMap["electronicSignature:SecureStringOverride"] = SecuredStringOverrideEditor;
  }

  var RemoteAuthenticationView = function RemoteAuthenticationView(params)
  {
    var that = this;
    BaseEditor.call(this, $.extend({
      keyName: 'RemoteAuthenticationView'
    }, params));

    subscriberMixIn(that);

    fillFEMap();
  };

  RemoteAuthenticationView.prototype = Object.create(BaseEditor.prototype);
  RemoteAuthenticationView.prototype.constructor = RemoteAuthenticationView;

  RemoteAuthenticationView.prototype.doInitialize = function () {
    var that = this, dom = that.jq(), context = {};
    showProcessedRequest=true;
    context.approvedCount = 0;
    context.pendingCount = 0;
    context.requests = [];
    context.showProcessedRequest=lex.get('showProcessedRequests.checkbox.label');
    context.autoRefresh=lex.get('autoRefresh.checkbox.label');

    dom.html(template(context));

    $(document).on('change', ':checkbox', function() { // jshint ignore:line
      if(this.id === "showProcessedRequest")
      {
        if (this.checked)
        {
          showProcessedRequest=true;
        }else
        {
         showProcessedRequest=false;
        }
      }
      // When showProcessedRequest checkbox is changed call filterRequests
      return timeRangeEditor.read().
        then(function(component){
        return that.filterRequests(component);
      });
     });

    return that.loadTimeRange(that.jq().find("#timeRangeEditor"));
  };

  RemoteAuthenticationView.prototype.doLoad = function (value)
  {
    // Build list of remote authentication requests
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //
    //  Pending request - 4/Approved Request - 2
    //
    //    Timestamp:20-Nov-17 12:17:07.820 PM                           Request Pending for MANAGER
    //    Point ORD: slot:/Drivers/TwoLevelSign/SBooleanWritable
    //    Value at the time of request: false
    //    Requested value: true
    //    Reason: R3
    //    Action: setWithAuthentication
    //    Primary authenticator: OPERATOR
    //
    //    Timestamp:20-Nov-17 12:27:07.820 PM                           Request Pending for MANAGER
    //    Point ORD: slot:/Drivers/TwoLevelSign/SBooleanWritable
    //    Value at the time of request: false
    //    Requested value: true
    //    Reason: R3
    //    Action: setWithAuthentication
    //    Primary authenticator: OPERATOR
    //
    ////////////////////////////////////////////////////////////////////////////////////////////////
    const that = this;
    remoteAuthentication = value;

    $('#autoRefresh').click(function(){
      if($('#autoRefresh')[0].checked)
      {
        remoteAuthentication.loadSlots().then(function(){
          autoRefreshTimer = setInterval(()=>{getFilterRequests(remoteAuthentication, that);},
              remoteAuthentication.getRefreshInterval().getMillis());
        });
      }
      else
      {
        if(autoRefreshTimer != null){
          clearInterval(autoRefreshTimer);
          autoRefreshTimer = null;
        }
      }
    });

    return getFilterRequests(value, that);
  };

  function getFilterRequests(value, that){
    return value.loadSlots().then(function(){
      return timeRangeEditor.read();
    }).then(function(component){
      return that.filterRequests(component);
    }).then(function(){
      return value;
    });
  }

    RemoteAuthenticationView.prototype.destroy = function () {
      if(autoRefreshTimer != null){
        clearInterval(autoRefreshTimer);
        autoRefreshTimer = null;
      }
      return BaseEditor.prototype.destroy.apply(this, arguments);
    };

  /**
   * This method creates the remote secondary request details grid with all required details.
   * @param component
   */
  RemoteAuthenticationView.prototype.makeRequestDetailsGrid = function(component)
  {
    var that = this;
    var requestJSON = {};
    requestJSON.componentHandle = component.get("componentHandle").toString();
    requestJSON.componentTypeSpec = component.getType().getTypeSpec().toString();
    requestJSON.FeComponentName = component.getName().toString();
    requestJSON.isPending = component.get("isPending").toString();
    requestJSON.showButtons = requestJSON.isPending === "true";
    requestJSON.primaryComments = "NA";
    if(component.getSlot("primaryComments") != null && component.get("primaryComments").toString() != "")
    {
      requestJSON.primaryComments = component.get("primaryComments").toString();
    }

    requestJSON.stationName = "local";
    if(component.getSlot("remoteJace") != null && component.get("remoteJace").toString() != "")
    {
      requestJSON.stationName = component.get("remoteJace").toString();
    }

    requestJSON.timeStamp = component.get("requestTime").toString();
    requestJSON.slotPath = component.get("pointOrd").toString();
    //requestJSON.oldValue = component.get("oldValue").toString();
    requestJSON.reason = component.get("reasonForChange").toString();
    requestJSON.actionName = component.get("actionName").toString();
    requestJSON.primaryAuthenticator = component.get("userName").toString();
    requestJSON.requestStatus = component.get("pointOrd").toString();
    requestJSON.rejectionComment = "";
    requestJSON.timeStampLabel = lex.get("timeStamp.label");
    requestJSON.remotePointOrdLabel = lex.get("remote.point.ord.label");
    requestJSON.atTheTimeOfRequestLabel = lex.get("value.atTheTimeOfRequest.label");
    requestJSON.RequestValueLabel = lex.get("remote.requested.value");
    requestJSON.ReasonLabel = lex.get("reason.label");
    requestJSON.ActionLabel = lex.get("remote.action.Label");
    requestJSON.levelOneAuthenticator = lex.get("prim.authenticator.label");
    requestJSON.levelOneCommentsLabel = lex.get("prim.authenticator.comments.label");
    requestJSON.stationNameLabel = lex.get("remote.station.label");

    if (component.getType().is("control:Override") || component.getType().is("electronicSignature:SecureNumericOverride") || component.getType().is("electronicSignature:SecureEnumOverride") || component.getType().is("electronicSignature:SecureStringOverride"))
    {
      requestJSON.requestedValue = component.get("value").toString();
      requestJSON.oldValue = component.get("oldValue").toString();
    }
    else if(component.getType().is("electronicSignature:SecureAuto"))
    {
      requestJSON.requestedValue = "NA";
      requestJSON.oldValue = component.get("oldValue").toString();
    }
    else if(component.getType().is("electronicSignature:SecureActiveInactive"))
    {
      //Fix for PROSVC-8008-	E - trueText and falseText facet values not shown in secondary authentication
      /* requestJSON.requestedValue = component.get("newValue").toString();
       requestJSON.oldValue = component.get("oldValueString").toString();*/
      var requestedValue = component.get("newValue").toString();
      var oldValue = component.get("oldValueString").toString();
      requestJSON.requestedValue = getTrueFalseText(component,requestedValue);
      requestJSON.oldValue = getTrueFalseText(component,oldValue);
    }
    else if(component.getType().is("electronicSignature:SecureBooleanOverride"))
    {
      //Fix for PROSVC-8008-	E - trueText and falseText facet values not shown in secondary authentication
      /*requestJSON.requestedValue = component.get("value").toString();
      requestJSON.oldValue = component.get("oldValueString").toString();*/
      var requestedValue = component.get("value").toString();
      var oldValue = component.get("oldValueString").toString();
      requestJSON.requestedValue = getTrueFalseText(component,requestedValue);
      requestJSON.oldValue = getTrueFalseText(component,oldValue);
    }else if(component.getType().is("electronicSignature:SecureBoolean"))
    {
      //Fix for PROSVC-8008-	E - trueText and falseText facet values not shown in secondary authentication
      /* requestJSON.requestedValue = component.get("newValue").toString();
        requestJSON.oldValue = component.get("oldValue").toString();*/
      var requestedValue = component.get("newValue").toString();
      var oldValue = component.get("oldValue").toString();
      requestJSON.requestedValue = getTrueFalseText(component,requestedValue);
      requestJSON.oldValue = getTrueFalseText(component,oldValue);
    }
    else
    {
      requestJSON.requestedValue = component.get("newValue").toString();
      requestJSON.oldValue = component.get("oldValue").toString();
    }

    if(component.get("processSecondary").toString() === "true" && component.get("isSecondaryApproved").toString() === "false")
    {
      if(component.get("rejectedComments")!==null)
      {
        requestJSON.rejectionComment = "Reason for rejection : " + component.get("rejectedComments").toString();
      }
    }

    if(component.get("isPending").toString() === "true")
    {
      requestJSON.requestStatus = "Request Pending  for " + component.get("manager").toString();
    }
    else if(component.get("processSecondary").toString() === "true" && component.get("isSecondaryApproved").toString() === "true")
    {
      requestJSON.requestStatus = "Request Approved by " + component.get("manager").toString();
    }
    else if(component.get("processSecondary").toString() === "true"   && component.get("isSecondaryApproved").toString() === "false")
    {
      requestJSON.requestStatus = "Request  Rejected by " + component.get("manager").toString();
    }

    requestJSON.proxyComponentHandle = "";
    if(component.getSlot("remoteJace")!=null)
    {
      return that.getProxyComponent(component).then(function(proxyPoint){
        requestJSON.proxyComponentHandle =  proxyPoint.getHandle().toString();
        if(component.getSlot("proxyComponentHandle")==null)
          component.add({slot:"proxyComponentHandle", value: proxyPoint.getHandle().toString()});
        else
          component.set({slot:"proxyComponentHandle", value: proxyPoint.getHandle().toString()});

        return requestJSON;
      });
    }
    else
    {
      return requestJSON;
    }
   };

    /**
     * This method returns true and false text for provided value
     * @param component
     * @param value
     * @returns {string}
     */
  function getTrueFalseText(component,value) {
    var trueText =  component.getSecureSlotFacets().$map.get("trueText");
    var falseText =  component.getSecureSlotFacets().$map.get("falseText");
    var newValText;
    if(trueText === null || trueText === undefined)
    {
      trueText = "true";
    }
    if(falseText === null || falseText === undefined)
    {
      falseText = "false";
    }
    if(value === "true"){
      newValText=trueText;
    }else if(value === "false"){
      newValText=falseText;
    }else if(value === "Null"){
      newValText="Null";
    }
    return newValText;

  }

  /**
   * Filters the secondary approval requests depending on whether currently logged in user
   * has ri permissions for operator level actions and RI permission for non-operator level actions.
   * @param secondaryRequest
   * @return True if the currently logged in user has rights to perform the action.
   */
  RemoteAuthenticationView.prototype.hasPermissions = function(secondaryRequest)
  {
    var hasPermission = false;

    // 1. Create a string in below format:
    // componentHandle;userName;actionName
    var parameter =secondaryRequest.get("componentHandle").toString() + ";" + secondaryRequest.get("manager").toString() +
        ";" + secondaryRequest.get("actionName").toString() + ";" + (baja.getUserName()==""?remoteAuthentication.getLoginUserName():baja.getUserName());

    // 2. Invoke the action hasPermission.
    hasPermission = remoteAuthentication.hasPermission(parameter);

    return hasPermission;
  };

  RemoteAuthenticationView.prototype.loadTimeRange = function(timeRangeDom)
  {
    var that = this;
    return fe.buildFor({
      dom: timeRangeDom,
      value: baja.$('bql:DynamicTimeRange', 'today'),
      type: DynamicTimeRangeEditor,
      formFactor: "mini"
    }).then(function(editor){
       timeRangeEditor = editor;

       timeRangeEditor.jq().on(events.MODIFY_EVENT, function(){
          return timeRangeEditor.read()./*then(function(timeRange){
            return remoteAuthentication.getTimeRange(timeRange);
          }).*/then(function(component){
            return that.filterRequests(component);
          });
       });
       return timeRangeEditor;
    });
  };

    function getUniqueHandle(secondaryRequest) {
      if(remoteAuthentication.getShouldStoreStationRemoteRequestInBogFile())
      {
        return secondaryRequest.get("remoteRequestHandle").toString();
      }
      else {
        return secondaryRequest.get("uniqueHash").toString();
      }
    }

    RemoteAuthenticationView.prototype.filterRequests = function(timeRange)
  {
      var that = this;
      var approvedCount = 0, pendingCount = 0, rejectCount = 0;
      requestList = baja.$('baja:Component');
      return remoteAuthentication.loadSlots().then(function(){
        return remoteAuthentication.lease();
      }).then(function(){
        var parameter = baja.$('electronicSignature:RemoteRequestParameter');
        parameter.setSearchJaces(true);
        parameter.setTimeRange(timeRange);
        parameter.setUser(baja.getUserName());

        return remoteAuthentication.getRemoteRequests(parameter);
      }).then(function(requests){
        requestList = requests;
        if(requestList.getSlot("downJaces") != null)
        {
          downJaces = requestList.get("downJaces").toString();
          return requestList.remove("downJaces").then(function () {
            return requests;
          });
        }
        return requests;
      }).then(function(requests){
        var permissionPromiseList = [];

        requestList.getSlots().is("baja:Complex").eachValue(function (secondaryRequest)
        {
          if(secondaryRequest.getSlot("remoteJace") != null)
          {
            permissionPromiseList.push(Promise.resolve({handle:getUniqueHandle(secondaryRequest), hasPermission: true }));
          }
          else
          {
            var promise = that.hasPermissions(secondaryRequest).then(function(result){ return {handle:getUniqueHandle(secondaryRequest), hasPermission: result };});
            permissionPromiseList.push(promise);
          }
        });

        return Promise.all(permissionPromiseList);
      }).then(function(permissionPromiseList){
        var timRangePromises = [];
        requestList.getSlots().is("baja:Complex").eachValue(function (secondaryRequest)
        {
          var permissionObject = _.find(permissionPromiseList, function(p)
          {
            return p.handle === getUniqueHandle(secondaryRequest);
          });
          permissionObject.isWithinTimeRange = true;
          timRangePromises.push(permissionObject);
        });
        return Promise.all(timRangePromises);
      }).then(function(permissionPromiseList) {
        var requests = [];
        var propertyArray =  requestList.getSlots().is("baja:Complex").toValueArray();
        var ArrayLength = propertyArray.length;
        propertyArray.sort((com1,com2) => {
          if ((com1.get("requestTime").getMillis()) > (com2.get("requestTime").getMillis())){
            return 1;
          }
          if ((com1.get("requestTime").getMillis()) < (com2.get("requestTime").getMillis())){
            return -1;
          }
          return 0;
        });
        for (var i = ArrayLength-1; i >= 0; i--) {
          var secondaryRequest = propertyArray[i];
          var permissionObject = _.find(permissionPromiseList, function (p) {
            return p.handle === getUniqueHandle(secondaryRequest);
          });

          if (permissionObject && permissionObject.hasPermission && permissionObject.isWithinTimeRange) {
            if (secondaryRequest.get("isSecondaryApproved").toString() === "true") {
              approvedCount = approvedCount + 1;
            } else {
              if (secondaryRequest.get("isPending").toString() === "false") {
                rejectCount = rejectCount + 1;
              }
            }

            if (secondaryRequest.get("isPending").toString() === "true") {
              pendingCount = pendingCount + 1;
            }
            var showRequest=true;
            if((!showProcessedRequest) && (secondaryRequest.get("isPending").toString()!=="true"))
            {
              showRequest=false;
            }
            if (showRequest) {
              var requestJSON  = that.makeRequestDetailsGrid(secondaryRequest);
             requests.push(requestJSON);
            }
           // requests.push(requestJSON);
          }
        };
        return Promise.all(requests);
      }).then(function(requests){
        var context = {};
        context.approvedCount = approvedCount;
        context.rejectCount = rejectCount;
        context.pendingCount = pendingCount;
        context.requests = requests;
        context.pendingRequestLabel = lex.get('pending.request.label');
        context.approvedRequestLabel = lex.get('approved.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');

        context.timeStampLabel = lex.get('timeStamp.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.declinedRequestLabel = lex.get('declined.request.label');
        context.messageforUnreachable = lex.get('Remote.unreachable');
        context.hasDownJaces = false;
        if(downJaces!=null && downJaces.length>0)
        {
          context.hasDownJaces = true;
          context.downJaces = downJaces;
        }
        return context;
      }).then(function(context){
        var dom = that.jq();
        dom.find("#requestGrid").empty();
        dom.find("#requestGrid").html(requestTemplate(context));
        dom.find("#requestHeader").empty();
        dom.find("#requestHeader").html(headerTemplate(context));
        return Promise.resolve(context);
      }).then(function(){
        var dom = that.jq();
        dom.find("#requestGrid button").each(function(){
          if($(this).attr("isPending") === "false")
            {
            $(this).attr("disabled","true");
            }
        });
        dom.find("#requestGrid button").click(function(){
          that.processRequest(this);
        });
        return Promise.resolve(true);
      }).then(function()
      {
        return remoteAuthentication;
      });
  };

  RemoteAuthenticationView.prototype.processRequest = function(button)
  {
    var that = this;
    var componentName = $(button).attr("parameterName");
    var sourceHandle = $(button).attr("sourceHandle");
    var parameterTypeSpec = $(button).attr("parameterTypeSpec");
    var proxyComponentHandle = $(button).attr("proxyComponentHandle");
    if(proxyComponentHandle != "")
    {
      sourceHandle = proxyComponentHandle;
    }
    var processAccept = false;
    var parameterForAction = null;
    if($(button).attr("process") === "accept")
    {
      processAccept = true;
    }
    var sourceComponent;
    var component = requestList.get(componentName);
    var parent= null;
    return component.loadSlots().then(function(){
      return baja.Ord.make("station:|h:"+sourceHandle).get({lease:true});
    }).then(function(source){
      sourceComponent = source;
      return sourceComponent.loadSlots();
    }).then(function(){
      parameterForAction = component.newCopy(true);
      return parameterForAction;
    }).then(function(s){
      var rejected = !processAccept;
      if(!parameterForAction.has("isRejected"))
      {
        return parameterForAction.add({slot:"isRejected", value:rejected});
      }
      else
      {
        return parameterForAction.set({slot:"isRejected", value:rejected});
      }
    }).then(function(){
      var param = sourceComponent.getHandle().toString() + ";" + component.get("actionName").toString();
      return remoteAuthentication.getSecureFacets(param);
    }).then(function(facets){

      var editor = feMap[parameterTypeSpec];
      //var parameterForAction = component.newCopy(true);
      var remoteFacets = baja.Facets.make(["IsRemoteRequest"],[true]);
      facets = baja.Facets.make(facets, remoteFacets);
      return parameterForAction.setSecureSlotFacets(facets).then(function(){
        return feDialogs.showFor({
          value: parameterForAction,
          type: editor
        });
      });
    }).then(function(parameterForAction){
      if (parameterForAction !== null)
      {
        return remoteAuthentication.lease().then(function(){
          return parameterForAction.loadSlots();
        }).then(function(){
          return parameterForAction.setProcessSecondary(true);
        }).then(function(){
          return parameterForAction.setIsValidUser(true);
        }).then(function(){
          return parameterForAction.setIsSecondaryApproved(processAccept);
        }).then(function(){
          return parameterForAction.setIsPending(false);
        }).then(function(){
          return remoteAuthentication.lease();
        })/*.then(function(){
          //return remoteAuthentication.set({slot: componentName,value:parameterForAction});
          return baja.$("baja:Component", {
            FeComponentName: baja.$("baja:String").make(componentName),
            parameterForAction: parameterForAction,
          });
        }).then(function(tempComp){
          return remoteAuthentication.invoke({slot:"processRemoteRequest", value: tempComp});
        })*/.then(function(){
           parent= remoteAuthentication.getParent();
           return parent.lease(3);
        }).then(function(){
          return parent.loadSlots(3);
        }).then(function(){
            remoteAuthentication= parent.get("secondaryRemoteAuthentication");
            return remoteAuthentication.lease(3);
        }).then(function(){
           return remoteAuthentication.loadSlots();
        }).then(function(){
          //if(processAccept)
          //{
            return sourceComponent.invoke({slot:parameterForAction.getActionName().toString(), value: parameterForAction});
          //}
          //else
          //{
          //  return Promise.resolve(true);
          //}
        }).then(function(){
            return timeRangeEditor.read();
        })/*.then(function(timeRange){
           return remoteAuthentication.getTimeRange(timeRange);
        })*/.then(function(component){
           return setTimeout(()=>{that.filterRequests(component)},1000);
        }).then(function(){
          return parameterForAction;
        });
      }
      else
      {
        return null;
      }
    });
  };

  RemoteAuthenticationView.prototype.getProxyComponent = function(component)
  {
    var that = this;
    if(component == null || component.getSlot("remoteJace")==null)
      return null;

    return Util.getComponentByOrd("station:|slot:/Drivers/NiagaraNetwork").then(function(niagaraNetwork){
          return niagaraNetwork.loadSlots().then(function(){
            return niagaraNetwork;
          });
    }).then(function(loadedNetwork){
      var station = null;
      loadedNetwork.getSlots().is("baja:Complex").eachValue(function (niagaraStation)
      {
        if(niagaraStation.getName() === component.get("remoteJace").toString())
        {
          station = niagaraStation;
        }
      });
      return station;
    }).then(function(niagaraStation){
      if(niagaraStation == null)
        return null;
      var ord = "station:|slot:|bql:select toPathString from electronicSignatureRemote:RemoteSecureNiagaraProxyExt where pointId='" + component.get("pointOrd").toString() + "'";
      return Util.getComponentByOrd(ord);
    }).then(function(pointsTable){
      var proxyComponentOrd = null;
      return pointsTable.cursor({
        each: function(){
          proxyComponentOrd = this.get("toPathString");
        },
        limit: 1
      }).then(function(){
        return proxyComponentOrd;
      });
    }).then(function(proxyComponentOrd){
      return Util.getComponentByOrd("station:|slot:"+proxyComponentOrd);
    }).then(function (proxy) {
      return proxy.getParent();
    }).then(function (parent) {
      return parent.loadSlots().then(function () {
        return parent;
      });
    });

  };

  return RemoteAuthenticationView;
});